Package: diffHTS
Type: Package
Title: Differential Drug Sensitivity Analysis for Two-Condition
        High-Throughput Screens
Version: 0.1.0
Authors@R: 
    person("Liang", "Tang", email = "xphdtangliang@gmail.com", role = c("aut", "cre"))
Description: A complete workflow for large-scale, two-condition
    high-throughput drug screening (HTS). It compares drug sensitivity between
    any two experimental conditions - for example irradiated versus
    non-irradiated cells, cancer versus normal cell lines, or treated versus
    untreated samples - across many plates and experiments. The package covers
    the full pipeline: control-based normalisation, plate-level quality-control
    metrics (Z-factor, Z-prime, signal-to-background, signal-to-noise and
    strictly standardised mean difference), replicate-consistency checks,
    four-parameter logistic dose-response fitting with area under the curve
    (AUC) estimation, differential (delta) AUC scoring with within-plate
    standardisation, cut-off and sigma-based hit selection, and
    publication-ready heatmap, scatter and quality-control visualisations.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1)
Imports: tidyr, rlang, ggplot2, graphics, stats, utils
Suggests: ComplexHeatmap, circlize, ggprism, ggrepel, grid, readxl,
        testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-18 17:37:57 UTC; wd979071
Author: Liang Tang [aut, cre]
Maintainer: Liang Tang <xphdtangliang@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-28 16:10:08 UTC
Built: R 4.5.2; ; 2026-07-28 18:42:34 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(coef,dsr_4pl)
S3method(fitted,dsr_4pl)
S3method(predict,dsr_4pl)
S3method(print,dsr_4pl)
S3method(residuals,dsr_4pl)
export(annotate_hit_info)
export(apply_plate_layout)
export(baseline_subtract)
export(build_auc_matrix)
export(calc_cv)
export(calc_drc_auc)
export(calc_plate_qc)
export(calc_replicate_correlation)
export(calc_replicate_cv)
export(calc_robust_z_prime)
export(calc_sb_ratio)
export(calc_sn_ratio)
export(calc_ssmd)
export(calc_well_zscore)
export(calc_z_factor)
export(calc_z_prime)
export(calculate_qc_metrics)
export(check_control_label)
export(clean_compound_id)
export(clean_filename)
export(cluster_auc_matrix)
export(compute_auc)
export(compute_delta_auc)
export(convert_conc_log10)
export(detect_outlier_wells)
export(export_hit_table)
export(extract_cluster_hit)
export(extract_drc_params)
export(filter_bad_replicate)
export(filter_low_quality_curve)
export(filter_valid_plates)
export(fit_4pl_curve)
export(fit_dose_response)
export(flag_qc_plates)
export(four_pl)
export(generate_hts_report)
export(hts_pal)
export(import_dose_response)
export(merge_plate_data)
export(norm_by_control)
export(plate_layout)
export(plate_layout_1536)
export(plate_layout_384)
export(plot_auc_heatmap)
export(plot_batch_drc_overlay)
export(plot_cluster_tree)
export(plot_condition_scatter)
export(plot_control_scatter)
export(plot_cv_distribution)
export(plot_delta_auc_heatmap)
export(plot_dose_response_curves)
export(plot_hit_bar_count)
export(plot_hit_stratify_bar)
export(plot_ic50_auc_cor)
export(plot_inhibition_hist)
export(plot_plate_heatmap)
export(plot_plate_heatmap_inhibition)
export(plot_plate_heatmap_raw)
export(plot_plate_qc)
export(plot_primary_inhibition_rank)
export(plot_primary_secondary_cor)
export(plot_qc_boxplot)
export(plot_qc_circular_heatmap)
export(plot_qc_metric_circular)
export(plot_qc_metric_trend)
export(plot_replicate_scatter)
export(plot_sigma_hits)
export(plot_single_drc)
export(rank_hit_compound)
export(read_hts_plate)
export(read_plate_layout)
export(select_hits_cutoff)
export(select_hits_sigma)
export(select_primary_hit)
export(select_sigma_hits)
export(summarize_plate_setup)
export(summarize_primary_hit)
export(summarize_sigma_hits)
export(theme_hts)
export(windows_to_linux_path)
importFrom(rlang,.data)

# diffHTS

<!-- badges: start -->
<!-- badges: end -->

**diffHTS** provides reusable methods for **large-scale**, **two-condition**
high-throughput drug screening (HTS). It compares drug sensitivity between any
two experimental conditions — for example irradiated versus non-irradiated
cells, cancer versus normal cell lines, or treated versus untreated samples —
across many plates and experiments, in a documented and tested toolkit.

## Installation

Install from a local source checkout:

```r
# install.packages("remotes")
remotes::install_local("diffHTS")

# or, from the built source tarball
install.packages("diffHTS_0.1.0.tar.gz", repos = NULL, type = "source")
```

Dose-response fitting is implemented in base R, so no external fitting package
is needed. Some visualisation and I/O features use packages that are only
*suggested*: the Bioconductor packages `ComplexHeatmap` and `circlize`
(heatmaps), `ggrepel` (labelled scatter plots), `readxl` (Excel import) and
`rmarkdown` (HTML reports).

## Standard pipeline

**diffHTS** organises a complete HTS analysis into seven modules that take raw
plate readouts all the way to an annotated, ranked hit list. It supports both
single-concentration **primary** screens and gradient **secondary**
(dose-response) confirmation screens. Each module pairs business functions with
matching visualisation functions.

| Module | Business functions | Visualisation |
|--------|--------------------|---------------|
| **1. Import & pre-QC** | `read_plate_layout()`, `apply_plate_layout()`, `read_hts_plate()`, `summarize_plate_setup()`, `baseline_subtract()`, `detect_outlier_wells()`, `calc_z_prime()`, `calc_robust_z_prime()`, `calc_z_factor()`, `calc_ssmd()`, `calc_sb_ratio()`, `calc_sn_ratio()`, `calc_cv()`, `calc_plate_qc()`, `calc_well_zscore()`, `filter_valid_plates()` | `plot_plate_heatmap_raw()`, `plot_plate_qc()` |
| **2. Normalisation** | `norm_by_control()`, `merge_plate_data()` | `plot_plate_heatmap_inhibition()`, `plot_inhibition_hist()` |
| **3. Replicate consistency** | `calc_replicate_cv()`, `calc_replicate_correlation()`, `filter_bad_replicate()` | `plot_replicate_scatter()`, `plot_cv_distribution()` |
| **4. Primary hit selection** | `select_primary_hit()`, `select_sigma_hits()`, `summarize_primary_hit()`, `summarize_sigma_hits()` | `plot_primary_inhibition_rank()`, `plot_hit_bar_count()`, `plot_sigma_hits()` |
| **5. Dose-response & AUC** | `import_dose_response()`, `fit_4pl_curve()`, `extract_drc_params()`, `calc_drc_auc()`, `filter_low_quality_curve()` | `plot_single_drc()`, `plot_batch_drc_overlay()`, `plot_ic50_auc_cor()` |
| **6. AUC matrix & clustering** | `build_auc_matrix()`, `cluster_auc_matrix()`, `extract_cluster_hit()` | `plot_auc_heatmap()`, `plot_cluster_tree()` |
| **7. Ranking, annotation & report** | `rank_hit_compound()`, `annotate_hit_info()`, `export_hit_table()`, `generate_hts_report()` | `plot_hit_stratify_bar()`, `plot_primary_secondary_cor()` |

**Utilities**: `plate_layout()` / `plate_layout_384()` / `plate_layout_1536()`,
`convert_conc_log10()`, `check_control_label()`, `clean_compound_id()`.

The original two-condition (differential) analysis remains available:
`four_pl()`, `compute_auc()`, `fit_dose_response()`, `compute_delta_auc()`,
`select_hits_cutoff()`, `select_hits_sigma()`, `calculate_qc_metrics()`,
`plot_dose_response_curves()`, `plot_delta_auc_heatmap()`,
`plot_condition_scatter()` and the QC plots.

## Quick start

```r
library(diffHTS)

## --- Primary screen: raw signals -> hits ---------------------------------
## Author a plate map (which wells are NC / PC / blank / compound) yourself,
## then stamp it onto the instrument readings:
layout <- read_plate_layout(
  system.file("extdata", "plate_layout_example.csv", package = "diffHTS"),
  plate_id = "P01")
# raw <- apply_plate_layout(signal_export, layout)  # merge roles onto readings

raw  <- read_hts_plate(hts_primary_raw) # three 96-well plates in one object
summarize_plate_setup(raw)             # per-plate layout: controls & compounds
raw  <- baseline_subtract(raw)
qc   <- calc_plate_qc(raw)             # full QC panel: Z'/robust-Z'/SSMD/S:B/S:N/CV%
plot_plate_qc(qc, metric = "ssmd")     # one plot fn, choose any metric
good <- filter_valid_plates(raw)       # drop failed plates (P03)
norm <- norm_by_control(good)          # % inhibition vs NC/PC
hits <- select_primary_hit(norm, threshold = 50)
summarize_primary_hit(hits)

## --- Secondary screen: dose-response -> ranked hits ----------------------
dr   <- import_dose_response(hts_dose_response,
                             response_col = "viability",
                             group_cols   = "cell_line")
drc  <- calc_drc_auc(fit_4pl_curve(dr, group_cols = "cell_line"))
params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])

ranked <- rank_hit_compound(params)                 # Strong/Moderate/Weak
annotate_hit_info(ranked, hts_compound_meta)        # add target/MoA
plot_single_drc(drc, drc$meta$curve_id[1])          # one fitted curve
plot_auc_heatmap(build_auc_matrix(drc$meta, zscore = "row"))
```

## Example data

- `hts_primary_raw` — simulated 96-well primary screen with raw signals
  (plates P01/P02 good, P03 failed).
- `hts_dose_response` — simulated gradient dose-response screen (10 compounds,
  3 cell lines, 2 replicates).
- `hts_compound_meta` — simulated compound annotation table.
- `screen_doseresponse` — a small simulated two-condition dose-response screen.
- `screen_delta_auc` — a small simulated differential-AUC table.
- `screen_plate_qc`, `screen_plate_layout` — QC / plate-layout demo data.

All are fully simulated (see `data-raw/make_datasets.R`) and do not correspond
to any real compound or cell line.

## Vignette

See `vignette("diffHTS")` for an end-to-end walkthrough.

## License

GPL-3

# Module 3: Replicate consistency analysis -----------------------------------

# Mean value per compound per plate (compound wells only).
.compound_plate_means <- function(data, value, compound_col, plate_col,
                                   compound_label = "compound") {
  df <- as.data.frame(data, stringsAsFactors = FALSE)
  if ("well_type" %in% names(df)) {
    df <- df[df$well_type == compound_label, , drop = FALSE]
  }
  for (col in c(value, compound_col, plate_col)) {
    if (!col %in% names(df)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }
  agg <- stats::aggregate(
    df[[value]],
    by = list(compound_id = df[[compound_col]], plate_id = df[[plate_col]]),
    FUN = function(z) mean(.as_numeric(z), na.rm = TRUE)
  )
  names(agg)[3] <- "value"
  agg
}

#' Replicate coefficient of variation per compound
#'
#' Computes, for each compound, the coefficient of variation (CV, percent)
#' across its technical replicates (repeated wells, whether on the same plate or
#' across plates), a standard measure of assay reproducibility.
#'
#' @param data An `hts_norm` object or a long data frame of well-level values.
#' @param value Name of the value column on which to compute the CV. Default
#'   `"viability"` (more stable than inhibition, which can sit near zero).
#' @param compound_col Name of the compound column. Default `"compound_id"`.
#' @param group_cols Optional additional grouping columns (for example
#'   `"cell_line"`) that define what counts as one compound series.
#'
#' @return A data frame with one row per compound (and any `group_cols`):
#'   `compound_id`, the grouping columns, `n`, `mean`, `sd` and `cv` (percent).
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' cv <- calc_replicate_cv(norm)
#' head(cv[order(-cv$cv), ])
#'
#' @seealso [calc_replicate_correlation()], [filter_bad_replicate()]
#' @export
calc_replicate_cv <- function(data, value = "viability",
                              compound_col = "compound_id",
                              group_cols = NULL) {
  df <- as.data.frame(data, stringsAsFactors = FALSE)
  if ("well_type" %in% names(df)) {
    df <- df[df$well_type == "compound", , drop = FALSE]
  }
  for (col in c(value, compound_col, group_cols)) {
    if (!col %in% names(df)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }
  keys <- c(compound_col, group_cols)
  by_list <- stats::setNames(lapply(keys, function(k) df[[k]]), keys)
  v <- .as_numeric(df[[value]])
  agg_mean <- stats::aggregate(v, by_list, function(z) mean(z, na.rm = TRUE))
  agg_sd <- stats::aggregate(v, by_list, function(z) stats::sd(z, na.rm = TRUE))
  agg_n <- stats::aggregate(v, by_list, function(z) sum(!is.na(z)))
  out <- agg_mean
  names(out)[length(keys) + 1] <- "mean"
  out$sd <- agg_sd$x
  out$n <- agg_n$x
  out$cv <- 100 * out$sd / abs(out$mean)
  names(out)[1] <- "compound_id"
  out[, c("compound_id", group_cols, "n", "mean", "sd", "cv")]
}

#' Pairwise replicate correlation between plates
#'
#' Computes the Pearson correlation (and its \eqn{R^2}) between the per-compound
#' activity of every pair of replicate plates, the standard check that technical
#' replicates agree.
#'
#' @param data An `hts_norm` object or long data frame.
#' @param value Name of the value column. Default `"viability"`.
#' @param compound_col Name of the compound column. Default `"compound_id"`.
#' @param plate_col Name of the plate column. Default `"plate_id"`.
#'
#' @return A data frame with one row per plate pair: `plate_x`, `plate_y`, `n`,
#'   `pearson_r` and `r_squared`. The wide compound-by-plate matrix is attached
#'   as `attr(x, "wide")` for use by [plot_replicate_scatter()].
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' calc_replicate_correlation(norm)
#'
#' @seealso [calc_replicate_cv()], [plot_replicate_scatter()]
#' @export
calc_replicate_correlation <- function(data, value = "viability",
                                       compound_col = "compound_id",
                                       plate_col = "plate_id") {
  means <- .compound_plate_means(data, value, compound_col, plate_col)
  wide <- tidyr::pivot_wider(means, names_from = "plate_id",
    values_from = "value")
  wide <- as.data.frame(wide)
  plates <- setdiff(names(wide), "compound_id")
  if (length(plates) < 2) {
    stop("At least two plates are required to compute replicate correlation.",
      call. = FALSE)
  }
  pairs <- utils::combn(plates, 2)
  res <- lapply(seq_len(ncol(pairs)), function(j) {
    a <- pairs[1, j]
    b <- pairs[2, j]
    ok <- stats::complete.cases(wide[[a]], wide[[b]])
    r <- if (sum(ok) > 2) stats::cor(wide[[a]][ok], wide[[b]][ok]) else NA_real_
    data.frame(plate_x = a, plate_y = b, n = sum(ok),
      pearson_r = r, r_squared = r^2, stringsAsFactors = FALSE)
  })
  out <- do.call(rbind, res)
  attr(out, "wide") <- wide
  out
}

#' Drop compounds with poor replicate reproducibility
#'
#' Removes compounds whose replicate coefficient of variation exceeds a
#' threshold. When a correlation table is supplied it also checks that the
#' replicate plates are globally concordant, warning if any pair falls below the
#' \eqn{R^2} threshold.
#'
#' @param cv A data frame from [calc_replicate_cv()].
#' @param cv_max Maximum acceptable CV (percent). Default `15`.
#' @param cor_result Optional data frame from [calc_replicate_correlation()].
#' @param r2_min Minimum acceptable pairwise \eqn{R^2}. Default `0.8`.
#'
#' @return The subset of `cv` for compounds passing the CV threshold, with the
#'   removed compound ids stored in `attr(x, "removed")`.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' cv <- calc_replicate_cv(norm)
#' good <- filter_bad_replicate(cv, cv_max = 20)
#' attr(good, "removed")
#'
#' @seealso [calc_replicate_cv()], [calc_replicate_correlation()]
#' @export
filter_bad_replicate <- function(cv, cv_max = 15, cor_result = NULL,
                                 r2_min = 0.8) {
  if (!"cv" %in% names(cv)) {
    stop("`cv` must contain a 'cv' column (see `calc_replicate_cv()`).",
      call. = FALSE)
  }
  if (!is.null(cor_result) && "r_squared" %in% names(cor_result)) {
    bad_pairs <- cor_result[which(cor_result$r_squared < r2_min), , drop = FALSE]
    if (nrow(bad_pairs) > 0) {
      warning(nrow(bad_pairs), " plate pair(s) below R^2 = ", r2_min,
        "; replicate concordance is weak.", call. = FALSE)
    }
  }
  keep <- is.finite(cv$cv) & cv$cv <= cv_max
  out <- cv[keep, , drop = FALSE]
  attr(out, "removed") <- cv$compound_id[!keep]
  rownames(out) <- NULL
  out
}

# Default well-type -> colour map, mirroring the report palette
# (compounds blue, negative controls black, positive controls red, blanks gold).
.well_type_palette <- c(
  compound = "#0072B2", Experimental = "#0072B2", sample = "#0072B2",
  NC = "black", negative_ctrl = "black", DMSO = "black",
  PC = "#D55E00", positive_ctrl = "#D55E00", kill_ctrl = "#D55E00",
  Blank = "#E1B000", edgewell = "#E1B000", empty = "#E1B000"
)

# Build a compound/well-by-plate wide table, optionally retaining a type column
# so points can be coloured. Replicate plates share a layout, so wells are paired
# by their address; this keeps the control wells in the scatter, as in the
# source reps_corr_plot() workflow.
.replicate_wide <- function(data, value, plate_col, key_col, type_col) {
  df <- as.data.frame(data, stringsAsFactors = FALSE)
  for (col in c(value, plate_col, key_col)) {
    if (!col %in% names(df)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }
  v <- .as_numeric(df[[value]])
  by <- list(key = df[[key_col]], plate = df[[plate_col]])
  if (!is.null(type_col)) by <- c(by, list(type = df[[type_col]]))
  agg <- stats::aggregate(v, by, function(z) mean(z, na.rm = TRUE))
  names(agg)[ncol(agg)] <- "value"
  id_cols <- if (is.null(type_col)) "key" else c("key", "type")
  wide <- tidyr::pivot_wider(agg, id_cols = tidyr::all_of(id_cols),
    names_from = "plate", values_from = "value")
  as.data.frame(wide, stringsAsFactors = FALSE)
}

#' Replicate scatter plot(s) with regression fit and correlation
#'
#' Plots the activity of replicate plates against one another in the house style
#' used across the screening reports: points coloured by well type (compounds,
#' negative/positive controls, blanks) at 50% opacity, a linear regression line
#' with a shaded 95% confidence band, and a Pearson \eqn{R} (with p-value)
#' annotated on each panel. When more than two replicate plates are present and
#' no specific pair is requested, every pairwise combination (replicate 1 vs 2,
#' 1 vs 3, 2 vs 3, ...) is drawn together in a single faceted figure, mirroring
#' the `combn()`-based `reps_corr_plot()` workflow in the source analyses.
#'
#' @param data An `hts_norm`/long data frame, or a data frame from
#'   [calc_replicate_correlation()] (its wide matrix is used).
#' @param plate_x,plate_y Plate identifiers for the two axes. When both are
#'   `NULL` (default) all pairwise combinations of the available plates are
#'   drawn; supply both to draw a single pair.
#' @param value Name of the value column when `data` is well-level. Default
#'   `"viability"`.
#' @param plate_col Name of the plate column. Default `"plate_id"`.
#' @param key_col Column identifying matching observations across replicate
#'   plates. Defaults to the well-address column `"well"` when present (so that
#'   control wells are paired and displayed), otherwise the compound column.
#' @param compound_col Compound column, used as the pairing key when no `well`
#'   column is available. Default `"compound_id"`.
#' @param color_col Column used to colour points, typically the well-role column.
#'   Defaults to `"well_type"` when present; set to `NULL` to disable colouring.
#' @param palette Named vector mapping well-type levels to colours. Defaults to
#'   the report palette (compounds blue, negative controls black, positive
#'   controls red, blanks gold).
#' @param line_color,band_color Colour of the regression line and of its 95%
#'   confidence band. Default light blue on light grey, as in the reports.
#' @param show_identity Whether to draw the dotted \eqn{y = x} identity line.
#'   Default `TRUE`.
#'
#' @return A single [ggplot2::ggplot] object. With three or more plates it is
#'   faceted, one panel per replicate pair. The annotated \eqn{R} and p-value are
#'   computed over every point shown in the panel.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' # All pairwise replicate comparisons at once, points coloured by well type:
#' plot_replicate_scatter(norm)
#' # A single chosen pair:
#' plot_replicate_scatter(norm, plate_x = "P01", plate_y = "P02")
#'
#' @seealso [calc_replicate_correlation()]
#' @export
plot_replicate_scatter <- function(data, plate_x = NULL, plate_y = NULL,
                                    value = "viability",
                                    plate_col = "plate_id",
                                    key_col = NULL,
                                    compound_col = "compound_id",
                                    color_col = NULL,
                                    palette = .well_type_palette,
                                    line_color = "lightblue",
                                    band_color = "lightgray",
                                    show_identity = TRUE) {
  wide <- attr(data, "wide")
  type_present <- FALSE
  if (is.null(wide)) {
    df <- as.data.frame(data, stringsAsFactors = FALSE)
    if (is.null(color_col) && "well_type" %in% names(df)) color_col <- "well_type"
    if (is.null(key_col)) {
      key_col <- if ("well" %in% names(df)) "well" else compound_col
    }
    type_col <- if (!is.null(color_col) && color_col %in% names(df)) color_col else NULL
    wide <- .replicate_wide(df, value, plate_col, key_col, type_col)
    type_present <- !is.null(type_col)
  }
  reserved <- c("key", "type", "compound_id")
  plates <- setdiff(names(wide), reserved)
  if (length(plates) < 2) {
    stop("At least two plates are required for a replicate scatter plot.",
      call. = FALSE)
  }

  # Which plate pairs to draw: one requested pair, or every combination.
  if (!is.null(plate_x) || !is.null(plate_y)) {
    if (is.null(plate_x)) plate_x <- plates[1]
    if (is.null(plate_y)) plate_y <- plates[2]
    pairs <- matrix(c(plate_x, plate_y), nrow = 2)
  } else {
    pairs <- utils::combn(plates, 2)
  }

  type_vals <- if (type_present) wide$type else rep(NA_character_, nrow(wide))
  panels <- vector("list", ncol(pairs))
  labels <- vector("list", ncol(pairs))
  for (j in seq_len(ncol(pairs))) {
    a <- pairs[1, j]
    b <- pairs[2, j]
    for (p in c(a, b)) {
      if (!p %in% names(wide)) stop("Plate '", p, "' not found.", call. = FALSE)
    }
    lab_pair <- paste0(a, " vs ", b)
    d <- data.frame(x = wide[[a]], y = wide[[b]], type = type_vals,
      pair = lab_pair, stringsAsFactors = FALSE)
    d <- d[stats::complete.cases(d[, c("x", "y")]), , drop = FALSE]
    panels[[j]] <- d
    if (nrow(d) > 2) {
      ct <- stats::cor.test(d$x, d$y, method = "pearson")
      r <- unname(ct$estimate)
      p_lab <- if (ct$p.value < 0.001) "p < 0.001" else
        sprintf("p = %.3f", ct$p.value)
      lab <- sprintf("R = %.2f, %s", r, p_lab)
    } else {
      lab <- "R = NA"
    }
    labels[[j]] <- data.frame(pair = lab_pair, label = lab,
      stringsAsFactors = FALSE)
  }
  df <- do.call(rbind, panels)
  lab_df <- do.call(rbind, labels)
  # Preserve pair ordering for the facets.
  lev <- vapply(seq_len(ncol(pairs)),
    function(j) paste0(pairs[1, j], " vs ", pairs[2, j]), character(1))
  df$pair <- factor(df$pair, levels = lev)
  lab_df$pair <- factor(lab_df$pair, levels = lev)

  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$x, y = .data$y))
  if (isTRUE(show_identity)) {
    p <- p + ggplot2::geom_abline(slope = 1, intercept = 0,
      linetype = "dotted", color = "grey60")
  }
  if (type_present) {
    p <- p + ggplot2::geom_point(ggplot2::aes(color = .data$type),
      alpha = 0.5, size = 1.6)
    types <- sort(unique(stats::na.omit(df$type)))
    pal <- palette[types]
    if (anyNA(pal)) {
      pal[is.na(pal)] <- hts_pal(sum(is.na(pal)))
    }
    names(pal) <- types
    p <- p + ggplot2::scale_color_manual(values = pal, name = NULL,
      na.translate = FALSE)
  } else {
    p <- p + ggplot2::geom_point(alpha = 0.6, color = "#0072B2", size = 1.8,
      stroke = 0.2)
  }
  p <- p +
    ggplot2::geom_smooth(method = "lm", formula = y ~ x, se = TRUE,
      color = line_color, fill = band_color, linewidth = 0.8) +
    ggplot2::geom_text(data = lab_df, ggplot2::aes(label = .data$label),
      x = -Inf, y = Inf, hjust = -0.08, vjust = 1.4, size = 3.4,
      inherit.aes = FALSE) +
    theme_hts(legend = "right")

  if (ncol(pairs) > 1) {
    p <- p +
      ggplot2::facet_wrap(~ .data$pair, scales = "free") +
      ggplot2::labs(x = "Replicate (x-axis)", y = "Replicate (y-axis)")
  } else {
    p <- p + ggplot2::labs(x = pairs[1, 1], y = pairs[2, 1])
  }
  p
}

#' Histogram of replicate CV values
#'
#' Shows the distribution of per-compound replicate CVs across the library, with
#' the acceptance threshold marked.
#'
#' @param cv A data frame from [calc_replicate_cv()].
#' @param cv_max Threshold to draw as a reference line. Default `15`.
#' @param bins Number of histogram bins. Default `30`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' plot_cv_distribution(calc_replicate_cv(norm))
#'
#' @seealso [calc_replicate_cv()], [filter_bad_replicate()]
#' @export
plot_cv_distribution <- function(cv, cv_max = 15, bins = 30) {
  if (!"cv" %in% names(cv)) {
    stop("`cv` must contain a 'cv' column.", call. = FALSE)
  }
  df <- cv[is.finite(cv$cv), , drop = FALSE]
  ggplot2::ggplot(df, ggplot2::aes(x = .data$cv)) +
    ggplot2::geom_histogram(bins = bins, fill = .hts_col("compound"),
      color = "white", linewidth = 0.2, alpha = 0.9) +
    ggplot2::geom_vline(xintercept = cv_max, linetype = "dashed",
      color = .hts_col("reference"), linewidth = 0.6) +
    ggplot2::labs(x = "Replicate CV (%)", y = "Compounds") +
    theme_hts()
}

#' Plate-level quality-control metrics for high-throughput screens
#'
#' Computes the standard set of assay quality metrics for a single plate from
#' per-control-type summary statistics: the Z-factor, Z-prime (Z'), the
#' signal-to-background ratio, the signal-to-noise ratio and the strictly
#' standardised mean difference (SSMD).
#'
#' @param df A data frame describing one plate with one row per control type. It
#'   must contain the columns `experiment_type`, `mean` and `sd`. The
#'   `experiment_type` column is expected to include the levels
#'   `"negative_ctrl"`, `"positive_cttl"` and `"sample"` (an optional
#'   `"edgewell"` level is ignored).
#'
#' @return A one-row data frame with columns `z_factor`, `z_prime`, `sb_value`,
#'   `sn_value` and `ssmd_value`.
#'
#' @details The metrics follow the usual HTS definitions, using the negative
#'   control and positive (kill) control populations:
#'   \deqn{Z\text{-}factor = 1 - \frac{3(sd_{neg} + sd_{sample})}{|m_{neg} - m_{sample}|}}
#'   \deqn{Z' = 1 - \frac{3(sd_{neg} + sd_{pos})}{|m_{neg} - m_{pos}|}}
#'   with signal-to-background \eqn{m_{neg}/m_{pos}}, signal-to-noise
#'   \eqn{(m_{pos} - m_{neg})/sd_{neg}} and
#'   \eqn{SSMD = (m_{pos} - m_{neg})/\sqrt{sd_{pos}^2 + sd_{neg}^2}}.
#'
#' @examples
#' plate <- data.frame(
#'   experiment_type = c("negative_ctrl", "positive_cttl", "sample"),
#'   mean = c(1.00, 0.10, 0.60),
#'   sd = c(0.05, 0.03, 0.20)
#' )
#' calculate_qc_metrics(plate)
#'
#' @seealso [flag_qc_plates()]
#' @export
calculate_qc_metrics <- function(df) {
  required <- c("experiment_type", "mean", "sd")
  missing <- setdiff(required, names(df))
  if (length(missing) > 0) {
    stop(
      "`df` is missing required column(s): ",
      paste(missing, collapse = ", "), ".",
      call. = FALSE
    )
  }

  pick <- function(type, col) {
    val <- df[[col]][df$experiment_type == type]
    if (length(val) == 0) NA_real_ else val[1]
  }

  m_negative <- pick("negative_ctrl", "mean")
  m_positive <- pick("positive_cttl", "mean")
  m_sample <- pick("sample", "mean")
  sd_negative <- pick("negative_ctrl", "sd")
  sd_positive <- pick("positive_cttl", "sd")
  sd_sample <- pick("sample", "sd")

  data.frame(
    z_factor = 1 - 3 * (sd_negative + sd_sample) / abs(m_negative - m_sample),
    z_prime = 1 - 3 * (sd_negative + sd_positive) / abs(m_negative - m_positive),
    sb_value = m_negative / m_positive,
    sn_value = (m_positive - m_negative) / sd_negative,
    ssmd_value = (m_positive - m_negative) / sqrt(sd_positive^2 + sd_negative^2)
  )
}

#' Flag plates that fail quality-control thresholds
#'
#' Adds pass/fail flags to a table of plate quality metrics such as the one
#' produced by [calculate_qc_metrics()], using conventional acceptance
#' thresholds.
#'
#' @param qc A data frame of plate quality metrics. Columns referenced by the
#'   active thresholds must be present (`z_prime`, `sb_value` and, if supplied,
#'   `cv_negative_ctrl`).
#' @param z_prime_min Minimum acceptable Z-prime. Plates below this are flagged
#'   `"Bad"`. Default `0.4`.
#' @param sb_min Minimum acceptable signal-to-background ratio. Default `3`.
#' @param cv_max Maximum acceptable coefficient of variation (percent) of the
#'   negative control. Only applied when a `cv_negative_ctrl` column is present.
#'   Default `10`.
#'
#' @return The input data frame with added character flag columns
#'   `z_primer_flag`, `sb_flag` and (when applicable) `cv_flag`, each taking the
#'   value `"Bad"` or `"Normal"`.
#'
#' @examples
#' qc <- data.frame(
#'   plateID = c("P01", "P02"),
#'   z_prime = c(0.62, 0.20),
#'   sb_value = c(5.1, 2.4),
#'   cv_negative_ctrl = c(6, 14)
#' )
#' flag_qc_plates(qc)
#'
#' @seealso [calculate_qc_metrics()]
#' @export
flag_qc_plates <- function(qc, z_prime_min = 0.4, sb_min = 3, cv_max = 10) {
  if ("z_prime" %in% names(qc)) {
    qc$z_primer_flag <- ifelse(qc$z_prime < z_prime_min, "Bad", "Normal")
  }
  if ("sb_value" %in% names(qc)) {
    qc$sb_flag <- ifelse(qc$sb_value < sb_min, "Bad", "Normal")
  }
  if ("cv_negative_ctrl" %in% names(qc)) {
    qc$cv_flag <- ifelse(qc$cv_negative_ctrl > cv_max, "Bad", "Normal")
  }
  qc
}

test_that("calculate_qc_metrics computes HTS metrics", {
  plate <- data.frame(
    experiment_type = c("negative_ctrl", "positive_cttl", "sample"),
    mean = c(1.00, 0.10, 0.60),
    sd = c(0.05, 0.03, 0.20)
  )
  out <- calculate_qc_metrics(plate)

  expect_named(out, c(
    "z_factor", "z_prime", "sb_value", "sn_value", "ssmd_value"
  ))
  expect_equal(out$z_factor, 1 - 3 * (0.05 + 0.20) / abs(1.00 - 0.60),
    tolerance = 1e-9)
  expect_equal(out$z_prime, 1 - 3 * (0.05 + 0.03) / abs(1.00 - 0.10),
    tolerance = 1e-9)
  expect_equal(out$sb_value, 1.00 / 0.10, tolerance = 1e-9)
  expect_equal(out$sn_value, (0.10 - 1.00) / 0.05, tolerance = 1e-9)
  expect_equal(
    out$ssmd_value,
    (0.10 - 1.00) / sqrt(0.03^2 + 0.05^2),
    tolerance = 1e-9
  )
})

test_that("calculate_qc_metrics errors on missing columns", {
  expect_error(
    calculate_qc_metrics(data.frame(experiment_type = "x", mean = 1)),
    "missing required"
  )
})

test_that("flag_qc_plates adds pass/fail flags", {
  qc <- data.frame(
    plateID = c("P01", "P02"),
    z_prime = c(0.62, 0.20),
    sb_value = c(5.1, 2.4),
    cv_negative_ctrl = c(6, 14)
  )
  out <- flag_qc_plates(qc)

  expect_equal(out$z_primer_flag, c("Normal", "Bad"))
  expect_equal(out$sb_flag, c("Normal", "Bad"))
  expect_equal(out$cv_flag, c("Normal", "Bad"))
})

test_that("flag_qc_plates only flags present columns", {
  qc <- data.frame(plateID = "P01", z_prime = 0.62)
  out <- flag_qc_plates(qc)
  expect_true("z_primer_flag" %in% names(out))
  expect_false("sb_flag" %in% names(out))
})

#' Four-parameter logistic (4PL) dose-response function
#'
#' Evaluates the classic four-parameter logistic model used to describe
#' drug dose-response curves in high-throughput screens.
#'
#' The model is
#' \deqn{f(x) = \mathrm{bottom} +
#'   \frac{\mathrm{top} - \mathrm{bottom}}{1 + (x / \mathrm{ic50})^{\mathrm{hill}}}}
#'
#' @param x Numeric vector of drug concentrations (on the same scale used to fit
#'   the model, typically the raw concentration).
#' @param top Response of the upper asymptote (e.g. viability with no drug).
#' @param ic50 Concentration producing the half-maximal response.
#' @param hill Hill slope controlling the steepness of the curve.
#' @param bottom Response of the lower asymptote (e.g. viability at saturating
#'   drug).
#'
#' @return A numeric vector of predicted responses, the same length as `x`.
#'
#' @examples
#' four_pl(x = c(0.01, 0.1, 1, 10), top = 1, ic50 = 1, hill = 1, bottom = 0)
#'
#' @seealso [compute_auc()], [fit_dose_response()]
#' @export
four_pl <- function(x, top, ic50, hill, bottom) {
  if (!is.numeric(x)) {
    stop("`x` must be numeric.", call. = FALSE)
  }
  bottom + (top - bottom) / (1 + (x / ic50)^hill)
}

#' Area under a four-parameter logistic dose-response curve
#'
#' Integrates a fitted [four_pl()] curve between two concentrations to obtain
#' the area under the curve (AUC), a robust summary of drug potency used for
#' between-condition comparisons.
#'
#' @param min_concentration Lower integration limit (minimum tested
#'   concentration).
#' @param max_concentration Upper integration limit (maximum tested
#'   concentration).
#' @param top,ic50,hill,bottom Four-parameter logistic parameters, as in
#'   [four_pl()].
#'
#' @return A single numeric value: the integrated area under the curve. When
#'   integration fails (for example a degenerate fit) `NA_real_` is returned
#'   with a warning.
#'
#' @examples
#' compute_auc(
#'   min_concentration = 0.01, max_concentration = 10,
#'   top = 1, ic50 = 1, hill = 1, bottom = 0
#' )
#'
#' @seealso [four_pl()], [fit_dose_response()]
#' @export
compute_auc <- function(min_concentration, max_concentration,
                        top, ic50, hill, bottom) {
  if (min_concentration >= max_concentration) {
    stop("`min_concentration` must be smaller than `max_concentration`.",
      call. = FALSE
    )
  }
  out <- tryCatch(
    stats::integrate(
      four_pl,
      lower = min_concentration, upper = max_concentration,
      top = top, ic50 = ic50, hill = hill, bottom = bottom
    )$value,
    error = function(e) {
      warning("AUC integration failed: ", conditionMessage(e), call. = FALSE)
      NA_real_
    }
  )
  out
}

#' Fit a four-parameter logistic dose-response model
#'
#' Fits a 4PL curve for a single drug under one condition by non-linear least
#' squares, entirely in base R (no external fitting package required). Robust
#' starting values are derived from the data and refined with a bounded
#' `L-BFGS-B` optimisation over several initial `IC50` guesses, falling back to
#' a derivative-free `Nelder-Mead` search if needed. The model uses the same
#' parameterisation as [four_pl()], which matches the `dr4pl` package
#' convention (`UpperLimit`, `IC50`, `Slope`, `LowerLimit`).
#'
#' @param data A data frame containing at least the response and concentration
#'   columns for one drug and one condition.
#' @param response Name of the (normalised) response column, as a string.
#' @param concentration Name of the concentration column, as a string.
#' @param bounds Optional named list overriding the box constraints used by the
#'   bounded optimiser. Recognised names are `top`, `bottom`, `ic50` and `hill`,
#'   each a numeric length-two vector `c(lower, upper)`.
#'
#' @return An object of class `dsr_4pl`: a list with elements `coefficients`
#'   (named numeric vector `top`, `ic50`, `hill`, `bottom`), `fitted`,
#'   `residuals`, `convergence` (logical), `sse`, `data` and `call`. The
#'   companion methods `coef()`, `predict()`, `fitted()`, `residuals()` and
#'   `print()` are provided for this class.
#'
#' @details The fitted parameters can be passed directly to [compute_auc()] to
#'   obtain a potency summary, for example
#'   `do.call(compute_auc, c(list(min(x), max(x)), as.list(coef(fit))))`.
#'
#' @examples
#' one <- subset(
#'   screen_doseresponse,
#'   drug_name == "DrugA" & condition == "Gy0" & experiment_type == "sample"
#' )
#' fit <- fit_dose_response(one, "normalized_cell_count", "concentration")
#' coef(fit)
#' compute_auc(
#'   min(one$concentration), max(one$concentration),
#'   top = coef(fit)[["top"]], ic50 = coef(fit)[["ic50"]],
#'   hill = coef(fit)[["hill"]], bottom = coef(fit)[["bottom"]]
#' )
#'
#' @seealso [four_pl()], [compute_auc()]
#' @export
fit_dose_response <- function(data, response, concentration, bounds = NULL) {
  for (col in c(response, concentration)) {
    if (!col %in% names(data)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }

  y <- .as_numeric(data[[response]])
  x <- .as_numeric(data[[concentration]])
  keep <- is.finite(x) & is.finite(y)
  x <- x[keep]
  y <- y[keep]

  if (length(x) < 4) {
    stop("At least 4 finite observations are required to fit a 4PL model.",
      call. = FALSE
    )
  }

  fit <- .fit_4pl(x, y, bounds = bounds)
  fit$call <- match.call()
  fit$data <- data.frame(concentration = x, response = y)
  fit
}

# Internal: coerce a column to numeric safely. Crucially, factors are converted
# via their *labels* (as.character) rather than their integer codes, so that a
# concentration column stored as a factor such as factor(c("0.01", "0.1", "1"))
# yields the real doses 0.01, 0.1, 1 and not the level codes 1, 2, 3.
.as_numeric <- function(v) {
  if (is.numeric(v)) {
    return(v)
  }
  if (is.factor(v)) {
    v <- as.character(v)
  }
  suppressWarnings(as.numeric(v))
}

# Internal: non-linear least-squares fit of the 4PL model in base R.
# The IC50 is optimised on the log10 scale, which conditions the problem well
# when concentrations span several orders of magnitude.
.fit_4pl <- function(x, y, bounds = NULL) {
  x_pos <- x[x > 0]
  rng_y <- range(y, na.rm = TRUE)
  span <- diff(rng_y)
  mid <- mean(rng_y)
  lx <- log10(x_pos)

  # Default box constraints, generous but keeping the curve well posed.
  default_bounds <- list(
    top = c(mid, rng_y[2] + 0.5 * span + 1e-3),
    bottom = c(rng_y[1] - 0.5 * span - 1e-3, mid),
    ic50 = c(10^(min(lx) - 1), 10^(max(lx) + 1)),
    hill = c(0.05, 25)
  )
  if (!is.null(bounds)) {
    default_bounds[names(bounds)] <- bounds
  }
  b <- default_bounds
  # Optimise over (top, log10_ic50, hill, bottom).
  lower <- c(b$top[1], log10(b$ic50[1]), b$hill[1], b$bottom[1])
  upper <- c(b$top[2], log10(b$ic50[2]), b$hill[2], b$bottom[2])

  sse <- function(p) {
    pred <- four_pl(x, top = p[1], ic50 = 10^p[2], hill = p[3], bottom = p[4])
    val <- sum((y - pred)^2)
    if (!is.finite(val)) 1e12 else val
  }

  # Multi-start over IC50 (log scale) spanning the tested concentration range.
  top0 <- rng_y[2]
  bottom0 <- rng_y[1]
  le_grid <- unique(as.numeric(stats::quantile(lx, c(0.25, 0.5, 0.75))))

  best <- NULL
  for (le0 in le_grid) {
    start <- c(top0, le0, 1, bottom0)
    start <- pmin(pmax(start, lower), upper)
    res <- tryCatch(
      stats::optim(start, sse,
        method = "L-BFGS-B",
        lower = lower, upper = upper
      ),
      error = function(e) NULL
    )
    if (!is.null(res) && (is.null(best) || res$value < best$value)) {
      best <- res
    }
  }

  # Fall back to a derivative-free search if the bounded optimiser failed.
  if (is.null(best)) {
    best <- stats::optim(c(top0, stats::median(lx), 1, bottom0), sse,
      method = "Nelder-Mead"
    )
  }

  par <- c(best$par[1], 10^best$par[2], best$par[3], best$par[4])
  names(par) <- c("top", "ic50", "hill", "bottom")
  fitted <- four_pl(x, top = par[1], ic50 = par[2], hill = par[3], bottom = par[4])

  structure(
    list(
      coefficients = par,
      fitted = as.numeric(fitted),
      residuals = as.numeric(y - fitted),
      convergence = isTRUE(best$convergence == 0),
      sse = best$value
    ),
    class = "dsr_4pl"
  )
}

#' @describeIn fit_dose_response Extract the fitted 4PL coefficients (`top`,
#'   `ic50`, `hill`, `bottom`).
#' @param object A `dsr_4pl` object.
#' @param ... Ignored, for S3 method consistency.
#' @export
coef.dsr_4pl <- function(object, ...) {
  object$coefficients
}

#' @describeIn fit_dose_response Predict responses from a fitted model.
#' @param newdata Optional numeric vector of concentrations at which to predict.
#'   Defaults to the concentrations used for fitting.
#' @export
predict.dsr_4pl <- function(object, newdata = NULL, ...) {
  x <- if (is.null(newdata)) object$data$concentration else newdata
  p <- object$coefficients
  four_pl(x, top = p[["top"]], ic50 = p[["ic50"]], hill = p[["hill"]],
    bottom = p[["bottom"]]
  )
}

#' @describeIn fit_dose_response Extract fitted values.
#' @export
fitted.dsr_4pl <- function(object, ...) {
  object$fitted
}

#' @describeIn fit_dose_response Extract residuals.
#' @export
residuals.dsr_4pl <- function(object, ...) {
  object$residuals
}

#' @describeIn fit_dose_response Print a short summary of the fitted model.
#' @param x A `dsr_4pl` object.
#' @export
print.dsr_4pl <- function(x, ...) {
  cat("Four-parameter logistic fit (diffHTS)\n")
  cat("  top   =", format(x$coefficients[["top"]], digits = 4), "\n")
  cat("  ic50  =", format(x$coefficients[["ic50"]], digits = 4), "\n")
  cat("  hill  =", format(x$coefficients[["hill"]], digits = 4), "\n")
  cat("  bottom=", format(x$coefficients[["bottom"]], digits = 4), "\n")
  cat("  SSE   =", format(x$sse, digits = 4),
    if (x$convergence) "(converged)" else "(not converged)", "\n"
  )
  invisible(x)
}

test_that("four_pl evaluates the 4PL model", {
  # At x = ic50 the response is the midpoint of top and bottom
  expect_equal(four_pl(1, top = 1, ic50 = 1, hill = 1, bottom = 0), 0.5)

  # Vectorised input returns the same length
  x <- c(0.01, 0.1, 1, 10)
  out <- four_pl(x, top = 1, ic50 = 1, hill = 1, bottom = 0)
  expect_length(out, length(x))
  # monotonic decreasing for a killing curve (top > bottom)
  expect_true(all(diff(out) < 0))
})

test_that("four_pl rejects non-numeric x", {
  expect_error(four_pl("a", top = 1, ic50 = 1, hill = 1, bottom = 0),
    "must be numeric")
})

test_that("compute_auc integrates the 4PL curve", {
  # integral of 1/(1+x) from 0.01 to 10 = ln(11) - ln(1.01)
  expected <- log(11) - log(1.01)
  expect_equal(
    compute_auc(0.01, 10, top = 1, ic50 = 1, hill = 1, bottom = 0),
    expected,
    tolerance = 1e-6
  )
})

test_that("compute_auc errors when bounds are inverted", {
  expect_error(
    compute_auc(10, 1, top = 1, ic50 = 1, hill = 1, bottom = 0),
    "must be smaller"
  )
})

test_that("fit_dose_response fits a 4PL without external packages", {
  one <- subset(
    screen_doseresponse,
    drug_name == "DrugA" & condition == "Gy0" & experiment_type == "sample"
  )
  fit <- fit_dose_response(one, "normalized_cell_count", "concentration")

  expect_s3_class(fit, "dsr_4pl")
  cf <- coef(fit)
  expect_named(cf, c("top", "ic50", "hill", "bottom"))
  expect_length(cf, 4)

  # A killing curve: top (no drug) should exceed bottom (saturating drug)
  expect_gt(cf[["top"]], cf[["bottom"]])
  # IC50 should sit within the tested concentration range
  expect_gte(cf[["ic50"]], min(one$concentration))
  expect_lte(cf[["ic50"]], max(one$concentration))

  # S3 helpers behave consistently
  expect_length(fitted(fit), nrow(fit$data))
  expect_equal(residuals(fit), fit$data$response - fitted(fit))
  expect_equal(
    predict(fit, newdata = fit$data$concentration), fitted(fit),
    tolerance = 1e-8
  )
  expect_output(print(fit), "logistic fit")
})

test_that("fit_dose_response handles factor/character concentration columns", {
  one <- subset(
    screen_doseresponse,
    drug_name == "DrugA" & condition == "Gy0" & experiment_type == "sample"
  )
  num_fit <- fit_dose_response(one, "normalized_cell_count", "concentration")

  # Concentration stored as a factor (a common raw-data format). Naive
  # as.numeric() would return level codes 1..5 and wreck the fit; the function
  # must recover the real doses and match the numeric fit.
  one_factor <- one
  one_factor$concentration <- factor(as.character(one_factor$concentration))
  fac_fit <- fit_dose_response(one_factor, "normalized_cell_count", "concentration")
  expect_equal(coef(fac_fit), coef(num_fit), tolerance = 1e-6)

  # Concentration stored as character behaves identically.
  one_chr <- one
  one_chr$concentration <- as.character(one_chr$concentration)
  chr_fit <- fit_dose_response(one_chr, "normalized_cell_count", "concentration")
  expect_equal(coef(chr_fit), coef(num_fit), tolerance = 1e-6)
})

test_that("fit_dose_response errors on too few observations", {
  expect_error(
    fit_dose_response(
      data.frame(
        normalized_cell_count = c(1, 0.5, 0.1),
        concentration = c(0.1, 1, 10)
      ),
      "normalized_cell_count", "concentration"
    ),
    "At least 4"
  )
})

test_that("fit_dose_response errors on a missing column", {
  expect_error(
    fit_dose_response(
      data.frame(x = 1:5, concentration = 1:5),
      "normalized_cell_count", "concentration"
    ),
    "not found"
  )
})

# Internal helpers shared across the HTS pipeline -----------------------------

# Canonical well-type labels used throughout the package.
.hts_well_types <- c(negative = "NC", positive = "PC", blank = "Blank",
                     compound = "compound")

# Read an HTS table from csv/txt/xlsx into a data frame.
.read_hts_file <- function(file, sep = "\t", sheet = 1) {
  ext <- tolower(tools::file_ext(file))
  if (ext %in% c("xls", "xlsx")) {
    if (!requireNamespace("readxl", quietly = TRUE)) {
      stop("Reading Excel files requires the 'readxl' package.", call. = FALSE)
    }
    as.data.frame(readxl::read_excel(file, sheet = sheet))
  } else if (ext == "csv") {
    utils::read.csv(file, stringsAsFactors = FALSE, check.names = FALSE)
  } else {
    utils::read.table(file, header = TRUE, sep = sep,
      stringsAsFactors = FALSE, check.names = FALSE)
  }
}

# Reshape a plate-map grid (first column = row letters, remaining column names =
# column indices) into a long data frame with `row`, `col`, `well` and `value`.
.grid_to_long <- function(grid, value_name = "content") {
  grid <- as.data.frame(grid, stringsAsFactors = FALSE, check.names = FALSE)
  if (ncol(grid) < 2L) {
    stop("A plate-map grid needs a row-label column plus one column per plate ",
      "column.", call. = FALSE)
  }
  row_labels <- toupper(trimws(as.character(grid[[1]])))
  body <- grid[, -1, drop = FALSE]
  col_idx <- suppressWarnings(
    as.integer(sub("^[^0-9]*([0-9]+).*$", "\\1", names(body)))
  )
  if (anyNA(col_idx)) {
    stop("Plate-map column headers must contain the plate column numbers ",
      "(e.g. 1, 2, ... 24).", call. = FALSE)
  }
  long <- do.call(rbind, lapply(seq_len(nrow(body)), function(i) {
    data.frame(
      row = row_labels[i],
      col = col_idx,
      value = as.character(unlist(body[i, ], use.names = FALSE)),
      stringsAsFactors = FALSE
    )
  }))
  long$value <- trimws(long$value)
  long$well <- paste0(long$row, long$col)
  names(long)[names(long) == "value"] <- value_name
  long[, c("well", "row", "col", value_name)]
}

# Physical dimensions (rows x columns) of a standard microplate format.
.plate_dims <- function(format) {
  dims <- list(
    "6" = c(2L, 3L), "12" = c(3L, 4L), "24" = c(4L, 6L), "48" = c(6L, 8L),
    "96" = c(8L, 12L), "384" = c(16L, 24L), "1536" = c(32L, 48L)
  )
  key <- as.character(format)
  if (!key %in% names(dims)) {
    stop("Unsupported plate format '", format, "'. Use one of ",
      paste(names(dims), collapse = ", "), ".",
      call. = FALSE
    )
  }
  dims[[key]]
}

# Row labels for a plate with `n` rows: A..Z then AA..AF for 1536-well plates.
.plate_row_labels <- function(n) {
  labels <- c(LETTERS, paste0("A", LETTERS))
  labels[seq_len(n)]
}

# Attach the standard hts classes to a data frame.
.as_hts <- function(df, subclass) {
  class(df) <- unique(c(subclass, "hts_tbl", "data.frame"))
  df
}

#' Standard microplate coordinate map
#'
#' Builds the full row/column coordinate table for a standard microplate,
#' ordered by row then column. This is useful for joining plate readouts onto a
#' complete layout so that empty wells are represented explicitly.
#'
#' @param format Plate format (number of wells): one of `6`, `12`, `24`, `48`,
#'   `96`, `384` or `1536`. Default `96`.
#'
#' @return A data frame with one row per well and the columns `well` (for
#'   example `"A1"`), `row` (row letter), `col` (column number) and `row_index`
#'   (numeric row position, `A = 1`).
#'
#' @examples
#' head(plate_layout(96))
#' nrow(plate_layout(384))
#'
#' @seealso [plate_layout_384()], [plate_layout_1536()]
#' @export
plate_layout <- function(format = 96) {
  d <- .plate_dims(format)
  n_row <- d[1]
  n_col <- d[2]
  rows <- .plate_row_labels(n_row)
  grid <- expand.grid(col = seq_len(n_col), row = rows,
    KEEP.OUT.ATTRS = FALSE, stringsAsFactors = FALSE
  )
  grid <- grid[order(match(grid$row, rows), grid$col), , drop = FALSE]
  data.frame(
    well = paste0(grid$row, grid$col),
    row = grid$row,
    col = as.integer(grid$col),
    row_index = match(grid$row, rows),
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

#' @rdname plate_layout
#' @export
plate_layout_384 <- function() plate_layout(384)

#' @rdname plate_layout
#' @export
plate_layout_1536 <- function() plate_layout(1536)

#' Convert concentrations to a log10 scale
#'
#' Transforms a concentration vector to `log10`, the scale on which
#' dose-response curves are usually drawn and fitted. Non-positive values (for
#' example the zero-dose controls) become `NA`.
#'
#' @param x Numeric vector of concentrations (factors/characters are coerced).
#' @param shift_min Logical; if `TRUE` the minimum finite `log10` value is
#'   subtracted so that the series starts at zero (matching the pipeline's
#'   `log10(conc) - min(log10(conc))` convention). Default `FALSE`.
#'
#' @return A numeric vector of `log10` concentrations.
#'
#' @examples
#' convert_conc_log10(c(0.01, 0.1, 1, 10, 100))
#' convert_conc_log10(c(0.01, 0.1, 1, 10, 100), shift_min = TRUE)
#'
#' @export
convert_conc_log10 <- function(x, shift_min = FALSE) {
  x <- .as_numeric(x)
  x[!is.finite(x) | x <= 0] <- NA_real_
  out <- log10(x)
  if (shift_min && any(is.finite(out))) {
    out <- out - min(out, na.rm = TRUE)
  }
  out
}

#' Validate control-well labelling
#'
#' Checks that a data frame contains the expected negative-, positive- and
#' blank-control labels in its well-type column, so that normalisation and QC
#' can be carried out. It is a light-weight guard to run right after import.
#'
#' @param data A data frame of well-level readouts.
#' @param type_col Name of the well-type column, as a string. Default
#'   `"well_type"`.
#' @param required Character vector of labels that must be present. Default
#'   `c("NC", "PC")`; add `"Blank"` if background subtraction is planned.
#'
#' @return Invisibly, a list with `ok` (logical), `present` and `missing`
#'   (character vectors) and `counts` (a table of well-type frequencies). A
#'   warning is emitted when any required label is missing.
#'
#' @examples
#' d <- data.frame(well_type = c("NC", "NC", "PC", "compound"))
#' check_control_label(d)
#'
#' @export
check_control_label <- function(data, type_col = "well_type",
                                required = c("NC", "PC")) {
  if (!type_col %in% names(data)) {
    stop("Column '", type_col, "' not found in `data`.", call. = FALSE)
  }
  present_labels <- unique(as.character(data[[type_col]]))
  missing <- setdiff(required, present_labels)
  if (length(missing) > 0) {
    warning("Missing required control label(s): ",
      paste(missing, collapse = ", "), ".",
      call. = FALSE
    )
  }
  invisible(list(
    ok = length(missing) == 0,
    present = intersect(required, present_labels),
    missing = missing,
    counts = table(as.character(data[[type_col]]))
  ))
}

#' Standardise compound identifiers
#'
#' Cleans free-text compound identifiers into a consistent form: trims
#' surrounding whitespace, collapses internal whitespace, and replaces illegal
#' or separator characters with underscores so that IDs can be matched and used
#' safely in file names.
#'
#' @param x A character vector of compound identifiers.
#' @param to_upper Logical; if `TRUE` (default) the result is upper-cased.
#'
#' @return A character vector of cleaned identifiers.
#'
#' @examples
#' clean_compound_id(c("  cpd 001 ", "Drug/A", "cpd-002!"))
#'
#' @export
clean_compound_id <- function(x, to_upper = TRUE) {
  x <- as.character(x)
  x <- trimws(x)
  x <- gsub("[[:space:]]+", "_", x)
  x <- gsub("[^A-Za-z0-9_.-]", "_", x)
  x <- gsub("_+", "_", x)
  x <- gsub("^_|_$", "", x)
  if (to_upper) x <- toupper(x)
  x
}

#' diffHTS: Differential Drug Sensitivity Analysis for Two-Condition
#' High-Throughput Screens
#'
#' The diffHTS package bundles reusable methods for analysing
#' high-throughput drug screening (HTS) experiments that contrast two
#' conditions, for example irradiated versus non-irradiated cells or cancer
#' versus normal cell lines. The typical workflow is:
#'
#' \enumerate{
#'   \item Fit four-parameter logistic dose-response curves per drug and
#'     condition with [fit_dose_response()] and summarise potency with
#'     [compute_auc()].
#'   \item Assess plate quality with [calculate_qc_metrics()] and
#'     [flag_qc_plates()].
#'   \item Contrast the two conditions with [compute_delta_auc()] to obtain a
#'     differential AUC and its within-experiment z-score.
#'   \item Nominate hits with [select_hits_cutoff()] or [select_hits_sigma()].
#'   \item Visualise results with [plot_delta_auc_heatmap()],
#'     [plot_condition_scatter()] and [plot_qc_boxplot()].
#' }
#'
#' Two small demonstration datasets, [screen_doseresponse] and
#' [screen_delta_auc], are provided to run the examples end to end.
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom rlang .data
## usethis namespace: end
NULL

#' Differential (delta) AUC between two screening conditions
#'
#' Contrasts drug potency between two experimental conditions by reshaping a
#' long per-drug/per-condition AUC table to one row per drug and computing the
#' differential AUC together with its within-experiment z-score. This is the
#' core scoring step for two-condition screens (for example irradiated versus
#' non-irradiated, or treated versus control).
#'
#' @param data A long-format data frame with one row per drug and condition.
#' @param drug_col Name of the drug identifier column, as a string. Default
#'   `"drug_name"`.
#' @param condition_col Name of the condition column, as a string. Default
#'   `"condition"`. The column must contain exactly the two values given in
#'   `conditions`.
#' @param auc_col Name of the AUC column, as a string. Default `"auc"`.
#' @param conditions Character vector of length two giving the baseline and
#'   treatment condition, in the order `c(baseline, treatment)`. The delta is
#'   computed as `treatment - baseline`. Defaults to the two levels found in
#'   `condition_col`.
#'
#' @return A data frame with one row per drug containing the two per-condition
#'   AUC columns (named `auc_<condition>`), `delta_auc` and `zscore_delta_auc`
#'   (the z-scored `delta_auc` across all drugs).
#'
#' @details A negative `delta_auc` indicates the treatment condition sensitises
#'   cells to the drug relative to baseline (lower AUC = more killing). The
#'   z-score standardises `delta_auc` across all drugs within the experiment so
#'   that hits can be selected on a common scale (see [select_hits_sigma()]).
#'
#' @examples
#' auc_long <- data.frame(
#'   drug_name = rep(c("DrugA", "DrugB", "DrugC"), each = 2),
#'   condition = rep(c("Gy0", "Gy2"), times = 3),
#'   auc = c(5.0, 3.1, 4.2, 4.0, 6.0, 2.0)
#' )
#' compute_delta_auc(auc_long, conditions = c("Gy0", "Gy2"))
#'
#' @seealso [select_hits_cutoff()], [select_hits_sigma()]
#' @export
compute_delta_auc <- function(data,
                              drug_col = "drug_name",
                              condition_col = "condition",
                              auc_col = "auc",
                              conditions = NULL) {
  for (col in c(drug_col, condition_col, auc_col)) {
    if (!col %in% names(data)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }

  if (is.null(conditions)) {
    conditions <- unique(as.character(data[[condition_col]]))
  }
  if (length(conditions) != 2) {
    stop(
      "`conditions` must identify exactly two conditions; found ",
      length(conditions), ".",
      call. = FALSE
    )
  }
  baseline <- conditions[1]
  treatment <- conditions[2]

  long <- data[as.character(data[[condition_col]]) %in% conditions,
    c(drug_col, condition_col, auc_col),
    drop = FALSE
  ]
  names(long) <- c("drug_name", "condition", "auc")
  long$condition <- as.character(long$condition)

  wide <- tidyr::pivot_wider(
    unique(long),
    names_from = "condition",
    values_from = "auc",
    names_prefix = "auc_"
  )

  base_col <- paste0("auc_", baseline)
  treat_col <- paste0("auc_", treatment)

  wide$delta_auc <- wide[[treat_col]] - wide[[base_col]]
  wide$zscore_delta_auc <- as.numeric(scale(wide$delta_auc))

  as.data.frame(wide)
}

test_that("compute_delta_auc reshapes and scores", {
  auc_long <- data.frame(
    drug_name = rep(c("DrugA", "DrugB", "DrugC"), each = 2),
    condition = rep(c("Gy0", "Gy2"), times = 3),
    auc = c(5.0, 3.1, 4.2, 4.0, 6.0, 2.0)
  )
  out <- compute_delta_auc(auc_long, conditions = c("Gy0", "Gy2"))

  expect_named(out, c(
    "drug_name", "auc_Gy0", "auc_Gy2", "delta_auc", "zscore_delta_auc"
  ))
  expect_equal(nrow(out), 3)
  # delta = treatment - baseline
  expect_equal(out$delta_auc, c(-1.9, -0.2, -4.0), tolerance = 1e-9)
  # z-score is centred and complete
  expect_false(any(is.na(out$zscore_delta_auc)))
  expect_equal(mean(out$zscore_delta_auc), 0, tolerance = 1e-9)
})

test_that("compute_delta_auc errors on missing columns", {
  expect_error(
    compute_delta_auc(data.frame(condition = "Gy0", auc = 1)),
    "not found"
  )
})

test_that("compute_delta_auc errors when conditions != 2", {
  df <- data.frame(
    drug_name = "D1",
    condition = c("Gy0", "Gy2", "Gy4"),
    auc = c(1, 2, 3)
  )
  expect_error(
    compute_delta_auc(df, conditions = c("Gy0", "Gy2", "Gy4")),
    "exactly two conditions"
  )
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6.5,
  fig.height = 4,
  dpi = 120,
  dev = "png"
)

## ----setup--------------------------------------------------------------------
library(diffHTS)

## ----styling------------------------------------------------------------------
library(ggplot2)
ggplot(mtcars, aes(factor(cyl), mpg, fill = factor(cyl))) +
  geom_boxplot(color = "white", linewidth = 0.3) +
  scale_fill_manual(values = hts_pal()) +
  labs(x = "Cylinders", y = "MPG", fill = "cyl") +
  theme_hts()

## ----m1-read------------------------------------------------------------------
raw <- read_hts_plate(hts_primary_raw)
unique(raw$plate_id)

## ----m1-layout----------------------------------------------------------------
layout_file <- system.file("extdata", "plate_layout_example.csv",
                           package = "diffHTS")
layout <- read_plate_layout(layout_file, plate_id = "P01")
table(layout$well_type)
head(layout)

## ----m1-apply-----------------------------------------------------------------
# A signal export carries only well + reading; the layout supplies the roles.
set.seed(1)
signal_export <- data.frame(well = layout$well,
                            signal = runif(nrow(layout), 1e4, 1e5))
raw_mapped <- apply_plate_layout(signal_export, layout, plate_id = "P01")
table(raw_mapped$well_type)

## ----m1-setup-----------------------------------------------------------------
summarize_plate_setup(raw)

## ----m1-----------------------------------------------------------------------
raw <- baseline_subtract(raw)
raw <- detect_outlier_wells(raw)
calc_z_prime(raw)[, c("plate_id", "z_prime", "pass")]
good <- filter_valid_plates(raw)
attr(good, "dropped_plates")   # P03 is dropped for failing Z'

## ----m1-qc--------------------------------------------------------------------
qc <- calc_plate_qc(raw)
qc[, c("plate_id", "cv_nc", "cv_pc", "sb", "sn",
       "z_prime", "robust_z_prime", "ssmd", "pass")]

## ----m1-qc-single-------------------------------------------------------------
calc_ssmd(raw)[, c("plate_id", "ssmd", "pass")]
calc_cv(raw)

## ----m1-qc-plots, fig.width = 6, fig.height = 3.5-----------------------------
plot_plate_qc(qc, metric = "z_prime")        # assay window (controls only)
plot_plate_qc(qc, metric = "robust_z_prime") # robust to outlier control wells
plot_plate_qc(qc, metric = "ssmd")           # control separation
plot_plate_qc(qc, metric = "sn")             # signal-to-noise
plot_plate_qc(qc, metric = "cv")             # control CV%, both controls

## ----m1-zscore, fig.width = 6, fig.height = 3.5-------------------------------
scored <- calc_well_zscore(raw)
plot_plate_qc(scored, metric = "robust_zscore")

## ----m1-heatmap, fig.width = 8, fig.height = 5--------------------------------
plot_plate_heatmap_raw(raw, plate = "P01")

## ----m1-384, fig.width = 9, fig.height = 5------------------------------------
plot_plate_heatmap(hts_plate_384, fill = "signal", well_col = "well",
  control_col = "well_type", plate_col = "plate_id", plate_type = 384)

## ----m2-----------------------------------------------------------------------
norm <- norm_by_control(good)
round(range(norm$inhibition, na.rm = TRUE), 1)

## ----m2-plots, fig.width = 7, fig.height = 4----------------------------------
plot_plate_heatmap_inhibition(norm, plate = "P01")
plot_inhibition_hist(norm, plate = "P01")

## ----m3-----------------------------------------------------------------------
cv <- calc_replicate_cv(norm)
calc_replicate_correlation(norm)

## ----m3-plots, fig.width = 9, fig.height = 3.4--------------------------------
plot_replicate_scatter(norm)

## ----m3-cv, fig.width = 6, fig.height = 4-------------------------------------
plot_cv_distribution(cv, cv_max = 15)

## ----m4-----------------------------------------------------------------------
hits <- select_primary_hit(norm, threshold = 50)
summarize_primary_hit(hits)

## ----m4-plots, fig.width = 6, fig.height = 4----------------------------------
plot_primary_inhibition_rank(hits)
plot_hit_bar_count(hits)

## ----m4-sigma-----------------------------------------------------------------
sig <- select_sigma_hits(norm, n_sigma = 3)
head(sig)
summarize_sigma_hits(sig)

## ----m4-sigma-compare---------------------------------------------------------
c(robust = sum(select_sigma_hits(norm, method = "robust")$is_hit),
  sd     = sum(select_sigma_hits(norm, method = "sd")$is_hit))

## ----m4-sigma-plot, fig.width = 7, fig.height = 4.5---------------------------
plot_sigma_hits(sig)
plot_hit_bar_count(sig)

## ----m5-----------------------------------------------------------------------
dr  <- import_dose_response(hts_dose_response, response_col = "viability",
                            group_cols = "cell_line")
drc <- fit_4pl_curve(dr, group_cols = "cell_line")
drc <- calc_drc_auc(drc)
head(extract_drc_params(drc))

## ----m5-drc, fig.width = 6, fig.height = 4------------------------------------
plot_single_drc(drc, drc$meta$curve_id[1])
plot_batch_drc_overlay(drc, drc$meta$curve_id[1:4])

## ----m5-cor, fig.width = 6, fig.height = 4------------------------------------
params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
plot_ic50_auc_cor(params)

## ----m6-----------------------------------------------------------------------
m  <- build_auc_matrix(drc$meta)
cl <- cluster_auc_matrix(m, k = 2)
table(cl$clusters)

## ----m6-plots, fig.width = 6, fig.height = 5, eval = requireNamespace("ComplexHeatmap", quietly = TRUE)----
plot_auc_heatmap(build_auc_matrix(drc$meta, zscore = "row"))

## ----m7-----------------------------------------------------------------------
ranked <- rank_hit_compound(params)
table(ranked$tier)
head(annotate_hit_info(ranked, hts_compound_meta))

## ----doseresponse-------------------------------------------------------------
four_pl(x = c(0.1, 1, 10), top = 1, ic50 = 1, hill = 1, bottom = 0)

compute_auc(
  min_concentration = 0.01, max_concentration = 100,
  top = 1, ic50 = 5, hill = 1, bottom = 0.05
)

one <- subset(
  screen_doseresponse,
  drug_name == "DrugA" & condition == "Gy0" & experiment_type == "sample"
)
fit <- fit_dose_response(one, "normalized_cell_count", "concentration")
coef(fit)

## ----qc-----------------------------------------------------------------------
head(screen_doseresponse)

plate <- data.frame(
  experiment_type = c("negative_ctrl", "positive_ctrl", "sample"),
  mean = c(1.00, 0.05, 0.55),
  sd = c(0.05, 0.02, 0.20)
)
calculate_qc_metrics(plate)

## ----qc-boxplot---------------------------------------------------------------
plot_qc_boxplot(screen_doseresponse)

## ----plate-map, fig.width = 8, fig.height = 5---------------------------------
plot_plate_heatmap(subset(screen_plate_layout, plateID == "EXP99_Gy0"),
  control_col = "experiment_type")

## ----qc-controls, fig.width = 7, fig.height = 4-------------------------------
plot_control_scatter(screen_plate_qc)

plot_qc_metric_trend(screen_plate_qc, metric = "z_prime", threshold = 0.5)

## ----qc-circular, fig.width = 6, fig.height = 6, eval = requireNamespace("circlize", quietly = TRUE)----
plot_qc_metric_circular(screen_plate_qc, metric = "z_prime")

plot_qc_circular_heatmap(screen_plate_qc)

## ----delta--------------------------------------------------------------------
auc_long <- data.frame(
  drug_name = rep(c("DrugA", "DrugB", "DrugC", "DrugD"), each = 2),
  condition = rep(c("Gy0", "Gy2"), times = 4),
  auc = c(5.0, 2.0, 4.2, 4.1, 6.0, 3.4, 3.0, 3.1)
)
compute_delta_auc(auc_long, conditions = c("Gy0", "Gy2"))

## ----hits---------------------------------------------------------------------
head(screen_delta_auc)

cancer <- c("A549", "H1299", "H1975", "H2030")
select_hits_cutoff(screen_delta_auc, score_cols = cancer,
                   cutoff = 0, min_pass = 4)

## ----curves-------------------------------------------------------------------
drugA <- subset(
  screen_doseresponse,
  drug_name == "DrugA" & experiment_type == "sample"
)
plot_dose_response_curves(drugA, drug = "DrugA")

## ----scatter, fig.width = 5.5, fig.height = 5.5-------------------------------
plot_condition_scatter(screen_delta_auc, x = "MRC5", y = "A549")

## ----heatmap, eval = requireNamespace("ComplexHeatmap", quietly = TRUE) && requireNamespace("circlize", quietly = TRUE)----
plot_delta_auc_heatmap(
  screen_delta_auc,
  value_cols = c("A549", "H1299", "H1975", "H2030", "MRC5"),
  cell_types = c(A549 = "Cancer", H1299 = "Cancer", H1975 = "Cancer",
                 H2030 = "Cancer", MRC5 = "Normal")
)

#' Convert a Windows path to a forward-slash path
#'
#' Replaces backslashes with forward slashes so that Windows-style paths can be
#' used consistently across platforms.
#'
#' @param path A character vector of file paths.
#'
#' @return A character vector of the same length with backslashes replaced by
#'   forward slashes.
#'
#' @examples
#' windows_to_linux_path("C:\\data\\EXP99\\raw.txt")
#'
#' @export
windows_to_linux_path <- function(path) {
  gsub("\\\\", "/", path)
}

#' Sanitise a string for use as a file name
#'
#' Removes newlines and characters that are illegal in file names on common
#' operating systems, so that drug names or plate labels can be turned into safe
#' output file names.
#'
#' @param filename_str A character vector of candidate file names.
#'
#' @return A character vector with newlines removed, `/` and `:` replaced by
#'   `_` and `-` respectively, and the characters `* ? " < > |` stripped.
#'
#' @examples
#' clean_filename("Drug A/B: 10\u00b5M?")
#'
#' @export
clean_filename <- function(filename_str) {
  filename_str <- gsub("\n", "", filename_str)
  filename_str <- gsub("/", "_", filename_str)
  filename_str <- gsub(":", "-", filename_str)
  gsub("[*?\"<>|]", "", filename_str)
}

test_that("windows_to_linux_path converts backslashes", {
  expect_equal(windows_to_linux_path("C:\\data\\file.txt"), "C:/data/file.txt")
  # vectorised
  out <- windows_to_linux_path(c("a\\b", "c\\d\\e"))
  expect_equal(out, c("a/b", "c/d/e"))
})

test_that("clean_filename sanitises illegal characters", {
  expect_equal(clean_filename("a/b:c?"), "a_b-c")
  expect_equal(clean_filename("Drug\nX"), "DrugX")
  expect_equal(clean_filename("p*q<s>"), "pqs")
})

# Module 4: Primary screen hit selection -------------------------------------

# Per-compound activity summary from well-level data.
.compound_activity <- function(data, value, compound_col,
                               compound_label = "compound") {
  df <- as.data.frame(data, stringsAsFactors = FALSE)
  if ("well_type" %in% names(df)) {
    df <- df[df$well_type == compound_label, , drop = FALSE]
  }
  for (col in c(value, compound_col)) {
    if (!col %in% names(df)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }
  agg <- stats::aggregate(
    .as_numeric(df[[value]]),
    by = list(compound_id = df[[compound_col]]),
    FUN = function(z) mean(z, na.rm = TRUE)
  )
  names(agg)[2] <- "activity"
  agg
}

#' Select primary-screen hits
#'
#' Flags active compounds from single-concentration primary-screen data using
#' either a fixed activity threshold or a percentile rank of the whole library.
#'
#' @param data An `hts_norm` object or long data frame.
#' @param value Name of the activity column. Default `"inhibition"`.
#' @param method Selection mode: `"threshold"` (default) keeps compounds with
#'   activity at or above `threshold`; `"percentile"` keeps the top fraction.
#' @param threshold Activity cut-off for `method = "threshold"`. Default `50`.
#' @param percentile Upper fraction to keep for `method = "percentile"` (for
#'   example `0.95` keeps the top 5 percent). Default `0.95`.
#' @param compound_col Name of the compound column. Default `"compound_id"`.
#'
#' @return A data frame with one row per compound: `compound_id`, `activity`,
#'   `is_hit`. The applied cut-off is stored in `attr(x, "cutoff")` and the
#'   method in `attr(x, "method")`.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' hits <- select_primary_hit(norm, threshold = 50)
#' sum(hits$is_hit)
#'
#' @seealso [summarize_primary_hit()], [plot_primary_inhibition_rank()]
#' @export
select_primary_hit <- function(data, value = "inhibition",
                               method = c("threshold", "percentile"),
                               threshold = 50, percentile = 0.95,
                               compound_col = "compound_id") {
  method <- match.arg(method)
  act <- .compound_activity(data, value, compound_col)
  if (method == "threshold") {
    cutoff <- threshold
  } else {
    cutoff <- stats::quantile(act$activity, probs = percentile, na.rm = TRUE)
  }
  act$is_hit <- is.finite(act$activity) & act$activity >= cutoff
  act <- act[order(-act$activity), , drop = FALSE]
  rownames(act) <- NULL
  attr(act, "cutoff") <- unname(cutoff)
  attr(act, "method") <- method
  act
}

#' Summarise primary-screen hits
#'
#' Reports the number of compounds screened, the number of hits and the hit
#' rate.
#'
#' @param hits A data frame from [select_primary_hit()].
#'
#' @return A one-row data frame with `n_compounds`, `n_hits`, `hit_rate`
#'   (fraction) and `cutoff`.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' summarize_primary_hit(select_primary_hit(norm))
#'
#' @seealso [select_primary_hit()]
#' @export
summarize_primary_hit <- function(hits) {
  if (!"is_hit" %in% names(hits)) {
    stop("`hits` must contain an 'is_hit' column.", call. = FALSE)
  }
  n <- nrow(hits)
  n_hit <- sum(hits$is_hit, na.rm = TRUE)
  cutoff <- attr(hits, "cutoff")
  if (is.null(cutoff)) cutoff <- NA_real_
  data.frame(
    n_compounds = n,
    n_hits = n_hit,
    hit_rate = if (n > 0) n_hit / n else NA_real_,
    cutoff = cutoff,
    stringsAsFactors = FALSE
  )
}

#' Ranked primary-activity curve
#'
#' Draws the whole library ranked by activity, with the hit cut-off marked, the
#' standard view of how many compounds clear the primary threshold.
#'
#' @param hits A data frame from [select_primary_hit()].
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' plot_primary_inhibition_rank(select_primary_hit(norm))
#'
#' @seealso [select_primary_hit()]
#' @export
plot_primary_inhibition_rank <- function(hits) {
  if (!"activity" %in% names(hits)) {
    stop("`hits` must contain an 'activity' column.", call. = FALSE)
  }
  df <- hits[order(-hits$activity), , drop = FALSE]
  df$rank <- seq_len(nrow(df))
  cutoff <- attr(hits, "cutoff")
  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$rank, y = .data$activity)) +
    ggplot2::geom_point(ggplot2::aes(color = .data$is_hit), size = 1.6,
      alpha = 0.8, stroke = 0.2) +
    ggplot2::scale_color_manual(values = c("FALSE" = .hts_col("nonhit"),
      "TRUE" = .hts_col("hit")), name = "Hit") +
    ggplot2::labs(x = "Compound rank", y = "Activity") +
    theme_hts()
  if (!is.null(cutoff) && is.finite(cutoff)) {
    p <- p + ggplot2::geom_hline(yintercept = cutoff, linetype = "dashed",
      color = .hts_col("reference"), linewidth = 0.6)
  }
  p
}

#' Hit-count bar chart
#'
#' Bar chart of the number of hit versus non-hit compounds from a primary
#' screen.
#'
#' @param hits A data frame from [select_primary_hit()].
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' plot_hit_bar_count(select_primary_hit(norm))
#'
#' @seealso [summarize_primary_hit()]
#' @export
plot_hit_bar_count <- function(hits) {
  if (!"is_hit" %in% names(hits)) {
    stop("`hits` must contain an 'is_hit' column.", call. = FALSE)
  }
  df <- as.data.frame(table(is_hit = ifelse(hits$is_hit, "Hit", "Non-hit")))
  names(df) <- c("category", "count")
  ggplot2::ggplot(df, ggplot2::aes(x = .data$category, y = .data$count,
    fill = .data$category)) +
    ggplot2::geom_col(width = 0.6, color = "white", linewidth = 0.3) +
    ggplot2::geom_text(ggplot2::aes(label = .data$count), vjust = -0.4,
      color = "#333333", size = 3.6) +
    ggplot2::scale_fill_manual(values = c("Hit" = .hts_col("hit"),
      "Non-hit" = .hts_col("nonhit")), guide = "none") +
    ggplot2::scale_y_continuous(expand = ggplot2::expansion(mult = c(0, 0.1))) +
    ggplot2::labs(x = NULL, y = "Compounds") +
    theme_hts()
}

#' Select primary-screen hits by distance from the library distribution
#'
#' Treats the per-compound activity of the whole library as an approximately
#' normal "inactive" distribution and flags compounds sitting far out in the
#' tail. Each compound receives a z-score (how many standard deviations it lies
#' from the library centre) and is binned by whether it clears 1, 2 or 3 sigma.
#' By default the centre and spread are estimated *robustly* (median and MAD),
#' the HTS-standard choice: a handful of genuine hits would otherwise inflate a
#' plain mean/SD and mask themselves (with too wide an SD nothing reaches 3
#' sigma).
#'
#' @param data An `hts_norm` object or long data frame.
#' @param value Name of the activity column. Default `"inhibition"`.
#' @param method How to estimate the distribution centre and spread:
#'   `"robust"` (default) uses the median and MAD (scaled by 1.4826 to match the
#'   SD of a normal); `"sd"` uses the plain mean and standard deviation.
#' @param n_sigma Number of sigma a compound must clear to be called a hit.
#'   Default `3`.
#' @param direction Tail to test: `"greater"` (default, high activity),
#'   `"less"` (low activity) or `"two.sided"` (either tail, scored on `abs(z)`).
#' @param compound_col Name of the compound column. Default `"compound_id"`.
#'
#' @return A data frame with one row per compound: `compound_id`, `activity`,
#'   `sigma` (signed z-score), `sigma_level` (0-3, how many sigma it clears in
#'   the tested direction), `band` (an ordered-factor label) and `is_hit`
#'   (`sigma_level >= n_sigma`). The distribution `center`, `scale`, `method`,
#'   `n_sigma` and `direction` are stored as attributes.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' hits <- select_sigma_hits(norm, n_sigma = 3)
#' sum(hits$is_hit)
#'
#' @seealso [summarize_sigma_hits()], [plot_sigma_hits()], [select_primary_hit()]
#' @export
select_sigma_hits <- function(data, value = "inhibition",
                              method = c("robust", "sd"),
                              n_sigma = 3,
                              direction = c("greater", "less", "two.sided"),
                              compound_col = "compound_id") {
  method <- match.arg(method)
  direction <- match.arg(direction)
  act <- .compound_activity(data, value, compound_col)
  x <- .as_numeric(act$activity)
  if (method == "robust") {
    center <- stats::median(x, na.rm = TRUE)
    scale <- stats::mad(x, center = center, constant = 1.4826, na.rm = TRUE)
  } else {
    center <- mean(x, na.rm = TRUE)
    scale <- stats::sd(x, na.rm = TRUE)
  }
  z <- if (is.finite(scale) && scale > 0) (x - center) / scale else
    rep(NA_real_, length(x))
  dev <- switch(direction, greater = z, less = -z, two.sided = abs(z))
  lvl <- ifelse(is.finite(dev), pmin(3L, pmax(0L, as.integer(floor(dev)))),
    NA_integer_)
  band_lab <- c("< 1 sigma", ">= 1 sigma", ">= 2 sigma", ">= 3 sigma")
  act$sigma <- z
  act$sigma_level <- lvl
  act$band <- factor(band_lab[lvl + 1L], levels = band_lab, ordered = TRUE)
  act$is_hit <- is.finite(dev) & lvl >= n_sigma
  act <- act[order(-dev), , drop = FALSE]
  rownames(act) <- NULL
  attr(act, "center") <- unname(center)
  attr(act, "scale") <- unname(scale)
  attr(act, "method") <- method
  attr(act, "n_sigma") <- n_sigma
  attr(act, "direction") <- direction
  act
}

#' Summarise sigma-based hits
#'
#' Counts how many compounds clear 1, 2 and 3 sigma from the library centre and
#' compares each count to what a perfectly normal, hit-free distribution would
#' predict, so enrichment of the tail over chance is obvious.
#'
#' @param hits A data frame from [select_sigma_hits()].
#'
#' @return A data frame with one row per sigma level (1, 2, 3): `sigma`,
#'   `n_beyond` (compounds clearing it), `frac_beyond`, `expected_frac`
#'   (normal-theory tail probability) and `expected_n` (expected count).
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' summarize_sigma_hits(select_sigma_hits(norm))
#'
#' @seealso [select_sigma_hits()], [plot_sigma_hits()]
#' @export
summarize_sigma_hits <- function(hits) {
  if (!"sigma_level" %in% names(hits)) {
    stop("`hits` must come from `select_sigma_hits()`.", call. = FALSE)
  }
  direction <- attr(hits, "direction")
  if (is.null(direction)) direction <- "greater"
  n <- sum(!is.na(hits$sigma_level))
  tail_p <- function(k) {
    if (direction == "two.sided") 2 * stats::pnorm(k, lower.tail = FALSE) else
      stats::pnorm(k, lower.tail = FALSE)
  }
  ks <- 1:3
  n_beyond <- vapply(ks,
    function(k) sum(hits$sigma_level >= k, na.rm = TRUE), integer(1))
  exp_frac <- vapply(ks, tail_p, numeric(1))
  data.frame(
    sigma = ks,
    n_beyond = n_beyond,
    frac_beyond = if (n > 0) n_beyond / n else NA_real_,
    expected_frac = exp_frac,
    expected_n = exp_frac * n,
    stringsAsFactors = FALSE
  )
}

#' Distribution plot of sigma-based hits
#'
#' Histogram of the per-compound activity with the library centre and the 1, 2
#' and 3 sigma cut-offs marked, bars coloured by how many sigma each compound
#' clears. Optionally overlays the fitted normal curve so the departure from
#' normality (the hit tail) stands out.
#'
#' @param hits A data frame from [select_sigma_hits()].
#' @param bins Number of histogram bins. Default `30`.
#' @param show_normal Overlay the fitted normal density curve. Default `TRUE`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' plot_sigma_hits(select_sigma_hits(norm))
#'
#' @seealso [select_sigma_hits()], [summarize_sigma_hits()]
#' @export
plot_sigma_hits <- function(hits, bins = 30, show_normal = TRUE) {
  if (!all(c("activity", "band") %in% names(hits))) {
    stop("`hits` must come from `select_sigma_hits()`.", call. = FALSE)
  }
  center <- attr(hits, "center")
  scale <- attr(hits, "scale")
  direction <- attr(hits, "direction")
  if (is.null(direction)) direction <- "greater"
  x <- .as_numeric(hits$activity)
  df <- data.frame(activity = x, band = hits$band)
  rng <- range(x[is.finite(x)])
  bw <- (rng[2] - rng[1]) / bins
  if (!is.finite(bw) || bw <= 0) bw <- 1
  sigma_cols <- c("< 1 sigma" = "#B4B4B4", ">= 1 sigma" = "#56B4E9",
    ">= 2 sigma" = "#E69F00", ">= 3 sigma" = "#D55E00")
  ks <- 1:3
  pos <- switch(direction,
    greater = center + ks * scale,
    less = center - ks * scale,
    two.sided = c(center - ks * scale, center + ks * scale))
  lab_pos <- switch(direction,
    less = center - ks * scale,
    center + ks * scale)
  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$activity)) +
    ggplot2::geom_histogram(ggplot2::aes(fill = .data$band), binwidth = bw,
      boundary = center, color = "white", linewidth = 0.2) +
    ggplot2::scale_fill_manual(values = sigma_cols, drop = FALSE,
      name = "Distance from centre") +
    ggplot2::scale_y_continuous(expand = ggplot2::expansion(mult = c(0, 0.08)))
  if (isTRUE(show_normal) && is.finite(scale) && scale > 0) {
    n <- sum(is.finite(x))
    p <- p + ggplot2::stat_function(fun = function(z)
      stats::dnorm(z, mean = center, sd = scale) * n * bw,
      color = .hts_col("neutral"), linewidth = 0.8, na.rm = TRUE)
  }
  p +
    ggplot2::geom_vline(xintercept = center, color = .hts_col("reference"),
      linewidth = 0.6) +
    ggplot2::geom_vline(xintercept = pos, linetype = "dashed",
      color = .hts_col("reference"), linewidth = 0.4) +
    ggplot2::annotate("text", x = lab_pos, y = Inf, label = paste0(ks, " sigma"),
      angle = 90, vjust = -0.4, hjust = 1.05, size = 2.9,
      color = .hts_col("reference")) +
    ggplot2::labs(x = "Per-compound activity", y = "Compounds") +
    theme_hts()
}

# Module 1: Raw data import and pre-QC ---------------------------------------

# Canonicalise a well-type vector against the configured control labels.
.canon_well_type <- function(x, nc_label, pc_label, blank_label) {
  x <- as.character(x)
  lx <- tolower(trimws(x))
  out <- rep("compound", length(x))
  out[lx %in% tolower(nc_label)] <- "NC"
  out[lx %in% tolower(pc_label)] <- "PC"
  out[lx %in% tolower(blank_label)] <- "Blank"
  out
}

#' Import a plate readout file into a standard HTS object
#'
#' Reads an instrument export (csv, txt/tsv or Excel) or an in-memory data frame
#' of well-level readouts and standardises it into an `hts_raw` object with a
#' fixed set of columns, ready for the rest of the pipeline. Well coordinates are
#' parsed from the well identifier and control wells are canonicalised to the
#' labels `"NC"`, `"PC"` and `"Blank"`.
#'
#' @param file A path to a `.csv`, `.txt`/`.tsv` or `.xlsx`/`.xls` file, or an
#'   existing data frame of well-level readings.
#' @param plate_id Optional plate identifier used when the data has no plate
#'   column.
#' @param well_col,signal_col,compound_col,conc_col,type_col,plate_col Names of
#'   the well, signal, compound, concentration, well-type and plate columns in
#'   `file`. Missing optional columns are filled with `NA`.
#' @param nc_label,pc_label,blank_label Character vectors of the labels that
#'   mark negative controls, positive controls and blanks (matched
#'   case-insensitively in `type_col`, or in `compound_col` when `type_col` is
#'   absent).
#' @param sep Field separator for `.txt`/`.tsv` files. Default tab.
#' @param sheet Worksheet passed to \pkg{readxl} for Excel files. Default `1`.
#'
#' @return An `hts_raw` object (a data frame subclass) with columns `plate_id`,
#'   `well`, `row`, `col`, `well_type`, `compound_id`, `concentration` and
#'   `signal`.
#'
#' @details Reading Excel files requires the suggested package \pkg{readxl}.
#'
#' @examples
#' raw <- read_hts_plate(hts_primary_raw)
#' table(raw$well_type)
#'
#' @seealso [baseline_subtract()], [calc_z_prime()], [norm_by_control()]
#' @export
read_hts_plate <- function(file, plate_id = NULL,
                           well_col = "well", signal_col = "signal",
                           compound_col = "compound_id",
                           conc_col = "concentration",
                           type_col = "well_type", plate_col = "plate_id",
                           nc_label = "NC", pc_label = "PC",
                           blank_label = "Blank", sep = "\t", sheet = 1) {
  if (is.data.frame(file)) {
    df <- as.data.frame(file, stringsAsFactors = FALSE)
  } else if (is.character(file) && length(file) == 1L) {
    ext <- tolower(tools::file_ext(file))
    df <- if (ext %in% c("xls", "xlsx")) {
      if (!requireNamespace("readxl", quietly = TRUE)) {
        stop("Reading Excel files requires the 'readxl' package.",
          call. = FALSE
        )
      }
      as.data.frame(readxl::read_excel(file, sheet = sheet))
    } else if (ext == "csv") {
      utils::read.csv(file, stringsAsFactors = FALSE, check.names = FALSE)
    } else {
      utils::read.table(file, header = TRUE, sep = sep,
        stringsAsFactors = FALSE, check.names = FALSE)
    }
  } else {
    stop("`file` must be a data frame or a single file path.", call. = FALSE)
  }

  if (!well_col %in% names(df)) {
    stop("Well column '", well_col, "' not found.", call. = FALSE)
  }
  if (!signal_col %in% names(df)) {
    stop("Signal column '", signal_col, "' not found.", call. = FALSE)
  }

  wells <- as.character(df[[well_col]])
  rows <- toupper(sub("^([A-Za-z]+).*$", "\\1", wells))
  cols <- as.integer(sub("^[A-Za-z]+0*([0-9]+)$", "\\1", wells))

  pid <- if (plate_col %in% names(df)) {
    as.character(df[[plate_col]])
  } else if (!is.null(plate_id)) {
    rep(as.character(plate_id), nrow(df))
  } else {
    rep("plate1", nrow(df))
  }
  compound <- if (compound_col %in% names(df)) {
    as.character(df[[compound_col]])
  } else {
    NA_character_
  }
  concentration <- if (conc_col %in% names(df)) {
    .as_numeric(df[[conc_col]])
  } else {
    NA_real_
  }
  well_type <- if (type_col %in% names(df)) {
    .canon_well_type(df[[type_col]], nc_label, pc_label, blank_label)
  } else {
    .canon_well_type(compound, nc_label, pc_label, blank_label)
  }

  out <- data.frame(
    plate_id = pid, well = wells, row = rows, col = cols,
    well_type = well_type, compound_id = compound,
    concentration = concentration,
    signal = .as_numeric(df[[signal_col]]),
    stringsAsFactors = FALSE
  )
  .as_hts(out, "hts_raw")
}

#' Read a user-defined plate-map layout file
#'
#' Reads a plate map that **you author yourself** in Excel or CSV to declare, for
#' every well, whether it holds a negative control, a positive control, a blank
#' (empty/edge) well or a test compound. Nothing about the layout is fixed by the
#' package: you are free to place the controls, blanks and compounds in whichever
#' wells your assay uses, and [apply_plate_layout()] then stamps those roles onto
#' the measured readings.
#'
#' The expected file is a **grid** that mirrors the physical plate: the first
#' column holds the row letters (`A`, `B`, ...), the remaining column headers are
#' the plate column numbers (`1`, `2`, ... `24`), and each cell contains the
#' label for that well. Labels are matched case-insensitively against
#' `nc_labels`, `pc_labels` and `blank_labels`; any other non-empty label is
#' treated as a compound identifier. An optional second grid of the same shape
#' supplies per-well concentrations.
#'
#' @param file Path to the plate-map grid (`.csv`, `.txt`/`.tsv` or
#'   `.xlsx`/`.xls`).
#' @param plate_id Optional plate identifier attached to every well of the
#'   layout, so the same map can be matched to a specific plate.
#' @param nc_labels,pc_labels,blank_labels Character vectors of the cell labels
#'   that mark negative controls, positive controls and blank/empty wells. All
#'   other non-empty labels become compound identifiers.
#' @param concentration_file Optional path to a second grid (same layout) giving
#'   the concentration of each well.
#' @param sheet,sep Worksheet (Excel) and field separator (`.txt`/`.tsv`) passed
#'   to the reader.
#'
#' @return An `hts_layout` data frame with one row per well and the columns
#'   `plate_id`, `well`, `row`, `col`, `well_type` (`"NC"`, `"PC"`, `"Blank"` or
#'   `"compound"`), `compound_id` and `concentration`.
#'
#' @examples
#' f <- system.file("extdata", "plate_layout_example.csv", package = "diffHTS")
#' layout <- read_plate_layout(f, plate_id = "P01")
#' table(layout$well_type)
#'
#' @seealso [apply_plate_layout()], [read_hts_plate()], [check_control_label()]
#' @export
read_plate_layout <- function(file, plate_id = NULL,
                              nc_labels = c("NC", "DMSO", "vehicle",
                                "negative_ctrl", "neg_ctrl"),
                              pc_labels = c("PC", "kill_ctrl",
                                "positive_ctrl", "pos_ctrl"),
                              blank_labels = c("Blank", "edgewell", "empty",
                                "blank", "na", ""),
                              concentration_file = NULL, sheet = 1,
                              sep = "\t") {
  grid <- if (is.data.frame(file)) file else .read_hts_file(file, sep, sheet)
  long <- .grid_to_long(grid, "content")

  content <- long$content
  lc <- tolower(trimws(content))
  well_type <- rep("compound", length(content))
  well_type[lc %in% tolower(nc_labels)] <- "NC"
  well_type[lc %in% tolower(pc_labels)] <- "PC"
  is_blank <- lc %in% tolower(blank_labels) | is.na(content) | content == ""
  well_type[is_blank] <- "Blank"

  compound_id <- content
  compound_id[is_blank] <- NA_character_

  layout <- data.frame(
    plate_id = if (!is.null(plate_id)) as.character(plate_id) else NA_character_,
    well = long$well, row = long$row, col = long$col,
    well_type = well_type, compound_id = compound_id,
    concentration = NA_real_, stringsAsFactors = FALSE
  )

  if (!is.null(concentration_file)) {
    cg <- if (is.data.frame(concentration_file)) {
      concentration_file
    } else {
      .read_hts_file(concentration_file, sep, sheet)
    }
    clong <- .grid_to_long(cg, "concentration")
    conc <- .as_numeric(clong$concentration)
    layout$concentration <- conc[match(layout$well, clong$well)]
  }

  .as_hts(layout, "hts_layout")
}

#' Stamp a plate-map layout onto measured readings
#'
#' Joins a user-defined layout from [read_plate_layout()] onto a table of raw
#' signal readings, matching by well address (and plate identifier when both
#' sides carry one), and returns a standard `hts_raw` object. This keeps the
#' *what was measured* (signal) and the *what each well is* (role, compound,
#' concentration) in separate files, exactly as a screening lab records them,
#' and lets the same signal export be re-interpreted against different layouts.
#'
#' @param readings A data frame (or `hts_raw`) of readings with at least a well
#'   column and a signal column.
#' @param layout An `hts_layout` from [read_plate_layout()], or a data frame with
#'   `well`, `well_type`, `compound_id` and (optionally) `concentration`.
#' @param well_col,signal_col Names of the well and signal columns in
#'   `readings`.
#' @param plate_col Name of the plate column. When present in both `readings`
#'   and a multi-plate `layout`, wells are matched within each plate.
#' @param plate_id Optional plate identifier used when `readings` has no plate
#'   column.
#'
#' @return An `hts_raw` object with columns `plate_id`, `well`, `row`, `col`,
#'   `well_type`, `compound_id`, `concentration` and `signal`, ready for the rest
#'   of the pipeline.
#'
#' @examples
#' f <- system.file("extdata", "plate_layout_example.csv", package = "diffHTS")
#' layout <- read_plate_layout(f, plate_id = "P01")
#' # Simulate a signal export for the same wells.
#' readings <- data.frame(well = layout$well,
#'                         signal = runif(nrow(layout), 1e4, 1e5))
#' raw <- apply_plate_layout(readings, layout, plate_id = "P01")
#' table(raw$well_type)
#'
#' @seealso [read_plate_layout()], [read_hts_plate()]
#' @export
apply_plate_layout <- function(readings, layout, well_col = "well",
                               signal_col = "signal", plate_col = "plate_id",
                               plate_id = NULL) {
  rd <- as.data.frame(readings, stringsAsFactors = FALSE)
  lay <- as.data.frame(layout, stringsAsFactors = FALSE)
  if (!well_col %in% names(rd)) {
    stop("Well column '", well_col, "' not found in `readings`.", call. = FALSE)
  }
  if (!signal_col %in% names(rd)) {
    stop("Signal column '", signal_col, "' not found in `readings`.",
      call. = FALSE)
  }
  if (!"well" %in% names(lay)) {
    stop("`layout` must contain a 'well' column.", call. = FALSE)
  }

  rd_well <- as.character(rd[[well_col]])
  rd_plate <- if (plate_col %in% names(rd)) {
    as.character(rd[[plate_col]])
  } else if (!is.null(plate_id)) {
    rep(as.character(plate_id), nrow(rd))
  } else {
    rep("plate1", nrow(rd))
  }

  layout_multi <- plate_col %in% names(lay) &&
    !all(is.na(lay[[plate_col]])) && length(unique(lay[[plate_col]])) > 1L
  key_rd <- if (layout_multi) paste(rd_plate, rd_well, sep = "\r") else rd_well
  key_lay <- if (layout_multi) {
    paste(as.character(lay[[plate_col]]), lay$well, sep = "\r")
  } else {
    lay$well
  }
  idx <- match(key_rd, key_lay)
  if (anyNA(idx)) {
    stop(sum(is.na(idx)), " reading(s) have no matching well in `layout`.",
      call. = FALSE)
  }

  well_type <- if ("well_type" %in% names(lay)) lay$well_type[idx] else "compound"
  compound <- if ("compound_id" %in% names(lay)) {
    as.character(lay$compound_id[idx])
  } else {
    NA_character_
  }
  concentration <- if ("concentration" %in% names(lay)) {
    .as_numeric(lay$concentration[idx])
  } else {
    NA_real_
  }

  rows <- toupper(sub("^([A-Za-z]+).*$", "\\1", rd_well))
  cols <- as.integer(sub("^[A-Za-z]+0*([0-9]+)$", "\\1", rd_well))

  out <- data.frame(
    plate_id = rd_plate, well = rd_well, row = rows, col = cols,
    well_type = well_type, compound_id = compound,
    concentration = concentration, signal = .as_numeric(rd[[signal_col]]),
    stringsAsFactors = FALSE
  )
  .as_hts(out, "hts_raw")
}

.infer_plate_format <- function(max_row, max_col) {
  for (f in c(6L, 12L, 24L, 48L, 96L, 384L, 1536L)) {
    d <- .plate_dims(f)
    if (max_row <= d[1] && max_col <= d[2]) return(f)
  }
  NA_integer_
}

# Collapse a vector of integers into a compact "1, 3-11, 12" style string.
.collapse_int_ranges <- function(x) {
  x <- sort(unique(stats::na.omit(as.integer(x))))
  if (length(x) == 0) return("")
  breaks <- c(0L, which(diff(x) != 1L), length(x))
  parts <- vapply(seq_len(length(breaks) - 1L), function(i) {
    seg <- x[(breaks[i] + 1L):breaks[i + 1L]]
    if (length(seg) == 1L) as.character(seg) else
      paste0(seg[1], "-", seg[length(seg)])
  }, character(1))
  paste(parts, collapse = ", ")
}

#' Summarise how each plate in a run is set up
#'
#' Reports, for every plate in an `hts_raw` object, how the plate is laid out:
#' its physical format, the number of wells of each role (negative/positive
#' controls, blanks and compounds), how many distinct compounds it carries, and
#' which columns the controls and compounds occupy. The positions are **read
#' back from your own layout** (see [read_plate_layout()]) rather than assumed by
#' the package: because a screening run normally spans **several plates at
#' once**, this is the quickest way to confirm that every plate carries the
#' layout you designed before QC and normalisation.
#'
#' @param hts_raw An `hts_raw` object from [read_hts_plate()] (or any well-level
#'   data frame with plate, well-coordinate and well-type columns).
#' @param plate_col,type_col,compound_col,col_col Names of the plate,
#'   well-type, compound and column-index columns. Defaults match
#'   [read_hts_plate()] output.
#' @param nc_label,pc_label,blank_label,compound_label Well-type labels for the
#'   four roles.
#'
#' @return A data frame with one row per plate and the columns `plate_id`,
#'   `n_wells`, `plate_format`, `n_rows`, `n_cols`, `n_nc`, `n_pc`, `n_blank`,
#'   `n_compound`, `n_unique_compound`, `nc_columns`, `pc_columns`,
#'   `blank_columns` and `compound_columns` (control/compound positions given as
#'   compact column ranges such as `"3-11"`).
#'
#' @examples
#' raw <- read_hts_plate(hts_primary_raw)
#' # One row per plate; the run holds three 96-well plates here.
#' summarize_plate_setup(raw)
#'
#' @seealso [read_hts_plate()], [check_control_label()], [calc_z_prime()]
#' @export
summarize_plate_setup <- function(hts_raw, plate_col = "plate_id",
                                  type_col = "well_type",
                                  compound_col = "compound_id",
                                  col_col = "col",
                                  nc_label = "NC", pc_label = "PC",
                                  blank_label = "Blank",
                                  compound_label = "compound") {
  df <- as.data.frame(hts_raw, stringsAsFactors = FALSE)
  for (col in c(plate_col, type_col)) {
    if (!col %in% names(df)) {
      stop("Column '", col, "' not found in `hts_raw`.", call. = FALSE)
    }
  }
  has_col <- col_col %in% names(df)
  plates <- unique(as.character(df[[plate_col]]))
  cols_for <- function(d, role) {
    if (!has_col) return("")
    .collapse_int_ranges(d[[col_col]][d[[type_col]] == role])
  }
  out <- lapply(plates, function(p) {
    d <- df[as.character(df[[plate_col]]) == p, , drop = FALSE]
    max_row <- if ("row" %in% names(d))
      max(match(toupper(d$row), .plate_row_labels(48)), na.rm = TRUE) else NA
    max_col <- if (has_col) max(.as_numeric(d[[col_col]]), na.rm = TRUE) else NA
    fmt <- if (is.finite(max_row) && is.finite(max_col))
      .infer_plate_format(max_row, max_col) else NA_integer_
    cpd <- if (compound_col %in% names(d))
      d[[compound_col]][d[[type_col]] == compound_label] else character(0)
    data.frame(
      plate_id = p,
      n_wells = nrow(d),
      plate_format = fmt,
      n_rows = if (is.finite(max_row)) max_row else NA_integer_,
      n_cols = if (is.finite(max_col)) as.integer(max_col) else NA_integer_,
      n_nc = sum(d[[type_col]] == nc_label),
      n_pc = sum(d[[type_col]] == pc_label),
      n_blank = sum(d[[type_col]] == blank_label),
      n_compound = sum(d[[type_col]] == compound_label),
      n_unique_compound = length(unique(stats::na.omit(cpd))),
      nc_columns = cols_for(d, nc_label),
      pc_columns = cols_for(d, pc_label),
      blank_columns = cols_for(d, blank_label),
      compound_columns = cols_for(d, compound_label),
      stringsAsFactors = FALSE
    )
  })
  res <- do.call(rbind, out)
  rownames(res) <- NULL
  res
}

#' Subtract the blank background signal from a plate
#'
#' Removes the per-plate instrument background by subtracting the mean blank
#' (no-cell) signal from every well, a standard first step for luminescence and
#' fluorescence readouts. The original signal is preserved in `signal_raw`.
#'
#' @param hts_raw An `hts_raw` object from [read_hts_plate()].
#' @param blank_label Well-type label identifying blank wells. Default
#'   `"Blank"`.
#' @param clip_zero Logical; if `TRUE` (default) background-subtracted signals
#'   are floored at zero.
#'
#' @return The input object with `signal` replaced by the background-subtracted
#'   value and a new `signal_raw` column holding the original signal.
#'
#' @examples
#' raw <- read_hts_plate(hts_primary_raw)
#' bs <- baseline_subtract(raw)
#' head(bs[, c("well_type", "signal_raw", "signal")])
#'
#' @seealso [read_hts_plate()], [norm_by_control()]
#' @export
baseline_subtract <- function(hts_raw, blank_label = "Blank",
                              clip_zero = TRUE) {
  if (!"signal" %in% names(hts_raw)) {
    stop("`hts_raw` has no 'signal' column.", call. = FALSE)
  }
  df <- hts_raw
  df$signal_raw <- df$signal
  plates <- split(seq_len(nrow(df)), df$plate_id)
  for (idx in plates) {
    blanks <- df$signal_raw[idx][df$well_type[idx] == blank_label]
    bg <- if (length(blanks) > 0) mean(blanks, na.rm = TRUE) else 0
    df$signal[idx] <- df$signal_raw[idx] - bg
  }
  if (clip_zero) df$signal[df$signal < 0] <- 0
  .as_hts(df, "hts_raw")
}

#' Flag outlier wells by the IQR rule
#'
#' Detects anomalous wells (for example contaminated, bubble or crystallisation
#' artefacts) using the interquartile-range rule within groups of comparable
#' wells, and optionally removes them.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column to test. Default `"signal"`.
#' @param group_cols Columns defining comparable well groups within which the
#'   IQR is computed. Default `c("plate_id", "well_type")`.
#' @param k IQR multiplier for the fences. Default `1.5`.
#' @param filter Logical; if `TRUE` outlier wells are dropped instead of only
#'   flagged. Default `FALSE`.
#'
#' @return The input object with a logical `outlier` column added (or with
#'   outlier rows removed when `filter = TRUE`).
#'
#' @examples
#' raw <- read_hts_plate(hts_primary_raw)
#' flagged <- detect_outlier_wells(raw)
#' sum(flagged$outlier)
#'
#' @seealso [calc_z_prime()], [filter_valid_plates()]
#' @export
detect_outlier_wells <- function(hts_raw, signal_col = "signal",
                                 group_cols = c("plate_id", "well_type"),
                                 k = 1.5, filter = FALSE) {
  if (!signal_col %in% names(hts_raw)) {
    stop("Column '", signal_col, "' not found in `hts_raw`.", call. = FALSE)
  }
  df <- hts_raw
  key <- interaction(df[group_cols], drop = TRUE)
  df$outlier <- FALSE
  for (g in levels(key)) {
    idx <- which(key == g)
    v <- .as_numeric(df[[signal_col]][idx])
    qs <- stats::quantile(v, c(0.25, 0.75), na.rm = TRUE)
    iqr <- qs[2] - qs[1]
    lo <- qs[1] - k * iqr
    hi <- qs[2] + k * iqr
    df$outlier[idx] <- is.finite(v) & (v < lo | v > hi)
  }
  if (filter) df <- df[!df$outlier, , drop = FALSE]
  .as_hts(df, "hts_raw")
}

#' Compute per-plate Z' factors
#'
#' Calculates the Z' (Z-prime) factor for each plate from its negative- and
#' positive-control populations and flags plates against an acceptance
#' threshold.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column. Default `"signal"`.
#' @param nc_label,pc_label Well-type labels for the negative and positive
#'   controls. Defaults `"NC"` and `"PC"`.
#' @param z_prime_min Acceptance threshold. Default `0.5`.
#'
#' @return A data frame with one row per plate: `plate_id`, `mean_nc`,
#'   `mean_pc`, `sd_nc`, `sd_pc`, `z_prime` and a logical `pass`.
#'
#' @examples
#' raw <- read_hts_plate(hts_primary_raw)
#' calc_z_prime(raw)
#'
#' @seealso [filter_valid_plates()], [plot_plate_qc()]
#' @export
calc_z_prime <- function(hts_raw, signal_col = "signal",
                         nc_label = "NC", pc_label = "PC",
                         z_prime_min = 0.5) {
  if (!signal_col %in% names(hts_raw)) {
    stop("Column '", signal_col, "' not found in `hts_raw`.", call. = FALSE)
  }
  plates <- unique(as.character(hts_raw$plate_id))
  out <- lapply(plates, function(p) {
    d <- hts_raw[hts_raw$plate_id == p, , drop = FALSE]
    nc <- .as_numeric(d[[signal_col]][d$well_type == nc_label])
    pc <- .as_numeric(d[[signal_col]][d$well_type == pc_label])
    m_nc <- mean(nc, na.rm = TRUE)
    m_pc <- mean(pc, na.rm = TRUE)
    s_nc <- stats::sd(nc, na.rm = TRUE)
    s_pc <- stats::sd(pc, na.rm = TRUE)
    data.frame(
      plate_id = p, mean_nc = m_nc, mean_pc = m_pc,
      sd_nc = s_nc, sd_pc = s_pc,
      z_prime = 1 - 3 * (s_nc + s_pc) / abs(m_nc - m_pc),
      stringsAsFactors = FALSE
    )
  })
  res <- do.call(rbind, out)
  res$pass <- res$z_prime >= z_prime_min
  rownames(res) <- NULL
  res
}

# Per-group location/scale summary used by the QC metrics.
.qc_group_stats <- function(x) {
  x <- .as_numeric(x)
  list(
    n = sum(is.finite(x)),
    mean = mean(x, na.rm = TRUE), sd = stats::sd(x, na.rm = TRUE),
    median = stats::median(x, na.rm = TRUE), mad = stats::mad(x, na.rm = TRUE)
  )
}

# Per-plate location/scale of the NC, PC and sample populations. Shared by every
# single-metric QC helper and by calc_plate_qc() so all formulas agree.
.plate_control_stats <- function(hts_raw, signal_col = "signal",
                                 nc_label = "NC", pc_label = "PC",
                                 sample_label = "compound") {
  if (!signal_col %in% names(hts_raw)) {
    stop("Column '", signal_col, "' not found in `hts_raw`.", call. = FALSE)
  }
  plates <- unique(as.character(hts_raw$plate_id))
  do.call(rbind, lapply(plates, function(p) {
    d <- hts_raw[as.character(hts_raw$plate_id) == p, , drop = FALSE]
    nc <- .qc_group_stats(d[[signal_col]][d$well_type == nc_label])
    pc <- .qc_group_stats(d[[signal_col]][d$well_type == pc_label])
    sm <- .qc_group_stats(d[[signal_col]][d$well_type == sample_label])
    data.frame(
      plate_id = p,
      mean_nc = nc$mean, mean_pc = pc$mean, sd_nc = nc$sd, sd_pc = pc$sd,
      median_nc = nc$median, median_pc = pc$median,
      mad_nc = nc$mad, mad_pc = pc$mad,
      mean_sample = sm$mean, sd_sample = sm$sd,
      stringsAsFactors = FALSE
    )
  }))
}

#' Robust Z'-factor per plate
#'
#' Median/MAD version of the Z'-factor. **What it is for:** it answers the same
#' question as [calc_z_prime()] — how cleanly the positive and negative controls
#' separate — but because it uses the median and the median absolute deviation
#' instead of the mean and SD, a couple of splashed or crystallised control
#' wells cannot single-handedly sink an otherwise good plate. Prefer it when
#' control replicates occasionally contain outliers.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column. Default `"signal"`.
#' @param nc_label,pc_label Well-type labels for the negative and positive
#'   controls. Defaults `"NC"` and `"PC"`.
#' @param z_prime_min Acceptance threshold (`>= 0.5` is the usual cut-off for an
#'   excellent assay). Default `0.5`.
#'
#' @return A data frame with one row per plate: `plate_id`, `median_nc`,
#'   `median_pc`, `mad_nc`, `mad_pc`, `robust_z_prime` and a logical `pass`.
#'
#' @examples
#' calc_robust_z_prime(read_hts_plate(hts_primary_raw))
#'
#' @seealso [calc_z_prime()], [calc_plate_qc()], [plot_plate_qc()]
#' @export
calc_robust_z_prime <- function(hts_raw, signal_col = "signal",
                                nc_label = "NC", pc_label = "PC",
                                z_prime_min = 0.5) {
  s <- .plate_control_stats(hts_raw, signal_col, nc_label, pc_label)
  res <- data.frame(
    plate_id = s$plate_id,
    median_nc = s$median_nc, median_pc = s$median_pc,
    mad_nc = s$mad_nc, mad_pc = s$mad_pc,
    robust_z_prime = 1 - 3 * (s$mad_nc + s$mad_pc) /
      abs(s$median_nc - s$median_pc),
    stringsAsFactors = FALSE
  )
  res$pass <- res$robust_z_prime >= z_prime_min
  rownames(res) <- NULL
  res
}

#' Z-factor per plate (sample-aware)
#'
#' The screening-window quality factor computed from the **test-compound**
#' population against the negative control. **What it is for:** unlike the
#' Z'-factor (controls only), the Z-factor tells you whether the assay window is
#' large enough relative to the spread of the actual library wells. Values above
#' `0.5` indicate an excellent screen and `0`--`0.5` a marginal one; it is
#' routinely low or negative in a single-point primary screen, because most
#' library compounds are inactive and therefore sit at the negative-control
#' level — that is expected, not a failure.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column. Default `"signal"`.
#' @param nc_label,sample_label Well-type labels for the negative control and
#'   the test wells. Defaults `"NC"` and `"compound"`.
#' @param z_factor_min Acceptance threshold. Default `0.5`.
#'
#' @return A data frame with one row per plate: `plate_id`, `mean_sample`,
#'   `mean_nc`, `sd_sample`, `sd_nc`, `z_factor` and a logical `pass`.
#'
#' @examples
#' calc_z_factor(read_hts_plate(hts_primary_raw))
#'
#' @seealso [calc_z_prime()], [calc_plate_qc()]
#' @export
calc_z_factor <- function(hts_raw, signal_col = "signal", nc_label = "NC",
                          sample_label = "compound", z_factor_min = 0.5) {
  s <- .plate_control_stats(hts_raw, signal_col, nc_label,
    sample_label = sample_label)
  res <- data.frame(
    plate_id = s$plate_id,
    mean_sample = s$mean_sample, mean_nc = s$mean_nc,
    sd_sample = s$sd_sample, sd_nc = s$sd_nc,
    z_factor = 1 - 3 * (s$sd_sample + s$sd_nc) /
      abs(s$mean_sample - s$mean_nc),
    stringsAsFactors = FALSE
  )
  res$pass <- res$z_factor >= z_factor_min
  rownames(res) <- NULL
  res
}

#' SSMD per plate (strictly standardised mean difference)
#'
#' The mean control difference divided by the pooled control spread. **What it
#' is for:** SSMD is the effect-size measure of control separation, and it is
#' the statistically principled companion to the Z'-factor for setting hit
#' cut-offs. Its magnitude grades the assay — roughly `|SSMD|` `>= 2` is
#' excellent, `1`--`2` good and `< 1` weak — independent of the number of
#' control replicates.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column. Default `"signal"`.
#' @param nc_label,pc_label Well-type labels for the negative and positive
#'   controls. Defaults `"NC"` and `"PC"`.
#' @param ssmd_min Acceptance threshold applied to `abs(ssmd)`. Default `2`.
#'
#' @return A data frame with one row per plate: `plate_id`, `mean_nc`,
#'   `mean_pc`, `sd_nc`, `sd_pc`, `ssmd` and a logical `pass`. The sign of
#'   `ssmd` follows `mean_nc - mean_pc`.
#'
#' @examples
#' calc_ssmd(read_hts_plate(hts_primary_raw))
#'
#' @seealso [calc_z_prime()], [calc_plate_qc()]
#' @export
calc_ssmd <- function(hts_raw, signal_col = "signal", nc_label = "NC",
                      pc_label = "PC", ssmd_min = 2) {
  s <- .plate_control_stats(hts_raw, signal_col, nc_label, pc_label)
  res <- data.frame(
    plate_id = s$plate_id,
    mean_nc = s$mean_nc, mean_pc = s$mean_pc, sd_nc = s$sd_nc, sd_pc = s$sd_pc,
    ssmd = (s$mean_nc - s$mean_pc) / sqrt(s$sd_nc^2 + s$sd_pc^2),
    stringsAsFactors = FALSE
  )
  res$pass <- abs(res$ssmd) >= ssmd_min
  rownames(res) <- NULL
  res
}

#' Signal-to-background ratio per plate
#'
#' The ratio of the high-signal control mean to the low-signal control mean.
#' **What it is for:** S/B is the quickest sanity check that the assay actually
#' produced a usable dynamic range — that the untreated (high) and killed (low)
#' controls are far enough apart on the raw scale. It ignores variability, so it
#' complements but does not replace the Z'-factor; a common minimum is `>= 2`.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column. Default `"signal"`.
#' @param nc_label,pc_label Well-type labels for the negative and positive
#'   controls. Defaults `"NC"` and `"PC"`.
#' @param sb_min Acceptance threshold. Default `2`.
#'
#' @return A data frame with one row per plate: `plate_id`, `mean_nc`,
#'   `mean_pc`, `sb` (high-mean / low-mean) and a logical `pass`.
#'
#' @examples
#' calc_sb_ratio(read_hts_plate(hts_primary_raw))
#'
#' @seealso [calc_sn_ratio()], [calc_plate_qc()]
#' @export
calc_sb_ratio <- function(hts_raw, signal_col = "signal", nc_label = "NC",
                          pc_label = "PC", sb_min = 2) {
  s <- .plate_control_stats(hts_raw, signal_col, nc_label, pc_label)
  hi <- pmax(s$mean_nc, s$mean_pc)
  lo <- pmin(s$mean_nc, s$mean_pc)
  res <- data.frame(
    plate_id = s$plate_id, mean_nc = s$mean_nc, mean_pc = s$mean_pc,
    sb = hi / lo, stringsAsFactors = FALSE
  )
  res$pass <- res$sb >= sb_min
  rownames(res) <- NULL
  res
}

#' Signal-to-noise ratio per plate
#'
#' The assay window divided by the noise of the low-signal control. **What it is
#' for:** S/N asks whether the separation between the two controls is large
#' compared with the background scatter, i.e. whether a real change would rise
#' above the noise floor. It is more informative than S/B because it accounts
#' for variability; higher is better, with `>= 3` a reasonable floor.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column. Default `"signal"`.
#' @param nc_label,pc_label Well-type labels for the negative and positive
#'   controls. Defaults `"NC"` and `"PC"`.
#' @param sn_min Acceptance threshold. Default `3`.
#'
#' @return A data frame with one row per plate: `plate_id`, `mean_nc`,
#'   `mean_pc`, `sn` (window / low-control SD) and a logical `pass`.
#'
#' @examples
#' calc_sn_ratio(read_hts_plate(hts_primary_raw))
#'
#' @seealso [calc_sb_ratio()], [calc_plate_qc()]
#' @export
calc_sn_ratio <- function(hts_raw, signal_col = "signal", nc_label = "NC",
                          pc_label = "PC", sn_min = 3) {
  s <- .plate_control_stats(hts_raw, signal_col, nc_label, pc_label)
  hi_is_nc <- s$mean_nc >= s$mean_pc
  hi_mean <- ifelse(hi_is_nc, s$mean_nc, s$mean_pc)
  lo_mean <- ifelse(hi_is_nc, s$mean_pc, s$mean_nc)
  lo_sd <- ifelse(hi_is_nc, s$sd_pc, s$sd_nc)
  res <- data.frame(
    plate_id = s$plate_id, mean_nc = s$mean_nc, mean_pc = s$mean_pc,
    sn = ifelse(is.finite(lo_sd) & lo_sd > 0, (hi_mean - lo_mean) / lo_sd,
      NA_real_),
    stringsAsFactors = FALSE
  )
  res$pass <- res$sn >= sn_min
  rownames(res) <- NULL
  res
}

#' Control coefficient of variation per plate
#'
#' The percent CV (`100 * SD / mean`) of each control population. **What it is
#' for:** CV% is the direct measure of pipetting and reagent-handling
#' reproducibility within a plate. Low, stable control CVs are a precondition
#' for every other metric; a control drifting above ~15--20% flags a liquid-
#' handling or edge-effect problem even when the Z'-factor still passes.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column. Default `"signal"`.
#' @param nc_label,pc_label Well-type labels for the negative and positive
#'   controls. Defaults `"NC"` and `"PC"`.
#' @param cv_max Acceptance threshold in percent, applied to both controls.
#'   Default `20`.
#'
#' @return A data frame with one row per plate: `plate_id`, `cv_nc`, `cv_pc`
#'   (percent) and a logical `pass` (both controls at or below `cv_max`).
#'
#' @examples
#' calc_cv(read_hts_plate(hts_primary_raw))
#'
#' @seealso [calc_plate_qc()], [plot_plate_qc()]
#' @export
calc_cv <- function(hts_raw, signal_col = "signal", nc_label = "NC",
                    pc_label = "PC", cv_max = 20) {
  s <- .plate_control_stats(hts_raw, signal_col, nc_label, pc_label)
  res <- data.frame(
    plate_id = s$plate_id,
    cv_nc = 100 * s$sd_nc / s$mean_nc,
    cv_pc = 100 * s$sd_pc / s$mean_pc,
    stringsAsFactors = FALSE
  )
  res$pass <- res$cv_nc <= cv_max & res$cv_pc <= cv_max
  rownames(res) <- NULL
  res
}

#' Comprehensive per-plate quality assessment
#'
#' Computes the full panel of assay-quality metrics used to judge a
#' high-throughput screening plate, in a single per-plate table. Alongside the
#' classic Z'-factor this reports the median/MAD-based **robust Z'-factor**, the
#' sample-aware **Z-factor**, **SSMD**, the **signal-to-background (S/B)** and
#' **signal-to-noise (S/N)** ratios, and the **coefficient of variation (CV%)**
#' of each control population. All of these can then be visualised through the
#' single entry point [plot_plate_qc()].
#'
#' Each metric also has its own dedicated function, useful when you only need one
#' number (and each with its own acceptance threshold): [calc_z_prime()],
#' [calc_robust_z_prime()], [calc_z_factor()], [calc_ssmd()], [calc_sb_ratio()],
#' [calc_sn_ratio()] and [calc_cv()]. `calc_plate_qc()` uses the very same
#' formulas so the numbers agree.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column. Default `"signal"`.
#' @param nc_label,pc_label Well-type labels for the negative and positive
#'   controls. Defaults `"NC"` and `"PC"`.
#' @param sample_label Well-type label for the test (compound) wells, used by
#'   the sample-aware Z-factor. Default `"compound"`.
#' @param z_prime_min Acceptance threshold applied to the Z'-factor to set the
#'   `pass` flag. Default `0.5`.
#'
#' @return A data frame with one row per plate and the columns `plate_id`,
#'   `mean_nc`, `mean_pc`, `sd_nc`, `sd_pc`, `cv_nc`, `cv_pc`, `signal_window`,
#'   `sb` (signal-to-background), `sn` (signal-to-noise), `z_prime`,
#'   `robust_z_prime`, `z_factor`, `ssmd` and a logical `pass`.
#'
#' @details What each metric tells you, with the high-signal control taken as
#'   whichever of NC/PC has the larger mean and the low-signal control as the
#'   other:
#'   \itemize{
#'     \item **CV%** \eqn{= 100\,\sigma/\mu} of each control — pipetting/reagent
#'       reproducibility; lower is better.
#'     \item **S/B** \eqn{= \mu_{hi}/\mu_{lo}} — raw dynamic range of the assay.
#'     \item **S/N** \eqn{= (\mu_{hi}-\mu_{lo})/\sigma_{lo}} — assay window
#'       relative to background noise.
#'     \item **Z'-factor** \eqn{= 1 - 3(\sigma_{nc}+\sigma_{pc})/|\mu_{nc}-\mu_{pc}|}
#'       — overall control-only assay quality (`>= 0.5` excellent).
#'     \item **Robust Z'-factor**: as Z'-factor but with medians and MADs,
#'       resistant to outlier control wells.
#'     \item **Z-factor** \eqn{= 1 - 3(\sigma_{s}+\sigma_{nc})/|\mu_{s}-\mu_{nc}|}
#'       using the sample population \eqn{s} — is the window large versus the
#'       library spread (often low/negative in primary screens).
#'     \item **SSMD** \eqn{= (\mu_{nc}-\mu_{pc})/\sqrt{\sigma_{nc}^2+\sigma_{pc}^2}}
#'       — effect-size measure of control separation for hit cut-offs.
#'   }
#'
#' @examples
#' raw <- read_hts_plate(hts_primary_raw)
#' calc_plate_qc(raw)
#'
#' @seealso [plot_plate_qc()], [calc_z_prime()], [calc_well_zscore()]
#' @export
calc_plate_qc <- function(hts_raw, signal_col = "signal",
                          nc_label = "NC", pc_label = "PC",
                          sample_label = "compound", z_prime_min = 0.5) {
  s <- .plate_control_stats(hts_raw, signal_col, nc_label, pc_label,
    sample_label)
  window <- abs(s$mean_nc - s$mean_pc)
  hi_is_nc <- s$mean_nc >= s$mean_pc
  hi_mean <- ifelse(hi_is_nc, s$mean_nc, s$mean_pc)
  lo_mean <- ifelse(hi_is_nc, s$mean_pc, s$mean_nc)
  lo_sd <- ifelse(hi_is_nc, s$sd_pc, s$sd_nc)
  res <- data.frame(
    plate_id = s$plate_id,
    mean_nc = s$mean_nc, mean_pc = s$mean_pc,
    sd_nc = s$sd_nc, sd_pc = s$sd_pc,
    cv_nc = 100 * s$sd_nc / s$mean_nc,
    cv_pc = 100 * s$sd_pc / s$mean_pc,
    signal_window = window,
    sb = hi_mean / lo_mean,
    sn = ifelse(is.finite(lo_sd) & lo_sd > 0, (hi_mean - lo_mean) / lo_sd,
      NA_real_),
    z_prime = 1 - 3 * (s$sd_nc + s$sd_pc) / window,
    robust_z_prime = 1 - 3 * (s$mad_nc + s$mad_pc) /
      abs(s$median_nc - s$median_pc),
    z_factor = 1 - 3 * (s$sd_sample + s$sd_nc) / abs(s$mean_sample - s$mean_nc),
    ssmd = (s$mean_nc - s$mean_pc) / sqrt(s$sd_nc^2 + s$sd_pc^2),
    stringsAsFactors = FALSE
  )
  res$pass <- res$z_prime >= z_prime_min
  rownames(res) <- NULL
  res
}

#' Per-well standard and robust Z-scores
#'
#' Standardises every well's signal within its plate, adding both a classic
#' **Z-score** (mean/SD based) and a **robust Z-score** (median/MAD based, which
#' is the preferred choice in HTS because it is not distorted by the very hits
#' the screen is looking for). Scores are computed against a per-plate reference
#' population, by default the test-compound wells.
#'
#' @param hts_raw An `hts_raw` object.
#' @param signal_col Name of the signal column. Default `"signal"`.
#' @param reference One of `"compound"`, `"negative"` or `"all"`, selecting the
#'   per-plate population that defines the centre and spread. Default
#'   `"compound"`.
#' @param nc_label,sample_label Well-type labels for the negative controls and
#'   test wells, used to pick the reference population.
#'
#' @return The input object with two added columns: `zscore` (mean/SD based) and
#'   `robust_zscore` (median/MAD based, MAD scaled to the normal distribution).
#'
#' @examples
#' raw <- read_hts_plate(hts_primary_raw)
#' scored <- calc_well_zscore(raw)
#' head(scored[, c("plate_id", "well", "zscore", "robust_zscore")])
#'
#' @seealso [calc_plate_qc()], [plot_plate_qc()]
#' @export
calc_well_zscore <- function(hts_raw, signal_col = "signal",
                             reference = c("compound", "negative", "all"),
                             nc_label = "NC", sample_label = "compound") {
  reference <- match.arg(reference)
  if (!signal_col %in% names(hts_raw)) {
    stop("Column '", signal_col, "' not found in `hts_raw`.", call. = FALSE)
  }
  df <- as.data.frame(hts_raw, stringsAsFactors = FALSE)
  df$zscore <- NA_real_
  df$robust_zscore <- NA_real_
  for (p in unique(as.character(df$plate_id))) {
    idx <- which(as.character(df$plate_id) == p)
    v <- .as_numeric(df[[signal_col]][idx])
    ref <- switch(reference,
      compound = v[df$well_type[idx] == sample_label],
      negative = v[df$well_type[idx] == nc_label],
      all = v
    )
    mu <- mean(ref, na.rm = TRUE)
    sdv <- stats::sd(ref, na.rm = TRUE)
    med <- stats::median(ref, na.rm = TRUE)
    madv <- stats::mad(ref, na.rm = TRUE)
    df$zscore[idx] <- if (is.finite(sdv) && sdv > 0) (v - mu) / sdv else NA_real_
    df$robust_zscore[idx] <- if (is.finite(madv) && madv > 0) {
      (v - med) / madv
    } else {
      NA_real_
    }
  }
  .as_hts(df, "hts_raw")
}

#' Keep only plates that pass the Z' threshold
#'
#' Removes whole plates whose Z' factor falls below the acceptance threshold,
#' returning a clean data set for downstream normalisation.
#'
#' @param hts_raw An `hts_raw` object.
#' @param z_prime_min Acceptance threshold. Default `0.5`.
#' @param signal_col,nc_label,pc_label Passed to [calc_z_prime()].
#'
#' @return The `hts_raw` object restricted to passing plates, with the Z'
#'   summary table stored in `attr(x, "zprime")` and the dropped plate ids in
#'   `attr(x, "dropped_plates")`.
#'
#' @examples
#' raw <- read_hts_plate(hts_primary_raw)
#' clean <- filter_valid_plates(raw)
#' attr(clean, "dropped_plates")
#'
#' @seealso [calc_z_prime()]
#' @export
filter_valid_plates <- function(hts_raw, z_prime_min = 0.5,
                                signal_col = "signal",
                                nc_label = "NC", pc_label = "PC") {
  zp <- calc_z_prime(hts_raw, signal_col = signal_col, nc_label = nc_label,
    pc_label = pc_label, z_prime_min = z_prime_min)
  keep <- zp$plate_id[zp$pass]
  dropped <- zp$plate_id[!zp$pass]
  out <- hts_raw[as.character(hts_raw$plate_id) %in% keep, , drop = FALSE]
  out <- .as_hts(out, "hts_raw")
  attr(out, "zprime") <- zp
  attr(out, "dropped_plates") <- dropped
  out
}

#' Raw-signal plate heatmap
#'
#' Draws a single plate's raw (or background-subtracted) readouts in plate
#' layout, optionally marking flagged outlier wells. A thin wrapper around
#' [plot_plate_heatmap()] for `hts_raw` objects.
#'
#' @param hts_raw An `hts_raw` object.
#' @param plate Plate identifier to show. Defaults to the first plate.
#' @param signal_col Name of the signal column to colour by. Default
#'   `"signal"`.
#' @param mark_outliers Logical; if `TRUE` and an `outlier` column is present,
#'   flagged wells are labelled with `"X"`. Default `TRUE`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' raw <- detect_outlier_wells(read_hts_plate(hts_primary_raw))
#' plot_plate_heatmap_raw(raw, plate = "P01")
#'
#' @seealso [plot_plate_heatmap()], [plot_plate_qc()]
#' @export
plot_plate_heatmap_raw <- function(hts_raw, plate = NULL, signal_col = "signal",
                                   mark_outliers = TRUE) {
  if (is.null(plate)) plate <- as.character(hts_raw$plate_id)[1]
  df <- hts_raw
  lab <- NULL
  if (mark_outliers && "outlier" %in% names(df)) {
    df$.outlier_mark <- ifelse(df$outlier, "X", "")
    lab <- ".outlier_mark"
  }
  plot_plate_heatmap(df, fill = signal_col, well_col = "well",
    plate = plate, plate_col = "plate_id", label = lab,
    control_col = if ("well_type" %in% names(df)) "well_type" else NULL,
    title = paste0("Raw signal: ", plate))
}

# Label / acceptance-threshold / direction metadata for each plate QC metric.
.qc_metric_meta <- function(metric) {
  list(
    z_prime = list(label = "Z'-factor", threshold = 0.5, direction = "higher"),
    robust_z_prime = list(label = "Robust Z'-factor", threshold = 0.5,
      direction = "higher"),
    z_factor = list(label = "Z-factor", threshold = 0.5, direction = "higher"),
    ssmd = list(label = "SSMD", threshold = 2, direction = "abs"),
    sb = list(label = "Signal/Background (S/B)", threshold = 2,
      direction = "higher"),
    sn = list(label = "Signal/Noise (S/N)", threshold = 3, direction = "higher"),
    cv_nc = list(label = "CV% (NC)", threshold = 20, direction = "lower"),
    cv_pc = list(label = "CV% (PC)", threshold = 20, direction = "lower")
  )[[metric]]
}

#' Visualise any plate quality-control metric
#'
#' A single entry point for plotting the plate-quality metrics produced by
#' [calc_plate_qc()] (and the per-well scores from [calc_well_zscore()]). Choose
#' which assessment to draw through the `metric` argument; the acceptance
#' threshold, axis label and pass/fail colouring are set automatically for each
#' metric but can be overridden.
#'
#' @param qc A data frame from [calc_plate_qc()] for the plate-level metrics, or
#'   from [calc_well_zscore()] for the per-well `zscore`/`robust_zscore`
#'   distributions.
#' @param metric The metric to plot: one of `"z_prime"`, `"robust_z_prime"`,
#'   `"z_factor"`, `"ssmd"`, `"sb"`, `"sn"`, `"cv_nc"`, `"cv_pc"`, `"cv"` (both
#'   control CVs side by side), or `"zscore"`/`"robust_zscore"` for per-well
#'   distributions. Default `"z_prime"`.
#' @param threshold Optional acceptance threshold drawn as a reference line;
#'   defaults to the standard cut-off for the chosen metric.
#' @param plate_col Name of the plate-identifier column. Default `"plate_id"`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' raw <- read_hts_plate(hts_primary_raw)
#' qc <- calc_plate_qc(raw)
#' plot_plate_qc(qc, metric = "z_prime")
#' plot_plate_qc(qc, metric = "ssmd")
#' plot_plate_qc(qc, metric = "cv")
#' plot_plate_qc(calc_well_zscore(raw), metric = "robust_zscore")
#'
#' @seealso [calc_plate_qc()], [calc_well_zscore()], [plot_plate_heatmap_raw()]
#' @export
plot_plate_qc <- function(qc, metric = "z_prime", threshold = NULL,
                          plate_col = "plate_id") {
  df <- as.data.frame(qc, stringsAsFactors = FALSE)
  if (!plate_col %in% names(df)) {
    stop("Column '", plate_col, "' not found in `qc`.", call. = FALSE)
  }
  df$.plate <- as.character(df[[plate_col]])

  # Per-well distribution metrics.
  if (metric %in% c("zscore", "robust_zscore")) {
    if (!metric %in% names(df)) {
      stop("Column '", metric, "' not found; run calc_well_zscore() first.",
        call. = FALSE)
    }
    lab <- if (metric == "zscore") "Z-score" else "Robust Z-score"
    df$.value <- .as_numeric(df[[metric]])
    p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$.plate,
      y = .data$.value)) +
      ggplot2::geom_hline(yintercept = 0, color = "#888888", linewidth = 0.4) +
      ggplot2::geom_boxplot(width = 0.55, fill = "#EAECEF", color = "#4C4C4C",
        linewidth = 0.5, outlier.color = "#4C4C4C", outlier.size = 0.9,
        outlier.alpha = 0.7) +
      ggplot2::labs(x = NULL, y = lab, title = paste0("Per-well ", lab)) +
      theme_hts(legend = "none")
    if (!is.null(threshold)) {
      p <- p + ggplot2::geom_hline(yintercept = c(-1, 1) * abs(threshold),
        linetype = "dashed", color = .hts_col("reference"), linewidth = 0.6)
    }
    return(p)
  }

  # Both control CVs side by side.
  if (metric == "cv") {
    if (!all(c("cv_nc", "cv_pc") %in% names(df))) {
      stop("`qc` must contain 'cv_nc' and 'cv_pc'; run calc_plate_qc() first.",
        call. = FALSE)
    }
    thr <- if (!is.null(threshold)) threshold else 20
    long <- data.frame(
      plate = rep(df$.plate, 2L),
      control = factor(rep(c("NC", "PC"), each = nrow(df)),
        levels = c("NC", "PC")),
      value = c(.as_numeric(df$cv_nc), .as_numeric(df$cv_pc)),
      stringsAsFactors = FALSE
    )
    long$label <- .fmt_qc(long$value)
    return(
      ggplot2::ggplot(long, ggplot2::aes(x = .data$plate, y = .data$value,
        fill = .data$control)) +
        ggplot2::geom_col(position = ggplot2::position_dodge(0.72),
          width = 0.68, color = "white", linewidth = 0.3) +
        ggplot2::geom_text(ggplot2::aes(label = .data$label),
          position = ggplot2::position_dodge(0.72), vjust = -0.4, size = 3,
          color = "#333333") +
        ggplot2::geom_hline(yintercept = thr, linetype = "dashed",
          color = .hts_col("reference"), linewidth = 0.6) +
        ggplot2::annotate("text", x = Inf, y = thr,
          label = paste0("\u2264 ", thr, "%"), hjust = 1.05, vjust = -0.4,
          size = 3, color = .hts_col("reference")) +
        ggplot2::scale_fill_manual(
          values = c(NC = .hts_col("NC"), PC = .hts_col("PC"))
        ) +
        ggplot2::scale_y_continuous(
          expand = ggplot2::expansion(mult = c(0.02, 0.12))
        ) +
        ggplot2::labs(x = NULL, y = "CV (%)", fill = "Control",
          title = "Plate QC: control CV%") +
        theme_hts(legend = "bottom")
    )
  }

  meta <- .qc_metric_meta(metric)
  if (is.null(meta)) {
    stop("Unknown metric '", metric, "'. See ?plot_plate_qc for valid values.",
      call. = FALSE)
  }
  if (!metric %in% names(df)) {
    stop("Column '", metric, "' not found; run calc_plate_qc() first.",
      call. = FALSE)
  }
  thr <- if (!is.null(threshold)) threshold else meta$threshold
  df$.value <- .as_numeric(df[[metric]])
  df$.pass <- factor(
    switch(meta$direction,
      higher = df$.value >= thr,
      lower = df$.value <= thr,
      abs = abs(df$.value) >= thr
    ),
    levels = c("TRUE", "FALSE")
  )
  df$.label <- .fmt_qc(df$.value)
  df$.vjust <- ifelse(df$.value >= 0, -0.45, 1.3)
  cut_label <- switch(meta$direction,
    higher = paste0("\u2265 ", thr),
    lower = paste0("\u2264 ", thr),
    abs = paste0("|", metric, "| \u2265 ", thr)
  )
  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$.plate, y = .data$.value,
    fill = .data$.pass)) +
    ggplot2::geom_col(width = 0.62, color = "white", linewidth = 0.3) +
    ggplot2::geom_text(ggplot2::aes(label = .data$.label,
      vjust = .data$.vjust), size = 3.2, color = "#333333") +
    ggplot2::scale_fill_manual(
      values = c(`TRUE` = .hts_col("pass"), `FALSE` = .hts_col("fail")),
      drop = FALSE, name = "QC pass", labels = c(`TRUE` = "Pass",
        `FALSE` = "Fail")
    ) +
    ggplot2::scale_y_continuous(
      expand = ggplot2::expansion(mult = c(0.06, 0.14))
    ) +
    ggplot2::labs(x = NULL, y = meta$label,
      title = paste0("Plate QC: ", meta$label)) +
    theme_hts(legend = "bottom")
  if (meta$direction == "abs") {
    p <- p +
      ggplot2::geom_hline(yintercept = c(-thr, thr), linetype = "dashed",
        color = .hts_col("reference"), linewidth = 0.6) +
      ggplot2::annotate("text", x = Inf, y = thr, label = cut_label,
        hjust = 1.05, vjust = -0.4, size = 3, color = .hts_col("reference"))
  } else {
    p <- p +
      ggplot2::geom_hline(yintercept = thr, linetype = "dashed",
        color = .hts_col("reference"), linewidth = 0.6) +
      ggplot2::annotate("text", x = Inf, y = thr, label = cut_label,
        hjust = 1.05, vjust = -0.4, size = 3, color = .hts_col("reference"))
  }
  p
}

test_that("utils_hts helpers behave", {
  expect_equal(nrow(plate_layout(96)), 96L)
  expect_equal(nrow(plate_layout_384()), 384L)
  expect_equal(nrow(plate_layout_1536()), 1536L)
  expect_true(all(c("row", "col", "well") %in% names(plate_layout(96))))
  expect_equal(convert_conc_log10(c(1, 10, 100)), c(0, 1, 2))
  expect_equal(clean_compound_id(c("cpd 1!", "a-b")), c("CPD_1", "A-B"))
  chk <- suppressWarnings(check_control_label(
    data.frame(well_type = c("NC", "PC", "compound")),
    required = c("NC", "PC", "Blank")
  ))
  expect_false(chk$ok)
})

test_that("module 1 pre-QC runs end to end", {
  raw <- read_hts_plate(hts_primary_raw)
  expect_s3_class(raw, "hts_raw")
  raw <- baseline_subtract(raw)
  expect_true("signal_raw" %in% names(raw))
  raw <- detect_outlier_wells(raw)
  zp <- calc_z_prime(raw)
  expect_true(all(c("plate_id", "z_prime", "pass") %in% names(zp)))
  # P01/P02 good, P03 fails
  expect_equal(sum(zp$pass), 2L)
  valid <- filter_valid_plates(raw)
  expect_false("P03" %in% valid$plate_id)
})

test_that("calc_plate_qc reports the full QC panel per plate", {
  raw <- read_hts_plate(hts_primary_raw)
  qc <- calc_plate_qc(raw)
  expect_equal(nrow(qc), 3L)
  expect_true(all(c("cv_nc", "cv_pc", "sb", "sn", "z_prime", "robust_z_prime",
    "z_factor", "ssmd", "pass") %in% names(qc)))
  # Good plates separate the controls: high SSMD, low control CV, Z' >= 0.5.
  good <- qc[qc$plate_id %in% c("P01", "P02"), ]
  bad <- qc[qc$plate_id == "P03", ]
  expect_true(all(good$z_prime >= 0.5))
  expect_true(all(good$ssmd > bad$ssmd))
  expect_true(all(good$cv_nc < bad$cv_nc))
  expect_true(all(good$sb > 1))
  expect_equal(sum(qc$pass), 2L)
  # Z' and its robust variant agree on the pass/fail verdict here.
  expect_equal(qc$z_prime >= 0.5, qc$robust_z_prime >= 0.5)
})

test_that("standalone QC metric functions match calc_plate_qc", {
  raw <- read_hts_plate(hts_primary_raw)
  qc <- calc_plate_qc(raw)
  expect_equal(calc_robust_z_prime(raw)$robust_z_prime, qc$robust_z_prime)
  expect_equal(calc_z_factor(raw)$z_factor, qc$z_factor)
  expect_equal(calc_ssmd(raw)$ssmd, qc$ssmd)
  expect_equal(calc_sb_ratio(raw)$sb, qc$sb)
  expect_equal(calc_sn_ratio(raw)$sn, qc$sn)
  expect_equal(calc_cv(raw)$cv_nc, qc$cv_nc)
  expect_equal(calc_cv(raw)$cv_pc, qc$cv_pc)
  # Each returns one row per plate with its own pass flag.
  for (fn in list(calc_robust_z_prime, calc_ssmd, calc_sb_ratio, calc_sn_ratio,
    calc_cv)) {
    res <- fn(raw)
    expect_equal(nrow(res), 3L)
    expect_true("pass" %in% names(res))
  }
  # SSMD passes on magnitude, so the flag is direction-independent.
  expect_true(all(calc_ssmd(raw)$pass == (abs(qc$ssmd) >= 2)))
})

test_that("calc_well_zscore adds standard and robust per-well scores", {
  raw <- read_hts_plate(hts_primary_raw)
  sc <- calc_well_zscore(raw)
  expect_true(all(c("zscore", "robust_zscore") %in% names(sc)))
  expect_equal(nrow(sc), nrow(raw))
  # Reference population is roughly centred near zero within a plate.
  cpd <- sc[sc$plate_id == "P01" & sc$well_type == "compound", ]
  expect_lt(abs(stats::median(cpd$robust_zscore, na.rm = TRUE)), 1)
})

test_that("plot_plate_qc dispatches on metric", {
  raw <- read_hts_plate(hts_primary_raw)
  qc <- calc_plate_qc(raw)
  for (m in c("z_prime", "robust_z_prime", "z_factor", "ssmd", "sb", "sn",
    "cv_nc", "cv")) {
    expect_s3_class(plot_plate_qc(qc, metric = m), "ggplot")
  }
  sc <- calc_well_zscore(raw)
  expect_s3_class(plot_plate_qc(sc, metric = "robust_zscore"), "ggplot")
  expect_error(plot_plate_qc(qc, metric = "nonsense"), "Unknown metric")
  expect_error(plot_plate_qc(qc, metric = "zscore"), "calc_well_zscore")
})

test_that("summarize_plate_setup describes each plate's layout", {
  raw <- read_hts_plate(hts_primary_raw)
  s <- summarize_plate_setup(raw)
  # One row per plate; the run holds three 96-well plates.
  expect_equal(nrow(s), 3L)
  expect_true(all(s$plate_format == 96))
  expect_true(all(s$n_wells == 96))
  expect_true(all(s$n_unique_compound == 48))
  # Positions are read back from the data as authored: an empty "Blank"
  # evaporation-buffer ring, diagonal controls in the two inset columns (2 and
  # 11) and the 48 compounds in the interior columns 3-10.
  expect_true(all(s$nc_columns == "2, 11"))
  expect_true(all(s$pc_columns == "2, 11"))
  expect_true(all(s$compound_columns == "3-10"))
  # The realistic 384-well plate infers its format and diagonal inset controls.
  s384 <- summarize_plate_setup(hts_plate_384)
  expect_equal(s384$plate_format, 384)
  expect_equal(s384$pc_columns, "3, 22")
  expect_equal(s384$nc_columns, "3, 22")
})

test_that("read_plate_layout parses a user-defined grid plate map", {
  f <- system.file("extdata", "plate_layout_example.csv", package = "diffHTS")
  if (!nzchar(f)) {
    f <- testthat::test_path("..", "..", "inst", "extdata",
      "plate_layout_example.csv")
  }
  skip_if_not(file.exists(f))
  layout <- read_plate_layout(f, plate_id = "P01")
  expect_s3_class(layout, "hts_layout")
  expect_equal(nrow(layout), 96L)
  # User-defined roles: inset controls (not on the plate edge).
  expect_equal(sum(layout$well_type == "NC"), 6L)
  expect_equal(sum(layout$well_type == "PC"), 6L)
  expect_equal(sum(layout$well_type == "compound"), 48L)
  expect_equal(sum(layout$well_type == "Blank"), 36L)
  # Diagonal controls: NC and PC each sit in interior columns 2 and 11.
  expect_true(all(layout$col[layout$well_type == "NC"] %in% c(2L, 11L)))
  expect_true(all(layout$col[layout$well_type == "PC"] %in% c(2L, 11L)))
  expect_true(all(is.na(layout$compound_id[layout$well_type == "Blank"])))
})

test_that("apply_plate_layout stamps roles onto readings", {
  f <- system.file("extdata", "plate_layout_example.csv", package = "diffHTS")
  if (!nzchar(f)) {
    f <- testthat::test_path("..", "..", "inst", "extdata",
      "plate_layout_example.csv")
  }
  skip_if_not(file.exists(f))
  layout <- read_plate_layout(f, plate_id = "P01")
  readings <- data.frame(well = layout$well,
    signal = seq_len(nrow(layout)) * 10)
  raw <- apply_plate_layout(readings, layout, plate_id = "P01")
  expect_s3_class(raw, "hts_raw")
  expect_equal(nrow(raw), nrow(layout))
  expect_equal(table(raw$well_type), table(layout$well_type))
  expect_true(all(raw$plate_id == "P01"))
  # Missing wells in the layout raise a clear error.
  expect_error(
    apply_plate_layout(data.frame(well = "Z99", signal = 1), layout),
    "no matching well"
  )
})

test_that("module 1 plots return objects", {
  raw <- baseline_subtract(read_hts_plate(hts_primary_raw))
  expect_s3_class(plot_plate_heatmap_raw(raw, plate = "P01"), "ggplot")
  expect_s3_class(plot_plate_qc(calc_plate_qc(raw), metric = "z_prime"), "ggplot")
})

test_that("module 2 normalisation anchors controls", {
  raw <- baseline_subtract(read_hts_plate(hts_primary_raw))
  norm <- norm_by_control(raw)
  expect_s3_class(norm, "hts_norm")
  # NC ~ 0% inhibition, PC ~ 100% inhibition on a good plate
  nc <- norm[norm$plate_id == "P01" & norm$well_type == "NC", ]
  pc <- norm[norm$plate_id == "P01" & norm$well_type == "PC", ]
  expect_lt(abs(mean(nc$inhibition)), 5)
  expect_lt(abs(mean(pc$inhibition) - 100), 5)
  merged <- merge_plate_data(norm, norm)
  expect_equal(nrow(merged), 2 * nrow(norm))
})

test_that("module 2 plots return objects", {
  norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
  expect_s3_class(plot_plate_heatmap_inhibition(norm, plate = "P01"), "ggplot")
  expect_s3_class(plot_inhibition_hist(norm, plate = "P01"), "ggplot")
})

test_that("module 3 replicate consistency", {
  norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
  cv <- calc_replicate_cv(norm)
  expect_true(all(c("compound_id", "cv") %in% names(cv)))
  cor <- calc_replicate_correlation(norm)
  expect_true(cor$r_squared[1] > 0.8)
  good <- filter_bad_replicate(cv, cv_max = 50)
  expect_true(nrow(good) <= nrow(cv))
  expect_s3_class(plot_replicate_scatter(norm, "P01", "P02"), "ggplot")
  # No pair requested -> all pairwise combinations faceted into one plot,
  # with points coloured by well type (controls included).
  p_all <- plot_replicate_scatter(norm)
  expect_s3_class(p_all, "ggplot")
  expect_equal(length(unique(p_all$data$pair)), 3L)
  expect_true(all(c("compound", "NC", "PC") %in% p_all$data$type))
  expect_s3_class(plot_cv_distribution(cv), "ggplot")
})

test_that("module 4 primary hit selection", {
  norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
  ph <- select_primary_hit(norm, threshold = 50)
  expect_true(all(c("compound_id", "activity", "is_hit") %in% names(ph)))
  s <- summarize_primary_hit(ph)
  expect_equal(s$n_hits, sum(ph$is_hit))
  pct <- select_primary_hit(norm, method = "percentile", percentile = 0.9)
  expect_true(sum(pct$is_hit) >= 1)
  expect_s3_class(plot_primary_inhibition_rank(ph), "ggplot")
  expect_s3_class(plot_hit_bar_count(ph), "ggplot")
})

test_that("module 4 sigma-based hit selection", {
  norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
  sh <- select_sigma_hits(norm, n_sigma = 3)
  expect_true(all(c("compound_id", "activity", "sigma", "sigma_level", "band",
    "is_hit") %in% names(sh)))
  expect_true(all(sh$sigma_level %in% 0:3))
  # is_hit is exactly the compounds clearing 3 sigma.
  expect_equal(sum(sh$is_hit), sum(sh$sigma_level >= 3, na.rm = TRUE))
  # Robust (median/MAD) estimation recovers a non-empty hit tail that the
  # SD-based estimate (inflated by the hits themselves) misses entirely.
  expect_gt(sum(sh$is_hit), 0)
  sd_based <- select_sigma_hits(norm, method = "sd", n_sigma = 3)
  expect_gte(sum(sh$is_hit), sum(sd_based$is_hit))
  ss <- summarize_sigma_hits(sh)
  expect_equal(ss$sigma, 1:3)
  expect_true(all(ss$n_beyond >= c(ss$n_beyond[-1], 0)))  # monotone non-increasing
  expect_s3_class(plot_sigma_hits(sh), "ggplot")
})

test_that("module 5 dose-response fitting recovers S-curves", {
  dr <- import_dose_response(hts_dose_response, response_col = "viability",
    group_cols = "cell_line")
  expect_s3_class(dr, "hts_dose")
  drc <- fit_4pl_curve(dr, group_cols = "cell_line")
  expect_s3_class(drc, "hts_drc")
  expect_equal(nrow(drc$meta), 30L)
  # Good simulated curves should fit well.
  expect_gt(median(drc$meta$rsq, na.rm = TRUE), 0.9)
  params <- extract_drc_params(drc)
  expect_true(all(c("ic50", "emax", "bottom", "hill") %in% names(params)))
  drc <- calc_drc_auc(drc)
  expect_true("auc" %in% names(drc$meta))
  expect_true(all(is.finite(drc$meta$auc)))
  clean <- filter_low_quality_curve(drc, rsq_min = 0.85)
  expect_true(nrow(clean$meta) <= nrow(drc$meta))
})

test_that("module 5 plots return objects", {
  drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
    response_col = "viability", group_cols = "cell_line"),
    group_cols = "cell_line"))
  expect_s3_class(plot_single_drc(drc, drc$meta$curve_id[1]), "ggplot")
  expect_s3_class(plot_batch_drc_overlay(drc, drc$meta$curve_id[1:3]), "ggplot")
  pp <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
  expect_s3_class(plot_ic50_auc_cor(pp), "ggplot")
})

test_that("module 6 AUC matrix and clustering", {
  drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
    response_col = "viability", group_cols = "cell_line"),
    group_cols = "cell_line"))
  m <- build_auc_matrix(drc$meta)
  expect_equal(dim(m), c(10L, 3L))
  mz <- build_auc_matrix(drc$meta, zscore = "row")
  expect_equal(attr(mz, "zscore"), "row")
  cl <- cluster_auc_matrix(m, k = 2)
  expect_equal(cl$k, 2L)
  expect_equal(length(cl$clusters), nrow(m))
  hits <- extract_cluster_hit(m, cl)
  expect_true(length(hits) >= 1)
})

test_that("module 6 plots draw without error", {
  skip_if_not_installed("ComplexHeatmap")
  drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
    response_col = "viability", group_cols = "cell_line"),
    group_cols = "cell_line"))
  m <- build_auc_matrix(drc$meta)
  pdf(tempfile(fileext = ".pdf"))
  on.exit(dev.off())
  expect_invisible(plot_auc_heatmap(m))
  expect_invisible(plot_cluster_tree(cluster_auc_matrix(m, k = 2)))
})

test_that("module 7 ranking, annotation and export", {
  drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
    response_col = "viability", group_cols = "cell_line"),
    group_cols = "cell_line"))
  pp <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
  ranked <- rank_hit_compound(pp)
  expect_true(all(c("score", "tier") %in% names(ranked)))
  expect_true(is.ordered(ranked$tier))
  ann <- annotate_hit_info(ranked, hts_compound_meta)
  expect_true("compound_name" %in% names(ann))
  expect_equal(nrow(ann), nrow(ranked))
  f <- export_hit_table(ann, tempfile(fileext = ".csv"))
  expect_true(file.exists(f))
  expect_s3_class(plot_hit_stratify_bar(ranked), "ggplot")
})

test_that("module 7 primary-secondary correlation plot", {
  norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
  ph <- select_primary_hit(norm)
  drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
    response_col = "viability", group_cols = "cell_line"),
    group_cols = "cell_line"))
  sec <- aggregate(auc ~ compound_id, drc$meta, mean)
  expect_s3_class(plot_primary_secondary_cor(ph, sec), "ggplot")
})

test_that("theme_hts and hts_pal are exported and well-formed", {
  expect_s3_class(theme_hts(), "theme")
  expect_s3_class(theme_hts(grid = "both", legend = "right"), "theme")
  pal <- hts_pal()
  expect_type(pal, "character")
  expect_true(length(pal) >= 8)
  expect_true(all(grepl("^#[0-9A-Fa-f]{6}$", pal)))
  expect_length(hts_pal(3), 3)
  library(ggplot2)
  g <- ggplot(mtcars, aes(factor(cyl), mpg)) + geom_boxplot() + theme_hts()
  expect_s3_class(g, "ggplot")
})

#' Heatmap of differential AUC across conditions or cell lines
#'
#' Draws a diverging heatmap of differential (delta) AUC values with drugs on
#' one axis and conditions/cell lines on the other, using \pkg{ComplexHeatmap}.
#' Colours are centred at zero so that sensitising (negative) and antagonising
#' (positive) responses are visually separated.
#'
#' @param data A data frame with one row per drug: a drug identifier column and
#'   one numeric delta-AUC column per condition or cell line.
#' @param drug_col Name of the drug identifier column, as a string. Default
#'   `"drug_name"`.
#' @param value_cols Character vector of the delta-AUC column names to display.
#'   Defaults to every numeric column other than `drug_col`.
#' @param cell_types Optional named character vector mapping each column in
#'   `value_cols` to a group label (for example `"Cancer"` or `"Normal"`) used
#'   to annotate the heatmap.
#' @param cluster_rows,cluster_columns Logical, whether to cluster drugs and
#'   conditions respectively. Defaults `TRUE`.
#' @param name Legend title. Default `"delta AUC"`.
#'
#' @return A [ComplexHeatmap::Heatmap-class] object, which is drawn when
#'   printed. Rows with missing values are dropped before plotting.
#'
#' @details Requires the suggested packages \pkg{ComplexHeatmap} and
#'   \pkg{circlize}.
#'
#' @examples
#' if (requireNamespace("ComplexHeatmap", quietly = TRUE) &&
#'     requireNamespace("circlize", quietly = TRUE)) {
#'   ht <- plot_delta_auc_heatmap(
#'     screen_delta_auc,
#'     value_cols = c("A549", "H1299", "H1975", "MRC5")
#'   )
#' }
#'
#' @seealso [plot_condition_scatter()]
#' @export
plot_delta_auc_heatmap <- function(data, drug_col = "drug_name",
                                    value_cols = NULL, cell_types = NULL,
                                    cluster_rows = TRUE, cluster_columns = TRUE,
                                    name = "delta AUC") {
  if (!requireNamespace("ComplexHeatmap", quietly = TRUE) ||
    !requireNamespace("circlize", quietly = TRUE)) {
    stop(
      "Packages 'ComplexHeatmap' and 'circlize' are required for ",
      "`plot_delta_auc_heatmap()`.",
      call. = FALSE
    )
  }
  if (is.null(value_cols)) {
    value_cols <- setdiff(names(data)[vapply(data, is.numeric, logical(1))], drug_col)
  }

  mat <- as.matrix(data[, value_cols, drop = FALSE])
  rownames(mat) <- data[[drug_col]]
  mat <- mat[stats::complete.cases(mat), , drop = FALSE]

  col_fun <- circlize::colorRamp2(
    c(min(mat, na.rm = TRUE), 0, max(mat, na.rm = TRUE)),
    c("navy", "white", "firebrick3")
  )

  top_annotation <- NULL
  if (!is.null(cell_types)) {
    types <- cell_types[colnames(mat)]
    top_annotation <- ComplexHeatmap::HeatmapAnnotation(
      Type = types, which = "column",
      col = list(Type = c("Normal" = "#0072B2", "Cancer" = "#D55E00"))
    )
  }

  ComplexHeatmap::Heatmap(
    mat,
    name = name,
    col = col_fun,
    cluster_rows = cluster_rows,
    cluster_columns = cluster_columns,
    column_names_rot = 45,
    top_annotation = top_annotation,
    border = TRUE
  )
}

#' Scatter plot comparing two conditions or cell lines
#'
#' Plots a per-drug scatter of a score (for example delta AUC) under two
#' conditions or cell lines, colouring points by quadrant and labelling the most
#' extreme drugs in each quadrant. This is useful for spotting drugs that
#' behave differently between, say, a cancer and a normal cell line.
#'
#' @param data A data frame with one row per drug.
#' @param x,y Names of the two numeric score columns to place on the x and y
#'   axes, as strings.
#' @param drug_col Name of the drug identifier column, as a string. Default
#'   `"drug_name"`.
#' @param label_n Number of most-extreme drugs (by distance from the origin) to
#'   label per quadrant. Default `5`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @details Point labelling uses \pkg{ggrepel} when available and falls back to
#'   [ggplot2::geom_text()] otherwise. The axes are centred on the origin with a
#'   fixed aspect ratio so the four quadrants are equal in size.
#'
#' @examples
#' df <- data.frame(
#'   drug_name = paste0("Drug", 1:8),
#'   MRC5 = c(-1, 0.5, -0.2, 1.2, -0.8, 0.3, -1.5, 0.9),
#'   A549 = c(-1.2, 0.1, 0.7, -0.9, -1.1, 1.0, -0.4, 0.6)
#' )
#' plot_condition_scatter(df, x = "MRC5", y = "A549")
#'
#' @seealso [plot_delta_auc_heatmap()]
#' @export
plot_condition_scatter <- function(data, x, y, drug_col = "drug_name",
                                    label_n = 5) {
  for (col in c(drug_col, x, y)) {
    if (!col %in% names(data)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }

  df <- data.frame(
    drug_name = data[[drug_col]],
    x = data[[x]],
    y = data[[y]]
  )
  df <- df[stats::complete.cases(df), , drop = FALSE]

  df$quadrant <- ifelse(df$x >= 0 & df$y >= 0, "Q1",
    ifelse(df$x < 0 & df$y >= 0, "Q2",
      ifelse(df$x < 0 & df$y < 0, "Q3", "Q4")
    )
  )
  df$distance <- sqrt(df$x^2 + df$y^2)

  labels <- do.call(rbind, lapply(split(df, df$quadrant), function(d) {
    d[order(d$distance, decreasing = TRUE), , drop = FALSE][
      seq_len(min(label_n, nrow(d))), ,
      drop = FALSE
    ]
  }))

  quadrant_colors <- c(
    Q1 = "#1b9e77", Q2 = "#d95f02",
    Q3 = "#7570b3", Q4 = "#e7298a"
  )

  p <- ggplot2::ggplot(
    df,
    ggplot2::aes(x = .data$x, y = .data$y, color = .data$quadrant)
  ) +
    ggplot2::geom_hline(yintercept = 0, linewidth = 0.6, color = "#888888") +
    ggplot2::geom_vline(xintercept = 0, linewidth = 0.6, color = "#888888") +
    ggplot2::geom_point(size = 2.6, alpha = 0.8, stroke = 0.2) +
    ggplot2::scale_color_manual(values = quadrant_colors) +
    ggplot2::labs(x = x, y = y, color = "Quadrant") +
    theme_hts(grid = "both", legend = "right")

  ## Symmetric, equal-sized quadrants: centre the axes on the origin and use a
  ## fixed aspect so all four quadrants cover the same area.
  lim <- max(abs(c(df$x, df$y)), na.rm = TRUE)
  if (!is.finite(lim) || lim == 0) lim <- 1
  lim <- lim * 1.08
  p <- p + ggplot2::coord_fixed(
    xlim = c(-lim, lim), ylim = c(-lim, lim), clip = "off"
  )

  if (requireNamespace("ggrepel", quietly = TRUE)) {
    p <- p + ggrepel::geom_text_repel(
      data = labels,
      ggplot2::aes(label = .data$drug_name),
      size = 3, max.overlaps = 20
    )
  } else {
    p <- p + ggplot2::geom_text(
      data = labels,
      ggplot2::aes(label = .data$drug_name),
      size = 3, vjust = -0.6, show.legend = FALSE
    )
  }
  p
}

#' Quality-control boxplot of control wells across plates
#'
#' Draws grouped boxplots of a per-well readout across plates, split by control
#' type, to inspect assay consistency and separation between negative and
#' positive controls.
#'
#' @param data A long-format data frame of well-level readouts.
#' @param plate_col Name of the plate identifier column, as a string. Default
#'   `"plateID"`.
#' @param value_col Name of the numeric readout column, as a string. Default
#'   `"normalized_cell_count"`.
#' @param group_col Name of the control-type / grouping column, as a string.
#'   Default `"experiment_type"`.
#' @param y_title Axis title for the readout. Default
#'   `"Normalized cell count"`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' plot_qc_boxplot(screen_doseresponse)
#'
#' @seealso [calculate_qc_metrics()]
#' @export
plot_qc_boxplot <- function(data, plate_col = "plateID",
                            value_col = "normalized_cell_count",
                            group_col = "experiment_type",
                            y_title = "Normalized cell count") {
  for (col in c(plate_col, value_col, group_col)) {
    if (!col %in% names(data)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }

  ggplot2::ggplot(
    data,
    ggplot2::aes(
      x = .data[[plate_col]],
      y = .data[[value_col]],
      fill = .data[[group_col]]
    )
  ) +
    ggplot2::geom_boxplot(outlier.shape = NA) +
    ggplot2::geom_point(
      position = ggplot2::position_jitterdodge(), alpha = 0.3, size = 0.6
    ) +
    ggplot2::scale_fill_manual(values = hts_pal()) +
    ggplot2::expand_limits(y = 0) +
    ggplot2::labs(x = NULL, y = y_title, fill = "Control") +
    theme_hts(legend = "bottom") +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
    )
}

#' Plot dose-response curves for two conditions
#'
#' Fits a four-parameter logistic model to each condition of a single drug and
#' overlays the observed responses with the smooth fitted curves, coloured by
#' condition. This is the key visual for comparing drug sensitivity between two
#' conditions (for example irradiated versus non-irradiated cells), making a
#' potency shift between conditions immediately apparent.
#'
#' The styling reproduces the plots used throughout the screening pipeline: a
#' logarithmic concentration axis, distinct colour and point shape per
#' condition, a horizontal 50 % reference line, and the fitted IC50/AUC of each
#' condition reported in the subtitle.
#'
#' @param data A long-format data frame with one row per well/replicate,
#'   containing drug, condition, concentration and response columns.
#' @param drug Optional drug name to display. If `NULL` (default) `data` is
#'   assumed to already contain a single drug.
#' @param response Name of the (normalised) response column, as a string.
#'   Default `"normalized_cell_count"`.
#' @param concentration Name of the concentration column, as a string. Default
#'   `"concentration"`.
#' @param condition Name of the condition column, as a string. Default
#'   `"condition"`.
#' @param drug_col Name of the drug identifier column, as a string. Default
#'   `"drug_name"`.
#' @param n_points Number of points used to draw each smooth fitted curve.
#'   Default `200`.
#' @param colors Character vector of colours, one per condition (recycled from
#'   the pipeline palette by default).
#' @param shapes Numeric vector of point shapes, one per condition.
#' @param ref_line Numeric response level at which to draw a horizontal dashed
#'   reference line (for example the 50 % effect level). Set to `NA` to omit.
#' @param legend_title Legend title. Default `"Condition"`.
#'
#' @return A [ggplot2::ggplot] object with a base-10 logarithmic concentration
#'   axis, observed points and fitted 4PL curves coloured by condition.
#'
#' @details Only concentrations that are finite and strictly positive are used
#'   (a log scale cannot show zero or negative doses). A condition with fewer
#'   than four usable observations is plotted as points only, with a warning,
#'   and no curve is fitted for it. Fitting is performed with
#'   [fit_dose_response()], so no external fitting package is required. The
#'   figure uses the package's modern-academic [theme_hts()] look with a
#'   colourblind-safe condition palette.
#'
#' @examples
#' one <- subset(
#'   screen_doseresponse,
#'   drug_name == "DrugA" & experiment_type == "sample"
#' )
#' plot_dose_response_curves(one, drug = "DrugA")
#'
#' @seealso [fit_dose_response()], [four_pl()], [compute_auc()]
#' @export
plot_dose_response_curves <- function(data, drug = NULL,
                                      response = "normalized_cell_count",
                                      concentration = "concentration",
                                      condition = "condition",
                                      drug_col = "drug_name",
                                      n_points = 200,
                                      colors = c("#0072B2", "#D55E00",
                                                 "#009E73", "#CC79A7"),
                                      shapes = c(19, 15, 17, 18),
                                      ref_line = 0.5,
                                      legend_title = "Condition") {
  for (col in c(response, concentration, condition)) {
    if (!col %in% names(data)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }

  if (!is.null(drug)) {
    if (!drug_col %in% names(data)) {
      stop("Column '", drug_col, "' not found in `data`.", call. = FALSE)
    }
    data <- data[data[[drug_col]] == drug, , drop = FALSE]
    if (nrow(data) == 0) {
      stop("No rows found for drug '", drug, "'.", call. = FALSE)
    }
  }

  obs <- data.frame(
    concentration = .as_numeric(data[[concentration]]),
    response = .as_numeric(data[[response]]),
    condition = as.character(data[[condition]]),
    stringsAsFactors = FALSE
  )
  obs <- obs[is.finite(obs$concentration) & obs$concentration > 0 &
    is.finite(obs$response), , drop = FALSE]
  if (nrow(obs) == 0) {
    stop("No finite, positive-concentration observations to plot.",
      call. = FALSE
    )
  }

  conditions <- sort(unique(obs$condition))
  obs$condition <- factor(obs$condition, levels = conditions)

  curves <- list()
  summaries <- character(0)
  for (cond in conditions) {
    sub <- obs[obs$condition == cond, , drop = FALSE]
    if (nrow(sub) < 4) {
      warning("Condition '", cond, "' has fewer than 4 usable observations; ",
        "plotting points only.",
        call. = FALSE
      )
      next
    }
    fit <- fit_dose_response(sub, "response", "concentration")
    cf <- stats::coef(fit)
    grid <- 10^seq(log10(min(sub$concentration)), log10(max(sub$concentration)),
      length.out = n_points
    )
    curves[[cond]] <- data.frame(
      concentration = grid,
      response = stats::predict(fit, newdata = grid),
      condition = factor(cond, levels = conditions),
      stringsAsFactors = FALSE
    )
    auc <- compute_auc(
      min(sub$concentration), max(sub$concentration),
      top = cf[["top"]], ic50 = cf[["ic50"]],
      hill = cf[["hill"]], bottom = cf[["bottom"]]
    )
    summaries <- c(summaries, sprintf(
      "%s  IC50: %.2f  AUC: %.2f", cond, cf[["ic50"]], auc
    ))
  }

  n_cond <- length(conditions)
  cond_colors <- stats::setNames(
    rep(colors, length.out = n_cond)[seq_len(n_cond)], conditions
  )
  cond_shapes <- stats::setNames(
    rep(shapes, length.out = n_cond)[seq_len(n_cond)], conditions
  )

  p <- ggplot2::ggplot() +
    ggplot2::geom_point(
      data = obs,
      ggplot2::aes(
        x = .data$concentration, y = .data$response,
        color = .data$condition, shape = .data$condition
      ),
      size = 3
    )

  if (length(curves) > 0) {
    curve_df <- do.call(rbind, curves)
    p <- p + ggplot2::geom_line(
      data = curve_df,
      ggplot2::aes(
        x = .data$concentration, y = .data$response,
        color = .data$condition
      ),
      linewidth = 1.2
    )
  }
  if (!is.na(ref_line)) {
    p <- p + ggplot2::geom_hline(
      yintercept = ref_line, linetype = "dashed", color = "#888888",
      linewidth = 0.6
    )
  }

  x_lab <- if (!is.null(drug)) {
    bquote("[" * .(drug) * "] Concentration" ~ (mu * M))
  } else {
    bquote("Concentration" ~ (mu * M))
  }
  subtitle <- if (length(summaries) > 0) paste(summaries, collapse = "\n") else NULL

  p <- p +
    ggplot2::expand_limits(y = 0) +
    ggplot2::scale_x_log10() +
    ggplot2::scale_color_manual(name = legend_title, values = cond_colors) +
    ggplot2::scale_shape_manual(name = legend_title, values = cond_shapes) +
    ggplot2::labs(
      x = x_lab, y = "Relative cell proliferation", subtitle = subtitle
    ) +
    ggplot2::geom_hline(yintercept = 0, color = "grey80", linewidth = 0.3) +
    theme_hts(base_size = 14, legend = "right")

  p + ggplot2::theme(
    plot.subtitle = ggplot2::element_text(size = 12, hjust = 0.5,
      color = "#333333"),
    legend.title = ggplot2::element_text(size = 13, color = "#333333")
  )
}

#' Scatter plot of control readouts across plates
#'
#' Draws, for each plate, the individual negative- and positive-control well
#' readouts (for example normalised cell counts) so that plate-to-plate control
#' behaviour and separation can be inspected at a glance. This is the standard
#' QC view used to confirm that negative controls stay high and positive
#' controls stay low across an entire screen.
#'
#' @param data A data frame with one row per plate and several negative- and
#'   positive-control replicate columns (wide format, such as
#'   [screen_plate_qc]).
#' @param plate_col Name of the plate identifier column, as a string. Default
#'   `"plateID"`.
#' @param neg_cols,pos_cols Character vectors naming the negative- and
#'   positive-control replicate columns. By default columns matching
#'   `"^neg_ctrl"` and `"^pos_ctrl"` are used.
#' @param colors Named character vector giving the point colours for the two
#'   control groups.
#' @param y_title Y-axis title. Default `"Normalized cell count"`.
#'
#' @return A [ggplot2::ggplot] object: control readouts (points, dodged and
#'   jittered) per plate, coloured by control type, with the y-axis expanded to
#'   include zero.
#'
#' @examples
#' plot_control_scatter(screen_plate_qc)
#'
#' @seealso [plot_qc_boxplot()], [plot_qc_circular_heatmap()]
#' @export
plot_control_scatter <- function(data, plate_col = "plateID",
                                 neg_cols = grep("^neg_ctrl", names(data),
                                   value = TRUE),
                                 pos_cols = grep("^pos_ctrl", names(data),
                                   value = TRUE),
                                 colors = c("Negative control" = "#0072B2",
                                            "Positive control" = "#D55E00"),
                                 y_title = "Normalized cell count") {
  if (!plate_col %in% names(data)) {
    stop("Column '", plate_col, "' not found in `data`.", call. = FALSE)
  }
  if (length(neg_cols) == 0 || length(pos_cols) == 0) {
    stop("No negative- or positive-control columns found; supply `neg_cols` ",
      "and `pos_cols`.",
      call. = FALSE
    )
  }

  plate <- factor(data[[plate_col]], levels = unique(data[[plate_col]]))
  long <- rbind(
    data.frame(
      plate = rep(plate, length(neg_cols)),
      value = .as_numeric(unlist(data[neg_cols], use.names = FALSE)),
      control = "Negative control",
      stringsAsFactors = FALSE
    ),
    data.frame(
      plate = rep(plate, length(pos_cols)),
      value = .as_numeric(unlist(data[pos_cols], use.names = FALSE)),
      control = "Positive control",
      stringsAsFactors = FALSE
    )
  )

  ggplot2::ggplot(
    long,
    ggplot2::aes(x = .data$plate, y = .data$value, color = .data$control)
  ) +
    ggplot2::geom_point(
      position = ggplot2::position_jitterdodge(
        jitter.width = 0.15, dodge.width = 0.6
      ),
      alpha = 0.7, size = 1.8
    ) +
    ggplot2::scale_color_manual(values = colors) +
    ggplot2::expand_limits(y = 0) +
    ggplot2::labs(x = NULL, y = y_title, color = "Control") +
    theme_hts(legend = "bottom") +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
    )
}

#' Trend plot of a QC metric across plates
#'
#' Plots a single QC metric (for example Z', Z-factor or negative-control CV)
#' for every plate in acquisition order, colouring each plate by a pass/fail
#' flag and marking the overall mean. This mirrors the per-metric QC review
#' plots used in the screening pipeline and makes drifting or failing plates
#' easy to spot.
#'
#' @param data A data frame with one row per plate, such as [screen_plate_qc].
#' @param metric Name of the metric column to plot, as a string. Default
#'   `"z_prime"`.
#' @param plate_col Name of the plate identifier column, as a string. Default
#'   `"plateID"`.
#' @param flag_col Optional name of a column of `"Bad"`/`"Normal"` flags used to
#'   colour points. If `NULL` (default) plates are flagged on the fly using
#'   `threshold` and `direction`.
#' @param threshold Numeric cut-off used to flag plates when `flag_col` is
#'   `NULL`. Default `0.5`.
#' @param direction Either `"min"` (values below `threshold` fail, the default)
#'   or `"max"` (values above `threshold` fail).
#' @param colors Named character vector giving the colours for `"Bad"` and
#'   `"Normal"` plates.
#'
#' @return A [ggplot2::ggplot] object: one bar per plate in plate order,
#'   coloured by flag, with a dashed line at the mean and a solid line at
#'   `threshold`.
#'
#' @examples
#' plot_qc_metric_trend(screen_plate_qc, metric = "z_prime")
#'
#' @seealso [calculate_qc_metrics()], [flag_qc_plates()]
#' @export
plot_qc_metric_trend <- function(data, metric = "z_prime",
                                 plate_col = "plateID", flag_col = NULL,
                                 threshold = 0.5, direction = c("min", "max"),
                                 colors = c(Bad = "#D55E00", Normal = "#0072B2")) {
  direction <- match.arg(direction)
  for (col in c(metric, plate_col)) {
    if (!col %in% names(data)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }

  df <- data.frame(
    plate = factor(data[[plate_col]], levels = unique(data[[plate_col]])),
    value = .as_numeric(data[[metric]]),
    stringsAsFactors = FALSE
  )
  if (!is.null(flag_col)) {
    if (!flag_col %in% names(data)) {
      stop("Column '", flag_col, "' not found in `data`.", call. = FALSE)
    }
    df$flag <- ifelse(as.character(data[[flag_col]]) == "Bad", "Bad", "Normal")
  } else {
    fail <- if (direction == "min") df$value < threshold else df$value > threshold
    df$flag <- ifelse(fail, "Bad", "Normal")
  }
  df$flag <- factor(df$flag, levels = c("Bad", "Normal"))
  mean_val <- mean(df$value, na.rm = TRUE)

  ggplot2::ggplot(
    df,
    ggplot2::aes(x = .data$plate, y = .data$value, fill = .data$flag)
  ) +
    ggplot2::geom_col(width = 0.7, color = "white", linewidth = 0.3) +
    ggplot2::geom_hline(yintercept = threshold, color = "#888888",
      linewidth = 0.6) +
    ggplot2::geom_hline(
      yintercept = mean_val, linetype = "dashed",
      color = .hts_col("reference"), linewidth = 0.6
    ) +
    ggplot2::scale_fill_manual(values = colors, drop = FALSE, name = NULL) +
    ggplot2::labs(x = NULL, y = metric) +
    theme_hts(legend = "bottom") +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
    )
}

#' Circular plot of a per-plate QC metric
#'
#' Draws a circular (\pkg{circlize}) scatter of a QC metric with one sector per
#' experiment and one point per plate, coloured by whether the plate passes a
#' threshold. This reproduces the whole-screen Z'-factor overview used in the
#' pipeline, where many plates across several experiments are compared on a
#' single ring.
#'
#' @param data A data frame with one row per plate, such as [screen_plate_qc].
#' @param metric Name of the metric column, as a string. Default `"z_prime"`.
#' @param sector_col Name of the column that defines the sectors (experiments),
#'   as a string. Default `"experimentID"`.
#' @param x_col Name of the numeric within-sector position column (plate order),
#'   as a string. Default `"plateNumber"`.
#' @param label_col Name of the column holding the plate identifier printed on
#'   the outside of each bar, as a string. Default `"plateID"`; if the column is
#'   absent the values of `x_col` are used instead.
#' @param threshold Numeric pass/fail cut-off. Plates whose metric is below
#'   `threshold` are drawn in `bad_color`. Default `0.5`.
#' @param transform Logical; if `TRUE` (default) strongly negative metric values
#'   (`< -1`) are compressed with `-log10(-x)` for display, matching the
#'   pipeline's Z'-factor plot.
#' @param colors Named character vector with entries `"Bad"` and `"Normal"`
#'   giving the bar colours.
#' @param bar_lwd Line width of each plate bar. Default `4`.
#' @param x_margin Numeric angular padding (in `x_col` units) added to each side
#'   of every sector so the first and last plate bars do not touch the sector
#'   edges. Default `0.8`.
#' @param show_labels Logical; if `TRUE` (default) the plate identifier from
#'   `label_col` is printed radially just outside each bar.
#' @param center_label Text drawn in the centre of the ring. Defaults to
#'   `metric`.
#'
#' @return Invisibly `NULL`. Called for the side effect of drawing to the active
#'   graphics device.
#'
#' @details Requires the suggested package \pkg{circlize}. Each experiment is a
#'   sector and each plate a thin vertical bar drawn at its own `plateNumber`
#'   position, rising from a zero baseline to the metric value, so the bars never
#'   overlap. An angular `x_margin` keeps the first and last bars clear of the
#'   sector edges, the plate identifier is printed radially just outside each
#'   bar, and a short Y-axis on the first sector plus a dashed reference line at
#'   `threshold` in every sector give the scale. The design scales to large
#'   campaigns of roughly ten plates per experiment.
#'
#' @examples
#' if (requireNamespace("circlize", quietly = TRUE)) {
#'   plot_qc_metric_circular(screen_plate_qc, metric = "z_prime")
#' }
#'
#' @seealso [plot_qc_metric_trend()], [plot_qc_circular_heatmap()]
#' @export
plot_qc_metric_circular <- function(data, metric = "z_prime",
                                    sector_col = "experimentID",
                                    x_col = "plateNumber",
                                    label_col = "plateID",
                                    threshold = 0.5, transform = TRUE,
                                    colors = c(Bad = "#D55E00",
                                               Normal = "#0072B2"),
                                    bar_lwd = 4,
                                    x_margin = 0.8,
                                    show_labels = TRUE,
                                    center_label = NULL) {
  if (!requireNamespace("circlize", quietly = TRUE)) {
    stop("Package 'circlize' is required for `plot_qc_metric_circular()`.",
      call. = FALSE
    )
  }
  for (col in c(metric, sector_col, x_col)) {
    if (!col %in% names(data)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }

  compress <- function(v) ifelse(v > -1, v, -log10(-v))
  value <- .as_numeric(data[[metric]])
  df <- data.frame(
    sector = as.character(data[[sector_col]]),
    x = .as_numeric(data[[x_col]]),
    value = value,
    y = if (transform) compress(value) else value,
    label = if (label_col %in% names(data)) {
      as.character(data[[label_col]])
    } else {
      as.character(data[[x_col]])
    },
    stringsAsFactors = FALSE
  )
  df$barcol <- ifelse(df$value < threshold, colors[["Bad"]], colors[["Normal"]])
  thr_disp <- if (transform) compress(threshold) else threshold
  if (is.null(center_label)) center_label <- metric

  yr <- range(c(df$y, thr_disp, 0), na.rm = TRUE)
  pad <- diff(yr) * 0.12
  if (!is.finite(pad) || pad == 0) pad <- 0.1
  ylim <- c(min(yr[1], 0) - pad, yr[2] + pad)
  first_sector <- df$sector[1]
  y_at <- unique(round(c(0, thr_disp, yr[2]), 2))

  ## Give each sector an explicit x range with a margin so the first and last
  ## bars are inset from the sector edges instead of overlapping them.
  secs <- unique(df$sector)
  xlim <- do.call(rbind, lapply(secs, function(s) {
    xs <- df$x[df$sector == s]
    c(min(xs) - x_margin, max(xs) + x_margin)
  }))

  on.exit(circlize::circos.clear(), add = TRUE)
  circlize::circos.clear()
  circlize::circos.par("track.height" = 0.5, gap.degree = 4,
    cell.padding = c(0.02, 0.1, 0.02, 0.1),
    points.overflow.warning = FALSE,
    canvas.xlim = c(-1.35, 1.35), canvas.ylim = c(-1.35, 1.35)
  )
  circlize::circos.initialize(secs, xlim = xlim)
  circlize::circos.track(ylim = ylim,
    panel.fun = function(x, y) {
      sec <- circlize::CELL_META$sector.index
      d <- df[df$sector == sec, , drop = FALSE]
      ## Zero baseline (the x-axis) that every bar rises from.
      circlize::circos.lines(
        circlize::CELL_META$cell.xlim, c(0, 0),
        col = "#555555", lwd = 1.1
      )
      lab_y <- circlize::CELL_META$cell.ylim[2] + circlize::mm_y(2)
      for (i in seq_len(nrow(d))) {
        circlize::circos.lines(
          c(d$x[i], d$x[i]), c(0, d$y[i]),
          col = d$barcol[i], lwd = bar_lwd, straight = TRUE
        )
        if (isTRUE(show_labels)) {
          circlize::circos.text(
            d$x[i], lab_y, d$label[i], cex = 0.5, col = "#555555",
            facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.5)
          )
        }
      }
      circlize::circos.lines(
        circlize::CELL_META$cell.xlim, c(thr_disp, thr_disp),
        col = "#666666", lty = 2
      )
      if (sec == first_sector) {
        circlize::circos.yaxis(
          side = "left", at = y_at, labels.cex = 0.55, col = "#888888",
          labels.col = "#555555"
        )
      }
    }
  )
  graphics::text(0, 0, center_label, cex = 1.3, col = "#222222")
  invisible(NULL)
}

#' Circular heatmap of control readouts across plates
#'
#' Draws a large circular (\pkg{circlize}) heatmap of the control-well readouts
#' for every plate, grouped into sectors by experiment. Each ring is one plate
#' and each cell one control well, so systematic control problems across a whole
#' screen become visible at once. This mirrors the summary `circos.heatmap`
#' used in the pipeline to display negative- and positive-control cell counts.
#'
#' @param data A data frame with one row per plate and numeric control columns,
#'   such as [screen_plate_qc].
#' @param value_cols Character vector of the control-value columns to display.
#'   By default all columns matching `"_ctrl_"` are used.
#' @param plate_col Name of the plate identifier column (used as row names), as
#'   a string. Default `"plateID"`.
#' @param split_col Optional name of a grouping column (experiment) used to
#'   split the ring into sectors. Default `"experimentID"`.
#' @param col_breaks,col_colors Numeric breakpoints and matching colours passed
#'   to [circlize::colorRamp2()]. Defaults map `0`/`0.5`/`1` to
#'   navy/white/firebrick.
#' @param name Legend title. Default `"Norm cell counts"`.
#' @param show_sector_labels Logical, whether to label each sector. Default
#'   `TRUE`.
#'
#' @return Invisibly `NULL`. Called for the side effect of drawing to the active
#'   graphics device.
#'
#' @details Requires the suggested package \pkg{circlize}; when
#'   \pkg{ComplexHeatmap} is also installed a colour legend is added.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("circlize", quietly = TRUE)) {
#'   plot_qc_circular_heatmap(screen_plate_qc)
#' }
#' }
#'
#' @seealso [plot_control_scatter()], [plot_delta_auc_heatmap()]
#' @export
plot_qc_circular_heatmap <- function(data,
                                     value_cols = grep("_ctrl_", names(data),
                                       value = TRUE),
                                     plate_col = "plateID",
                                     split_col = "experimentID",
                                     col_breaks = c(0, 0.5, 1),
                                     col_colors = c("navy", "white",
                                       "firebrick3"),
                                     name = "Norm cell counts",
                                     show_sector_labels = TRUE) {
  if (!requireNamespace("circlize", quietly = TRUE)) {
    stop("Package 'circlize' is required for `plot_qc_circular_heatmap()`.",
      call. = FALSE
    )
  }
  if (!plate_col %in% names(data)) {
    stop("Column '", plate_col, "' not found in `data`.", call. = FALSE)
  }
  if (length(value_cols) == 0) {
    stop("No control-value columns found; supply `value_cols`.", call. = FALSE)
  }

  mat <- as.matrix(data[, value_cols, drop = FALSE])
  storage.mode(mat) <- "double"
  rownames(mat) <- data[[plate_col]]
  split <- if (!is.null(split_col) && split_col %in% names(data)) {
    factor(data[[split_col]])
  } else {
    NULL
  }
  col_fun <- circlize::colorRamp2(col_breaks, col_colors)

  on.exit(circlize::circos.clear(), add = TRUE)
  circlize::circos.clear()
  circlize::circos.heatmap(mat, split = split, col = col_fun,
    track.height = 0.4, bg.border = "#CCCCCC", bg.lwd = 0.6, bg.lty = 1,
    rownames.side = "outside", show.sector.labels = show_sector_labels
  )
  circlize::circos.clear()

  if (requireNamespace("ComplexHeatmap", quietly = TRUE) &&
    requireNamespace("grid", quietly = TRUE)) {
    lgd <- ComplexHeatmap::Legend(title = name, col_fun = col_fun)
    grid::grid.draw(lgd)
  }
  invisible(NULL)
}

#' Plate-layout heatmap of well readouts
#'
#' Draws a microplate as a grid of wells (rows lettered top-to-bottom, columns
#' numbered left-to-right) coloured by a per-well readout such as normalised
#' cell count, so that drug-treated wells and their positive/negative controls
#' can be inspected in their physical plate positions. Missing wells are shown
#' as empty cells, making layout errors and edge effects easy to spot. This
#' abstracts the per-plate `heatmap.2()` plate maps used in the screening
#' pipeline, but is built on \pkg{ggplot2} and needs no external heatmap
#' package.
#'
#' @param data A long-format data frame with one row per well.
#' @param fill Name of the column mapped to tile colour, as a string. Default
#'   `"normalized_cell_count"`. Numeric columns use a continuous viridis scale;
#'   character/factor columns use a discrete viridis scale (useful for showing
#'   the well-type layout).
#' @param well_col Optional name of a single well-identifier column such as
#'   `"A1"` or `"B12"`. If supplied it is split into a row letter and column
#'   number and `row_col`/`col_col` are ignored.
#' @param row_col,col_col Names of the row (letter) and column (number)
#'   columns used when `well_col` is `NULL`. Defaults `"plate_rows"` and
#'   `"plate_columns"`. Any non-digit characters in the column values (for
#'   example an `"X"` prefix) are stripped.
#' @param label Optional name of a column whose values are printed inside each
#'   tile (for example a drug name or a QC flag). Default `NULL`.
#' @param control_col Optional name of a column identifying each well's role
#'   (for example `"well_type"` or `"experiment_type"`). When supplied, the
#'   negative- and positive-control wells are circled on the plot so that they
#'   can be told apart from the drug-treated wells at a glance. Default `NULL`
#'   (no wells circled).
#' @param nc_values,pc_values Character vectors of the labels in `control_col`
#'   that mark negative and positive controls. Defaults cover the package's
#'   canonical labels (`"NC"`/`"PC"`) and the common long forms
#'   (`"negative_ctrl"`/`"positive_ctrl"`).
#' @param circle_size Size of the control-marking circles. When `NULL`
#'   (default) it is scaled to the plate so that 96- and 384-well plates are
#'   both legible.
#' @param plate Optional plate identifier; if supplied `data` is filtered to
#'   this plate. When `NULL` and several plates are present the plot is
#'   facetted by `plate_col`.
#' @param plate_col Name of the plate identifier column, as a string. Default
#'   `"plateID"`.
#' @param plate_type Plate format, either `96` or `384`, controlling the size
#'   of the drawn grid. When `NULL` (default) it is inferred from the observed
#'   rows and columns.
#' @param title Optional plot title.
#' @param na_value Colour used for wells with no data. Default `"grey90"`.
#'
#' @return A [ggplot2::ggplot] object: one tile per well, coloured by `fill`,
#'   with row A at the top and column 1 at the top-left, drawn with a fixed
#'   aspect ratio. When `control_col` is supplied the negative- and
#'   positive-control wells are outlined with coloured circles.
#'
#' @details Duplicate wells (for example replicates sharing a position) are
#'   summarised by their mean for numeric `fill` and concatenated for
#'   categorical `fill`/`label`.
#'
#' @examples
#' # A single 96-well plate coloured by normalised cell count.
#' plot_plate_heatmap(subset(screen_plate_layout, plateID == "EXP99_Gy0"))
#'
#' # Show the well-type layout instead, both plates side by side.
#' plot_plate_heatmap(screen_plate_layout, fill = "experiment_type")
#'
#' # Circle the negative and positive controls on a raw 96-well plate.
#' plot_plate_heatmap(
#'   subset(hts_primary_raw, plate_id == "P01"),
#'   fill = "signal", well_col = "well", plate_col = "plate_id",
#'   control_col = "well_type"
#' )
#'
#' # A realistic 384-well plate (the outer 2-well ring is a compound-free buffer
#' # that reads at the low, no-cell level; controls sit on a diagonal in the two
#' # inset columns 3 and 22).
#' plot_plate_heatmap(hts_plate_384, fill = "signal", well_col = "well",
#'   control_col = "well_type", plate_col = "plate_id", plate_type = 384)
#'
#' @seealso [plot_qc_boxplot()], [plot_control_scatter()]
#' @export
plot_plate_heatmap <- function(data, fill = "normalized_cell_count",
                               well_col = NULL, row_col = "plate_rows",
                               col_col = "plate_columns", label = NULL,
                               control_col = NULL,
                               nc_values = c("NC", "negative_ctrl"),
                               pc_values = c("PC", "positive_ctrl"),
                               circle_size = NULL,
                               plate = NULL, plate_col = "plateID",
                               plate_type = NULL, title = NULL,
                               na_value = "grey90") {
  if (!fill %in% names(data)) {
    stop("Column '", fill, "' not found in `data`.", call. = FALSE)
  }
  df <- data

  facet <- FALSE
  if (!is.null(plate)) {
    if (!plate_col %in% names(df)) {
      stop("Column '", plate_col, "' not found in `data`.", call. = FALSE)
    }
    df <- df[as.character(df[[plate_col]]) == plate, , drop = FALSE]
    if (nrow(df) == 0) {
      stop("No rows found for plate '", plate, "'.", call. = FALSE)
    }
  } else if (plate_col %in% names(df) &&
    length(unique(df[[plate_col]])) > 1) {
    facet <- TRUE
  }

  if (!is.null(well_col)) {
    if (!well_col %in% names(df)) {
      stop("Column '", well_col, "' not found in `data`.", call. = FALSE)
    }
    wells <- as.character(df[[well_col]])
    rows <- toupper(sub("^([A-Za-z]+).*$", "\\1", wells))
    cols <- as.integer(sub("^[A-Za-z]+0*([0-9]+)$", "\\1", wells))
  } else {
    for (col in c(row_col, col_col)) {
      if (!col %in% names(df)) {
        stop("Column '", col, "' not found in `data`.", call. = FALSE)
      }
    }
    rows <- toupper(gsub("[^A-Za-z]", "", as.character(df[[row_col]])))
    cols <- as.integer(gsub("\\D", "", as.character(df[[col_col]])))
  }

  numeric_fill <- is.numeric(df[[fill]])
  plot_df <- data.frame(
    row = rows, col = cols,
    value = if (numeric_fill) .as_numeric(df[[fill]]) else as.character(df[[fill]]),
    stringsAsFactors = FALSE
  )
  if (!is.null(label)) {
    if (!label %in% names(df)) {
      stop("Column '", label, "' not found in `data`.", call. = FALSE)
    }
    plot_df$lab <- as.character(df[[label]])
  }
  if (!is.null(control_col)) {
    if (!control_col %in% names(df)) {
      stop("Column '", control_col, "' not found in `data`.", call. = FALSE)
    }
    cvals <- as.character(df[[control_col]])
    ctrl <- rep(NA_character_, length(cvals))
    ctrl[cvals %in% nc_values] <- "Negative control"
    ctrl[cvals %in% pc_values] <- "Positive control"
    plot_df$ctrl <- ctrl
  }
  if (facet) plot_df$plate <- as.character(df[[plate_col]])
  plot_df <- plot_df[!is.na(plot_df$row) & !is.na(plot_df$col), , drop = FALSE]
  if (nrow(plot_df) == 0) {
    stop("No wells with a valid row and column could be parsed.", call. = FALSE)
  }

  by_cols <- c("row", "col", if (facet) "plate")
  collapse_chr <- function(z) paste(unique(z[!is.na(z)]), collapse = "/")
  agg_one <- function(col, fun) {
    a <- stats::aggregate(plot_df[[col]],
      by = plot_df[by_cols], FUN = fun
    )
    names(a)[ncol(a)] <- col
    a
  }
  agg <- agg_one("value", if (numeric_fill) {
    function(z) mean(z, na.rm = TRUE)
  } else {
    collapse_chr
  })
  if (!is.null(label)) {
    agg <- merge(agg, agg_one("lab", collapse_chr), by = by_cols)
  }
  if (!is.null(control_col)) {
    ctrl_agg <- agg_one("ctrl", collapse_chr)
    ctrl_agg$ctrl[ctrl_agg$ctrl == ""] <- NA
    agg <- merge(agg, ctrl_agg, by = by_cols)
  }

  max_row <- max(match(agg$row, LETTERS), na.rm = TRUE)
  max_col <- max(agg$col, na.rm = TRUE)
  if (is.null(plate_type)) {
    plate_type <- if (max_row <= 8 && max_col <= 12) 96L else 384L
  }
  dims <- switch(as.character(plate_type),
    "96" = c(8L, 12L), "384" = c(16L, 24L),
    c(max(max_row, 1L), max(max_col, 1L))
  )
  n_row <- max(dims[1], max_row)
  n_col <- max(dims[2], max_col)
  row_levels <- LETTERS[seq_len(n_row)]
  col_levels <- seq_len(n_col)

  base_grid <- expand.grid(row = row_levels, col = col_levels,
    KEEP.OUT.ATTRS = FALSE, stringsAsFactors = FALSE
  )
  if (facet) {
    plates <- unique(agg$plate)
    grid_df <- do.call(rbind, lapply(plates, function(p) {
      cbind(base_grid, plate = p)
    }))
  } else {
    grid_df <- base_grid
  }
  full <- merge(grid_df, agg, by = by_cols, all.x = TRUE)

  full$row <- factor(full$row, levels = rev(row_levels))
  full$col <- factor(full$col, levels = col_levels)

  p <- ggplot2::ggplot(
    full,
    ggplot2::aes(x = .data$col, y = .data$row, fill = .data$value)
  ) +
    ggplot2::geom_tile(color = "grey70", linewidth = 0.3)

  if (!is.null(control_col) && any(!is.na(full$ctrl))) {
    if (is.null(circle_size)) {
      circle_size <- max(1.4, min(6, 90 / n_col))
    }
    ctrl_df <- full[!is.na(full$ctrl), , drop = FALSE]
    p <- p +
      ggplot2::geom_point(
        data = ctrl_df,
        ggplot2::aes(color = .data$ctrl),
        shape = 21, fill = NA, size = circle_size, stroke = 1.3,
        inherit.aes = TRUE, show.legend = TRUE
      ) +
      ggplot2::scale_color_manual(
        values = c("Negative control" = "#CC79A7",
          "Positive control" = "#D55E00"),
        name = "Control", na.translate = FALSE
      )
  }

  if (!is.null(label)) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = .data$lab), size = 2, na.rm = TRUE
    )
  }

  fill_scale <- if (numeric_fill) {
    ggplot2::scale_fill_viridis_c(na.value = na_value)
  } else {
    ggplot2::scale_fill_viridis_d(na.value = na_value)
  }

  p +
    fill_scale +
    ggplot2::scale_x_discrete(position = "top") +
    ggplot2::coord_fixed() +
    (if (facet) ggplot2::facet_wrap(ggplot2::vars(.data$plate)) else NULL) +
    ggplot2::labs(x = NULL, y = NULL, fill = fill, title = title) +
    ggplot2::theme_minimal(base_size = 11) +
    ggplot2::theme(
      panel.grid = ggplot2::element_blank(),
      plot.background = ggplot2::element_rect(fill = "#FFFFFF", color = NA),
      text = ggplot2::element_text(color = "#333333"),
      plot.title = ggplot2::element_text(face = "bold", color = "#222222"),
      axis.text = ggplot2::element_text(size = 8, color = "#555555")
    )
}

test_that("plot_dose_response_curves returns a ggplot with fitted curves", {
  one <- subset(
    screen_doseresponse,
    drug_name == "DrugA" & experiment_type == "sample"
  )
  p <- plot_dose_response_curves(one, drug = "DrugA")

  expect_s3_class(p, "ggplot")
  # observed points plus fitted lines for both conditions
  geoms <- vapply(p$layers, function(l) class(l$geom)[1], character(1))
  expect_true("GeomPoint" %in% geoms)
  expect_true("GeomLine" %in% geoms)
  # log10 concentration axis
  expect_true(any(vapply(
    p$scales$scales, function(s) "x" %in% s$aesthetics, logical(1)
  )))
})

test_that("plot_dose_response_curves errors on missing columns", {
  expect_error(
    plot_dose_response_curves(data.frame(a = 1, b = 2)),
    "not found"
  )
})

test_that("plot_dose_response_curves errors for an unknown drug", {
  expect_error(
    plot_dose_response_curves(screen_doseresponse, drug = "DoesNotExist"),
    "No rows found"
  )
})

test_that("plot_control_scatter returns a ggplot with both control groups", {
  p <- plot_control_scatter(screen_plate_qc)
  expect_s3_class(p, "ggplot")
  groups <- unique(as.character(p$data$control))
  expect_setequal(groups, c("Negative control", "Positive control"))
})

test_that("plot_control_scatter errors when control columns are absent", {
  expect_error(
    plot_control_scatter(data.frame(plateID = "P1"), plate_col = "plateID"),
    "No negative- or positive-control columns"
  )
})

test_that("plot_qc_metric_trend flags failing plates", {
  p <- plot_qc_metric_trend(screen_plate_qc, metric = "z_prime",
    threshold = 0.5)
  expect_s3_class(p, "ggplot")
  expected_bad <- sum(screen_plate_qc$z_prime < 0.5)
  expect_equal(sum(p$data$flag == "Bad"), expected_bad)
})

test_that("plot_qc_metric_trend respects direction = 'max'", {
  df <- data.frame(
    plateID = c("P1", "P2", "P3"),
    cv_negative_ctrl = c(5, 12, 8)
  )
  p <- plot_qc_metric_trend(df, metric = "cv_negative_ctrl",
    threshold = 10, direction = "max")
  # only the plate above the cut-off (12) should fail
  expect_equal(as.character(p$data$flag), c("Normal", "Bad", "Normal"))
})

test_that("plot_qc_metric_trend can use a precomputed flag column", {
  df <- data.frame(
    plateID = c("P1", "P2"),
    z_prime = c(0.7, 0.3),
    z_primer_flag = c("Normal", "Bad")
  )
  p <- plot_qc_metric_trend(df, metric = "z_prime", flag_col = "z_primer_flag")
  expect_equal(as.character(p$data$flag), c("Normal", "Bad"))
})

test_that("plot_qc_metric_trend errors on a missing metric column", {
  expect_error(
    plot_qc_metric_trend(screen_plate_qc, metric = "not_a_column"),
    "not found"
  )
})

test_that("plot_qc_metric_circular draws without error", {
  skip_if_not_installed("circlize")
  tmp <- tempfile(fileext = ".pdf")
  grDevices::pdf(tmp)
  on.exit({
    grDevices::dev.off()
    unlink(tmp)
  })
  expect_invisible(
    suppressMessages(plot_qc_metric_circular(screen_plate_qc))
  )
})

test_that("plot_qc_metric_circular errors on missing columns", {
  skip_if_not_installed("circlize")
  expect_error(
    plot_qc_metric_circular(data.frame(experimentID = "E", plateNumber = 1)),
    "not found"
  )
})

test_that("plot_qc_circular_heatmap draws without error", {
  skip_if_not_installed("circlize")
  tmp <- tempfile(fileext = ".pdf")
  grDevices::pdf(tmp)
  on.exit({
    grDevices::dev.off()
    unlink(tmp)
  })
  expect_invisible(
    suppressMessages(plot_qc_circular_heatmap(screen_plate_qc))
  )
})

test_that("plot_qc_circular_heatmap errors when no value columns match", {
  skip_if_not_installed("circlize")
  expect_error(
    plot_qc_circular_heatmap(
      data.frame(plateID = "P1", experimentID = "E1"),
      value_cols = character(0)
    ),
    "No control-value columns"
  )
})

test_that("plot_plate_heatmap builds a full 96-well grid", {
  p <- plot_plate_heatmap(
    subset(screen_plate_layout, plateID == "EXP99_Gy0")
  )
  expect_s3_class(p, "ggplot")
  # 8 rows x 12 columns, every well populated
  expect_equal(nrow(p$data), 96)
  expect_equal(sum(!is.na(p$data$value)), 96)
  expect_setequal(levels(p$data$row), LETTERS[1:8])
})

test_that("plot_plate_heatmap facets when several plates are present", {
  p <- plot_plate_heatmap(screen_plate_layout, fill = "experiment_type")
  expect_s3_class(p, "ggplot")
  expect_true("plate" %in% names(p$data))
  expect_equal(nrow(p$data), 192)
})

test_that("plot_plate_heatmap parses a single well-identifier column", {
  df <- data.frame(
    well = c("A1", "B2", "H12"),
    normalized_cell_count = c(1, 0.5, 0.05)
  )
  p <- plot_plate_heatmap(df, well_col = "well")
  expect_s3_class(p, "ggplot")
  filled <- p$data[!is.na(p$data$value), ]
  expect_equal(nrow(filled), 3)
})

test_that("plot_plate_heatmap circles controls when control_col is given", {
  has_points <- function(p) {
    any(vapply(p$layers, function(l) inherits(l$geom, "GeomPoint"), logical(1)))
  }
  # No control_col: no circle layer (backward compatible).
  p0 <- plot_plate_heatmap(subset(screen_plate_layout, plateID == "EXP99_Gy0"))
  expect_false(has_points(p0))

  # With control_col: a point layer marks NC/PC wells.
  p1 <- plot_plate_heatmap(subset(screen_plate_layout, plateID == "EXP99_Gy0"),
    control_col = "experiment_type")
  expect_true(has_points(p1))
  built <- ggplot2::ggplot_build(p1)$data
  # one circle per control well on the 96-well plate
  sub <- subset(screen_plate_layout, plateID == "EXP99_Gy0")
  n_ctrl <- sum(sub$experiment_type %in% c("negative_ctrl", "positive_ctrl"))
  expect_equal(nrow(built[[2]]), n_ctrl)
})

test_that("plot_plate_heatmap circles controls on a 384-well plate", {
  data("hts_plate_384", package = "diffHTS", envir = environment())
  p <- plot_plate_heatmap(hts_plate_384, fill = "signal", well_col = "well",
    control_col = "well_type", plate_col = "plate_id", plate_type = 384)
  expect_s3_class(p, "ggplot")
  # Full 384 grid: every well (including the buffer ring) carries a reading.
  expect_equal(nrow(p$data), 384)
  # The outer buffer ring is populated with a low, no-cell reading, not NA.
  expect_equal(sum(is.na(p$data$value)), 0)
  built <- ggplot2::ggplot_build(p)$data
  # 12 NC + 12 PC (diagonal in columns 3 and 22) circled
  expect_equal(nrow(built[[2]]), 24)
})

test_that("plot_plate_heatmap_raw and _inhibition circle controls", {
  has_points <- function(p) {
    any(vapply(p$layers, function(l) inherits(l$geom, "GeomPoint"), logical(1)))
  }
  raw <- baseline_subtract(read_hts_plate(hts_primary_raw))
  expect_true(has_points(plot_plate_heatmap_raw(raw, plate = "P01")))
  norm <- norm_by_control(raw)
  expect_true(has_points(plot_plate_heatmap_inhibition(norm, plate = "P01")))
})

test_that("plot_plate_heatmap errors on a missing control_col", {
  expect_error(
    plot_plate_heatmap(subset(screen_plate_layout, plateID == "EXP99_Gy0"),
      control_col = "not_a_column"),
    "not found"
  )
})

# Module 7: Hit ranking, annotation and reporting ----------------------------

# Directional, scale-free score contribution for one metric.
# `better` = "low" means smaller values are more active.
.metric_score <- function(v, better = c("low", "high")) {
  better <- match.arg(better)
  s <- stats::sd(v, na.rm = TRUE)
  z <- if (!is.finite(s) || s == 0) rep(0, length(v)) else
    (v - mean(v, na.rm = TRUE)) / s
  z[is.na(z)] <- 0
  if (better == "low") -z else z
}

#' Rank and stratify hit compounds
#'
#' Combines potency, efficacy and quality metrics into a single activity score
#' and assigns each compound to a Strong, Moderate or Weak tier, the final
#' prioritisation step of the pipeline. Lower AUC and IC50 and higher Emax count
#' as more active; lower replicate CV and higher fit \eqn{R^2} raise confidence.
#'
#' @param data A per-compound data frame of metrics.
#' @param auc_col,ic50_col,emax_col,cv_col,rsq_col Column names for the metrics
#'   to use; set any to `NULL` to omit it. Metrics absent from `data` are
#'   silently skipped.
#' @param weights Optional named numeric vector of per-metric weights (names
#'   matching the metric column names). Defaults to equal weights.
#' @param probs Two increasing probabilities splitting Weak/Moderate/Strong by
#'   score quantile. Default `c(1/3, 2/3)`.
#'
#' @return `data` with added `score` (numeric) and `tier` (ordered factor
#'   Weak < Moderate < Strong), sorted by descending score.
#'
#' @examples
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
#' ranked <- rank_hit_compound(params)
#' table(ranked$tier)
#'
#' @seealso [annotate_hit_info()], [plot_hit_stratify_bar()]
#' @export
rank_hit_compound <- function(data, auc_col = "auc", ic50_col = "ic50",
                              emax_col = "emax", cv_col = "cv",
                              rsq_col = "rsq", weights = NULL,
                              probs = c(1 / 3, 2 / 3)) {
  df <- as.data.frame(data, stringsAsFactors = FALSE)
  spec <- list(
    auc = list(col = auc_col, better = "low"),
    ic50 = list(col = ic50_col, better = "low"),
    emax = list(col = emax_col, better = "high"),
    cv = list(col = cv_col, better = "low"),
    rsq = list(col = rsq_col, better = "high")
  )
  score <- rep(0, nrow(df))
  used <- character(0)
  for (nm in names(spec)) {
    col <- spec[[nm]]$col
    if (is.null(col) || !col %in% names(df)) next
    w <- if (!is.null(weights) && col %in% names(weights)) weights[[col]] else 1
    score <- score + w * .metric_score(.as_numeric(df[[col]]),
      better = spec[[nm]]$better)
    used <- c(used, col)
  }
  if (length(used) == 0) {
    stop("None of the requested metric columns are present in `data`.",
      call. = FALSE)
  }
  df$score <- score
  qs <- stats::quantile(score, probs = probs, na.rm = TRUE)
  tier <- ifelse(score >= qs[2], "Strong",
    ifelse(score >= qs[1], "Moderate", "Weak"))
  df$tier <- factor(tier, levels = c("Weak", "Moderate", "Strong"),
    ordered = TRUE)
  df <- df[order(-df$score), , drop = FALSE]
  rownames(df) <- NULL
  attr(df, "score_metrics") <- used
  df
}

#' Annotate hits with compound metadata
#'
#' Joins external compound metadata (name, target, mechanism, library source)
#' onto a hit table.
#'
#' @param hits A hit data frame (for example from [rank_hit_compound()]).
#' @param meta A metadata data frame such as [hts_compound_meta].
#' @param by Join key present in both. Default `"compound_id"`.
#'
#' @return `hits` with the metadata columns merged in (left join), row order
#'   preserved.
#'
#' @examples
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
#' annotate_hit_info(rank_hit_compound(params), hts_compound_meta)
#'
#' @seealso [rank_hit_compound()], [export_hit_table()]
#' @export
annotate_hit_info <- function(hits, meta, by = "compound_id") {
  if (!by %in% names(hits) || !by %in% names(meta)) {
    stop("Join column '", by, "' must be present in both `hits` and `meta`.",
      call. = FALSE)
  }
  hits <- as.data.frame(hits, stringsAsFactors = FALSE)
  hits$.order <- seq_len(nrow(hits))
  out <- merge(hits, meta, by = by, all.x = TRUE, sort = FALSE)
  out <- out[order(out$.order), , drop = FALSE]
  out$.order <- NULL
  rownames(out) <- NULL
  out
}

#' Export a hit summary table
#'
#' Writes a complete hit table (all QC and pharmacology columns) to a CSV file.
#'
#' @param hits A hit data frame.
#' @param file Output path.
#' @param ... Passed to [utils::write.csv()].
#'
#' @return The `file` path, invisibly.
#'
#' @examples
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
#' export_hit_table(rank_hit_compound(params), tempfile(fileext = ".csv"))
#'
#' @seealso [rank_hit_compound()], [annotate_hit_info()]
#' @export
export_hit_table <- function(hits, file, ...) {
  utils::write.csv(as.data.frame(hits), file = file, row.names = FALSE, ...)
  invisible(file)
}

#' Generate an HTML screening report
#'
#' Renders a standard HTML analysis report from the bundled R Markdown template,
#' embedding the supplied QC, dose-response and heatmap results. Requires the
#' suggested \pkg{rmarkdown} package.
#'
#' @param params A named list of objects made available to the report template
#'   (passed as `params`), for example `list(hits = ranked, drc = drc)`.
#' @param output_file Output HTML path. Default a temporary file.
#' @param template Path to an `.Rmd` template. Defaults to the package template
#'   in `inst/rmd/report_template.Rmd`.
#' @param quiet Passed to [rmarkdown::render()]. Default `TRUE`.
#'
#' @return The path to the rendered report, invisibly.
#'
#' @examples
#' \dontrun{
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
#' generate_hts_report(list(hits = rank_hit_compound(params)))
#' }
#'
#' @seealso [export_hit_table()]
#' @export
generate_hts_report <- function(params = list(),
                                output_file = tempfile(fileext = ".html"),
                                template = NULL, quiet = TRUE) {
  if (!requireNamespace("rmarkdown", quietly = TRUE)) {
    stop("Generating reports requires the 'rmarkdown' package.", call. = FALSE)
  }
  if (is.null(template)) {
    template <- system.file("rmd", "report_template.Rmd", package = "diffHTS")
  }
  if (!nzchar(template) || !file.exists(template)) {
    stop("Report template not found.", call. = FALSE)
  }
  rmarkdown::render(template, output_file = output_file,
    params = params, envir = new.env(parent = globalenv()), quiet = quiet)
  invisible(output_file)
}

#' Stratified hit-count bar chart
#'
#' Bar chart of the number of Strong, Moderate and Weak hits.
#'
#' @param ranked A data frame from [rank_hit_compound()] with a `tier` column.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
#' plot_hit_stratify_bar(rank_hit_compound(params))
#'
#' @seealso [rank_hit_compound()]
#' @export
plot_hit_stratify_bar <- function(ranked) {
  if (!"tier" %in% names(ranked)) {
    stop("`ranked` must contain a 'tier' column.", call. = FALSE)
  }
  df <- as.data.frame(table(tier = ranked$tier))
  ggplot2::ggplot(df, ggplot2::aes(x = .data$tier, y = .data$Freq,
    fill = .data$tier)) +
    ggplot2::geom_col(width = 0.65, color = "white", linewidth = 0.3) +
    ggplot2::geom_text(ggplot2::aes(label = .data$Freq), vjust = -0.4,
      color = "#333333", size = 3.6) +
    ggplot2::scale_fill_manual(values = c("Weak" = .hts_col("Weak"),
      "Moderate" = .hts_col("Moderate"), "Strong" = .hts_col("Strong")),
      guide = "none") +
    ggplot2::scale_y_continuous(expand = ggplot2::expansion(mult = c(0, 0.1))) +
    ggplot2::labs(x = NULL, y = "Compounds") +
    theme_hts()
}

#' Primary versus secondary activity correlation
#'
#' Scatter of primary-screen activity against secondary-screen AUC for the same
#' compounds, the standard check that primary hits reproduce in the dose-response
#' confirmation screen.
#'
#' @param primary A data frame with a compound column and a primary-activity
#'   column.
#' @param secondary A data frame with a compound column and a secondary-activity
#'   (AUC) column.
#' @param by Join key. Default `"compound_id"`.
#' @param primary_col Primary-activity column. Default `"activity"`.
#' @param secondary_col Secondary-activity column. Default `"auc"`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' primary <- select_primary_hit(norm)
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' sec <- aggregate(auc ~ compound_id, drc$meta, mean)
#' plot_primary_secondary_cor(primary, sec)
#'
#' @seealso [select_primary_hit()], [calc_drc_auc()]
#' @export
plot_primary_secondary_cor <- function(primary, secondary, by = "compound_id",
                                       primary_col = "activity",
                                       secondary_col = "auc") {
  if (!by %in% names(primary) || !by %in% names(secondary)) {
    stop("Join column '", by, "' must be present in both inputs.",
      call. = FALSE)
  }
  m <- merge(primary[, c(by, primary_col)], secondary[, c(by, secondary_col)],
    by = by)
  m <- m[stats::complete.cases(m), , drop = FALSE]
  r2 <- if (nrow(m) > 2) {
    stats::cor(m[[primary_col]], m[[secondary_col]])^2
  } else {
    NA_real_
  }
  ggplot2::ggplot(m, ggplot2::aes(x = .data[[primary_col]],
    y = .data[[secondary_col]])) +
    ggplot2::geom_point(color = .hts_col("accent"), alpha = 0.8, size = 2.6,
      stroke = 0.2) +
    ggplot2::geom_smooth(method = "lm", formula = y ~ x, se = FALSE,
      color = .hts_col("fail"), linewidth = 0.8) +
    ggplot2::labs(x = "Primary activity", y = "Secondary AUC",
      subtitle = if (is.na(r2)) NULL else sprintf("R^2 = %.3f", r2)) +
    theme_hts()
}

# Module 2: Normalisation and activity calculation ---------------------------

#' Normalise wells to plate controls
#'
#' Converts raw or background-subtracted signals to percent inhibition and
#' percent viability using each plate's own negative (NC) and positive (PC)
#' controls, so that readouts are comparable across plates. Inhibition is
#' \eqn{100 \times (m_{NC} - x) / (m_{NC} - m_{PC})} and viability is its
#' complement relative to the control window.
#'
#' @param hts_raw An `hts_raw` object (typically after [baseline_subtract()]).
#' @param signal_col Name of the signal column to normalise. Default
#'   `"signal"`.
#' @param nc_label,pc_label Well-type labels for the negative and positive
#'   controls. Defaults `"NC"` and `"PC"`.
#'
#' @return An `hts_norm` object: the input data with added numeric columns
#'   `inhibition` (percent) and `viability` (percent). Multiple plates are
#'   normalised independently.
#'
#' @examples
#' raw <- baseline_subtract(read_hts_plate(hts_primary_raw))
#' norm <- norm_by_control(raw)
#' summary(norm$inhibition)
#'
#' @seealso [merge_plate_data()], [plot_plate_heatmap_inhibition()]
#' @export
norm_by_control <- function(hts_raw, signal_col = "signal",
                            nc_label = "NC", pc_label = "PC") {
  if (!signal_col %in% names(hts_raw)) {
    stop("Column '", signal_col, "' not found in `hts_raw`.", call. = FALSE)
  }
  df <- as.data.frame(hts_raw, stringsAsFactors = FALSE)
  df$inhibition <- NA_real_
  df$viability <- NA_real_
  for (p in unique(as.character(df$plate_id))) {
    idx <- which(as.character(df$plate_id) == p)
    sig <- .as_numeric(df[[signal_col]][idx])
    m_nc <- mean(sig[df$well_type[idx] == nc_label], na.rm = TRUE)
    m_pc <- mean(sig[df$well_type[idx] == pc_label], na.rm = TRUE)
    window <- m_nc - m_pc
    if (!is.finite(window) || window == 0) {
      warning("Plate '", p, "' has a degenerate control window; ",
        "normalisation set to NA.", call. = FALSE)
      next
    }
    df$inhibition[idx] <- 100 * (m_nc - sig) / window
    df$viability[idx] <- 100 * (sig - m_pc) / window
  }
  .as_hts(df, c("hts_norm", "hts_raw"))
}

#' Combine normalised plates into one data set
#'
#' Row-binds several normalised plates (or a list of them) into a single
#' `hts_norm` object holding the activity of the whole screen, keeping only the
#' columns shared by all inputs.
#'
#' @param ... One or more `hts_norm`/data-frame objects, or a single list of
#'   them.
#'
#' @return A combined `hts_norm` object.
#'
#' @examples
#' n1 <- norm_by_control(read_hts_plate(
#'   hts_primary_raw[hts_primary_raw$plate_id == "P01", ]))
#' n2 <- norm_by_control(read_hts_plate(
#'   hts_primary_raw[hts_primary_raw$plate_id == "P02", ]))
#' merged <- merge_plate_data(n1, n2)
#' table(merged$plate_id)
#'
#' @seealso [norm_by_control()]
#' @export
merge_plate_data <- function(...) {
  args <- list(...)
  if (length(args) == 1 && is.list(args[[1]]) &&
    !is.data.frame(args[[1]])) {
    args <- args[[1]]
  }
  if (length(args) == 0) {
    stop("No plates supplied to `merge_plate_data()`.", call. = FALSE)
  }
  args <- lapply(args, as.data.frame, stringsAsFactors = FALSE)
  common <- Reduce(intersect, lapply(args, names))
  if (length(common) == 0) {
    stop("The supplied plates share no common columns.", call. = FALSE)
  }
  out <- do.call(rbind, lapply(args, function(d) d[, common, drop = FALSE]))
  rownames(out) <- NULL
  .as_hts(out, c("hts_norm", "hts_raw"))
}

#' Percent-inhibition plate heatmap
#'
#' Draws a plate's normalised percent-inhibition values in plate layout. A thin
#' wrapper around [plot_plate_heatmap()] for `hts_norm` objects.
#'
#' @param hts_norm An `hts_norm` object from [norm_by_control()].
#' @param plate Plate identifier to show. Defaults to the first plate.
#' @param value Name of the value column to colour by. Default `"inhibition"`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' plot_plate_heatmap_inhibition(norm, plate = "P01")
#'
#' @seealso [plot_plate_heatmap()], [plot_inhibition_hist()]
#' @export
plot_plate_heatmap_inhibition <- function(hts_norm, plate = NULL,
                                          value = "inhibition") {
  if (is.null(plate)) plate <- as.character(hts_norm$plate_id)[1]
  df <- as.data.frame(hts_norm)
  plot_plate_heatmap(df, fill = value, well_col = "well",
    plate = plate, plate_col = "plate_id",
    control_col = if ("well_type" %in% names(df)) "well_type" else NULL,
    title = paste0("% ", value, ": ", plate))
}

#' Histogram of compound inhibition on a plate
#'
#' Shows the distribution of percent inhibition across the compound wells of a
#' plate (controls excluded), the standard view for judging overall hit density.
#'
#' @param hts_norm An `hts_norm` object.
#' @param plate Optional plate identifier; if `NULL` (default) all compound
#'   wells are pooled.
#' @param value Name of the value column. Default `"inhibition"`.
#' @param compound_label Well-type label marking compound wells. Default
#'   `"compound"`.
#' @param bins Number of histogram bins. Default `30`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
#' plot_inhibition_hist(norm, plate = "P01")
#'
#' @seealso [norm_by_control()]
#' @export
plot_inhibition_hist <- function(hts_norm, plate = NULL, value = "inhibition",
                                 compound_label = "compound", bins = 30) {
  df <- as.data.frame(hts_norm)
  if ("well_type" %in% names(df)) {
    df <- df[df$well_type == compound_label, , drop = FALSE]
  }
  if (!is.null(plate)) {
    df <- df[as.character(df$plate_id) == plate, , drop = FALSE]
  }
  if (!value %in% names(df) || nrow(df) == 0) {
    stop("No compound '", value, "' values to plot.", call. = FALSE)
  }
  ggplot2::ggplot(df, ggplot2::aes(x = .data[[value]])) +
    ggplot2::geom_histogram(bins = bins, fill = .hts_col("compound"),
      color = "white", linewidth = 0.2, alpha = 0.9) +
    ggplot2::labs(x = paste0("% ", value), y = "Compounds",
      title = if (is.null(plate)) NULL else paste0("Plate ", plate)) +
    theme_hts()
}

#' Select hits by an absolute cut-off across conditions
#'
#' Nominates drugs whose score is below a fixed cut-off in a required number of
#' the supplied score columns. This implements the cut-off based hit-selection
#' strategy used to intersect differential responses across several cancer cell
#' lines.
#'
#' @param data A data frame with one row per drug: a drug identifier column plus
#'   one numeric score column per condition or cell line (for example
#'   `delta_auc` or a z-scored value).
#' @param score_cols Character vector of the score column names to test.
#' @param cutoff Numeric threshold. A drug passes a column when its value is
#'   strictly below `cutoff`. Default `0`.
#' @param min_pass Minimum number of `score_cols` that must be below `cutoff`
#'   for a drug to be selected. Defaults to all of them.
#' @param drug_col Name of the drug identifier column, as a string. Default
#'   `"drug_name"`.
#'
#' @return The subset of `data` (rows) meeting the criterion, with an added
#'   integer column `n_pass` giving the number of score columns below `cutoff`.
#'   Rows containing missing scores in the tested columns are dropped.
#'
#' @examples
#' df <- data.frame(
#'   drug_name = c("DrugA", "DrugB", "DrugC"),
#'   A549 = c(-0.5, 0.2, -0.9),
#'   H1299 = c(-0.3, -0.1, -0.8)
#' )
#' select_hits_cutoff(df, score_cols = c("A549", "H1299"), cutoff = 0,
#'                    min_pass = 2)
#'
#' @seealso [select_hits_sigma()]
#' @export
select_hits_cutoff <- function(data, score_cols, cutoff = 0,
                               min_pass = length(score_cols),
                               drug_col = "drug_name") {
  missing <- setdiff(c(drug_col, score_cols), names(data))
  if (length(missing) > 0) {
    stop(
      "Column(s) not found in `data`: ",
      paste(missing, collapse = ", "), ".",
      call. = FALSE
    )
  }

  mat <- as.matrix(data[, score_cols, drop = FALSE])
  complete <- stats::complete.cases(mat)
  data <- data[complete, , drop = FALSE]
  mat <- mat[complete, , drop = FALSE]

  n_pass <- rowSums(mat < cutoff)
  keep <- n_pass >= min_pass

  out <- data[keep, , drop = FALSE]
  out$n_pass <- n_pass[keep]
  rownames(out) <- NULL
  out
}

#' Select hits by a sigma (z-score) threshold
#'
#' Nominates drugs whose standardised score exceeds a given number of standard
#' deviations from the mean, in the desired direction. This implements the
#' sigma-based (for example 2-sigma or 3-sigma) hit-selection strategy applied
#' to z-scored differential AUC values.
#'
#' @param data A data frame with one row per drug.
#' @param score_col Name of the numeric (typically already z-scored) score
#'   column, as a string. Default `"zscore_delta_auc"`.
#' @param n_sigma Number of standard deviations defining the threshold. Default
#'   `3`.
#' @param direction Either `"less"` (default, select strongly negative scores,
#'   i.e. sensitising drugs) or `"greater"` (select strongly positive scores).
#' @param standardize Logical. If `TRUE`, the score column is z-scored before
#'   thresholding; if `FALSE` (default) the column is assumed to already be on a
#'   standardised (sigma) scale and is thresholded directly.
#' @param drug_col Name of the drug identifier column, as a string. Default
#'   `"drug_name"`.
#'
#' @return The subset of `data` whose score is beyond the sigma threshold in the
#'   requested direction, ordered by score.
#'
#' @examples
#' df <- data.frame(
#'   drug_name = paste0("Drug", 1:6),
#'   zscore_delta_auc = c(-3.4, -1.2, 0.1, 0.5, 1.1, -2.8)
#' )
#' select_hits_sigma(df, n_sigma = 2, direction = "less")
#'
#' @seealso [select_hits_cutoff()], [compute_delta_auc()]
#' @export
select_hits_sigma <- function(data, score_col = "zscore_delta_auc",
                              n_sigma = 3, direction = c("less", "greater"),
                              standardize = FALSE, drug_col = "drug_name") {
  direction <- match.arg(direction)
  if (!score_col %in% names(data)) {
    stop("Column '", score_col, "' not found in `data`.", call. = FALSE)
  }

  x <- data[[score_col]]
  score <- if (standardize) {
    (x - mean(x, na.rm = TRUE)) / stats::sd(x, na.rm = TRUE)
  } else {
    x
  }

  keep <- if (direction == "less") score <= -n_sigma else score >= n_sigma
  keep[is.na(keep)] <- FALSE

  out <- data[keep, , drop = FALSE]
  ord <- order(out[[score_col]], decreasing = (direction == "greater"))
  out <- out[ord, , drop = FALSE]
  rownames(out) <- NULL
  out
}

test_that("select_hits_cutoff selects by count of passing columns", {
  df <- data.frame(
    drug_name = c("DrugA", "DrugB", "DrugC"),
    A549 = c(-0.5, 0.2, -0.9),
    H1299 = c(-0.3, -0.1, -0.8)
  )
  out <- select_hits_cutoff(df, score_cols = c("A549", "H1299"),
    cutoff = 0, min_pass = 2)

  expect_equal(out$drug_name, c("DrugA", "DrugC"))
  expect_equal(out$n_pass, c(2L, 2L))
})

test_that("select_hits_cutoff drops rows with missing scores", {
  df <- data.frame(
    drug_name = c("DrugA", "DrugB"),
    A549 = c(-0.5, NA),
    H1299 = c(-0.3, -0.8)
  )
  out <- select_hits_cutoff(df, score_cols = c("A549", "H1299"),
    cutoff = 0, min_pass = 2)
  expect_equal(out$drug_name, "DrugA")
})

test_that("select_hits_cutoff errors on unknown columns", {
  df <- data.frame(drug_name = "D1", A549 = -0.5)
  expect_error(
    select_hits_cutoff(df, score_cols = c("A549", "MISSING")),
    "not found"
  )
})

test_that("select_hits_sigma selects in a direction", {
  df <- data.frame(
    drug_name = paste0("Drug", 1:6),
    zscore_delta_auc = c(-3.4, -1.2, 0.1, 0.5, 1.1, -2.8)
  )
  less <- select_hits_sigma(df, n_sigma = 2, direction = "less")
  expect_equal(less$drug_name, c("Drug1", "Drug6"))

  greater <- select_hits_sigma(df, n_sigma = 2, direction = "greater")
  expect_equal(nrow(greater), 0)
})

test_that("select_hits_sigma can standardise raw scores", {
  # one strong outlier among five near-zero values crosses the 2-sigma threshold
  df <- data.frame(
    drug_name = paste0("D", 1:6),
    score = c(-100, 0, 0, 0, 0, 0)
  )
  out <- select_hits_sigma(df, score_col = "score", n_sigma = 2,
    direction = "less", standardize = TRUE)
  expect_equal(out$drug_name, "D1")
})

#' Example two-condition dose-response screen
#'
#' A small, simulated high-throughput drug-screening dataset in long (tidy)
#' format, mimicking a two-condition experiment in which each drug is tested
#' across a concentration series under a baseline (`"Gy0"`) and a treatment
#' (`"Gy2"`) condition, alongside negative and positive control wells. It is
#' sized to run the package examples and vignette quickly.
#'
#' @format A data frame with one row per well and the following columns:
#' \describe{
#'   \item{plateID}{Plate identifier (character).}
#'   \item{drug_name}{Drug identifier, or a control label such as `"DMSO"`
#'     (negative) or `"kill_ctrl"` (positive).}
#'   \item{experiment_type}{Well role: `"sample"`, `"negative_ctrl"` or
#'     `"positive_cttl"`.}
#'   \item{condition}{Experimental condition, `"Gy0"` (baseline) or `"Gy2"`
#'     (treatment).}
#'   \item{concentration}{Drug concentration (micromolar).}
#'   \item{normalized_cell_count}{Viability normalised to the negative control
#'     (roughly 0-1).}
#' }
#'
#' @details The data are simulated (see `data-raw/make_datasets.R`) and do not
#'   correspond to any real compound or cell line.
#'
#' @seealso [screen_delta_auc]
#' @keywords datasets
"screen_doseresponse"

#' Example differential-AUC hit table
#'
#' A small, simulated table of differential (delta) AUC values with one row per
#' drug and one column per cell line, used to demonstrate hit selection and
#' heatmap/scatter visualisation across several cancer cell lines and a normal
#' (`MRC5`) cell line.
#'
#' @format A data frame with one row per drug and the following columns:
#' \describe{
#'   \item{drug_name}{Drug identifier (character).}
#'   \item{A549}{Delta AUC in the A549 lung cancer cell line (numeric).}
#'   \item{H1299}{Delta AUC in the H1299 lung cancer cell line (numeric).}
#'   \item{H1975}{Delta AUC in the H1975 lung cancer cell line (numeric).}
#'   \item{H2030}{Delta AUC in the H2030 lung cancer cell line (numeric).}
#'   \item{MRC5}{Delta AUC in the MRC5 normal lung fibroblast cell line
#'     (numeric).}
#' }
#'
#' @details The data are simulated (see `data-raw/make_datasets.R`) and do not
#'   correspond to any real compound. Negative values indicate treatment-induced
#'   sensitisation.
#'
#' @seealso [screen_doseresponse]
#' @keywords datasets
"screen_delta_auc"

#' Example per-plate quality-control summary
#'
#' A small, simulated per-plate quality-control (QC) table spanning several
#' experiments (cell lines), each with a few plates. Every plate carries its
#' negative- and positive-control replicate readouts together with the QC
#' metrics derived from them, so the table can drive the control scatter, the
#' circular control heatmap and the circular Z'-factor plot. A couple of plates
#' are deliberately noisy so that passing and failing plates are both present.
#'
#' @format A data frame with one row per plate and the following columns:
#' \describe{
#'   \item{plateID}{Plate identifier such as `"EXP76_01"` (character).}
#'   \item{experimentID}{Experiment identifier (character), the plate prefix.}
#'   \item{cell_line}{Cell line assayed on the experiment (character).}
#'   \item{plateNumber}{Plate order within the experiment (integer).}
#'   \item{z_factor}{Assay Z-factor from sample and negative-control wells.}
#'   \item{z_prime}{Z-prime factor from negative- and positive-control wells.}
#'   \item{sb_value}{Signal-to-background ratio.}
#'   \item{sn_value}{Signal-to-noise value.}
#'   \item{ssmd_value}{Strictly standardised mean difference.}
#'   \item{cv_negative_ctrl}{Coefficient of variation of the negative controls
#'     (percent).}
#'   \item{neg_ctrl_1, neg_ctrl_2, neg_ctrl_3, neg_ctrl_4, neg_ctrl_5,
#'     neg_ctrl_6}{Negative-control well readouts.}
#'   \item{pos_ctrl_1, pos_ctrl_2, pos_ctrl_3, pos_ctrl_4, pos_ctrl_5,
#'     pos_ctrl_6}{Positive-control well readouts.}
#' }
#'
#' @details The data are simulated (see `data-raw/make_datasets.R`) and do not
#'   correspond to any real compound or cell line.
#'
#' @seealso [plot_control_scatter()], [plot_qc_metric_circular()],
#'   [plot_qc_circular_heatmap()]
#' @keywords datasets
"screen_plate_qc"

#' Example 96-well plate layout with well positions
#'
#' A small, simulated 96-well plate (8 rows by 12 columns) assayed under two
#' conditions (`"Gy0"` and `"Gy2"`), in long (one row per well) format. The
#' layout follows good HTS practice: the outer ring (row A, row H, columns 1 and
#' 12) is an empty evaporation buffer, the two flanking columns (2 and 11) carry
#' the positive (kill) and negative (DMSO) controls in a rotationally balanced
#' diagonal pattern, and the interior block (rows B-G, columns 3-10) holds an
#' 8-point dose series of one drug per row. It is used to demonstrate the
#' plate-layout heatmap [plot_plate_heatmap()].
#'
#' @format A data frame with one row per well and the following columns:
#' \describe{
#'   \item{plateID}{Plate identifier such as `"EXP99_Gy0"` (character).}
#'   \item{condition}{Experimental condition, `"Gy0"` or `"Gy2"`.}
#'   \item{well}{Well identifier such as `"A1"` (character).}
#'   \item{plate_rows}{Plate row letter, `"A"`-`"H"` (character).}
#'   \item{plate_columns}{Plate column number, `1`-`12` (integer).}
#'   \item{drug_name}{Drug identifier, or a control label (`"DMSO"`,
#'     `"kill_ctrl"`), or `NA` for empty buffer wells.}
#'   \item{experiment_type}{Well role: `"sample"`, `"negative_ctrl"`,
#'     `"positive_ctrl"` or `"empty"` (evaporation-buffer ring).}
#'   \item{concentration}{Drug concentration (micromolar), `NA` for controls
#'     and empty wells.}
#'   \item{normalized_cell_count}{Viability normalised to the negative control;
#'     empty buffer wells hold no live cells and read near the kill control.}
#' }
#'
#' @details The data are simulated (see `data-raw/make_datasets.R`) and do not
#'   correspond to any real compound or cell line.
#'
#' @seealso [plot_plate_heatmap()]
#' @keywords datasets
"screen_plate_layout"

#' Example single-concentration primary screen (raw signals)
#'
#' A small, simulated primary high-throughput screen in 96-well plate format
#' with raw readout signals, used to demonstrate the pre-QC, normalisation,
#' replicate and primary-hit modules. The layout follows good HTS practice: the
#' outer ring of wells (row A, row H and columns 1 and 12) is a compound-free
#' evaporation buffer stored as `"Blank"` wells, the two inset columns 2 and 11
#' carry the controls in a rotationally balanced *diagonal* pattern (positive
#' controls top-left/bottom-right, negative controls top-right/bottom-left), and
#' the 48 compounds fill the interior block (rows B-G, columns 3-10). Plates
#' `"P01"` and `"P02"` are good replicates (Z-prime around 0.8) while `"P03"` is
#' a failed plate.
#'
#' @format A data frame with one row per well and the following columns:
#' \describe{
#'   \item{plate_id}{Plate identifier, `"P01"`, `"P02"` or `"P03"` (character).}
#'   \item{well}{Well identifier such as `"A1"` (character).}
#'   \item{row}{Plate row letter, `"A"`-`"H"` (character).}
#'   \item{col}{Plate column number, `1`-`12` (integer).}
#'   \item{well_type}{Well role: `"NC"` (negative), `"PC"` (positive),
#'     `"Blank"` or `"compound"` (character).}
#'   \item{compound_id}{Compound identifier such as `"CPD001"`, `NA` for
#'     control and blank wells (character).}
#'   \item{concentration}{Compound concentration (micromolar), `NA` for
#'     controls (numeric).}
#'   \item{signal}{Raw instrument signal (numeric).}
#' }
#'
#' @details The data are simulated (see `data-raw/make_datasets.R`) and do not
#'   correspond to any real compound or cell line.
"hts_primary_raw"

#' Example single 384-well screening plate (realistic layout)
#'
#' A single simulated 384-well plate (`"EXP87P01"`) whose layout mirrors a real
#' cell-based high-throughput screen (see `EXP87/`). The two outermost rings of
#' wells (rows A, B, O, P and columns 1, 2, 23, 24) are kept free of compound to
#' avoid edge-evaporation artefacts. They still hold medium but no live cells,
#' so the instrument reads them at the same low, "dead" level as the kill
#' control; they are stored as `"edgewell"` wells (not omitted). The controls
#' sit inset from the true plate edge in the two innermost flanking columns
#' (3 and 22), arranged in a rotationally balanced *diagonal* pattern: column 3
#' holds the positive (kill) control in its top half (rows C-H) and the negative
#' (DMSO) control in its bottom half (rows I-N), while column 22 mirrors it
#' (negative top, positive bottom). This diagonal split averages out
#' spatial/edge signal gradients. Columns 4-21 across the interior rows C-N hold
#' the test compounds. Used to demonstrate plate-layout heatmaps with control
#' circling on a 384-well plate.
#'
#' @format A data frame with one row per well (all 384) and the following
#'   columns:
#' \describe{
#'   \item{plate_id}{Plate identifier, `"EXP87P01"` (character).}
#'   \item{well}{Well address such as `"C03"` (character).}
#'   \item{row}{Plate row letter, `"A"`-`"P"` (character).}
#'   \item{col}{Plate column number, `1`-`24` (integer).}
#'   \item{well_type}{Well role: `"PC"` (positive control), `"NC"` (negative
#'     control), `"compound"` or `"edgewell"` (empty buffer) (character).}
#'   \item{compound_id}{Compound identifier for test wells, `NA` for controls
#'     and edge wells (character).}
#'   \item{concentration}{Compound concentration (micromolar); `NA` for
#'     controls and edge wells (numeric).}
#'   \item{signal}{Raw luminescence read-out (numeric).}
#' }
#'
#' @details The data are simulated (see `data-raw/make_datasets.R`) and do not
#'   correspond to any real compound. All 384 wells are stored; the empty edge
#'   wells carry only a low background/no-cell reading.
"hts_plate_384"

#' Example gradient dose-response secondary screen
#'
#' A small, simulated dose-response confirmation screen: ten compounds tested
#' across a concentration gradient in three cell lines with two replicates each.
#' Used to demonstrate the dose-response fitting, AUC and clustering modules.
#'
#' @format A data frame with one row per measurement and the following columns:
#' \describe{
#'   \item{compound_id}{Compound identifier such as `"CPD001"` (character).}
#'   \item{cell_line}{Cell line, `"A549"`, `"H1299"` or `"MRC5"` (character).}
#'   \item{concentration}{Compound concentration (micromolar) (numeric).}
#'   \item{replicate}{Technical replicate index, `1` or `2` (integer).}
#'   \item{viability}{Percent viability relative to vehicle control (numeric).}
#' }
#'
#' @details The data are simulated (see `data-raw/make_datasets.R`); the cell
#'   line `"MRC5"` is made comparatively resistant.
"hts_dose_response"

#' Example compound metadata
#'
#' A small, simulated compound annotation table used to demonstrate hit
#' annotation. Keyed by `compound_id` to join onto screening results.
#'
#' @format A data frame with one row per compound and the following columns:
#' \describe{
#'   \item{compound_id}{Compound identifier such as `"CPD001"` (character).}
#'   \item{compound_name}{Human-readable compound name (character).}
#'   \item{target}{Nominal molecular target (character).}
#'   \item{moa}{Mechanism of action (character).}
#'   \item{library_source}{Source library the compound came from (character).}
#' }
#'
#' @details The data are simulated (see `data-raw/make_datasets.R`) and do not
#'   correspond to any real compound.
"hts_compound_meta"

# Publication-ready theming and palettes for diffHTS figures -------------------

# Okabe-Ito colourblind-safe qualitative palette (8 hues).
.hts_okabe_ito <- c(
  blue = "#0072B2", vermillion = "#D55E00", green = "#009E73",
  purple = "#CC79A7", orange = "#E69F00", skyblue = "#56B4E9",
  yellow = "#F0E442", grey = "#999999"
)

# Semantic colours shared across diffHTS plots: low-saturation and
# colourblind-safe, never pure primaries or rainbow.
.hts_semantic <- c(
  pass = "#0072B2", fail = "#D55E00",
  NC = "#0072B2", PC = "#D55E00", compound = "#3C3C3C", Blank = "#E1B000",
  hit = "#D55E00", nonhit = "#B4B4B4",
  Weak = "#B4B4B4", Moderate = "#E69F00", Strong = "#D55E00",
  accent = "#0072B2", reference = "#666666", neutral = "#4C72B0"
)

# Look up one or more semantic colours by name.
.hts_col <- function(...) {
  keys <- c(...)
  unname(.hts_semantic[keys])
}

# Compact numeric labels for direct on-plot annotation.
.fmt_qc <- function(x) {
  x <- .as_numeric(x)
  out <- ifelse(abs(x) >= 100, formatC(x, format = "d"),
    formatC(signif(x, 3), format = "g"))
  out[!is.finite(x)] <- ""
  out
}

#' Colourblind-safe qualitative palette for diffHTS
#'
#' Returns the package's low-saturation, colourblind-safe qualitative palette
#' (an Okabe-Ito ordering). Use it whenever a plot maps a discrete variable to
#' colour so that figures stay consistent and accessible.
#'
#' @param n Number of colours to return (recycled if `n` exceeds the eight
#'   base hues). If `NULL` (default) the full palette is returned.
#'
#' @return A character vector of hex colours.
#'
#' @examples
#' hts_pal(3)
#'
#' @seealso [theme_hts()]
#' @export
hts_pal <- function(n = NULL) {
  pal <- unname(.hts_okabe_ito)
  if (is.null(n)) {
    return(pal)
  }
  rep(pal, length.out = n)
}

#' Modern academic ggplot2 theme for diffHTS figures
#'
#' A minimal, publication-ready theme: pure-white background, an L-shaped axis
#' (left and bottom only), light dashed horizontal major grid lines with no
#' minor grid, restrained dark-grey typography and generous margins. Pair it
#' with [hts_pal()] for discrete colours or a `viridis`/muted-blue gradient for
#' continuous ones.
#'
#' @param base_size Base font size in points. Default `12`.
#' @param base_family Base font family; defaults to the device sans-serif.
#' @param legend Legend position passed to `ggplot2::theme()`. Default
#'   `"bottom"`.
#' @param grid Which major grid lines to keep: `"y"` (default), `"x"`, `"both"`
#'   or `"none"`.
#'
#' @return A [ggplot2::theme] object that can be added to any ggplot.
#'
#' @examples
#' library(ggplot2)
#' ggplot(mtcars, aes(factor(cyl), mpg)) +
#'   geom_boxplot() +
#'   theme_hts()
#'
#' @seealso [hts_pal()], [plot_plate_qc()]
#' @export
theme_hts <- function(base_size = 12, base_family = "", legend = "bottom",
                      grid = c("y", "x", "both", "none")) {
  grid <- match.arg(grid)
  half <- base_size / 2
  grid_line <- ggplot2::element_line(color = "#E5E5E5", linewidth = 0.3,
    linetype = "dashed")
  th <- ggplot2::theme_minimal(base_size = base_size,
    base_family = base_family) +
    ggplot2::theme(
      text = ggplot2::element_text(color = "#333333", lineheight = 1.2),
      plot.background = ggplot2::element_rect(fill = "#FFFFFF", color = NA),
      panel.background = ggplot2::element_rect(fill = "#FFFFFF", color = NA),
      panel.grid.major = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank(),
      axis.line = ggplot2::element_line(color = "#888888", linewidth = 0.6,
        lineend = "round"),
      axis.ticks = ggplot2::element_line(color = "#888888", linewidth = 0.4),
      plot.title = ggplot2::element_text(size = ggplot2::rel(1.25),
        face = "bold", color = "#222222",
        margin = ggplot2::margin(b = half)),
      plot.subtitle = ggplot2::element_text(size = ggplot2::rel(1),
        color = "#555555", margin = ggplot2::margin(b = half)),
      plot.caption = ggplot2::element_text(size = ggplot2::rel(0.8),
        color = "#777777"),
      axis.title = ggplot2::element_text(size = ggplot2::rel(1.02),
        color = "#333333"),
      axis.title.x = ggplot2::element_text(margin = ggplot2::margin(t = half)),
      axis.title.y = ggplot2::element_text(margin = ggplot2::margin(r = half)),
      axis.text = ggplot2::element_text(size = ggplot2::rel(0.88),
        color = "#555555"),
      legend.position = legend,
      legend.title = ggplot2::element_text(size = ggplot2::rel(0.88),
        color = "#333333"),
      legend.text = ggplot2::element_text(size = ggplot2::rel(0.88),
        color = "#555555"),
      legend.key = ggplot2::element_blank(),
      legend.background = ggplot2::element_blank(),
      plot.margin = ggplot2::margin(16, 16, 16, 16)
    )
  if (grid %in% c("y", "both")) {
    th <- th + ggplot2::theme(panel.grid.major.y = grid_line)
  }
  if (grid %in% c("x", "both")) {
    th <- th + ggplot2::theme(panel.grid.major.x = grid_line)
  }
  th
}

# Module 5: Dose response curve fitting and AUC ------------------------------

# Build a stable per-curve key from the compound and any grouping columns.
.curve_keys <- function(df, compound_col, group_cols) {
  keys <- c(compound_col, group_cols)
  parts <- lapply(keys, function(k) as.character(df[[k]]))
  do.call(paste, c(parts, sep = " | "))
}

#' Import gradient dose-response data
#'
#' Standardises a secondary (dose-response) screening table into the columns
#' expected by the fitting engine, matching each compound to its concentration
#' gradient and response, ready for [fit_4pl_curve()].
#'
#' @param data A data frame, or a path to a csv/txt/xlsx file.
#' @param compound_col,concentration_col,response_col Names of the compound,
#'   concentration and response columns in `data`.
#' @param group_cols Optional grouping columns kept alongside each curve (for
#'   example `"cell_line"`).
#'
#' @return An `hts_dose` object (data frame) with columns `compound_id`,
#'   `concentration`, `response`, any `group_cols`, and canonical class.
#'
#' @examples
#' dr <- import_dose_response(hts_dose_response, response_col = "viability",
#'   group_cols = "cell_line")
#' head(dr)
#'
#' @seealso [fit_4pl_curve()]
#' @export
import_dose_response <- function(data, compound_col = "compound_id",
                                 concentration_col = "concentration",
                                 response_col = "viability",
                                 group_cols = NULL) {
  if (is.character(data) && length(data) == 1) {
    data <- .read_hts_file(data)
  }
  df <- as.data.frame(data, stringsAsFactors = FALSE)
  for (col in c(compound_col, concentration_col, response_col, group_cols)) {
    if (!col %in% names(df)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }
  out <- data.frame(
    compound_id = as.character(df[[compound_col]]),
    concentration = .as_numeric(df[[concentration_col]]),
    response = .as_numeric(df[[response_col]]),
    stringsAsFactors = FALSE
  )
  for (g in group_cols) out[[g]] <- df[[g]]
  .as_hts(out, "hts_dose")
}

#' Fit four-parameter logistic curves per compound
#'
#' Fits a 4PL dose-response model to every compound (optionally split by
#' grouping columns such as cell line), reusing the package's self-contained
#' fitter, and records the goodness of fit \eqn{R^2}. No external fitting
#' package is required.
#'
#' @param data An `hts_dose` object from [import_dose_response()] (or a long
#'   data frame with `compound_id`, `concentration`, `response`).
#' @param response,concentration,compound_col Column names. Default
#'   `"response"`, `"concentration"`, `"compound_id"`.
#' @param group_cols Optional grouping columns.
#' @param rsq_min Curves with \eqn{R^2} below this are flagged (not dropped
#'   here; use [filter_low_quality_curve()]). Default `0.85`.
#'
#' @return An `hts_drc` object: a list with `meta` (one row per curve: keys,
#'   `n`, `rsq`, `converged`, `pass`), `fits` (named list of `dsr_4pl` models)
#'   and `data` (the input). `group_cols` is stored as an attribute.
#'
#' @examples
#' dr <- import_dose_response(hts_dose_response, response_col = "viability",
#'   group_cols = "cell_line")
#' drc <- fit_4pl_curve(dr, group_cols = "cell_line")
#' head(drc$meta)
#'
#' @seealso [extract_drc_params()], [calc_drc_auc()], [plot_single_drc()]
#' @export
fit_4pl_curve <- function(data, response = "response",
                          concentration = "concentration",
                          compound_col = "compound_id", group_cols = NULL,
                          rsq_min = 0.85) {
  df <- as.data.frame(data, stringsAsFactors = FALSE)
  for (col in c(response, concentration, compound_col, group_cols)) {
    if (!col %in% names(df)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }
  df$.curve <- .curve_keys(df, compound_col, group_cols)
  keys <- unique(df$.curve)

  fits <- list()
  meta_rows <- list()
  for (k in keys) {
    sub <- df[df$.curve == k, , drop = FALSE]
    x <- .as_numeric(sub[[concentration]])
    y <- .as_numeric(sub[[response]])
    ok <- is.finite(x) & is.finite(y)
    x <- x[ok]
    y <- y[ok]
    row <- data.frame(curve_id = k,
      compound_id = as.character(sub[[compound_col]][1]),
      stringsAsFactors = FALSE)
    for (g in group_cols) row[[g]] <- sub[[g]][1]
    row$n <- length(x)
    if (length(x) < 4 || length(unique(x)) < 4) {
      row$rsq <- NA_real_
      row$converged <- FALSE
      fits[[k]] <- NULL
    } else {
      fit <- tryCatch(fit_dose_response(
        data.frame(response = y, concentration = x),
        "response", "concentration"), error = function(e) NULL)
      if (is.null(fit)) {
        row$rsq <- NA_real_
        row$converged <- FALSE
      } else {
        sst <- sum((y - mean(y))^2)
        rsq <- if (sst > 0) 1 - fit$sse / sst else NA_real_
        row$rsq <- rsq
        row$converged <- isTRUE(fit$convergence)
        fits[[k]] <- fit
      }
    }
    meta_rows[[k]] <- row
  }
  meta <- do.call(rbind, meta_rows)
  meta$pass <- is.finite(meta$rsq) & meta$rsq >= rsq_min
  rownames(meta) <- NULL

  structure(list(meta = meta, fits = fits,
    data = .as_hts(df[, setdiff(names(df), ".curve"), drop = FALSE],
      "hts_dose")),
    group_cols = group_cols, rsq_min = rsq_min, class = "hts_drc")
}

#' Extract dose-response parameters
#'
#' Pulls the fitted potency and efficacy parameters from every curve of an
#' `hts_drc` object.
#'
#' @param drc An `hts_drc` object from [fit_4pl_curve()].
#'
#' @return A data frame with one row per curve: keys plus `ic50`, `emax` (`top`),
#'   `bottom`, `hill`, `rsq` and `pass`. Curves that failed to fit yield `NA`
#'   parameters.
#'
#' @examples
#' drc <- fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line")
#' head(extract_drc_params(drc))
#'
#' @seealso [fit_4pl_curve()], [calc_drc_auc()]
#' @export
extract_drc_params <- function(drc) {
  stopifnot(inherits(drc, "hts_drc"))
  meta <- drc$meta
  params <- lapply(meta$curve_id, function(k) {
    fit <- drc$fits[[k]]
    if (is.null(fit)) {
      return(data.frame(ic50 = NA_real_, emax = NA_real_, bottom = NA_real_,
        hill = NA_real_))
    }
    cf <- stats::coef(fit)
    data.frame(ic50 = cf[["ic50"]], emax = cf[["top"]], bottom = cf[["bottom"]],
      hill = cf[["hill"]])
  })
  out <- cbind(meta, do.call(rbind, params))
  out[, c(setdiff(names(meta), c("converged", "pass")),
    "ic50", "emax", "bottom", "hill", "pass")]
}

#' Area under the dose-response curve
#'
#' Computes the area under each fitted curve by trapezoidal integration of the
#' fitted 4PL over the tested concentration range (on the log10 scale), a
#' robust single-number summary of overall potency.
#'
#' @param drc An `hts_drc` object from [fit_4pl_curve()].
#'
#' @return The `hts_drc` object with an `auc` column added to its `meta`, also
#'   returned invisibly as a data frame via `drc$meta`.
#'
#' @examples
#' drc <- fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line")
#' calc_drc_auc(drc)$meta
#'
#' @seealso [compute_auc()], [extract_drc_params()]
#' @export
calc_drc_auc <- function(drc) {
  stopifnot(inherits(drc, "hts_drc"))
  auc <- vapply(drc$meta$curve_id, function(k) {
    fit <- drc$fits[[k]]
    if (is.null(fit)) return(NA_real_)
    x <- fit$data$concentration
    x <- x[x > 0]
    if (length(x) < 2) return(NA_real_)
    lx <- log10(range(x))
    grid_l <- seq(lx[1], lx[2], length.out = 200)
    grid <- 10^grid_l
    yhat <- stats::predict(fit, newdata = grid)
    # Trapezoid over the log10-concentration axis.
    sum(diff(grid_l) * (utils::head(yhat, -1) + utils::tail(yhat, -1)) / 2)
  }, numeric(1))
  drc$meta$auc <- unname(auc)
  drc
}

#' Drop low-quality dose-response curves
#'
#' Removes curves that failed to fit or whose \eqn{R^2} falls below the
#' threshold, leaving only trustworthy dose-response models.
#'
#' @param drc An `hts_drc` object.
#' @param rsq_min Minimum acceptable \eqn{R^2}. Defaults to the value stored at
#'   fit time.
#'
#' @return The filtered `hts_drc` object; removed curve ids are stored in
#'   `attr(drc, "removed")`.
#'
#' @examples
#' drc <- fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line")
#' clean <- filter_low_quality_curve(drc)
#' nrow(clean$meta)
#'
#' @seealso [fit_4pl_curve()]
#' @export
filter_low_quality_curve <- function(drc, rsq_min = NULL) {
  stopifnot(inherits(drc, "hts_drc"))
  if (is.null(rsq_min)) rsq_min <- attr(drc, "rsq_min")
  if (is.null(rsq_min)) rsq_min <- 0.85
  keep <- is.finite(drc$meta$rsq) & drc$meta$rsq >= rsq_min
  removed <- drc$meta$curve_id[!keep]
  drc$meta <- drc$meta[keep, , drop = FALSE]
  rownames(drc$meta) <- NULL
  drc$fits <- drc$fits[drc$meta$curve_id]
  attr(drc, "removed") <- removed
  drc
}

# Assemble points + fitted line for one curve.
.drc_curve_frame <- function(drc, curve_id, n_points = 200) {
  fit <- drc$fits[[curve_id]]
  if (is.null(fit)) return(NULL)
  obs <- fit$data
  x <- obs$concentration[obs$concentration > 0]
  grid <- 10^seq(log10(min(x)), log10(max(x)), length.out = n_points)
  list(
    obs = obs[obs$concentration > 0, , drop = FALSE],
    fit = data.frame(concentration = grid,
      response = stats::predict(fit, newdata = grid))
  )
}

#' Plot a single dose-response curve
#'
#' Plots one compound's observed points and fitted 4PL curve on a log10
#' concentration axis, annotating IC50, AUC and \eqn{R^2}.
#'
#' @param drc An `hts_drc` object (ideally after [calc_drc_auc()]).
#' @param curve_id Curve identifier (from `drc$meta$curve_id`). Defaults to the
#'   first curve.
#' @param color Curve colour. Default `"#0072B2"`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' plot_single_drc(drc, drc$meta$curve_id[1])
#'
#' @seealso [plot_batch_drc_overlay()]
#' @export
plot_single_drc <- function(drc, curve_id = NULL, color = "#0072B2") {
  stopifnot(inherits(drc, "hts_drc"))
  if (is.null(curve_id)) curve_id <- drc$meta$curve_id[1]
  cf <- .drc_curve_frame(drc, curve_id)
  if (is.null(cf)) stop("No fitted model for curve '", curve_id, "'.",
    call. = FALSE)
  meta <- drc$meta[drc$meta$curve_id == curve_id, , drop = FALSE]
  fit <- drc$fits[[curve_id]]
  ic50 <- stats::coef(fit)[["ic50"]]
  auc <- if ("auc" %in% names(meta)) meta$auc[1] else NA_real_
  rsq <- meta$rsq[1]
  subtitle <- sprintf("IC50: %.3g   AUC: %s   R^2: %.3f", ic50,
    if (is.na(auc)) "NA" else sprintf("%.2f", auc), rsq)

  p <- ggplot2::ggplot() +
    ggplot2::geom_point(data = cf$obs,
      ggplot2::aes(x = .data$concentration, y = .data$response),
      color = color, size = 2.5, alpha = 0.85, stroke = 0.2) +
    ggplot2::geom_line(data = cf$fit,
      ggplot2::aes(x = .data$concentration, y = .data$response),
      color = color, linewidth = 1.1, lineend = "round") +
    ggplot2::geom_vline(xintercept = ic50, linetype = "dashed",
      color = .hts_col("reference"), linewidth = 0.6) +
    ggplot2::scale_x_log10() +
    ggplot2::labs(x = "Concentration", y = "Response",
      title = curve_id, subtitle = subtitle) +
    theme_hts()
  p
}

#' Overlay multiple dose-response curves
#'
#' Overlays the fitted curves of several compounds for side-by-side comparison
#' of activity.
#'
#' @param drc An `hts_drc` object.
#' @param curve_ids Curve identifiers to overlay. Defaults to all fitted curves.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' drc <- fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line")
#' plot_batch_drc_overlay(drc, drc$meta$curve_id[1:4])
#'
#' @seealso [plot_single_drc()]
#' @export
plot_batch_drc_overlay <- function(drc, curve_ids = NULL) {
  stopifnot(inherits(drc, "hts_drc"))
  if (is.null(curve_ids)) curve_ids <- names(drc$fits)
  curve_ids <- curve_ids[!vapply(drc$fits[curve_ids], is.null, logical(1))]
  if (length(curve_ids) == 0) {
    stop("No fitted curves available to overlay.", call. = FALSE)
  }
  parts <- lapply(curve_ids, function(k) {
    cf <- .drc_curve_frame(drc, k)
    if (is.null(cf)) return(NULL)
    cf$fit$curve <- k
    cf$fit
  })
  line_df <- do.call(rbind, parts)
  ggplot2::ggplot(line_df, ggplot2::aes(x = .data$concentration,
    y = .data$response, color = .data$curve)) +
    ggplot2::geom_line(linewidth = 1, lineend = "round") +
    ggplot2::scale_x_log10() +
    ggplot2::scale_color_viridis_d(name = NULL, option = "D", end = 0.9) +
    ggplot2::labs(x = "Concentration", y = "Response") +
    theme_hts(legend = "right")
}

#' IC50 versus AUC scatter
#'
#' Scatter plot of IC50 against AUC across curves, a compact view for triaging
#' strong, moderate and weak compounds. IC50 marks the potency (the
#' concentration for half-maximal effect) while AUC integrates potency *and*
#' efficacy over the whole tested range; plotting them together shows whether the
#' two agree and flags compounds that are potent but only weakly efficacious (or
#' vice versa). A regression trend line and the Pearson correlation (computed on
#' the log10 IC50, matching the log x-axis) are drawn on the figure by default.
#'
#' @param params A data frame from [extract_drc_params()] joined with an `auc`
#'   column (for example `merge(extract_drc_params(drc), calc_drc_auc(drc)$meta)`),
#'   or the `meta` of an `hts_drc` after [calc_drc_auc()] plus IC50.
#' @param auc_col,ic50_col Column names. Default `"auc"`, `"ic50"`.
#' @param add_trend Add a linear regression trend line with a 95% confidence
#'   band. Default `TRUE`.
#' @param annotate_cor Annotate the Pearson correlation coefficient and p-value
#'   on the plot. Default `TRUE`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @examples
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
#' plot_ic50_auc_cor(params)
#'
#' @seealso [extract_drc_params()], [calc_drc_auc()]
#' @export
plot_ic50_auc_cor <- function(params, auc_col = "auc", ic50_col = "ic50",
                              add_trend = TRUE, annotate_cor = TRUE) {
  for (col in c(auc_col, ic50_col)) {
    if (!col %in% names(params)) {
      stop("Column '", col, "' not found in `params`.", call. = FALSE)
    }
  }
  df <- params[is.finite(params[[auc_col]]) & is.finite(params[[ic50_col]]) &
    params[[ic50_col]] > 0, , drop = FALSE]
  p <- ggplot2::ggplot(df,
    ggplot2::aes(x = .data[[ic50_col]], y = .data[[auc_col]]))
  if (isTRUE(add_trend) && nrow(df) > 2) {
    p <- p + ggplot2::geom_smooth(method = "lm", formula = y ~ x, se = TRUE,
      color = .hts_col("reference"), fill = "#E5E5E5", linewidth = 0.8)
  }
  p <- p +
    ggplot2::geom_point(color = .hts_col("accent"), size = 2.6, alpha = 0.8,
      stroke = 0.2) +
    ggplot2::scale_x_log10() +
    ggplot2::labs(x = "IC50", y = "AUC") +
    theme_hts()
  if (isTRUE(annotate_cor) && nrow(df) > 2) {
    ct <- suppressWarnings(stats::cor.test(log10(df[[ic50_col]]),
      df[[auc_col]], method = "pearson"))
    lab <- sprintf("R = %.2f, p %s", unname(ct$estimate),
      if (ct$p.value < 0.001) "< 0.001" else sprintf("= %.3f", ct$p.value))
    x_pos <- min(df[[ic50_col]], na.rm = TRUE)
    y_pos <- max(df[[auc_col]], na.rm = TRUE)
    p <- p + ggplot2::annotate("text", x = x_pos, y = y_pos, label = lab,
      hjust = 0, vjust = 1, size = 3.4, color = "#333333")
  }
  p
}

# Module 6: AUC matrix and global activity heatmap ---------------------------

#' Build a compound-by-sample AUC matrix
#'
#' Reshapes long AUC results into a numeric matrix with compounds as rows and
#' samples (for example cell lines or batches) as columns, optionally
#' standardised to Z-scores for comparability.
#'
#' @param data A data frame with compound, sample and AUC columns (typically
#'   `drc$meta` after [calc_drc_auc()]).
#' @param row_col Compound column. Default `"compound_id"`.
#' @param col_col Sample column. Default `"cell_line"`.
#' @param value_col AUC column. Default `"auc"`.
#' @param zscore Standardisation: `"none"` (default), `"row"` or `"column"`.
#'
#' @return A numeric matrix (compounds x samples). The standardisation used is
#'   stored in `attr(x, "zscore")`.
#'
#' @examples
#' \donttest{
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' build_auc_matrix(drc$meta)
#' }
#'
#' @seealso [cluster_auc_matrix()], [plot_auc_heatmap()]
#' @export
build_auc_matrix <- function(data, row_col = "compound_id",
                             col_col = "cell_line", value_col = "auc",
                             zscore = c("none", "row", "column")) {
  zscore <- match.arg(zscore)
  df <- as.data.frame(data, stringsAsFactors = FALSE)
  for (col in c(row_col, col_col, value_col)) {
    if (!col %in% names(df)) {
      stop("Column '", col, "' not found in `data`.", call. = FALSE)
    }
  }
  wide <- tidyr::pivot_wider(
    df[, c(row_col, col_col, value_col)],
    names_from = tidyr::all_of(col_col),
    values_from = tidyr::all_of(value_col),
    values_fn = function(z) mean(z, na.rm = TRUE)
  )
  wide <- as.data.frame(wide)
  rn <- as.character(wide[[row_col]])
  mat <- as.matrix(wide[, setdiff(names(wide), row_col), drop = FALSE])
  rownames(mat) <- rn
  z <- function(v) {
    s <- stats::sd(v, na.rm = TRUE)
    if (!is.finite(s) || s == 0) return(v - mean(v, na.rm = TRUE))
    (v - mean(v, na.rm = TRUE)) / s
  }
  if (zscore == "row") {
    mat <- t(apply(mat, 1, z))
  } else if (zscore == "column") {
    mat <- apply(mat, 2, z)
  }
  attr(mat, "zscore") <- zscore
  mat
}

#' Hierarchically cluster an AUC matrix
#'
#' Performs hierarchical clustering of the compounds (rows) and samples
#' (columns) of an AUC matrix and cuts the compound tree into groups.
#'
#' @param mat A numeric matrix from [build_auc_matrix()].
#' @param k Number of compound clusters to cut. Default `3`.
#' @param distance Distance method for [stats::dist()]. Default `"euclidean"`.
#' @param method Linkage method for [stats::hclust()]. Default `"complete"`.
#'
#' @return A list with `row_hclust`, `col_hclust` (or `NULL` if too few
#'   columns), `clusters` (named integer vector of compound cluster ids) and
#'   `k`.
#'
#' @examples
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' cl <- cluster_auc_matrix(build_auc_matrix(drc$meta), k = 2)
#' table(cl$clusters)
#'
#' @seealso [extract_cluster_hit()], [plot_cluster_tree()]
#' @export
cluster_auc_matrix <- function(mat, k = 3, distance = "euclidean",
                               method = "complete") {
  mat <- mat[stats::complete.cases(mat), , drop = FALSE]
  if (nrow(mat) < 2) {
    stop("At least two complete compound rows are required to cluster.",
      call. = FALSE)
  }
  row_h <- stats::hclust(stats::dist(mat, method = distance), method = method)
  col_h <- if (ncol(mat) >= 2) {
    stats::hclust(stats::dist(t(mat), method = distance), method = method)
  } else {
    NULL
  }
  k <- min(k, nrow(mat))
  clusters <- stats::cutree(row_h, k = k)
  list(row_hclust = row_h, col_hclust = col_h, clusters = clusters, k = k)
}

#' Extract the most active compound cluster
#'
#' Returns the compounds belonging to the cluster with the most extreme mean
#' activity, that is the lowest mean AUC (strongest potency) by default.
#'
#' @param mat The AUC matrix used for clustering.
#' @param cluster_result The list returned by [cluster_auc_matrix()].
#' @param direction `"low"` (default) selects the lowest-AUC cluster (most
#'   active); `"high"` selects the highest-AUC cluster.
#'
#' @return A character vector of compound ids in the selected cluster.
#'
#' @examples
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' m <- build_auc_matrix(drc$meta)
#' extract_cluster_hit(m, cluster_auc_matrix(m, k = 2))
#'
#' @seealso [cluster_auc_matrix()]
#' @export
extract_cluster_hit <- function(mat, cluster_result, direction = c("low",
                                "high")) {
  direction <- match.arg(direction)
  clusters <- cluster_result$clusters
  mat <- mat[names(clusters), , drop = FALSE]
  cluster_mean <- tapply(rowMeans(mat, na.rm = TRUE), clusters,
    function(z) mean(z, na.rm = TRUE))
  target <- if (direction == "low") {
    names(cluster_mean)[which.min(cluster_mean)]
  } else {
    names(cluster_mean)[which.max(cluster_mean)]
  }
  names(clusters)[clusters == as.integer(target)]
}

#' AUC clustering heatmap
#'
#' Draws a clustered heatmap of the AUC matrix. Uses \pkg{ComplexHeatmap} (with
#' \pkg{circlize} for the colour scale) when available for a publication-quality
#' figure with a diverging navy-white-firebrick palette matching
#' [plot_delta_auc_heatmap()], otherwise falls back to base [stats::heatmap()].
#'
#' @param mat A numeric matrix from [build_auc_matrix()].
#' @param title Plot title. Default `"AUC heatmap"`.
#' @param cluster_rows,cluster_columns Whether to cluster rows/columns. Default
#'   `TRUE`.
#'
#' @return Invisibly, the drawn object. Called for its side effect (a plot on
#'   the active graphics device).
#'
#' @examples
#' \donttest{
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' plot_auc_heatmap(build_auc_matrix(drc$meta, zscore = "row"))
#' }
#'
#' @seealso [build_auc_matrix()], [plot_cluster_tree()]
#' @export
plot_auc_heatmap <- function(mat, title = "AUC heatmap",
                             cluster_rows = TRUE, cluster_columns = TRUE) {
  mat <- mat[stats::complete.cases(mat), , drop = FALSE]
  if (requireNamespace("ComplexHeatmap", quietly = TRUE) &&
    requireNamespace("circlize", quietly = TRUE)) {
    rng <- range(mat, na.rm = TRUE)
    mid <- if (rng[1] < 0 && rng[2] > 0) 0 else stats::median(mat, na.rm = TRUE)
    col_fun <- circlize::colorRamp2(
      c(rng[1], mid, rng[2]), c("navy", "white", "firebrick3")
    )
    ht <- ComplexHeatmap::Heatmap(mat, name = "AUC",
      col = col_fun,
      column_title = title,
      column_names_rot = 45,
      border = TRUE,
      cluster_rows = cluster_rows, cluster_columns = cluster_columns)
    ComplexHeatmap::draw(ht)
    return(invisible(ht))
  }
  stats::heatmap(mat, main = title,
    Rowv = if (cluster_rows) NULL else NA,
    Colv = if (cluster_columns) NULL else NA,
    scale = "none")
  invisible(mat)
}

#' Plot the compound clustering dendrogram
#'
#' Draws the hierarchical clustering tree of compounds from an AUC matrix.
#'
#' @param cluster_result The list returned by [cluster_auc_matrix()].
#' @param main Plot title. Default `"Compound clustering"`.
#'
#' @return Invisibly `NULL`; called for its side effect (a base-graphics plot).
#'
#' @examples
#' drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
#'   response_col = "viability", group_cols = "cell_line"),
#'   group_cols = "cell_line"))
#' m <- build_auc_matrix(drc$meta)
#' plot_cluster_tree(cluster_auc_matrix(m, k = 2))
#'
#' @seealso [cluster_auc_matrix()]
#' @export
plot_cluster_tree <- function(cluster_result, main = "Compound clustering") {
  if (is.null(cluster_result$row_hclust)) {
    stop("`cluster_result` has no row dendrogram.", call. = FALSE)
  }
  graphics::plot(cluster_result$row_hclust, main = main, xlab = "", sub = "")
  invisible(NULL)
}

---
title: "Analysing a two-condition drug screen with diffHTS"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Analysing a two-condition drug screen with diffHTS}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6.5,
  fig.height = 4,
  dpi = 120,
  dev = "png"
)
```

```{r setup}
library(diffHTS)
```

## Overview

`diffHTS` is a one-stop toolkit for **large-scale** microplate
(96/384/1536-well) high-throughput drug screening. It is built for campaigns
that span **many plates and several experiments at once**, and covers the whole
chain from raw luminescence/fluorescence readouts through rigorous
**quality control**, replicate checks, dose-response fitting, AUC, clustering
and hit ranking. It supports both single-concentration **primary** screens and
gradient **secondary** (dose-response) screens.

Two comparisons sit at the heart of the package and shape every module:

- **Two conditions.** Drugs are compared between two experimental conditions —
  for example irradiated (`Gy2`) versus non-irradiated (`Gy0`) cells — via the
  differential (delta-AUC) workflow, to find compounds whose effect *depends* on
  the condition (e.g. radiosensitisers).
- **Cancer versus normal cell lines.** The same compounds are screened across
  cancer and normal cell lines, so that hits can be selected for a
  **therapeutic window** — potent against cancer cells while sparing normal
  cells — rather than for raw potency alone.

The analysis is organised into seven modules (each with business + companion
plotting functions), shown first as an end-to-end pipeline. The
**two-condition differential** workflow (delta-AUC between, e.g., `Gy0` vs
`Gy2`) is described afterwards.

### Publication-ready figures

Every companion plotting function returns a standard `ggplot` object drawn in a
consistent, colourblind-safe *modern academic* style: a white background with an
L-shaped axis and light dashed grid (`theme_hts()`), a low-saturation Okabe-Ito
qualitative palette (`hts_pal()`), muted semantic colours (blue for pass/NC,
vermillion for fail/PC), direct on-plot value labels and thin white bar
separators. Both helpers are exported, so you can restyle your own figures to
match:

```{r styling}
library(ggplot2)
ggplot(mtcars, aes(factor(cyl), mpg, fill = factor(cyl))) +
  geom_boxplot(color = "white", linewidth = 0.3) +
  scale_fill_manual(values = hts_pal()) +
  labs(x = "Cylinders", y = "MPG", fill = "cyl") +
  theme_hts()
```

## The seven-module pipeline

### Module 1 — Import and pre-QC

`read_hts_plate()` ingests a plate table (data frame or csv/txt/xlsx) into a
standardised `hts_raw` object. `baseline_subtract()` removes the blank-well
background, `detect_outlier_wells()` flags anomalies by the IQR rule,
`calc_z_prime()` computes each plate's Z' factor and `filter_valid_plates()`
drops plates below the threshold.

**A screening run normally spans several plates at once, and every Module 1
function is plate-aware.** A single `hts_raw` object can hold any number of
plates stacked in its `plate_id` column; background subtraction, outlier
detection and Z' scoring are all computed *within each plate*, and
`filter_valid_plates()` drops whole failing plates while keeping the rest. The
bundled `hts_primary_raw` demonstrates this with **three 96-well plates**:

```{r m1-read}
raw <- read_hts_plate(hts_primary_raw)
unique(raw$plate_id)
```

**How the plates are set up — you define the layout.** Nothing about where the
controls, blanks and compounds sit is fixed by the package: you decide it, and
record it in a **plate-map file** you author in Excel or CSV. The map is a grid
that mirrors the physical plate — the first column holds the row letters, the
remaining headers are the plate column numbers, and each cell names what that
well contains (a negative-control label such as `DMSO`, a positive-control
label such as `kill_ctrl`, a blank/edge label such as `edgewell`, or a compound
identifier). `read_plate_layout()` turns that grid into a tidy layout, and
`apply_plate_layout()` stamps those roles onto the measured signal so the *what
each well is* and the *what was measured* stay in separate files, exactly as a
lab records them.

**Two layout rules worth following.** However you design a plate, two spatial
conventions make the data more trustworthy and are used throughout the bundled
demos:

1. **Leave the outer ring empty.** Wells on the plate edge evaporate faster than
   interior wells, which biases their readings (the "edge effect"). Reserve the
   outer ring purely as a buffer — no drugs, no controls — and fill it only with
   medium/blank. The demos leave row A, row H and columns 1 and 12 empty on the
   96-well plate (a two-well ring on 384-well).
2. **Place controls on a diagonal.** Rather than parking all negative controls
   in one column and all positive controls in another, split each control across
   two inset columns diagonally — positive controls top-left and bottom-right,
   negative controls top-right and bottom-left. This *rotationally balanced*
   arrangement averages out any left-to-right or top-to-bottom signal gradient,
   so the controls report the true plate-wide window.

The package ships a small example map that follows both rules: an empty
`edgewell` ring with diagonal controls in the two inset columns (2 and 11):

```{r m1-layout}
layout_file <- system.file("extdata", "plate_layout_example.csv",
                           package = "diffHTS")
layout <- read_plate_layout(layout_file, plate_id = "P01")
table(layout$well_type)
head(layout)
```

```{r m1-apply}
# A signal export carries only well + reading; the layout supplies the roles.
set.seed(1)
signal_export <- data.frame(well = layout$well,
                            signal = runif(nrow(layout), 1e4, 1e5))
raw_mapped <- apply_plate_layout(signal_export, layout, plate_id = "P01")
table(raw_mapped$well_type)
```

The bundled `hts_primary_raw` already carries its well roles, and its layout
follows the same two rules: the outer ring (row A, row H, columns 1 and 12) is
an empty `Blank` evaporation buffer, the two inset columns 2 and 11 hold the
controls on a diagonal, and the 48 compounds fill the interior (rows B–G,
columns 3–10). `P01` and `P02` are good replicate plates; `P03` was
deliberately simulated with poor control separation so that QC can reject it.
`summarize_plate_setup()` reads whatever layout each plate actually uses
straight off the data — one row per plate — so you can confirm every plate was
built the way you intended before going further:

```{r m1-setup}
summarize_plate_setup(raw)
```

With the layout confirmed, run the per-plate pre-QC over all three plates at
once:

```{r m1}
raw <- baseline_subtract(raw)
raw <- detect_outlier_wells(raw)
calc_z_prime(raw)[, c("plate_id", "z_prime", "pass")]
good <- filter_valid_plates(raw)
attr(good, "dropped_plates")   # P03 is dropped for failing Z'
```

**A comprehensive quality panel.** The Z'-factor is only one of several
complementary ways to judge a plate. `calc_plate_qc()` returns the whole panel
in one per-plate table — the classic **Z'-factor** and its outlier-resistant
**robust Z'-factor** (median/MAD based), the sample-aware **Z-factor**,
**SSMD**, the **signal-to-background (S/B)** and **signal-to-noise (S/N)**
ratios, and the **CV%** of each control:

```{r m1-qc}
qc <- calc_plate_qc(raw)
qc[, c("plate_id", "cv_nc", "cv_pc", "sb", "sn",
       "z_prime", "robust_z_prime", "ssmd", "pass")]
```

Each metric answers a different question, so a plate can pass one and fail
another (here `P03` keeps an adequate S/B dynamic range yet fails everything
that accounts for variability):

| Metric | Function | What it tells you | Good |
|---|---|---|---|
| Z'-factor | `calc_z_prime()` | Overall control-only assay quality | >= 0.5 |
| Robust Z'-factor | `calc_robust_z_prime()` | Same, but immune to outlier control wells | >= 0.5 |
| Z-factor | `calc_z_factor()` | Window vs. the actual library spread (often low in primary screens) | >= 0.5 |
| SSMD | `calc_ssmd()` | Effect-size of control separation, for hit cut-offs | abs >= 2 |
| S/B | `calc_sb_ratio()` | Raw dynamic range of the assay | >= 2 |
| S/N | `calc_sn_ratio()` | Assay window relative to background noise | >= 3 |
| CV% | `calc_cv()` | Control reproducibility (pipetting/reagents) | <= 20% |

Each metric also has its own standalone function (mirroring `calc_z_prime()`)
for when you need just one number with its own threshold:

```{r m1-qc-single}
calc_ssmd(raw)[, c("plate_id", "ssmd", "pass")]
calc_cv(raw)
```

`plot_plate_qc()` is a single visualiser: pick any metric with the `metric`
argument and it draws the plates with the right acceptance line and pass/fail
colouring. The failing plate `P03` stands out on every metric:

```{r m1-qc-plots, fig.width = 6, fig.height = 3.5}
plot_plate_qc(qc, metric = "z_prime")        # assay window (controls only)
plot_plate_qc(qc, metric = "robust_z_prime") # robust to outlier control wells
plot_plate_qc(qc, metric = "ssmd")           # control separation
plot_plate_qc(qc, metric = "sn")             # signal-to-noise
plot_plate_qc(qc, metric = "cv")             # control CV%, both controls
```

Per-well **Z-scores** and **robust Z-scores** (the median/MAD score preferred in
HTS because hits do not distort it) come from `calc_well_zscore()` and plug into
the same visualiser:

```{r m1-zscore, fig.width = 6, fig.height = 3.5}
scored <- calc_well_zscore(raw)
plot_plate_qc(scored, metric = "robust_zscore")
```

```{r m1-heatmap, fig.width = 8, fig.height = 5}
plot_plate_heatmap_raw(raw, plate = "P01")
```

The negative (`NC`) and positive (`PC`) control wells are automatically circled
so they can be told apart from the drug-treated wells. The same works for
larger formats. The bundled `hts_plate_384` dataset mirrors a real 384-well
screen (EXP87) and applies exactly the same two rules: the outer two-well ring
(rows A, B, O, P and columns 1, 2, 23, 24) is kept free of compound to avoid
edge-evaporation artefacts. Those wells still hold medium but no live cells, so
they read at the same low, "dead" level as the kill control (dark, not missing).
The controls sit on a diagonal in the two inset
columns 3 and 22 — column 3 holds the positive (kill) control in its top half
(rows C–H) and the negative (vehicle) control in its bottom half (rows I–N),
while column 22 mirrors it. The circled controls trace out the tell-tale
diagonal:

```{r m1-384, fig.width = 9, fig.height = 5}
plot_plate_heatmap(hts_plate_384, fill = "signal", well_col = "well",
  control_col = "well_type", plate_col = "plate_id", plate_type = 384)
```

### Module 2 — Normalisation and activity

Raw signals cannot be compared across plates: each plate has its own cell
number, reagent batch and instrument gain. `norm_by_control()` removes that
plate-to-plate offset by rescaling every well against **that plate's own
controls**, so all plates land on a common 0–100% activity scale. With the
negative control (vehicle, full growth) defining 0% inhibition and the positive
control (kill) defining 100% inhibition, a well with signal `x` is scored as

`inhibition (%) = 100 * (mean(NC) - x) / (mean(NC) - mean(PC))`

so a well behaving like the vehicle scores about 0, a fully killed well about
100, and viability is simply `100 - inhibition`. Because the controls are taken
per plate, this both normalises the scale **and** corrects for plate-level
effects in one step. `merge_plate_data()` then stacks the normalised plates into
a single library-wide table.

```{r m2}
norm <- norm_by_control(good)
round(range(norm$inhibition, na.rm = TRUE), 1)
```

**Why plot the inhibition histogram?** `plot_inhibition_hist()` shows the
distribution of per-well inhibition across a plate, and it is the quickest sanity
check that normalisation worked. In a well-behaved primary screen the vast
majority of compounds are inactive, so the histogram should show a **tall peak
near 0% inhibition** (the inactive library) with only a **thin right-hand tail**
of active compounds approaching 100%. A peak sitting away from 0, a bimodal
shape, or a shifted centre signals a normalisation or control problem on that
plate before any hit is called. `plot_plate_heatmap_inhibition()` shows the same
values back in their physical well positions to expose any spatial pattern.

```{r m2-plots, fig.width = 7, fig.height = 4}
plot_plate_heatmap_inhibition(norm, plate = "P01")
plot_inhibition_hist(norm, plate = "P01")
```

### Module 3 — Replicate consistency

Screening decisions are only as trustworthy as the measurements behind them, so
before calling any hits we check that independent replicates of the same
compound agree. `calc_replicate_cv()` computes each compound's **coefficient of
variation** (CV = SD / mean) across its replicate wells, and
`calc_replicate_correlation()` reports the plate-to-plate Pearson correlation;
`filter_bad_replicate()` then removes compounds whose replicates disagree.

```{r m3}
cv <- calc_replicate_cv(norm)
calc_replicate_correlation(norm)
```

`plot_replicate_scatter()` draws every pairwise replicate comparison at once
(P01 vs P02, P01 vs P03, P02 vs P03) in one faceted figure. Following the house
style of the screening reports, points are coloured by well type (compounds
blue, negative controls black, positive controls red, blanks gold), each panel
carries a regression line with a shaded 95% confidence band, and the Pearson
`R` and its p-value are annotated. Pass `plate_x` and `plate_y` to focus on a
single pair.

```{r m3-plots, fig.width = 9, fig.height = 3.4}
plot_replicate_scatter(norm)
```

**Why look at the CV distribution?** A single global CV cut-off hides how the
whole library behaves. Plotting the distribution of per-compound CVs shows
whether the assay is tight (most mass well below the threshold) or noisy (a long
tail crossing it), reveals whether a small number of erratic compounds — rather
than a systemic problem — drive the failures, and lets you set an
evidence-based, rather than arbitrary, acceptance threshold (dashed line, `15%`
by convention). Compounds to the right of the line are dropped by
`filter_bad_replicate()`.

```{r m3-cv, fig.width = 6, fig.height = 4}
plot_cv_distribution(cv, cv_max = 15)
```

### Module 4 — Primary hit selection

`select_primary_hit()` flags actives by a fixed threshold or a percentile rank;
`summarize_primary_hit()` reports the hit rate.

```{r m4}
hits <- select_primary_hit(norm, threshold = 50)
summarize_primary_hit(hits)
```

```{r m4-plots, fig.width = 6, fig.height = 4}
plot_primary_inhibition_rank(hits)
plot_hit_bar_count(hits)
```

`plot_hit_bar_count()` simply tallies **hit vs non-hit** from any table carrying
an `is_hit` column, so it works for *both* ways of calling hits: the
**rank/threshold** method above (`select_primary_hit()`) and the
**distribution (sigma)** method below (`select_sigma_hits()`). Drawing it for
each makes the two definitions directly comparable — a fixed threshold and a
sigma tail usually agree on the strongest actives but disagree at the margin, so
seeing both counts side by side tells you how sensitive the hit list is to the
selection rule.

**A distribution-based (sigma) alternative.** A fixed cut-off ignores how noisy
the library actually is. The classic HTS view instead treats the per-compound
activity of the whole library as an approximately normal *inactive*
distribution and calls anything far out in the tail — conventionally beyond
**3 sigma** — a hit. `select_sigma_hits()` scores every compound by how many
standard deviations it sits from the library centre and bins it by whether it
clears 1, 2 or 3 sigma:

```{r m4-sigma}
sig <- select_sigma_hits(norm, n_sigma = 3)
head(sig)
summarize_sigma_hits(sig)
```

The `summarize_sigma_hits()` table contrasts the observed tail counts with what
a perfectly normal (hit-free) library would predict, so the enrichment beyond
3 sigma is obvious — far more compounds sit in the tail than chance allows.

By default the centre and spread are estimated **robustly** (median and MAD),
which is deliberate. A handful of genuine hits inflate a plain mean/SD so much
that they hide themselves: with `method = "sd"` the standard deviation is
stretched by the very hits we are hunting and *nothing* reaches 3 sigma, whereas
the outlier-resistant median/MAD keeps a tight null and recovers the real tail:

```{r m4-sigma-compare}
c(robust = sum(select_sigma_hits(norm, method = "robust")$is_hit),
  sd     = sum(select_sigma_hits(norm, method = "sd")$is_hit))
```

`plot_sigma_hits()` shows the whole picture: the activity histogram coloured by
sigma band, the fitted normal curve, and the 1/2/3 sigma cut-offs. The bump of
orange/red bars poking out past the normal curve on the right is the hit tail.
Passing the sigma table to `plot_hit_bar_count()` gives the matching hit vs
non-hit tally for this distribution-based rule, directly comparable to the
threshold-based count above:

```{r m4-sigma-plot, fig.width = 7, fig.height = 4.5}
plot_sigma_hits(sig)
plot_hit_bar_count(sig)
```

### Module 5 — Dose-response fitting and AUC

A primary screen at a single concentration only tells you *whether* a compound is
active; to know *how* active, the confirmed hits are re-tested across a
concentration gradient and fitted with a dose-response curve. `import_dose_response()`
standardises the gradient data, `fit_4pl_curve()` fits a 4PL model per compound
(and group, here cell line), `extract_drc_params()` pulls IC50/Emax/Hill and
`calc_drc_auc()` integrates each curve.

**Why compute both IC50 and AUC?** They describe *different* aspects of a
compound and neither is sufficient alone:

- **IC50** (the concentration giving half-maximal effect) measures **potency** —
  how little compound is needed. It is read from a single point on the curve and
  ignores what happens at the plateau, so a very potent compound that only kills
  40% of cells still gets a small IC50.
- **Emax / Hill slope** describe the **efficacy** (maximum achievable effect) and
  the steepness of the response.
- **AUC** (area under the curve) integrates the whole curve, combining potency
  *and* efficacy into one robust number. Because it uses every concentration it
  is far less sensitive to a single noisy point or to a poorly-constrained IC50
  when a curve does not fully plateau, which makes it the more stable ranking
  metric in HTS.

```{r m5}
dr  <- import_dose_response(hts_dose_response, response_col = "viability",
                            group_cols = "cell_line")
drc <- fit_4pl_curve(dr, group_cols = "cell_line")
drc <- calc_drc_auc(drc)
head(extract_drc_params(drc))
```

```{r m5-drc, fig.width = 6, fig.height = 4}
plot_single_drc(drc, drc$meta$curve_id[1])
plot_batch_drc_overlay(drc, drc$meta$curve_id[1:4])
```

Because the 4PL is fitted **per cell line**, `plot_batch_drc_overlay()` lets you
overlay the *same* compound's curves across the different cell lines on one axis.
This is the quickest way to read selectivity directly off the dose-response: a
curve that drops steeply in the cancer lines but stays high (flat, near full
viability) in the normal line marks a compound that kills cancer cells while
sparing normal ones — exactly the profile the screen is looking for.

**Why correlate IC50 against AUC?** Because the two metrics capture potency and
integrated activity, comparing them is a built-in consistency check. For most
compounds a lower IC50 (more potent) goes with a larger integrated effect, so
the two are correlated — `plot_ic50_auc_cor()` draws the trend line and prints
the Pearson `R` and p-value (computed on `log10(IC50)` to match the log x-axis)
directly on the figure. The value of the plot is in the **outliers**: a compound
that is potent (small IC50) but sits off the trend with a modest AUC is efficacy-
limited (steep but shallow curve), while a weak-IC50 compound with a
surprisingly large AUC has a broad, gradual response. These are exactly the
molecules worth a second look, and the plot separates strong / moderate / weak
candidates at a glance.

```{r m5-cor, fig.width = 6, fig.height = 4}
params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
plot_ic50_auc_cor(params)
```

### Module 6 — AUC matrix and clustering

`build_auc_matrix()` reshapes AUCs into a compound-by-sample matrix,
`cluster_auc_matrix()` clusters it and `extract_cluster_hit()` pulls the most
active cluster.

```{r m6}
m  <- build_auc_matrix(drc$meta)
cl <- cluster_auc_matrix(m, k = 2)
table(cl$clusters)
```

Reshaping every compound's AUC into one **compound-by-cell-line matrix** is what
turns selectivity into a *batch* read-out. Each row is a compound, each column a
cell line, and clustering the rows groups compounds by their **whole
cross-cell-line profile** at once — you no longer inspect drugs one at a time.
In the row-scaled heatmap below, a compound that is active only in the cancer
columns (dark) while staying inactive in the normal `MRC5` column (light) stands
out immediately as a differential, cancer-selective hit; broadly toxic compounds
are dark across *all* columns and are easy to set aside. That side-by-side
contrast across the full library is what lets a large screen be triaged quickly.

```{r m6-plots, fig.width = 6, fig.height = 5, eval = requireNamespace("ComplexHeatmap", quietly = TRUE)}
plot_auc_heatmap(build_auc_matrix(drc$meta, zscore = "row"))
```

### Module 7 — Ranking, annotation and report

`rank_hit_compound()` combines the metrics into a Strong/Moderate/Weak score,
`annotate_hit_info()` joins compound metadata, `export_hit_table()` writes a
CSV, and `generate_hts_report()` renders a full HTML report.

```{r m7}
ranked <- rank_hit_compound(params)
table(ranked$tier)
head(annotate_hit_info(ranked, hts_compound_meta))
```

A trustworthy final hit is not defined by any single test but by **agreement
across the evidence gathered in the previous modules**:

1. **Primary rank / threshold** (Module 4) — the compound is active in the
   single-concentration screen.
2. **Sigma tail** (Module 4) — its activity sits well beyond the library's null
   distribution (e.g. past 3 sigma), so it is unlikely to be noise.
3. **Cancer-versus-normal differential** (Modules 5–6 and the two-condition
   workflow below) — it is potent in the cancer lines/treated condition while
   sparing the normal line, i.e. it has a therapeutic window.

`rank_hit_compound()` produces the tiered score, `annotate_hit_info()` attaches
the library metadata, and the compounds that clear **all three** criteria are
the ones worth carrying forward — a shortlist that a large primary library can
be reduced to with confidence.

## Two-condition differential workflow

The remainder of this vignette covers the original differential (delta-AUC)
analysis used to compare drug sensitivity between **two conditions** -- for
example irradiated (`Gy2`) versus non-irradiated (`Gy0`) cells, or a cancer
versus a normal cell line.

### Experimental design: cancer versus normal cell lines

The screen is built around a **therapeutic-window** question rather than raw
potency. Each compound is tested in parallel on one or more **cancer** cell
lines (here `A549`, `H1299`, `H1975`, `H2030`) and on a **normal** cell line
(here `MRC5`), and the same 4PL dose-response fit gives an AUC (or IC50) per
compound per line. A useful hit is one that is **potent against the cancer
cells** (low cancer AUC/IC50, i.e. it kills them) while **sparing the normal
cells** (high `MRC5` AUC/IC50). Comparing the two rather than looking at a
single line filters out compounds that are simply broadly cytotoxic: those lower
the AUC everywhere, including in `MRC5`, and offer no selectivity. In the demo
data `MRC5` is deliberately made more resistant (a higher IC50) than the cancer
lines, so selective compounds separate out as a large gap between the cancer and
normal AUC/IC50. The same logic applies to the radiation setting: keep drugs
that sensitise the treated condition (`Gy2`) while leaving the untreated
baseline (`Gy0`) largely unaffected.

## 1. Dose-response and AUC

`four_pl()` evaluates the four-parameter logistic model and `compute_auc()`
integrates it. `fit_dose_response()` fits a curve directly from well-level data
using a self-contained base-R optimiser (no external fitting package required),
returning a `dsr_4pl` object with the usual `coef()`, `predict()`, `fitted()`
and `residuals()` methods.

```{r doseresponse}
four_pl(x = c(0.1, 1, 10), top = 1, ic50 = 1, hill = 1, bottom = 0)

compute_auc(
  min_concentration = 0.01, max_concentration = 100,
  top = 1, ic50 = 5, hill = 1, bottom = 0.05
)

one <- subset(
  screen_doseresponse,
  drug_name == "DrugA" & condition == "Gy0" & experiment_type == "sample"
)
fit <- fit_dose_response(one, "normalized_cell_count", "concentration")
coef(fit)
```

## 2. Quality control

`calculate_qc_metrics()` derives the standard plate metrics (Z-factor, Z',
signal-to-background, signal-to-noise, SSMD) from per-control summary
statistics, and `flag_qc_plates()` applies acceptance thresholds.

```{r qc}
head(screen_doseresponse)

plate <- data.frame(
  experiment_type = c("negative_ctrl", "positive_ctrl", "sample"),
  mean = c(1.00, 0.05, 0.55),
  sd = c(0.05, 0.02, 0.20)
)
calculate_qc_metrics(plate)
```

```{r qc-boxplot}
plot_qc_boxplot(screen_doseresponse)
```

`plot_plate_heatmap()` shows a whole plate in its physical layout, colouring
each well by cell growth. The demo plate follows standard HTS practice, so three
zones are visible at a glance: an **empty evaporation-buffer ring** around the
outer edge (row A, row H, columns 1 and 12), which holds no live cells and so
reads dark like the kill control; the **controls** placed in the two flanking
columns (2 and 11) in a rotationally balanced *diagonal* pattern -- positive
(kill) controls top-left and bottom-right, negative (DMSO) controls top-right
and bottom-left -- so any spatial gradient averages out; and the **dose series**
of one drug per row filling the interior block (rows B-G, columns 3-10). Keeping
compounds off the edge avoids the well-known edge effect (faster evaporation and
uneven readouts around the plate rim).

```{r plate-map, fig.width = 8, fig.height = 5}
plot_plate_heatmap(subset(screen_plate_layout, plateID == "EXP99_Gy0"),
  control_col = "experiment_type")
```

For a whole screen of many plates, `screen_plate_qc` holds one row per plate
with its control readouts and QC metrics. `plot_control_scatter()` compares the
negative- and positive-control readouts across plates, and
`plot_qc_metric_trend()` tracks any single metric (here Z') with failing plates
highlighted.

```{r qc-controls, fig.width = 7, fig.height = 4}
plot_control_scatter(screen_plate_qc)

plot_qc_metric_trend(screen_plate_qc, metric = "z_prime", threshold = 0.5)
```

When several experiments are screened together, circular summaries make the
whole campaign legible at once. `plot_qc_metric_circular()` places one sector
per experiment and one thin bar per plate rising from a zero baseline to its
Z' value, with the value printed at each bar tip (vermillion = fails the Z'
threshold, blue = passes) and a generous outer margin so no bar touches the
sector edge, while `plot_qc_circular_heatmap()` shows every control well of
every plate as a ring.

```{r qc-circular, fig.width = 6, fig.height = 6, eval = requireNamespace("circlize", quietly = TRUE)}
plot_qc_metric_circular(screen_plate_qc, metric = "z_prime")

plot_qc_circular_heatmap(screen_plate_qc)
```

## 3. Differential AUC between the two conditions

Given a long table of per-drug, per-condition AUCs, `compute_delta_auc()`
returns one row per drug with `delta_auc` (`treatment - baseline`) and its
z-score.

```{r delta}
auc_long <- data.frame(
  drug_name = rep(c("DrugA", "DrugB", "DrugC", "DrugD"), each = 2),
  condition = rep(c("Gy0", "Gy2"), times = 4),
  auc = c(5.0, 2.0, 4.2, 4.1, 6.0, 3.4, 3.0, 3.1)
)
compute_delta_auc(auc_long, conditions = c("Gy0", "Gy2"))
```

## 4. Hit selection

Across several cell lines, `select_hits_cutoff()` keeps drugs below a threshold
in enough cell lines, while `select_hits_sigma()` keeps drugs beyond a sigma
cut-off. Here the cancer lines are scored together: a hit must have a negative
delta-AUC (more sensitive under treatment) in at least four of them. Reading the
result alongside the normal line `MRC5` is what reveals selectivity -- prefer
compounds that pass in the cancer lines but leave `MRC5` near zero (spared).

```{r hits}
head(screen_delta_auc)

cancer <- c("A549", "H1299", "H1975", "H2030")
select_hits_cutoff(screen_delta_auc, score_cols = cancer,
                   cutoff = 0, min_pass = 4)
```

## 5. Visualisation

`plot_dose_response_curves()` overlays the observed responses and fitted 4PL
curves for a single drug under both conditions. In the demo data `DrugA` is a
strong radiosensitiser — its `Gy2` IC50 is about ten-fold lower than its `Gy0`
IC50 — so the two curves are **clearly separated**: the treated (`Gy2`) curve
sits well to the left, killing cells at much lower concentrations than the
untreated (`Gy0`) curve. That visible left-shift is the signature of a compound
whose effect depends on the condition:

```{r curves}
drugA <- subset(
  screen_doseresponse,
  drug_name == "DrugA" & experiment_type == "sample"
)
plot_dose_response_curves(drugA, drug = "DrugA")
```

`plot_condition_scatter()` compares each drug across two cell lines at once, with
the axes centred on the origin so the **four quadrants are equal in size** and
easy to read. Here the normal line `MRC5` is on the x-axis and the cancer line
`A549` on the y-axis, and the values are delta-AUCs (negative = more killing).
The quadrant a drug falls in tells you its selectivity at a glance:

- **Lower-right** (normal `MRC5` change near zero or positive = *spared*, cancer
  `A549` change strongly negative = *killed*): the therapeutic-window drugs — the
  ones the screen is after.
- **Lower-left**: kills both cancer and normal cells (broadly toxic,
  non-selective).
- **Upper-right / near the origin**: little effect on either line.

```{r scatter, fig.width = 5.5, fig.height = 5.5}
plot_condition_scatter(screen_delta_auc, x = "MRC5", y = "A549")
```

Finally, `plot_delta_auc_heatmap()` lays the whole library out as a
compound-by-cell-line grid with the cancer and normal columns annotated
separately. Cancer-selective hits appear as rows that are **strongly coloured
across the cancer columns but pale in the `MRC5` (normal) column**, so the
differential between normal and cancer cells is visible for every compound at
once — turning a large screen into a quick, side-by-side triage (requires the
suggested `ComplexHeatmap` and `circlize` packages):

```{r heatmap, eval = requireNamespace("ComplexHeatmap", quietly = TRUE) && requireNamespace("circlize", quietly = TRUE)}
plot_delta_auc_heatmap(
  screen_delta_auc,
  value_cols = c("A549", "H1299", "H1975", "H2030", "MRC5"),
  cell_types = c(A549 = "Cancer", H1299 = "Cancer", H1975 = "Cancer",
                 H2030 = "Cancer", MRC5 = "Normal")
)
```

---
title: "diffHTS screening report"
output: html_document
params:
  hits: null
  drc: null
  primary: null
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)
library(diffHTS)
```

# Overview

This report was generated by `diffHTS::generate_hts_report()`. Sections are
rendered only when the corresponding object is supplied via `params`.

```{r primary, eval = !is.null(params$primary)}
knitr::kable(summarize_primary_hit(params$primary),
  caption = "Primary screen summary")
plot_primary_inhibition_rank(params$primary)
```

```{r drc, eval = !is.null(params$drc)}
knitr::kable(head(params$drc$meta, 20), caption = "Dose-response curve metrics")
plot_batch_drc_overlay(params$drc)
```

```{r hits, eval = !is.null(params$hits)}
knitr::kable(head(params$hits, 30), caption = "Ranked hit compounds")
plot_hit_stratify_bar(params$hits)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_qc_metric_trend}
\alias{plot_qc_metric_trend}
\title{Trend plot of a QC metric across plates}
\usage{
plot_qc_metric_trend(
  data,
  metric = "z_prime",
  plate_col = "plateID",
  flag_col = NULL,
  threshold = 0.5,
  direction = c("min", "max"),
  colors = c(Bad = "#D55E00", Normal = "#0072B2")
)
}
\arguments{
\item{data}{A data frame with one row per plate, such as \link{screen_plate_qc}.}

\item{metric}{Name of the metric column to plot, as a string. Default
\code{"z_prime"}.}

\item{plate_col}{Name of the plate identifier column, as a string. Default
\code{"plateID"}.}

\item{flag_col}{Optional name of a column of \code{"Bad"}/\code{"Normal"} flags used to
colour points. If \code{NULL} (default) plates are flagged on the fly using
\code{threshold} and \code{direction}.}

\item{threshold}{Numeric cut-off used to flag plates when \code{flag_col} is
\code{NULL}. Default \code{0.5}.}

\item{direction}{Either \code{"min"} (values below \code{threshold} fail, the default)
or \code{"max"} (values above \code{threshold} fail).}

\item{colors}{Named character vector giving the colours for \code{"Bad"} and
\code{"Normal"} plates.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object: one bar per plate in plate order,
coloured by flag, with a dashed line at the mean and a solid line at
\code{threshold}.
}
\description{
Plots a single QC metric (for example Z', Z-factor or negative-control CV)
for every plate in acquisition order, colouring each plate by a pass/fail
flag and marking the overall mean. This mirrors the per-metric QC review
plots used in the screening pipeline and makes drifting or failing plates
easy to spot.
}
\examples{
plot_qc_metric_trend(screen_plate_qc, metric = "z_prime")

}
\seealso{
\code{\link[=calculate_qc_metrics]{calculate_qc_metrics()}}, \code{\link[=flag_qc_plates]{flag_qc_plates()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{clean_filename}
\alias{clean_filename}
\title{Sanitise a string for use as a file name}
\usage{
clean_filename(filename_str)
}
\arguments{
\item{filename_str}{A character vector of candidate file names.}
}
\value{
A character vector with newlines removed, \code{/} and \code{:} replaced by
\verb{_} and \code{-} respectively, and the characters \verb{* ? " < > |} stripped.
}
\description{
Removes newlines and characters that are illegal in file names on common
operating systems, so that drug names or plate labels can be turned into safe
output file names.
}
\examples{
clean_filename("Drug A/B: 10\u00b5M?")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{calc_plate_qc}
\alias{calc_plate_qc}
\title{Comprehensive per-plate quality assessment}
\usage{
calc_plate_qc(
  hts_raw,
  signal_col = "signal",
  nc_label = "NC",
  pc_label = "PC",
  sample_label = "compound",
  z_prime_min = 0.5
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column. Default \code{"signal"}.}

\item{nc_label, pc_label}{Well-type labels for the negative and positive
controls. Defaults \code{"NC"} and \code{"PC"}.}

\item{sample_label}{Well-type label for the test (compound) wells, used by
the sample-aware Z-factor. Default \code{"compound"}.}

\item{z_prime_min}{Acceptance threshold applied to the Z'-factor to set the
\code{pass} flag. Default \code{0.5}.}
}
\value{
A data frame with one row per plate and the columns \code{plate_id},
\code{mean_nc}, \code{mean_pc}, \code{sd_nc}, \code{sd_pc}, \code{cv_nc}, \code{cv_pc}, \code{signal_window},
\code{sb} (signal-to-background), \code{sn} (signal-to-noise), \code{z_prime},
\code{robust_z_prime}, \code{z_factor}, \code{ssmd} and a logical \code{pass}.
}
\description{
Computes the full panel of assay-quality metrics used to judge a
high-throughput screening plate, in a single per-plate table. Alongside the
classic Z'-factor this reports the median/MAD-based \strong{robust Z'-factor}, the
sample-aware \strong{Z-factor}, \strong{SSMD}, the \strong{signal-to-background (S/B)} and
\strong{signal-to-noise (S/N)} ratios, and the \strong{coefficient of variation (CV\%)}
of each control population. All of these can then be visualised through the
single entry point \code{\link[=plot_plate_qc]{plot_plate_qc()}}.
}
\details{
Each metric also has its own dedicated function, useful when you only need one
number (and each with its own acceptance threshold): \code{\link[=calc_z_prime]{calc_z_prime()}},
\code{\link[=calc_robust_z_prime]{calc_robust_z_prime()}}, \code{\link[=calc_z_factor]{calc_z_factor()}}, \code{\link[=calc_ssmd]{calc_ssmd()}}, \code{\link[=calc_sb_ratio]{calc_sb_ratio()}},
\code{\link[=calc_sn_ratio]{calc_sn_ratio()}} and \code{\link[=calc_cv]{calc_cv()}}. \code{calc_plate_qc()} uses the very same
formulas so the numbers agree.

What each metric tells you, with the high-signal control taken as
whichever of NC/PC has the larger mean and the low-signal control as the
other:
\itemize{
\item \strong{CV\%} \eqn{= 100\,\sigma/\mu} of each control — pipetting/reagent
reproducibility; lower is better.
\item \strong{S/B} \eqn{= \mu_{hi}/\mu_{lo}} — raw dynamic range of the assay.
\item \strong{S/N} \eqn{= (\mu_{hi}-\mu_{lo})/\sigma_{lo}} — assay window
relative to background noise.
\item \strong{Z'-factor} \eqn{= 1 - 3(\sigma_{nc}+\sigma_{pc})/|\mu_{nc}-\mu_{pc}|}
— overall control-only assay quality (\verb{>= 0.5} excellent).
\item \strong{Robust Z'-factor}: as Z'-factor but with medians and MADs,
resistant to outlier control wells.
\item \strong{Z-factor} \eqn{= 1 - 3(\sigma_{s}+\sigma_{nc})/|\mu_{s}-\mu_{nc}|}
using the sample population \eqn{s} — is the window large versus the
library spread (often low/negative in primary screens).
\item \strong{SSMD} \eqn{= (\mu_{nc}-\mu_{pc})/\sqrt{\sigma_{nc}^2+\sigma_{pc}^2}}
— effect-size measure of control separation for hit cut-offs.
}
}
\examples{
raw <- read_hts_plate(hts_primary_raw)
calc_plate_qc(raw)

}
\seealso{
\code{\link[=plot_plate_qc]{plot_plate_qc()}}, \code{\link[=calc_z_prime]{calc_z_prime()}}, \code{\link[=calc_well_zscore]{calc_well_zscore()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffHTS-package.R
\docType{package}
\name{diffHTS-package}
\alias{diffHTS}
\alias{diffHTS-package}
\title{diffHTS: Differential Drug Sensitivity Analysis for Two-Condition
High-Throughput Screens}
\description{
The diffHTS package bundles reusable methods for analysing
high-throughput drug screening (HTS) experiments that contrast two
conditions, for example irradiated versus non-irradiated cells or cancer
versus normal cell lines. The typical workflow is:
}
\details{
\enumerate{
\item Fit four-parameter logistic dose-response curves per drug and
condition with \code{\link[=fit_dose_response]{fit_dose_response()}} and summarise potency with
\code{\link[=compute_auc]{compute_auc()}}.
\item Assess plate quality with \code{\link[=calculate_qc_metrics]{calculate_qc_metrics()}} and
\code{\link[=flag_qc_plates]{flag_qc_plates()}}.
\item Contrast the two conditions with \code{\link[=compute_delta_auc]{compute_delta_auc()}} to obtain a
differential AUC and its within-experiment z-score.
\item Nominate hits with \code{\link[=select_hits_cutoff]{select_hits_cutoff()}} or \code{\link[=select_hits_sigma]{select_hits_sigma()}}.
\item Visualise results with \code{\link[=plot_delta_auc_heatmap]{plot_delta_auc_heatmap()}},
\code{\link[=plot_condition_scatter]{plot_condition_scatter()}} and \code{\link[=plot_qc_boxplot]{plot_qc_boxplot()}}.
}

Two small demonstration datasets, \link{screen_doseresponse} and
\link{screen_delta_auc}, are provided to run the examples end to end.
}
\author{
\strong{Maintainer}: Liang Tang \email{xphdtangliang@gmail.com}

Authors:
\itemize{
  \item Liang Tang \email{xphdtangliang@gmail.com}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module3_replicate.R
\name{plot_cv_distribution}
\alias{plot_cv_distribution}
\title{Histogram of replicate CV values}
\usage{
plot_cv_distribution(cv, cv_max = 15, bins = 30)
}
\arguments{
\item{cv}{A data frame from \code{\link[=calc_replicate_cv]{calc_replicate_cv()}}.}

\item{cv_max}{Threshold to draw as a reference line. Default \code{15}.}

\item{bins}{Number of histogram bins. Default \code{30}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Shows the distribution of per-compound replicate CVs across the library, with
the acceptance threshold marked.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
plot_cv_distribution(calc_replicate_cv(norm))

}
\seealso{
\code{\link[=calc_replicate_cv]{calc_replicate_cv()}}, \code{\link[=filter_bad_replicate]{filter_bad_replicate()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module4_primary.R
\name{summarize_sigma_hits}
\alias{summarize_sigma_hits}
\title{Summarise sigma-based hits}
\usage{
summarize_sigma_hits(hits)
}
\arguments{
\item{hits}{A data frame from \code{\link[=select_sigma_hits]{select_sigma_hits()}}.}
}
\value{
A data frame with one row per sigma level (1, 2, 3): \code{sigma},
\code{n_beyond} (compounds clearing it), \code{frac_beyond}, \code{expected_frac}
(normal-theory tail probability) and \code{expected_n} (expected count).
}
\description{
Counts how many compounds clear 1, 2 and 3 sigma from the library centre and
compares each count to what a perfectly normal, hit-free distribution would
predict, so enrichment of the tail over chance is obvious.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
summarize_sigma_hits(select_sigma_hits(norm))

}
\seealso{
\code{\link[=select_sigma_hits]{select_sigma_hits()}}, \code{\link[=plot_sigma_hits]{plot_sigma_hits()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module2_norm.R
\name{norm_by_control}
\alias{norm_by_control}
\title{Normalise wells to plate controls}
\usage{
norm_by_control(
  hts_raw,
  signal_col = "signal",
  nc_label = "NC",
  pc_label = "PC"
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object (typically after \code{\link[=baseline_subtract]{baseline_subtract()}}).}

\item{signal_col}{Name of the signal column to normalise. Default
\code{"signal"}.}

\item{nc_label, pc_label}{Well-type labels for the negative and positive
controls. Defaults \code{"NC"} and \code{"PC"}.}
}
\value{
An \code{hts_norm} object: the input data with added numeric columns
\code{inhibition} (percent) and \code{viability} (percent). Multiple plates are
normalised independently.
}
\description{
Converts raw or background-subtracted signals to percent inhibition and
percent viability using each plate's own negative (NC) and positive (PC)
controls, so that readouts are comparable across plates. Inhibition is
\eqn{100 \times (m_{NC} - x) / (m_{NC} - m_{PC})} and viability is its
complement relative to the control window.
}
\examples{
raw <- baseline_subtract(read_hts_plate(hts_primary_raw))
norm <- norm_by_control(raw)
summary(norm$inhibition)

}
\seealso{
\code{\link[=merge_plate_data]{merge_plate_data()}}, \code{\link[=plot_plate_heatmap_inhibition]{plot_plate_heatmap_inhibition()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{plot_plate_heatmap_raw}
\alias{plot_plate_heatmap_raw}
\title{Raw-signal plate heatmap}
\usage{
plot_plate_heatmap_raw(
  hts_raw,
  plate = NULL,
  signal_col = "signal",
  mark_outliers = TRUE
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{plate}{Plate identifier to show. Defaults to the first plate.}

\item{signal_col}{Name of the signal column to colour by. Default
\code{"signal"}.}

\item{mark_outliers}{Logical; if \code{TRUE} and an \code{outlier} column is present,
flagged wells are labelled with \code{"X"}. Default \code{TRUE}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Draws a single plate's raw (or background-subtracted) readouts in plate
layout, optionally marking flagged outlier wells. A thin wrapper around
\code{\link[=plot_plate_heatmap]{plot_plate_heatmap()}} for \code{hts_raw} objects.
}
\examples{
raw <- detect_outlier_wells(read_hts_plate(hts_primary_raw))
plot_plate_heatmap_raw(raw, plate = "P01")

}
\seealso{
\code{\link[=plot_plate_heatmap]{plot_plate_heatmap()}}, \code{\link[=plot_plate_qc]{plot_plate_qc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{screen_delta_auc}
\alias{screen_delta_auc}
\title{Example differential-AUC hit table}
\format{
A data frame with one row per drug and the following columns:
\describe{
\item{drug_name}{Drug identifier (character).}
\item{A549}{Delta AUC in the A549 lung cancer cell line (numeric).}
\item{H1299}{Delta AUC in the H1299 lung cancer cell line (numeric).}
\item{H1975}{Delta AUC in the H1975 lung cancer cell line (numeric).}
\item{H2030}{Delta AUC in the H2030 lung cancer cell line (numeric).}
\item{MRC5}{Delta AUC in the MRC5 normal lung fibroblast cell line
(numeric).}
}
}
\usage{
screen_delta_auc
}
\description{
A small, simulated table of differential (delta) AUC values with one row per
drug and one column per cell line, used to demonstrate hit selection and
heatmap/scatter visualisation across several cancer cell lines and a normal
(\code{MRC5}) cell line.
}
\details{
The data are simulated (see \code{data-raw/make_datasets.R}) and do not
correspond to any real compound. Negative values indicate treatment-induced
sensitisation.
}
\seealso{
\link{screen_doseresponse}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{calc_sb_ratio}
\alias{calc_sb_ratio}
\title{Signal-to-background ratio per plate}
\usage{
calc_sb_ratio(
  hts_raw,
  signal_col = "signal",
  nc_label = "NC",
  pc_label = "PC",
  sb_min = 2
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column. Default \code{"signal"}.}

\item{nc_label, pc_label}{Well-type labels for the negative and positive
controls. Defaults \code{"NC"} and \code{"PC"}.}

\item{sb_min}{Acceptance threshold. Default \code{2}.}
}
\value{
A data frame with one row per plate: \code{plate_id}, \code{mean_nc},
\code{mean_pc}, \code{sb} (high-mean / low-mean) and a logical \code{pass}.
}
\description{
The ratio of the high-signal control mean to the low-signal control mean.
\strong{What it is for:} S/B is the quickest sanity check that the assay actually
produced a usable dynamic range — that the untreated (high) and killed (low)
controls are far enough apart on the raw scale. It ignores variability, so it
complements but does not replace the Z'-factor; a common minimum is \verb{>= 2}.
}
\examples{
calc_sb_ratio(read_hts_plate(hts_primary_raw))

}
\seealso{
\code{\link[=calc_sn_ratio]{calc_sn_ratio()}}, \code{\link[=calc_plate_qc]{calc_plate_qc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{baseline_subtract}
\alias{baseline_subtract}
\title{Subtract the blank background signal from a plate}
\usage{
baseline_subtract(hts_raw, blank_label = "Blank", clip_zero = TRUE)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object from \code{\link[=read_hts_plate]{read_hts_plate()}}.}

\item{blank_label}{Well-type label identifying blank wells. Default
\code{"Blank"}.}

\item{clip_zero}{Logical; if \code{TRUE} (default) background-subtracted signals
are floored at zero.}
}
\value{
The input object with \code{signal} replaced by the background-subtracted
value and a new \code{signal_raw} column holding the original signal.
}
\description{
Removes the per-plate instrument background by subtracting the mean blank
(no-cell) signal from every well, a standard first step for luminescence and
fluorescence readouts. The original signal is preserved in \code{signal_raw}.
}
\examples{
raw <- read_hts_plate(hts_primary_raw)
bs <- baseline_subtract(raw)
head(bs[, c("well_type", "signal_raw", "signal")])

}
\seealso{
\code{\link[=read_hts_plate]{read_hts_plate()}}, \code{\link[=norm_by_control]{norm_by_control()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_hts.R
\name{hts_pal}
\alias{hts_pal}
\title{Colourblind-safe qualitative palette for diffHTS}
\usage{
hts_pal(n = NULL)
}
\arguments{
\item{n}{Number of colours to return (recycled if \code{n} exceeds the eight
base hues). If \code{NULL} (default) the full palette is returned.}
}
\value{
A character vector of hex colours.
}
\description{
Returns the package's low-saturation, colourblind-safe qualitative palette
(an Okabe-Ito ordering). Use it whenever a plot maps a discrete variable to
colour so that figures stay consistent and accessible.
}
\examples{
hts_pal(3)

}
\seealso{
\code{\link[=theme_hts]{theme_hts()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{screen_plate_layout}
\alias{screen_plate_layout}
\title{Example 96-well plate layout with well positions}
\format{
A data frame with one row per well and the following columns:
\describe{
\item{plateID}{Plate identifier such as \code{"EXP99_Gy0"} (character).}
\item{condition}{Experimental condition, \code{"Gy0"} or \code{"Gy2"}.}
\item{well}{Well identifier such as \code{"A1"} (character).}
\item{plate_rows}{Plate row letter, \code{"A"}-\code{"H"} (character).}
\item{plate_columns}{Plate column number, \code{1}-\code{12} (integer).}
\item{drug_name}{Drug identifier, or a control label (\code{"DMSO"},
\code{"kill_ctrl"}), or \code{NA} for empty buffer wells.}
\item{experiment_type}{Well role: \code{"sample"}, \code{"negative_ctrl"},
\code{"positive_ctrl"} or \code{"empty"} (evaporation-buffer ring).}
\item{concentration}{Drug concentration (micromolar), \code{NA} for controls
and empty wells.}
\item{normalized_cell_count}{Viability normalised to the negative control;
empty buffer wells hold no live cells and read near the kill control.}
}
}
\usage{
screen_plate_layout
}
\description{
A small, simulated 96-well plate (8 rows by 12 columns) assayed under two
conditions (\code{"Gy0"} and \code{"Gy2"}), in long (one row per well) format. The
layout follows good HTS practice: the outer ring (row A, row H, columns 1 and
12) is an empty evaporation buffer, the two flanking columns (2 and 11) carry
the positive (kill) and negative (DMSO) controls in a rotationally balanced
diagonal pattern, and the interior block (rows B-G, columns 3-10) holds an
8-point dose series of one drug per row. It is used to demonstrate the
plate-layout heatmap \code{\link[=plot_plate_heatmap]{plot_plate_heatmap()}}.
}
\details{
The data are simulated (see \code{data-raw/make_datasets.R}) and do not
correspond to any real compound or cell line.
}
\seealso{
\code{\link[=plot_plate_heatmap]{plot_plate_heatmap()}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{summarize_plate_setup}
\alias{summarize_plate_setup}
\title{Summarise how each plate in a run is set up}
\usage{
summarize_plate_setup(
  hts_raw,
  plate_col = "plate_id",
  type_col = "well_type",
  compound_col = "compound_id",
  col_col = "col",
  nc_label = "NC",
  pc_label = "PC",
  blank_label = "Blank",
  compound_label = "compound"
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object from \code{\link[=read_hts_plate]{read_hts_plate()}} (or any well-level
data frame with plate, well-coordinate and well-type columns).}

\item{plate_col, type_col, compound_col, col_col}{Names of the plate,
well-type, compound and column-index columns. Defaults match
\code{\link[=read_hts_plate]{read_hts_plate()}} output.}

\item{nc_label, pc_label, blank_label, compound_label}{Well-type labels for the
four roles.}
}
\value{
A data frame with one row per plate and the columns \code{plate_id},
\code{n_wells}, \code{plate_format}, \code{n_rows}, \code{n_cols}, \code{n_nc}, \code{n_pc}, \code{n_blank},
\code{n_compound}, \code{n_unique_compound}, \code{nc_columns}, \code{pc_columns},
\code{blank_columns} and \code{compound_columns} (control/compound positions given as
compact column ranges such as \code{"3-11"}).
}
\description{
Reports, for every plate in an \code{hts_raw} object, how the plate is laid out:
its physical format, the number of wells of each role (negative/positive
controls, blanks and compounds), how many distinct compounds it carries, and
which columns the controls and compounds occupy. The positions are \strong{read
back from your own layout} (see \code{\link[=read_plate_layout]{read_plate_layout()}}) rather than assumed by
the package: because a screening run normally spans \strong{several plates at
once}, this is the quickest way to confirm that every plate carries the
layout you designed before QC and normalisation.
}
\examples{
raw <- read_hts_plate(hts_primary_raw)
# One row per plate; the run holds three 96-well plates here.
summarize_plate_setup(raw)

}
\seealso{
\code{\link[=read_hts_plate]{read_hts_plate()}}, \code{\link[=check_control_label]{check_control_label()}}, \code{\link[=calc_z_prime]{calc_z_prime()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module4_primary.R
\name{select_sigma_hits}
\alias{select_sigma_hits}
\title{Select primary-screen hits by distance from the library distribution}
\usage{
select_sigma_hits(
  data,
  value = "inhibition",
  method = c("robust", "sd"),
  n_sigma = 3,
  direction = c("greater", "less", "two.sided"),
  compound_col = "compound_id"
)
}
\arguments{
\item{data}{An \code{hts_norm} object or long data frame.}

\item{value}{Name of the activity column. Default \code{"inhibition"}.}

\item{method}{How to estimate the distribution centre and spread:
\code{"robust"} (default) uses the median and MAD (scaled by 1.4826 to match the
SD of a normal); \code{"sd"} uses the plain mean and standard deviation.}

\item{n_sigma}{Number of sigma a compound must clear to be called a hit.
Default \code{3}.}

\item{direction}{Tail to test: \code{"greater"} (default, high activity),
\code{"less"} (low activity) or \code{"two.sided"} (either tail, scored on \code{abs(z)}).}

\item{compound_col}{Name of the compound column. Default \code{"compound_id"}.}
}
\value{
A data frame with one row per compound: \code{compound_id}, \code{activity},
\code{sigma} (signed z-score), \code{sigma_level} (0-3, how many sigma it clears in
the tested direction), \code{band} (an ordered-factor label) and \code{is_hit}
(\code{sigma_level >= n_sigma}). The distribution \code{center}, \code{scale}, \code{method},
\code{n_sigma} and \code{direction} are stored as attributes.
}
\description{
Treats the per-compound activity of the whole library as an approximately
normal "inactive" distribution and flags compounds sitting far out in the
tail. Each compound receives a z-score (how many standard deviations it lies
from the library centre) and is binned by whether it clears 1, 2 or 3 sigma.
By default the centre and spread are estimated \emph{robustly} (median and MAD),
the HTS-standard choice: a handful of genuine hits would otherwise inflate a
plain mean/SD and mask themselves (with too wide an SD nothing reaches 3
sigma).
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
hits <- select_sigma_hits(norm, n_sigma = 3)
sum(hits$is_hit)

}
\seealso{
\code{\link[=summarize_sigma_hits]{summarize_sigma_hits()}}, \code{\link[=plot_sigma_hits]{plot_sigma_hits()}}, \code{\link[=select_primary_hit]{select_primary_hit()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{filter_valid_plates}
\alias{filter_valid_plates}
\title{Keep only plates that pass the Z' threshold}
\usage{
filter_valid_plates(
  hts_raw,
  z_prime_min = 0.5,
  signal_col = "signal",
  nc_label = "NC",
  pc_label = "PC"
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{z_prime_min}{Acceptance threshold. Default \code{0.5}.}

\item{signal_col, nc_label, pc_label}{Passed to \code{\link[=calc_z_prime]{calc_z_prime()}}.}
}
\value{
The \code{hts_raw} object restricted to passing plates, with the Z'
summary table stored in \code{attr(x, "zprime")} and the dropped plate ids in
\code{attr(x, "dropped_plates")}.
}
\description{
Removes whole plates whose Z' factor falls below the acceptance threshold,
returning a clean data set for downstream normalisation.
}
\examples{
raw <- read_hts_plate(hts_primary_raw)
clean <- filter_valid_plates(raw)
attr(clean, "dropped_plates")

}
\seealso{
\code{\link[=calc_z_prime]{calc_z_prime()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{calc_ssmd}
\alias{calc_ssmd}
\title{SSMD per plate (strictly standardised mean difference)}
\usage{
calc_ssmd(
  hts_raw,
  signal_col = "signal",
  nc_label = "NC",
  pc_label = "PC",
  ssmd_min = 2
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column. Default \code{"signal"}.}

\item{nc_label, pc_label}{Well-type labels for the negative and positive
controls. Defaults \code{"NC"} and \code{"PC"}.}

\item{ssmd_min}{Acceptance threshold applied to \code{abs(ssmd)}. Default \code{2}.}
}
\value{
A data frame with one row per plate: \code{plate_id}, \code{mean_nc},
\code{mean_pc}, \code{sd_nc}, \code{sd_pc}, \code{ssmd} and a logical \code{pass}. The sign of
\code{ssmd} follows \code{mean_nc - mean_pc}.
}
\description{
The mean control difference divided by the pooled control spread. \strong{What it
is for:} SSMD is the effect-size measure of control separation, and it is
the statistically principled companion to the Z'-factor for setting hit
cut-offs. Its magnitude grades the assay — roughly \verb{|SSMD|} \verb{>= 2} is
excellent, \code{1}--\code{2} good and \verb{< 1} weak — independent of the number of
control replicates.
}
\examples{
calc_ssmd(read_hts_plate(hts_primary_raw))

}
\seealso{
\code{\link[=calc_z_prime]{calc_z_prime()}}, \code{\link[=calc_plate_qc]{calc_plate_qc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module4_primary.R
\name{plot_primary_inhibition_rank}
\alias{plot_primary_inhibition_rank}
\title{Ranked primary-activity curve}
\usage{
plot_primary_inhibition_rank(hits)
}
\arguments{
\item{hits}{A data frame from \code{\link[=select_primary_hit]{select_primary_hit()}}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Draws the whole library ranked by activity, with the hit cut-off marked, the
standard view of how many compounds clear the primary threshold.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
plot_primary_inhibition_rank(select_primary_hit(norm))

}
\seealso{
\code{\link[=select_primary_hit]{select_primary_hit()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module6_auc_matrix.R
\name{extract_cluster_hit}
\alias{extract_cluster_hit}
\title{Extract the most active compound cluster}
\usage{
extract_cluster_hit(mat, cluster_result, direction = c("low", "high"))
}
\arguments{
\item{mat}{The AUC matrix used for clustering.}

\item{cluster_result}{The list returned by \code{\link[=cluster_auc_matrix]{cluster_auc_matrix()}}.}

\item{direction}{\code{"low"} (default) selects the lowest-AUC cluster (most
active); \code{"high"} selects the highest-AUC cluster.}
}
\value{
A character vector of compound ids in the selected cluster.
}
\description{
Returns the compounds belonging to the cluster with the most extreme mean
activity, that is the lowest mean AUC (strongest potency) by default.
}
\examples{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
m <- build_auc_matrix(drc$meta)
extract_cluster_hit(m, cluster_auc_matrix(m, k = 2))

}
\seealso{
\code{\link[=cluster_auc_matrix]{cluster_auc_matrix()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{read_plate_layout}
\alias{read_plate_layout}
\title{Read a user-defined plate-map layout file}
\usage{
read_plate_layout(
  file,
  plate_id = NULL,
  nc_labels = c("NC", "DMSO", "vehicle", "negative_ctrl", "neg_ctrl"),
  pc_labels = c("PC", "kill_ctrl", "positive_ctrl", "pos_ctrl"),
  blank_labels = c("Blank", "edgewell", "empty", "blank", "na", ""),
  concentration_file = NULL,
  sheet = 1,
  sep = "\\t"
)
}
\arguments{
\item{file}{Path to the plate-map grid (\code{.csv}, \code{.txt}/\code{.tsv} or
\code{.xlsx}/\code{.xls}).}

\item{plate_id}{Optional plate identifier attached to every well of the
layout, so the same map can be matched to a specific plate.}

\item{nc_labels, pc_labels, blank_labels}{Character vectors of the cell labels
that mark negative controls, positive controls and blank/empty wells. All
other non-empty labels become compound identifiers.}

\item{concentration_file}{Optional path to a second grid (same layout) giving
the concentration of each well.}

\item{sheet, sep}{Worksheet (Excel) and field separator (\code{.txt}/\code{.tsv}) passed
to the reader.}
}
\value{
An \code{hts_layout} data frame with one row per well and the columns
\code{plate_id}, \code{well}, \code{row}, \code{col}, \code{well_type} (\code{"NC"}, \code{"PC"}, \code{"Blank"} or
\code{"compound"}), \code{compound_id} and \code{concentration}.
}
\description{
Reads a plate map that \strong{you author yourself} in Excel or CSV to declare, for
every well, whether it holds a negative control, a positive control, a blank
(empty/edge) well or a test compound. Nothing about the layout is fixed by the
package: you are free to place the controls, blanks and compounds in whichever
wells your assay uses, and \code{\link[=apply_plate_layout]{apply_plate_layout()}} then stamps those roles onto
the measured readings.
}
\details{
The expected file is a \strong{grid} that mirrors the physical plate: the first
column holds the row letters (\code{A}, \code{B}, ...), the remaining column headers are
the plate column numbers (\code{1}, \code{2}, ... \code{24}), and each cell contains the
label for that well. Labels are matched case-insensitively against
\code{nc_labels}, \code{pc_labels} and \code{blank_labels}; any other non-empty label is
treated as a compound identifier. An optional second grid of the same shape
supplies per-well concentrations.
}
\examples{
f <- system.file("extdata", "plate_layout_example.csv", package = "diffHTS")
layout <- read_plate_layout(f, plate_id = "P01")
table(layout$well_type)

}
\seealso{
\code{\link[=apply_plate_layout]{apply_plate_layout()}}, \code{\link[=read_hts_plate]{read_hts_plate()}}, \code{\link[=check_control_label]{check_control_label()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_plate_heatmap}
\alias{plot_plate_heatmap}
\title{Plate-layout heatmap of well readouts}
\usage{
plot_plate_heatmap(
  data,
  fill = "normalized_cell_count",
  well_col = NULL,
  row_col = "plate_rows",
  col_col = "plate_columns",
  label = NULL,
  control_col = NULL,
  nc_values = c("NC", "negative_ctrl"),
  pc_values = c("PC", "positive_ctrl"),
  circle_size = NULL,
  plate = NULL,
  plate_col = "plateID",
  plate_type = NULL,
  title = NULL,
  na_value = "grey90"
)
}
\arguments{
\item{data}{A long-format data frame with one row per well.}

\item{fill}{Name of the column mapped to tile colour, as a string. Default
\code{"normalized_cell_count"}. Numeric columns use a continuous viridis scale;
character/factor columns use a discrete viridis scale (useful for showing
the well-type layout).}

\item{well_col}{Optional name of a single well-identifier column such as
\code{"A1"} or \code{"B12"}. If supplied it is split into a row letter and column
number and \code{row_col}/\code{col_col} are ignored.}

\item{row_col, col_col}{Names of the row (letter) and column (number)
columns used when \code{well_col} is \code{NULL}. Defaults \code{"plate_rows"} and
\code{"plate_columns"}. Any non-digit characters in the column values (for
example an \code{"X"} prefix) are stripped.}

\item{label}{Optional name of a column whose values are printed inside each
tile (for example a drug name or a QC flag). Default \code{NULL}.}

\item{control_col}{Optional name of a column identifying each well's role
(for example \code{"well_type"} or \code{"experiment_type"}). When supplied, the
negative- and positive-control wells are circled on the plot so that they
can be told apart from the drug-treated wells at a glance. Default \code{NULL}
(no wells circled).}

\item{nc_values, pc_values}{Character vectors of the labels in \code{control_col}
that mark negative and positive controls. Defaults cover the package's
canonical labels (\code{"NC"}/\code{"PC"}) and the common long forms
(\code{"negative_ctrl"}/\code{"positive_ctrl"}).}

\item{circle_size}{Size of the control-marking circles. When \code{NULL}
(default) it is scaled to the plate so that 96- and 384-well plates are
both legible.}

\item{plate}{Optional plate identifier; if supplied \code{data} is filtered to
this plate. When \code{NULL} and several plates are present the plot is
facetted by \code{plate_col}.}

\item{plate_col}{Name of the plate identifier column, as a string. Default
\code{"plateID"}.}

\item{plate_type}{Plate format, either \code{96} or \code{384}, controlling the size
of the drawn grid. When \code{NULL} (default) it is inferred from the observed
rows and columns.}

\item{title}{Optional plot title.}

\item{na_value}{Colour used for wells with no data. Default \code{"grey90"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object: one tile per well, coloured by \code{fill},
with row A at the top and column 1 at the top-left, drawn with a fixed
aspect ratio. When \code{control_col} is supplied the negative- and
positive-control wells are outlined with coloured circles.
}
\description{
Draws a microplate as a grid of wells (rows lettered top-to-bottom, columns
numbered left-to-right) coloured by a per-well readout such as normalised
cell count, so that drug-treated wells and their positive/negative controls
can be inspected in their physical plate positions. Missing wells are shown
as empty cells, making layout errors and edge effects easy to spot. This
abstracts the per-plate \code{heatmap.2()} plate maps used in the screening
pipeline, but is built on \pkg{ggplot2} and needs no external heatmap
package.
}
\details{
Duplicate wells (for example replicates sharing a position) are
summarised by their mean for numeric \code{fill} and concatenated for
categorical \code{fill}/\code{label}.
}
\examples{
# A single 96-well plate coloured by normalised cell count.
plot_plate_heatmap(subset(screen_plate_layout, plateID == "EXP99_Gy0"))

# Show the well-type layout instead, both plates side by side.
plot_plate_heatmap(screen_plate_layout, fill = "experiment_type")

# Circle the negative and positive controls on a raw 96-well plate.
plot_plate_heatmap(
  subset(hts_primary_raw, plate_id == "P01"),
  fill = "signal", well_col = "well", plate_col = "plate_id",
  control_col = "well_type"
)

# A realistic 384-well plate (the outer 2-well ring is a compound-free buffer
# that reads at the low, no-cell level; controls sit on a diagonal in the two
# inset columns 3 and 22).
plot_plate_heatmap(hts_plate_384, fill = "signal", well_col = "well",
  control_col = "well_type", plate_col = "plate_id", plate_type = 384)

}
\seealso{
\code{\link[=plot_qc_boxplot]{plot_qc_boxplot()}}, \code{\link[=plot_control_scatter]{plot_control_scatter()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module5_drc.R
\name{plot_single_drc}
\alias{plot_single_drc}
\title{Plot a single dose-response curve}
\usage{
plot_single_drc(drc, curve_id = NULL, color = "#0072B2")
}
\arguments{
\item{drc}{An \code{hts_drc} object (ideally after \code{\link[=calc_drc_auc]{calc_drc_auc()}}).}

\item{curve_id}{Curve identifier (from \code{drc$meta$curve_id}). Defaults to the
first curve.}

\item{color}{Curve colour. Default \code{"#0072B2"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Plots one compound's observed points and fitted 4PL curve on a log10
concentration axis, annotating IC50, AUC and \eqn{R^2}.
}
\examples{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
plot_single_drc(drc, drc$meta$curve_id[1])

}
\seealso{
\code{\link[=plot_batch_drc_overlay]{plot_batch_drc_overlay()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module7_report.R
\name{plot_hit_stratify_bar}
\alias{plot_hit_stratify_bar}
\title{Stratified hit-count bar chart}
\usage{
plot_hit_stratify_bar(ranked)
}
\arguments{
\item{ranked}{A data frame from \code{\link[=rank_hit_compound]{rank_hit_compound()}} with a \code{tier} column.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Bar chart of the number of Strong, Moderate and Weak hits.
}
\examples{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
plot_hit_stratify_bar(rank_hit_compound(params))

}
\seealso{
\code{\link[=rank_hit_compound]{rank_hit_compound()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_delta_auc_heatmap}
\alias{plot_delta_auc_heatmap}
\title{Heatmap of differential AUC across conditions or cell lines}
\usage{
plot_delta_auc_heatmap(
  data,
  drug_col = "drug_name",
  value_cols = NULL,
  cell_types = NULL,
  cluster_rows = TRUE,
  cluster_columns = TRUE,
  name = "delta AUC"
)
}
\arguments{
\item{data}{A data frame with one row per drug: a drug identifier column and
one numeric delta-AUC column per condition or cell line.}

\item{drug_col}{Name of the drug identifier column, as a string. Default
\code{"drug_name"}.}

\item{value_cols}{Character vector of the delta-AUC column names to display.
Defaults to every numeric column other than \code{drug_col}.}

\item{cell_types}{Optional named character vector mapping each column in
\code{value_cols} to a group label (for example \code{"Cancer"} or \code{"Normal"}) used
to annotate the heatmap.}

\item{cluster_rows, cluster_columns}{Logical, whether to cluster drugs and
conditions respectively. Defaults \code{TRUE}.}

\item{name}{Legend title. Default \code{"delta AUC"}.}
}
\value{
A \link[ComplexHeatmap:Heatmap-class]{ComplexHeatmap::Heatmap} object, which is drawn when
printed. Rows with missing values are dropped before plotting.
}
\description{
Draws a diverging heatmap of differential (delta) AUC values with drugs on
one axis and conditions/cell lines on the other, using \pkg{ComplexHeatmap}.
Colours are centred at zero so that sensitising (negative) and antagonising
(positive) responses are visually separated.
}
\details{
Requires the suggested packages \pkg{ComplexHeatmap} and
\pkg{circlize}.
}
\examples{
if (requireNamespace("ComplexHeatmap", quietly = TRUE) &&
    requireNamespace("circlize", quietly = TRUE)) {
  ht <- plot_delta_auc_heatmap(
    screen_delta_auc,
    value_cols = c("A549", "H1299", "H1975", "MRC5")
  )
}

}
\seealso{
\code{\link[=plot_condition_scatter]{plot_condition_scatter()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module5_drc.R
\name{plot_ic50_auc_cor}
\alias{plot_ic50_auc_cor}
\title{IC50 versus AUC scatter}
\usage{
plot_ic50_auc_cor(
  params,
  auc_col = "auc",
  ic50_col = "ic50",
  add_trend = TRUE,
  annotate_cor = TRUE
)
}
\arguments{
\item{params}{A data frame from \code{\link[=extract_drc_params]{extract_drc_params()}} joined with an \code{auc}
column (for example \code{merge(extract_drc_params(drc), calc_drc_auc(drc)$meta)}),
or the \code{meta} of an \code{hts_drc} after \code{\link[=calc_drc_auc]{calc_drc_auc()}} plus IC50.}

\item{auc_col, ic50_col}{Column names. Default \code{"auc"}, \code{"ic50"}.}

\item{add_trend}{Add a linear regression trend line with a 95\% confidence
band. Default \code{TRUE}.}

\item{annotate_cor}{Annotate the Pearson correlation coefficient and p-value
on the plot. Default \code{TRUE}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Scatter plot of IC50 against AUC across curves, a compact view for triaging
strong, moderate and weak compounds. IC50 marks the potency (the
concentration for half-maximal effect) while AUC integrates potency \emph{and}
efficacy over the whole tested range; plotting them together shows whether the
two agree and flags compounds that are potent but only weakly efficacious (or
vice versa). A regression trend line and the Pearson correlation (computed on
the log10 IC50, matching the log x-axis) are drawn on the figure by default.
}
\examples{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
plot_ic50_auc_cor(params)

}
\seealso{
\code{\link[=extract_drc_params]{extract_drc_params()}}, \code{\link[=calc_drc_auc]{calc_drc_auc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module7_report.R
\name{export_hit_table}
\alias{export_hit_table}
\title{Export a hit summary table}
\usage{
export_hit_table(hits, file, ...)
}
\arguments{
\item{hits}{A hit data frame.}

\item{file}{Output path.}

\item{...}{Passed to \code{\link[utils:write.csv]{utils::write.csv()}}.}
}
\value{
The \code{file} path, invisibly.
}
\description{
Writes a complete hit table (all QC and pharmacology columns) to a CSV file.
}
\examples{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
export_hit_table(rank_hit_compound(params), tempfile(fileext = ".csv"))

}
\seealso{
\code{\link[=rank_hit_compound]{rank_hit_compound()}}, \code{\link[=annotate_hit_info]{annotate_hit_info()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{detect_outlier_wells}
\alias{detect_outlier_wells}
\title{Flag outlier wells by the IQR rule}
\usage{
detect_outlier_wells(
  hts_raw,
  signal_col = "signal",
  group_cols = c("plate_id", "well_type"),
  k = 1.5,
  filter = FALSE
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column to test. Default \code{"signal"}.}

\item{group_cols}{Columns defining comparable well groups within which the
IQR is computed. Default \code{c("plate_id", "well_type")}.}

\item{k}{IQR multiplier for the fences. Default \code{1.5}.}

\item{filter}{Logical; if \code{TRUE} outlier wells are dropped instead of only
flagged. Default \code{FALSE}.}
}
\value{
The input object with a logical \code{outlier} column added (or with
outlier rows removed when \code{filter = TRUE}).
}
\description{
Detects anomalous wells (for example contaminated, bubble or crystallisation
artefacts) using the interquartile-range rule within groups of comparable
wells, and optionally removes them.
}
\examples{
raw <- read_hts_plate(hts_primary_raw)
flagged <- detect_outlier_wells(raw)
sum(flagged$outlier)

}
\seealso{
\code{\link[=calc_z_prime]{calc_z_prime()}}, \code{\link[=filter_valid_plates]{filter_valid_plates()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hit_selection.R
\name{select_hits_sigma}
\alias{select_hits_sigma}
\title{Select hits by a sigma (z-score) threshold}
\usage{
select_hits_sigma(
  data,
  score_col = "zscore_delta_auc",
  n_sigma = 3,
  direction = c("less", "greater"),
  standardize = FALSE,
  drug_col = "drug_name"
)
}
\arguments{
\item{data}{A data frame with one row per drug.}

\item{score_col}{Name of the numeric (typically already z-scored) score
column, as a string. Default \code{"zscore_delta_auc"}.}

\item{n_sigma}{Number of standard deviations defining the threshold. Default
\code{3}.}

\item{direction}{Either \code{"less"} (default, select strongly negative scores,
i.e. sensitising drugs) or \code{"greater"} (select strongly positive scores).}

\item{standardize}{Logical. If \code{TRUE}, the score column is z-scored before
thresholding; if \code{FALSE} (default) the column is assumed to already be on a
standardised (sigma) scale and is thresholded directly.}

\item{drug_col}{Name of the drug identifier column, as a string. Default
\code{"drug_name"}.}
}
\value{
The subset of \code{data} whose score is beyond the sigma threshold in the
requested direction, ordered by score.
}
\description{
Nominates drugs whose standardised score exceeds a given number of standard
deviations from the mean, in the desired direction. This implements the
sigma-based (for example 2-sigma or 3-sigma) hit-selection strategy applied
to z-scored differential AUC values.
}
\examples{
df <- data.frame(
  drug_name = paste0("Drug", 1:6),
  zscore_delta_auc = c(-3.4, -1.2, 0.1, 0.5, 1.1, -2.8)
)
select_hits_sigma(df, n_sigma = 2, direction = "less")

}
\seealso{
\code{\link[=select_hits_cutoff]{select_hits_cutoff()}}, \code{\link[=compute_delta_auc]{compute_delta_auc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module2_norm.R
\name{plot_plate_heatmap_inhibition}
\alias{plot_plate_heatmap_inhibition}
\title{Percent-inhibition plate heatmap}
\usage{
plot_plate_heatmap_inhibition(hts_norm, plate = NULL, value = "inhibition")
}
\arguments{
\item{hts_norm}{An \code{hts_norm} object from \code{\link[=norm_by_control]{norm_by_control()}}.}

\item{plate}{Plate identifier to show. Defaults to the first plate.}

\item{value}{Name of the value column to colour by. Default \code{"inhibition"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Draws a plate's normalised percent-inhibition values in plate layout. A thin
wrapper around \code{\link[=plot_plate_heatmap]{plot_plate_heatmap()}} for \code{hts_norm} objects.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
plot_plate_heatmap_inhibition(norm, plate = "P01")

}
\seealso{
\code{\link[=plot_plate_heatmap]{plot_plate_heatmap()}}, \code{\link[=plot_inhibition_hist]{plot_inhibition_hist()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hts_plate_384}
\alias{hts_plate_384}
\title{Example single 384-well screening plate (realistic layout)}
\format{
A data frame with one row per well (all 384) and the following
columns:
\describe{
\item{plate_id}{Plate identifier, \code{"EXP87P01"} (character).}
\item{well}{Well address such as \code{"C03"} (character).}
\item{row}{Plate row letter, \code{"A"}-\code{"P"} (character).}
\item{col}{Plate column number, \code{1}-\code{24} (integer).}
\item{well_type}{Well role: \code{"PC"} (positive control), \code{"NC"} (negative
control), \code{"compound"} or \code{"edgewell"} (empty buffer) (character).}
\item{compound_id}{Compound identifier for test wells, \code{NA} for controls
and edge wells (character).}
\item{concentration}{Compound concentration (micromolar); \code{NA} for
controls and edge wells (numeric).}
\item{signal}{Raw luminescence read-out (numeric).}
}
}
\usage{
hts_plate_384
}
\description{
A single simulated 384-well plate (\code{"EXP87P01"}) whose layout mirrors a real
cell-based high-throughput screen (see \verb{EXP87/}). The two outermost rings of
wells (rows A, B, O, P and columns 1, 2, 23, 24) are kept free of compound to
avoid edge-evaporation artefacts. They still hold medium but no live cells,
so the instrument reads them at the same low, "dead" level as the kill
control; they are stored as \code{"edgewell"} wells (not omitted). The controls
sit inset from the true plate edge in the two innermost flanking columns
(3 and 22), arranged in a rotationally balanced \emph{diagonal} pattern: column 3
holds the positive (kill) control in its top half (rows C-H) and the negative
(DMSO) control in its bottom half (rows I-N), while column 22 mirrors it
(negative top, positive bottom). This diagonal split averages out
spatial/edge signal gradients. Columns 4-21 across the interior rows C-N hold
the test compounds. Used to demonstrate plate-layout heatmaps with control
circling on a 384-well plate.
}
\details{
The data are simulated (see \code{data-raw/make_datasets.R}) and do not
correspond to any real compound. All 384 wells are stored; the empty edge
wells carry only a low background/no-cell reading.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module7_report.R
\name{annotate_hit_info}
\alias{annotate_hit_info}
\title{Annotate hits with compound metadata}
\usage{
annotate_hit_info(hits, meta, by = "compound_id")
}
\arguments{
\item{hits}{A hit data frame (for example from \code{\link[=rank_hit_compound]{rank_hit_compound()}}).}

\item{meta}{A metadata data frame such as \link{hts_compound_meta}.}

\item{by}{Join key present in both. Default \code{"compound_id"}.}
}
\value{
\code{hits} with the metadata columns merged in (left join), row order
preserved.
}
\description{
Joins external compound metadata (name, target, mechanism, library source)
onto a hit table.
}
\examples{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
annotate_hit_info(rank_hit_compound(params), hts_compound_meta)

}
\seealso{
\code{\link[=rank_hit_compound]{rank_hit_compound()}}, \code{\link[=export_hit_table]{export_hit_table()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_qc_metric_circular}
\alias{plot_qc_metric_circular}
\title{Circular plot of a per-plate QC metric}
\usage{
plot_qc_metric_circular(
  data,
  metric = "z_prime",
  sector_col = "experimentID",
  x_col = "plateNumber",
  label_col = "plateID",
  threshold = 0.5,
  transform = TRUE,
  colors = c(Bad = "#D55E00", Normal = "#0072B2"),
  bar_lwd = 4,
  x_margin = 0.8,
  show_labels = TRUE,
  center_label = NULL
)
}
\arguments{
\item{data}{A data frame with one row per plate, such as \link{screen_plate_qc}.}

\item{metric}{Name of the metric column, as a string. Default \code{"z_prime"}.}

\item{sector_col}{Name of the column that defines the sectors (experiments),
as a string. Default \code{"experimentID"}.}

\item{x_col}{Name of the numeric within-sector position column (plate order),
as a string. Default \code{"plateNumber"}.}

\item{label_col}{Name of the column holding the plate identifier printed on
the outside of each bar, as a string. Default \code{"plateID"}; if the column is
absent the values of \code{x_col} are used instead.}

\item{threshold}{Numeric pass/fail cut-off. Plates whose metric is below
\code{threshold} are drawn in \code{bad_color}. Default \code{0.5}.}

\item{transform}{Logical; if \code{TRUE} (default) strongly negative metric values
(\verb{< -1}) are compressed with \code{-log10(-x)} for display, matching the
pipeline's Z'-factor plot.}

\item{colors}{Named character vector with entries \code{"Bad"} and \code{"Normal"}
giving the bar colours.}

\item{bar_lwd}{Line width of each plate bar. Default \code{4}.}

\item{x_margin}{Numeric angular padding (in \code{x_col} units) added to each side
of every sector so the first and last plate bars do not touch the sector
edges. Default \code{0.8}.}

\item{show_labels}{Logical; if \code{TRUE} (default) the plate identifier from
\code{label_col} is printed radially just outside each bar.}

\item{center_label}{Text drawn in the centre of the ring. Defaults to
\code{metric}.}
}
\value{
Invisibly \code{NULL}. Called for the side effect of drawing to the active
graphics device.
}
\description{
Draws a circular (\pkg{circlize}) scatter of a QC metric with one sector per
experiment and one point per plate, coloured by whether the plate passes a
threshold. This reproduces the whole-screen Z'-factor overview used in the
pipeline, where many plates across several experiments are compared on a
single ring.
}
\details{
Requires the suggested package \pkg{circlize}. Each experiment is a
sector and each plate a thin vertical bar drawn at its own \code{plateNumber}
position, rising from a zero baseline to the metric value, so the bars never
overlap. An angular \code{x_margin} keeps the first and last bars clear of the
sector edges, the plate identifier is printed radially just outside each
bar, and a short Y-axis on the first sector plus a dashed reference line at
\code{threshold} in every sector give the scale. The design scales to large
campaigns of roughly ten plates per experiment.
}
\examples{
if (requireNamespace("circlize", quietly = TRUE)) {
  plot_qc_metric_circular(screen_plate_qc, metric = "z_prime")
}

}
\seealso{
\code{\link[=plot_qc_metric_trend]{plot_qc_metric_trend()}}, \code{\link[=plot_qc_circular_heatmap]{plot_qc_circular_heatmap()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module4_primary.R
\name{plot_hit_bar_count}
\alias{plot_hit_bar_count}
\title{Hit-count bar chart}
\usage{
plot_hit_bar_count(hits)
}
\arguments{
\item{hits}{A data frame from \code{\link[=select_primary_hit]{select_primary_hit()}}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Bar chart of the number of hit versus non-hit compounds from a primary
screen.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
plot_hit_bar_count(select_primary_hit(norm))

}
\seealso{
\code{\link[=summarize_primary_hit]{summarize_primary_hit()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module5_drc.R
\name{calc_drc_auc}
\alias{calc_drc_auc}
\title{Area under the dose-response curve}
\usage{
calc_drc_auc(drc)
}
\arguments{
\item{drc}{An \code{hts_drc} object from \code{\link[=fit_4pl_curve]{fit_4pl_curve()}}.}
}
\value{
The \code{hts_drc} object with an \code{auc} column added to its \code{meta}, also
returned invisibly as a data frame via \code{drc$meta}.
}
\description{
Computes the area under each fitted curve by trapezoidal integration of the
fitted 4PL over the tested concentration range (on the log10 scale), a
robust single-number summary of overall potency.
}
\examples{
drc <- fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line")
calc_drc_auc(drc)$meta

}
\seealso{
\code{\link[=compute_auc]{compute_auc()}}, \code{\link[=extract_drc_params]{extract_drc_params()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qc.R
\name{calculate_qc_metrics}
\alias{calculate_qc_metrics}
\title{Plate-level quality-control metrics for high-throughput screens}
\usage{
calculate_qc_metrics(df)
}
\arguments{
\item{df}{A data frame describing one plate with one row per control type. It
must contain the columns \code{experiment_type}, \code{mean} and \code{sd}. The
\code{experiment_type} column is expected to include the levels
\code{"negative_ctrl"}, \code{"positive_cttl"} and \code{"sample"} (an optional
\code{"edgewell"} level is ignored).}
}
\value{
A one-row data frame with columns \code{z_factor}, \code{z_prime}, \code{sb_value},
\code{sn_value} and \code{ssmd_value}.
}
\description{
Computes the standard set of assay quality metrics for a single plate from
per-control-type summary statistics: the Z-factor, Z-prime (Z'), the
signal-to-background ratio, the signal-to-noise ratio and the strictly
standardised mean difference (SSMD).
}
\details{
The metrics follow the usual HTS definitions, using the negative
control and positive (kill) control populations:
\deqn{Z\text{-}factor = 1 - \frac{3(sd_{neg} + sd_{sample})}{|m_{neg} - m_{sample}|}}
\deqn{Z' = 1 - \frac{3(sd_{neg} + sd_{pos})}{|m_{neg} - m_{pos}|}}
with signal-to-background \eqn{m_{neg}/m_{pos}}, signal-to-noise
\eqn{(m_{pos} - m_{neg})/sd_{neg}} and
\eqn{SSMD = (m_{pos} - m_{neg})/\sqrt{sd_{pos}^2 + sd_{neg}^2}}.
}
\examples{
plate <- data.frame(
  experiment_type = c("negative_ctrl", "positive_cttl", "sample"),
  mean = c(1.00, 0.10, 0.60),
  sd = c(0.05, 0.03, 0.20)
)
calculate_qc_metrics(plate)

}
\seealso{
\code{\link[=flag_qc_plates]{flag_qc_plates()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_qc_circular_heatmap}
\alias{plot_qc_circular_heatmap}
\title{Circular heatmap of control readouts across plates}
\usage{
plot_qc_circular_heatmap(
  data,
  value_cols = grep("_ctrl_", names(data), value = TRUE),
  plate_col = "plateID",
  split_col = "experimentID",
  col_breaks = c(0, 0.5, 1),
  col_colors = c("navy", "white", "firebrick3"),
  name = "Norm cell counts",
  show_sector_labels = TRUE
)
}
\arguments{
\item{data}{A data frame with one row per plate and numeric control columns,
such as \link{screen_plate_qc}.}

\item{value_cols}{Character vector of the control-value columns to display.
By default all columns matching \code{"_ctrl_"} are used.}

\item{plate_col}{Name of the plate identifier column (used as row names), as
a string. Default \code{"plateID"}.}

\item{split_col}{Optional name of a grouping column (experiment) used to
split the ring into sectors. Default \code{"experimentID"}.}

\item{col_breaks, col_colors}{Numeric breakpoints and matching colours passed
to \code{\link[circlize:colorRamp2]{circlize::colorRamp2()}}. Defaults map \code{0}/\code{0.5}/\code{1} to
navy/white/firebrick.}

\item{name}{Legend title. Default \code{"Norm cell counts"}.}

\item{show_sector_labels}{Logical, whether to label each sector. Default
\code{TRUE}.}
}
\value{
Invisibly \code{NULL}. Called for the side effect of drawing to the active
graphics device.
}
\description{
Draws a large circular (\pkg{circlize}) heatmap of the control-well readouts
for every plate, grouped into sectors by experiment. Each ring is one plate
and each cell one control well, so systematic control problems across a whole
screen become visible at once. This mirrors the summary \code{circos.heatmap}
used in the pipeline to display negative- and positive-control cell counts.
}
\details{
Requires the suggested package \pkg{circlize}; when
\pkg{ComplexHeatmap} is also installed a colour legend is added.
}
\examples{
\donttest{
if (requireNamespace("circlize", quietly = TRUE)) {
  plot_qc_circular_heatmap(screen_plate_qc)
}
}

}
\seealso{
\code{\link[=plot_control_scatter]{plot_control_scatter()}}, \code{\link[=plot_delta_auc_heatmap]{plot_delta_auc_heatmap()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{windows_to_linux_path}
\alias{windows_to_linux_path}
\title{Convert a Windows path to a forward-slash path}
\usage{
windows_to_linux_path(path)
}
\arguments{
\item{path}{A character vector of file paths.}
}
\value{
A character vector of the same length with backslashes replaced by
forward slashes.
}
\description{
Replaces backslashes with forward slashes so that Windows-style paths can be
used consistently across platforms.
}
\examples{
windows_to_linux_path("C:\\\\data\\\\EXP99\\\\raw.txt")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module5_drc.R
\name{plot_batch_drc_overlay}
\alias{plot_batch_drc_overlay}
\title{Overlay multiple dose-response curves}
\usage{
plot_batch_drc_overlay(drc, curve_ids = NULL)
}
\arguments{
\item{drc}{An \code{hts_drc} object.}

\item{curve_ids}{Curve identifiers to overlay. Defaults to all fitted curves.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Overlays the fitted curves of several compounds for side-by-side comparison
of activity.
}
\examples{
drc <- fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line")
plot_batch_drc_overlay(drc, drc$meta$curve_id[1:4])

}
\seealso{
\code{\link[=plot_single_drc]{plot_single_drc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hts_primary_raw}
\alias{hts_primary_raw}
\title{Example single-concentration primary screen (raw signals)}
\format{
A data frame with one row per well and the following columns:
\describe{
\item{plate_id}{Plate identifier, \code{"P01"}, \code{"P02"} or \code{"P03"} (character).}
\item{well}{Well identifier such as \code{"A1"} (character).}
\item{row}{Plate row letter, \code{"A"}-\code{"H"} (character).}
\item{col}{Plate column number, \code{1}-\code{12} (integer).}
\item{well_type}{Well role: \code{"NC"} (negative), \code{"PC"} (positive),
\code{"Blank"} or \code{"compound"} (character).}
\item{compound_id}{Compound identifier such as \code{"CPD001"}, \code{NA} for
control and blank wells (character).}
\item{concentration}{Compound concentration (micromolar), \code{NA} for
controls (numeric).}
\item{signal}{Raw instrument signal (numeric).}
}
}
\usage{
hts_primary_raw
}
\description{
A small, simulated primary high-throughput screen in 96-well plate format
with raw readout signals, used to demonstrate the pre-QC, normalisation,
replicate and primary-hit modules. The layout follows good HTS practice: the
outer ring of wells (row A, row H and columns 1 and 12) is a compound-free
evaporation buffer stored as \code{"Blank"} wells, the two inset columns 2 and 11
carry the controls in a rotationally balanced \emph{diagonal} pattern (positive
controls top-left/bottom-right, negative controls top-right/bottom-left), and
the 48 compounds fill the interior block (rows B-G, columns 3-10). Plates
\code{"P01"} and \code{"P02"} are good replicates (Z-prime around 0.8) while \code{"P03"} is
a failed plate.
}
\details{
The data are simulated (see \code{data-raw/make_datasets.R}) and do not
correspond to any real compound or cell line.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module3_replicate.R
\name{plot_replicate_scatter}
\alias{plot_replicate_scatter}
\title{Replicate scatter plot(s) with regression fit and correlation}
\usage{
plot_replicate_scatter(
  data,
  plate_x = NULL,
  plate_y = NULL,
  value = "viability",
  plate_col = "plate_id",
  key_col = NULL,
  compound_col = "compound_id",
  color_col = NULL,
  palette = .well_type_palette,
  line_color = "lightblue",
  band_color = "lightgray",
  show_identity = TRUE
)
}
\arguments{
\item{data}{An \code{hts_norm}/long data frame, or a data frame from
\code{\link[=calc_replicate_correlation]{calc_replicate_correlation()}} (its wide matrix is used).}

\item{plate_x, plate_y}{Plate identifiers for the two axes. When both are
\code{NULL} (default) all pairwise combinations of the available plates are
drawn; supply both to draw a single pair.}

\item{value}{Name of the value column when \code{data} is well-level. Default
\code{"viability"}.}

\item{plate_col}{Name of the plate column. Default \code{"plate_id"}.}

\item{key_col}{Column identifying matching observations across replicate
plates. Defaults to the well-address column \code{"well"} when present (so that
control wells are paired and displayed), otherwise the compound column.}

\item{compound_col}{Compound column, used as the pairing key when no \code{well}
column is available. Default \code{"compound_id"}.}

\item{color_col}{Column used to colour points, typically the well-role column.
Defaults to \code{"well_type"} when present; set to \code{NULL} to disable colouring.}

\item{palette}{Named vector mapping well-type levels to colours. Defaults to
the report palette (compounds blue, negative controls black, positive
controls red, blanks gold).}

\item{line_color, band_color}{Colour of the regression line and of its 95\%
confidence band. Default light blue on light grey, as in the reports.}

\item{show_identity}{Whether to draw the dotted \eqn{y = x} identity line.
Default \code{TRUE}.}
}
\value{
A single \link[ggplot2:ggplot]{ggplot2::ggplot} object. With three or more plates it is
faceted, one panel per replicate pair. The annotated \eqn{R} and p-value are
computed over every point shown in the panel.
}
\description{
Plots the activity of replicate plates against one another in the house style
used across the screening reports: points coloured by well type (compounds,
negative/positive controls, blanks) at 50\% opacity, a linear regression line
with a shaded 95\% confidence band, and a Pearson \eqn{R} (with p-value)
annotated on each panel. When more than two replicate plates are present and
no specific pair is requested, every pairwise combination (replicate 1 vs 2,
1 vs 3, 2 vs 3, ...) is drawn together in a single faceted figure, mirroring
the \code{combn()}-based \code{reps_corr_plot()} workflow in the source analyses.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
# All pairwise replicate comparisons at once, points coloured by well type:
plot_replicate_scatter(norm)
# A single chosen pair:
plot_replicate_scatter(norm, plate_x = "P01", plate_y = "P02")

}
\seealso{
\code{\link[=calc_replicate_correlation]{calc_replicate_correlation()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{screen_plate_qc}
\alias{screen_plate_qc}
\title{Example per-plate quality-control summary}
\format{
A data frame with one row per plate and the following columns:
\describe{
\item{plateID}{Plate identifier such as \code{"EXP76_01"} (character).}
\item{experimentID}{Experiment identifier (character), the plate prefix.}
\item{cell_line}{Cell line assayed on the experiment (character).}
\item{plateNumber}{Plate order within the experiment (integer).}
\item{z_factor}{Assay Z-factor from sample and negative-control wells.}
\item{z_prime}{Z-prime factor from negative- and positive-control wells.}
\item{sb_value}{Signal-to-background ratio.}
\item{sn_value}{Signal-to-noise value.}
\item{ssmd_value}{Strictly standardised mean difference.}
\item{cv_negative_ctrl}{Coefficient of variation of the negative controls
(percent).}
\item{neg_ctrl_1, neg_ctrl_2, neg_ctrl_3, neg_ctrl_4, neg_ctrl_5,
neg_ctrl_6}{Negative-control well readouts.}
\item{pos_ctrl_1, pos_ctrl_2, pos_ctrl_3, pos_ctrl_4, pos_ctrl_5,
pos_ctrl_6}{Positive-control well readouts.}
}
}
\usage{
screen_plate_qc
}
\description{
A small, simulated per-plate quality-control (QC) table spanning several
experiments (cell lines), each with a few plates. Every plate carries its
negative- and positive-control replicate readouts together with the QC
metrics derived from them, so the table can drive the control scatter, the
circular control heatmap and the circular Z'-factor plot. A couple of plates
are deliberately noisy so that passing and failing plates are both present.
}
\details{
The data are simulated (see \code{data-raw/make_datasets.R}) and do not
correspond to any real compound or cell line.
}
\seealso{
\code{\link[=plot_control_scatter]{plot_control_scatter()}}, \code{\link[=plot_qc_metric_circular]{plot_qc_metric_circular()}},
\code{\link[=plot_qc_circular_heatmap]{plot_qc_circular_heatmap()}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module7_report.R
\name{rank_hit_compound}
\alias{rank_hit_compound}
\title{Rank and stratify hit compounds}
\usage{
rank_hit_compound(
  data,
  auc_col = "auc",
  ic50_col = "ic50",
  emax_col = "emax",
  cv_col = "cv",
  rsq_col = "rsq",
  weights = NULL,
  probs = c(1/3, 2/3)
)
}
\arguments{
\item{data}{A per-compound data frame of metrics.}

\item{auc_col, ic50_col, emax_col, cv_col, rsq_col}{Column names for the metrics
to use; set any to \code{NULL} to omit it. Metrics absent from \code{data} are
silently skipped.}

\item{weights}{Optional named numeric vector of per-metric weights (names
matching the metric column names). Defaults to equal weights.}

\item{probs}{Two increasing probabilities splitting Weak/Moderate/Strong by
score quantile. Default \code{c(1/3, 2/3)}.}
}
\value{
\code{data} with added \code{score} (numeric) and \code{tier} (ordered factor
Weak < Moderate < Strong), sorted by descending score.
}
\description{
Combines potency, efficacy and quality metrics into a single activity score
and assigns each compound to a Strong, Moderate or Weak tier, the final
prioritisation step of the pipeline. Lower AUC and IC50 and higher Emax count
as more active; lower replicate CV and higher fit \eqn{R^2} raise confidence.
}
\examples{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
ranked <- rank_hit_compound(params)
table(ranked$tier)

}
\seealso{
\code{\link[=annotate_hit_info]{annotate_hit_info()}}, \code{\link[=plot_hit_stratify_bar]{plot_hit_stratify_bar()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hts.R
\name{clean_compound_id}
\alias{clean_compound_id}
\title{Standardise compound identifiers}
\usage{
clean_compound_id(x, to_upper = TRUE)
}
\arguments{
\item{x}{A character vector of compound identifiers.}

\item{to_upper}{Logical; if \code{TRUE} (default) the result is upper-cased.}
}
\value{
A character vector of cleaned identifiers.
}
\description{
Cleans free-text compound identifiers into a consistent form: trims
surrounding whitespace, collapses internal whitespace, and replaces illegal
or separator characters with underscores so that IDs can be matched and used
safely in file names.
}
\examples{
clean_compound_id(c("  cpd 001 ", "Drug/A", "cpd-002!"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{plot_plate_qc}
\alias{plot_plate_qc}
\title{Visualise any plate quality-control metric}
\usage{
plot_plate_qc(qc, metric = "z_prime", threshold = NULL, plate_col = "plate_id")
}
\arguments{
\item{qc}{A data frame from \code{\link[=calc_plate_qc]{calc_plate_qc()}} for the plate-level metrics, or
from \code{\link[=calc_well_zscore]{calc_well_zscore()}} for the per-well \code{zscore}/\code{robust_zscore}
distributions.}

\item{metric}{The metric to plot: one of \code{"z_prime"}, \code{"robust_z_prime"},
\code{"z_factor"}, \code{"ssmd"}, \code{"sb"}, \code{"sn"}, \code{"cv_nc"}, \code{"cv_pc"}, \code{"cv"} (both
control CVs side by side), or \code{"zscore"}/\code{"robust_zscore"} for per-well
distributions. Default \code{"z_prime"}.}

\item{threshold}{Optional acceptance threshold drawn as a reference line;
defaults to the standard cut-off for the chosen metric.}

\item{plate_col}{Name of the plate-identifier column. Default \code{"plate_id"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
A single entry point for plotting the plate-quality metrics produced by
\code{\link[=calc_plate_qc]{calc_plate_qc()}} (and the per-well scores from \code{\link[=calc_well_zscore]{calc_well_zscore()}}). Choose
which assessment to draw through the \code{metric} argument; the acceptance
threshold, axis label and pass/fail colouring are set automatically for each
metric but can be overridden.
}
\examples{
raw <- read_hts_plate(hts_primary_raw)
qc <- calc_plate_qc(raw)
plot_plate_qc(qc, metric = "z_prime")
plot_plate_qc(qc, metric = "ssmd")
plot_plate_qc(qc, metric = "cv")
plot_plate_qc(calc_well_zscore(raw), metric = "robust_zscore")

}
\seealso{
\code{\link[=calc_plate_qc]{calc_plate_qc()}}, \code{\link[=calc_well_zscore]{calc_well_zscore()}}, \code{\link[=plot_plate_heatmap_raw]{plot_plate_heatmap_raw()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{calc_well_zscore}
\alias{calc_well_zscore}
\title{Per-well standard and robust Z-scores}
\usage{
calc_well_zscore(
  hts_raw,
  signal_col = "signal",
  reference = c("compound", "negative", "all"),
  nc_label = "NC",
  sample_label = "compound"
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column. Default \code{"signal"}.}

\item{reference}{One of \code{"compound"}, \code{"negative"} or \code{"all"}, selecting the
per-plate population that defines the centre and spread. Default
\code{"compound"}.}

\item{nc_label, sample_label}{Well-type labels for the negative controls and
test wells, used to pick the reference population.}
}
\value{
The input object with two added columns: \code{zscore} (mean/SD based) and
\code{robust_zscore} (median/MAD based, MAD scaled to the normal distribution).
}
\description{
Standardises every well's signal within its plate, adding both a classic
\strong{Z-score} (mean/SD based) and a \strong{robust Z-score} (median/MAD based, which
is the preferred choice in HTS because it is not distorted by the very hits
the screen is looking for). Scores are computed against a per-plate reference
population, by default the test-compound wells.
}
\examples{
raw <- read_hts_plate(hts_primary_raw)
scored <- calc_well_zscore(raw)
head(scored[, c("plate_id", "well", "zscore", "robust_zscore")])

}
\seealso{
\code{\link[=calc_plate_qc]{calc_plate_qc()}}, \code{\link[=plot_plate_qc]{plot_plate_qc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_dose_response_curves}
\alias{plot_dose_response_curves}
\title{Plot dose-response curves for two conditions}
\usage{
plot_dose_response_curves(
  data,
  drug = NULL,
  response = "normalized_cell_count",
  concentration = "concentration",
  condition = "condition",
  drug_col = "drug_name",
  n_points = 200,
  colors = c("#0072B2", "#D55E00", "#009E73", "#CC79A7"),
  shapes = c(19, 15, 17, 18),
  ref_line = 0.5,
  legend_title = "Condition"
)
}
\arguments{
\item{data}{A long-format data frame with one row per well/replicate,
containing drug, condition, concentration and response columns.}

\item{drug}{Optional drug name to display. If \code{NULL} (default) \code{data} is
assumed to already contain a single drug.}

\item{response}{Name of the (normalised) response column, as a string.
Default \code{"normalized_cell_count"}.}

\item{concentration}{Name of the concentration column, as a string. Default
\code{"concentration"}.}

\item{condition}{Name of the condition column, as a string. Default
\code{"condition"}.}

\item{drug_col}{Name of the drug identifier column, as a string. Default
\code{"drug_name"}.}

\item{n_points}{Number of points used to draw each smooth fitted curve.
Default \code{200}.}

\item{colors}{Character vector of colours, one per condition (recycled from
the pipeline palette by default).}

\item{shapes}{Numeric vector of point shapes, one per condition.}

\item{ref_line}{Numeric response level at which to draw a horizontal dashed
reference line (for example the 50 \% effect level). Set to \code{NA} to omit.}

\item{legend_title}{Legend title. Default \code{"Condition"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object with a base-10 logarithmic concentration
axis, observed points and fitted 4PL curves coloured by condition.
}
\description{
Fits a four-parameter logistic model to each condition of a single drug and
overlays the observed responses with the smooth fitted curves, coloured by
condition. This is the key visual for comparing drug sensitivity between two
conditions (for example irradiated versus non-irradiated cells), making a
potency shift between conditions immediately apparent.
}
\details{
The styling reproduces the plots used throughout the screening pipeline: a
logarithmic concentration axis, distinct colour and point shape per
condition, a horizontal 50 \% reference line, and the fitted IC50/AUC of each
condition reported in the subtitle.

Only concentrations that are finite and strictly positive are used
(a log scale cannot show zero or negative doses). A condition with fewer
than four usable observations is plotted as points only, with a warning,
and no curve is fitted for it. Fitting is performed with
\code{\link[=fit_dose_response]{fit_dose_response()}}, so no external fitting package is required. The
figure uses the package's modern-academic \code{\link[=theme_hts]{theme_hts()}} look with a
colourblind-safe condition palette.
}
\examples{
one <- subset(
  screen_doseresponse,
  drug_name == "DrugA" & experiment_type == "sample"
)
plot_dose_response_curves(one, drug = "DrugA")

}
\seealso{
\code{\link[=fit_dose_response]{fit_dose_response()}}, \code{\link[=four_pl]{four_pl()}}, \code{\link[=compute_auc]{compute_auc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module6_auc_matrix.R
\name{plot_auc_heatmap}
\alias{plot_auc_heatmap}
\title{AUC clustering heatmap}
\usage{
plot_auc_heatmap(
  mat,
  title = "AUC heatmap",
  cluster_rows = TRUE,
  cluster_columns = TRUE
)
}
\arguments{
\item{mat}{A numeric matrix from \code{\link[=build_auc_matrix]{build_auc_matrix()}}.}

\item{title}{Plot title. Default \code{"AUC heatmap"}.}

\item{cluster_rows, cluster_columns}{Whether to cluster rows/columns. Default
\code{TRUE}.}
}
\value{
Invisibly, the drawn object. Called for its side effect (a plot on
the active graphics device).
}
\description{
Draws a clustered heatmap of the AUC matrix. Uses \pkg{ComplexHeatmap} (with
\pkg{circlize} for the colour scale) when available for a publication-quality
figure with a diverging navy-white-firebrick palette matching
\code{\link[=plot_delta_auc_heatmap]{plot_delta_auc_heatmap()}}, otherwise falls back to base \code{\link[stats:heatmap]{stats::heatmap()}}.
}
\examples{
\donttest{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
plot_auc_heatmap(build_auc_matrix(drc$meta, zscore = "row"))
}

}
\seealso{
\code{\link[=build_auc_matrix]{build_auc_matrix()}}, \code{\link[=plot_cluster_tree]{plot_cluster_tree()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hit_selection.R
\name{select_hits_cutoff}
\alias{select_hits_cutoff}
\title{Select hits by an absolute cut-off across conditions}
\usage{
select_hits_cutoff(
  data,
  score_cols,
  cutoff = 0,
  min_pass = length(score_cols),
  drug_col = "drug_name"
)
}
\arguments{
\item{data}{A data frame with one row per drug: a drug identifier column plus
one numeric score column per condition or cell line (for example
\code{delta_auc} or a z-scored value).}

\item{score_cols}{Character vector of the score column names to test.}

\item{cutoff}{Numeric threshold. A drug passes a column when its value is
strictly below \code{cutoff}. Default \code{0}.}

\item{min_pass}{Minimum number of \code{score_cols} that must be below \code{cutoff}
for a drug to be selected. Defaults to all of them.}

\item{drug_col}{Name of the drug identifier column, as a string. Default
\code{"drug_name"}.}
}
\value{
The subset of \code{data} (rows) meeting the criterion, with an added
integer column \code{n_pass} giving the number of score columns below \code{cutoff}.
Rows containing missing scores in the tested columns are dropped.
}
\description{
Nominates drugs whose score is below a fixed cut-off in a required number of
the supplied score columns. This implements the cut-off based hit-selection
strategy used to intersect differential responses across several cancer cell
lines.
}
\examples{
df <- data.frame(
  drug_name = c("DrugA", "DrugB", "DrugC"),
  A549 = c(-0.5, 0.2, -0.9),
  H1299 = c(-0.3, -0.1, -0.8)
)
select_hits_cutoff(df, score_cols = c("A549", "H1299"), cutoff = 0,
                   min_pass = 2)

}
\seealso{
\code{\link[=select_hits_sigma]{select_hits_sigma()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module4_primary.R
\name{plot_sigma_hits}
\alias{plot_sigma_hits}
\title{Distribution plot of sigma-based hits}
\usage{
plot_sigma_hits(hits, bins = 30, show_normal = TRUE)
}
\arguments{
\item{hits}{A data frame from \code{\link[=select_sigma_hits]{select_sigma_hits()}}.}

\item{bins}{Number of histogram bins. Default \code{30}.}

\item{show_normal}{Overlay the fitted normal density curve. Default \code{TRUE}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Histogram of the per-compound activity with the library centre and the 1, 2
and 3 sigma cut-offs marked, bars coloured by how many sigma each compound
clears. Optionally overlays the fitted normal curve so the departure from
normality (the hit tail) stands out.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
plot_sigma_hits(select_sigma_hits(norm))

}
\seealso{
\code{\link[=select_sigma_hits]{select_sigma_hits()}}, \code{\link[=summarize_sigma_hits]{summarize_sigma_hits()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module2_norm.R
\name{plot_inhibition_hist}
\alias{plot_inhibition_hist}
\title{Histogram of compound inhibition on a plate}
\usage{
plot_inhibition_hist(
  hts_norm,
  plate = NULL,
  value = "inhibition",
  compound_label = "compound",
  bins = 30
)
}
\arguments{
\item{hts_norm}{An \code{hts_norm} object.}

\item{plate}{Optional plate identifier; if \code{NULL} (default) all compound
wells are pooled.}

\item{value}{Name of the value column. Default \code{"inhibition"}.}

\item{compound_label}{Well-type label marking compound wells. Default
\code{"compound"}.}

\item{bins}{Number of histogram bins. Default \code{30}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Shows the distribution of percent inhibition across the compound wells of a
plate (controls excluded), the standard view for judging overall hit density.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
plot_inhibition_hist(norm, plate = "P01")

}
\seealso{
\code{\link[=norm_by_control]{norm_by_control()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_control_scatter}
\alias{plot_control_scatter}
\title{Scatter plot of control readouts across plates}
\usage{
plot_control_scatter(
  data,
  plate_col = "plateID",
  neg_cols = grep("^neg_ctrl", names(data), value = TRUE),
  pos_cols = grep("^pos_ctrl", names(data), value = TRUE),
  colors = c(`Negative control` = "#0072B2", `Positive control` = "#D55E00"),
  y_title = "Normalized cell count"
)
}
\arguments{
\item{data}{A data frame with one row per plate and several negative- and
positive-control replicate columns (wide format, such as
\link{screen_plate_qc}).}

\item{plate_col}{Name of the plate identifier column, as a string. Default
\code{"plateID"}.}

\item{neg_cols, pos_cols}{Character vectors naming the negative- and
positive-control replicate columns. By default columns matching
\code{"^neg_ctrl"} and \code{"^pos_ctrl"} are used.}

\item{colors}{Named character vector giving the point colours for the two
control groups.}

\item{y_title}{Y-axis title. Default \code{"Normalized cell count"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object: control readouts (points, dodged and
jittered) per plate, coloured by control type, with the y-axis expanded to
include zero.
}
\description{
Draws, for each plate, the individual negative- and positive-control well
readouts (for example normalised cell counts) so that plate-to-plate control
behaviour and separation can be inspected at a glance. This is the standard
QC view used to confirm that negative controls stay high and positive
controls stay low across an entire screen.
}
\examples{
plot_control_scatter(screen_plate_qc)

}
\seealso{
\code{\link[=plot_qc_boxplot]{plot_qc_boxplot()}}, \code{\link[=plot_qc_circular_heatmap]{plot_qc_circular_heatmap()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module3_replicate.R
\name{calc_replicate_cv}
\alias{calc_replicate_cv}
\title{Replicate coefficient of variation per compound}
\usage{
calc_replicate_cv(
  data,
  value = "viability",
  compound_col = "compound_id",
  group_cols = NULL
)
}
\arguments{
\item{data}{An \code{hts_norm} object or a long data frame of well-level values.}

\item{value}{Name of the value column on which to compute the CV. Default
\code{"viability"} (more stable than inhibition, which can sit near zero).}

\item{compound_col}{Name of the compound column. Default \code{"compound_id"}.}

\item{group_cols}{Optional additional grouping columns (for example
\code{"cell_line"}) that define what counts as one compound series.}
}
\value{
A data frame with one row per compound (and any \code{group_cols}):
\code{compound_id}, the grouping columns, \code{n}, \code{mean}, \code{sd} and \code{cv} (percent).
}
\description{
Computes, for each compound, the coefficient of variation (CV, percent)
across its technical replicates (repeated wells, whether on the same plate or
across plates), a standard measure of assay reproducibility.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
cv <- calc_replicate_cv(norm)
head(cv[order(-cv$cv), ])

}
\seealso{
\code{\link[=calc_replicate_correlation]{calc_replicate_correlation()}}, \code{\link[=filter_bad_replicate]{filter_bad_replicate()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{screen_doseresponse}
\alias{screen_doseresponse}
\title{Example two-condition dose-response screen}
\format{
A data frame with one row per well and the following columns:
\describe{
\item{plateID}{Plate identifier (character).}
\item{drug_name}{Drug identifier, or a control label such as \code{"DMSO"}
(negative) or \code{"kill_ctrl"} (positive).}
\item{experiment_type}{Well role: \code{"sample"}, \code{"negative_ctrl"} or
\code{"positive_cttl"}.}
\item{condition}{Experimental condition, \code{"Gy0"} (baseline) or \code{"Gy2"}
(treatment).}
\item{concentration}{Drug concentration (micromolar).}
\item{normalized_cell_count}{Viability normalised to the negative control
(roughly 0-1).}
}
}
\usage{
screen_doseresponse
}
\description{
A small, simulated high-throughput drug-screening dataset in long (tidy)
format, mimicking a two-condition experiment in which each drug is tested
across a concentration series under a baseline (\code{"Gy0"}) and a treatment
(\code{"Gy2"}) condition, alongside negative and positive control wells. It is
sized to run the package examples and vignette quickly.
}
\details{
The data are simulated (see \code{data-raw/make_datasets.R}) and do not
correspond to any real compound or cell line.
}
\seealso{
\link{screen_delta_auc}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module7_report.R
\name{plot_primary_secondary_cor}
\alias{plot_primary_secondary_cor}
\title{Primary versus secondary activity correlation}
\usage{
plot_primary_secondary_cor(
  primary,
  secondary,
  by = "compound_id",
  primary_col = "activity",
  secondary_col = "auc"
)
}
\arguments{
\item{primary}{A data frame with a compound column and a primary-activity
column.}

\item{secondary}{A data frame with a compound column and a secondary-activity
(AUC) column.}

\item{by}{Join key. Default \code{"compound_id"}.}

\item{primary_col}{Primary-activity column. Default \code{"activity"}.}

\item{secondary_col}{Secondary-activity column. Default \code{"auc"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Scatter of primary-screen activity against secondary-screen AUC for the same
compounds, the standard check that primary hits reproduce in the dose-response
confirmation screen.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
primary <- select_primary_hit(norm)
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
sec <- aggregate(auc ~ compound_id, drc$meta, mean)
plot_primary_secondary_cor(primary, sec)

}
\seealso{
\code{\link[=select_primary_hit]{select_primary_hit()}}, \code{\link[=calc_drc_auc]{calc_drc_auc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dose_response.R
\name{compute_auc}
\alias{compute_auc}
\title{Area under a four-parameter logistic dose-response curve}
\usage{
compute_auc(min_concentration, max_concentration, top, ic50, hill, bottom)
}
\arguments{
\item{min_concentration}{Lower integration limit (minimum tested
concentration).}

\item{max_concentration}{Upper integration limit (maximum tested
concentration).}

\item{top, ic50, hill, bottom}{Four-parameter logistic parameters, as in
\code{\link[=four_pl]{four_pl()}}.}
}
\value{
A single numeric value: the integrated area under the curve. When
integration fails (for example a degenerate fit) \code{NA_real_} is returned
with a warning.
}
\description{
Integrates a fitted \code{\link[=four_pl]{four_pl()}} curve between two concentrations to obtain
the area under the curve (AUC), a robust summary of drug potency used for
between-condition comparisons.
}
\examples{
compute_auc(
  min_concentration = 0.01, max_concentration = 10,
  top = 1, ic50 = 1, hill = 1, bottom = 0
)

}
\seealso{
\code{\link[=four_pl]{four_pl()}}, \code{\link[=fit_dose_response]{fit_dose_response()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module2_norm.R
\name{merge_plate_data}
\alias{merge_plate_data}
\title{Combine normalised plates into one data set}
\usage{
merge_plate_data(...)
}
\arguments{
\item{...}{One or more \code{hts_norm}/data-frame objects, or a single list of
them.}
}
\value{
A combined \code{hts_norm} object.
}
\description{
Row-binds several normalised plates (or a list of them) into a single
\code{hts_norm} object holding the activity of the whole screen, keeping only the
columns shared by all inputs.
}
\examples{
n1 <- norm_by_control(read_hts_plate(
  hts_primary_raw[hts_primary_raw$plate_id == "P01", ]))
n2 <- norm_by_control(read_hts_plate(
  hts_primary_raw[hts_primary_raw$plate_id == "P02", ]))
merged <- merge_plate_data(n1, n2)
table(merged$plate_id)

}
\seealso{
\code{\link[=norm_by_control]{norm_by_control()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module3_replicate.R
\name{filter_bad_replicate}
\alias{filter_bad_replicate}
\title{Drop compounds with poor replicate reproducibility}
\usage{
filter_bad_replicate(cv, cv_max = 15, cor_result = NULL, r2_min = 0.8)
}
\arguments{
\item{cv}{A data frame from \code{\link[=calc_replicate_cv]{calc_replicate_cv()}}.}

\item{cv_max}{Maximum acceptable CV (percent). Default \code{15}.}

\item{cor_result}{Optional data frame from \code{\link[=calc_replicate_correlation]{calc_replicate_correlation()}}.}

\item{r2_min}{Minimum acceptable pairwise \eqn{R^2}. Default \code{0.8}.}
}
\value{
The subset of \code{cv} for compounds passing the CV threshold, with the
removed compound ids stored in \code{attr(x, "removed")}.
}
\description{
Removes compounds whose replicate coefficient of variation exceeds a
threshold. When a correlation table is supplied it also checks that the
replicate plates are globally concordant, warning if any pair falls below the
\eqn{R^2} threshold.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
cv <- calc_replicate_cv(norm)
good <- filter_bad_replicate(cv, cv_max = 20)
attr(good, "removed")

}
\seealso{
\code{\link[=calc_replicate_cv]{calc_replicate_cv()}}, \code{\link[=calc_replicate_correlation]{calc_replicate_correlation()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{calc_cv}
\alias{calc_cv}
\title{Control coefficient of variation per plate}
\usage{
calc_cv(
  hts_raw,
  signal_col = "signal",
  nc_label = "NC",
  pc_label = "PC",
  cv_max = 20
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column. Default \code{"signal"}.}

\item{nc_label, pc_label}{Well-type labels for the negative and positive
controls. Defaults \code{"NC"} and \code{"PC"}.}

\item{cv_max}{Acceptance threshold in percent, applied to both controls.
Default \code{20}.}
}
\value{
A data frame with one row per plate: \code{plate_id}, \code{cv_nc}, \code{cv_pc}
(percent) and a logical \code{pass} (both controls at or below \code{cv_max}).
}
\description{
The percent CV (\code{100 * SD / mean}) of each control population. \strong{What it is
for:} CV\% is the direct measure of pipetting and reagent-handling
reproducibility within a plate. Low, stable control CVs are a precondition
for every other metric; a control drifting above ~15--20\% flags a liquid-
handling or edge-effect problem even when the Z'-factor still passes.
}
\examples{
calc_cv(read_hts_plate(hts_primary_raw))

}
\seealso{
\code{\link[=calc_plate_qc]{calc_plate_qc()}}, \code{\link[=plot_plate_qc]{plot_plate_qc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module4_primary.R
\name{select_primary_hit}
\alias{select_primary_hit}
\title{Select primary-screen hits}
\usage{
select_primary_hit(
  data,
  value = "inhibition",
  method = c("threshold", "percentile"),
  threshold = 50,
  percentile = 0.95,
  compound_col = "compound_id"
)
}
\arguments{
\item{data}{An \code{hts_norm} object or long data frame.}

\item{value}{Name of the activity column. Default \code{"inhibition"}.}

\item{method}{Selection mode: \code{"threshold"} (default) keeps compounds with
activity at or above \code{threshold}; \code{"percentile"} keeps the top fraction.}

\item{threshold}{Activity cut-off for \code{method = "threshold"}. Default \code{50}.}

\item{percentile}{Upper fraction to keep for \code{method = "percentile"} (for
example \code{0.95} keeps the top 5 percent). Default \code{0.95}.}

\item{compound_col}{Name of the compound column. Default \code{"compound_id"}.}
}
\value{
A data frame with one row per compound: \code{compound_id}, \code{activity},
\code{is_hit}. The applied cut-off is stored in \code{attr(x, "cutoff")} and the
method in \code{attr(x, "method")}.
}
\description{
Flags active compounds from single-concentration primary-screen data using
either a fixed activity threshold or a percentile rank of the whole library.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
hits <- select_primary_hit(norm, threshold = 50)
sum(hits$is_hit)

}
\seealso{
\code{\link[=summarize_primary_hit]{summarize_primary_hit()}}, \code{\link[=plot_primary_inhibition_rank]{plot_primary_inhibition_rank()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module6_auc_matrix.R
\name{plot_cluster_tree}
\alias{plot_cluster_tree}
\title{Plot the compound clustering dendrogram}
\usage{
plot_cluster_tree(cluster_result, main = "Compound clustering")
}
\arguments{
\item{cluster_result}{The list returned by \code{\link[=cluster_auc_matrix]{cluster_auc_matrix()}}.}

\item{main}{Plot title. Default \code{"Compound clustering"}.}
}
\value{
Invisibly \code{NULL}; called for its side effect (a base-graphics plot).
}
\description{
Draws the hierarchical clustering tree of compounds from an AUC matrix.
}
\examples{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
m <- build_auc_matrix(drc$meta)
plot_cluster_tree(cluster_auc_matrix(m, k = 2))

}
\seealso{
\code{\link[=cluster_auc_matrix]{cluster_auc_matrix()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dose_response.R
\name{fit_dose_response}
\alias{fit_dose_response}
\alias{coef.dsr_4pl}
\alias{predict.dsr_4pl}
\alias{fitted.dsr_4pl}
\alias{residuals.dsr_4pl}
\alias{print.dsr_4pl}
\title{Fit a four-parameter logistic dose-response model}
\usage{
fit_dose_response(data, response, concentration, bounds = NULL)

\method{coef}{dsr_4pl}(object, ...)

\method{predict}{dsr_4pl}(object, newdata = NULL, ...)

\method{fitted}{dsr_4pl}(object, ...)

\method{residuals}{dsr_4pl}(object, ...)

\method{print}{dsr_4pl}(x, ...)
}
\arguments{
\item{data}{A data frame containing at least the response and concentration
columns for one drug and one condition.}

\item{response}{Name of the (normalised) response column, as a string.}

\item{concentration}{Name of the concentration column, as a string.}

\item{bounds}{Optional named list overriding the box constraints used by the
bounded optimiser. Recognised names are \code{top}, \code{bottom}, \code{ic50} and \code{hill},
each a numeric length-two vector \code{c(lower, upper)}.}

\item{object}{A \code{dsr_4pl} object.}

\item{...}{Ignored, for S3 method consistency.}

\item{newdata}{Optional numeric vector of concentrations at which to predict.
Defaults to the concentrations used for fitting.}

\item{x}{A \code{dsr_4pl} object.}
}
\value{
An object of class \code{dsr_4pl}: a list with elements \code{coefficients}
(named numeric vector \code{top}, \code{ic50}, \code{hill}, \code{bottom}), \code{fitted},
\code{residuals}, \code{convergence} (logical), \code{sse}, \code{data} and \code{call}. The
companion methods \code{coef()}, \code{predict()}, \code{fitted()}, \code{residuals()} and
\code{print()} are provided for this class.
}
\description{
Fits a 4PL curve for a single drug under one condition by non-linear least
squares, entirely in base R (no external fitting package required). Robust
starting values are derived from the data and refined with a bounded
\code{L-BFGS-B} optimisation over several initial \code{IC50} guesses, falling back to
a derivative-free \code{Nelder-Mead} search if needed. The model uses the same
parameterisation as \code{\link[=four_pl]{four_pl()}}, which matches the \code{dr4pl} package
convention (\code{UpperLimit}, \code{IC50}, \code{Slope}, \code{LowerLimit}).
}
\details{
The fitted parameters can be passed directly to \code{\link[=compute_auc]{compute_auc()}} to
obtain a potency summary, for example
\code{do.call(compute_auc, c(list(min(x), max(x)), as.list(coef(fit))))}.
}
\section{Functions}{
\itemize{
\item \code{coef(dsr_4pl)}: Extract the fitted 4PL coefficients (\code{top},
\code{ic50}, \code{hill}, \code{bottom}).

\item \code{predict(dsr_4pl)}: Predict responses from a fitted model.

\item \code{fitted(dsr_4pl)}: Extract fitted values.

\item \code{residuals(dsr_4pl)}: Extract residuals.

\item \code{print(dsr_4pl)}: Print a short summary of the fitted model.

}}
\examples{
one <- subset(
  screen_doseresponse,
  drug_name == "DrugA" & condition == "Gy0" & experiment_type == "sample"
)
fit <- fit_dose_response(one, "normalized_cell_count", "concentration")
coef(fit)
compute_auc(
  min(one$concentration), max(one$concentration),
  top = coef(fit)[["top"]], ic50 = coef(fit)[["ic50"]],
  hill = coef(fit)[["hill"]], bottom = coef(fit)[["bottom"]]
)

}
\seealso{
\code{\link[=four_pl]{four_pl()}}, \code{\link[=compute_auc]{compute_auc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hts.R
\name{plate_layout}
\alias{plate_layout}
\alias{plate_layout_384}
\alias{plate_layout_1536}
\title{Standard microplate coordinate map}
\usage{
plate_layout(format = 96)

plate_layout_384()

plate_layout_1536()
}
\arguments{
\item{format}{Plate format (number of wells): one of \code{6}, \code{12}, \code{24}, \code{48},
\code{96}, \code{384} or \code{1536}. Default \code{96}.}
}
\value{
A data frame with one row per well and the columns \code{well} (for
example \code{"A1"}), \code{row} (row letter), \code{col} (column number) and \code{row_index}
(numeric row position, \code{A = 1}).
}
\description{
Builds the full row/column coordinate table for a standard microplate,
ordered by row then column. This is useful for joining plate readouts onto a
complete layout so that empty wells are represented explicitly.
}
\examples{
head(plate_layout(96))
nrow(plate_layout(384))

}
\seealso{
\code{\link[=plate_layout_384]{plate_layout_384()}}, \code{\link[=plate_layout_1536]{plate_layout_1536()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module4_primary.R
\name{summarize_primary_hit}
\alias{summarize_primary_hit}
\title{Summarise primary-screen hits}
\usage{
summarize_primary_hit(hits)
}
\arguments{
\item{hits}{A data frame from \code{\link[=select_primary_hit]{select_primary_hit()}}.}
}
\value{
A one-row data frame with \code{n_compounds}, \code{n_hits}, \code{hit_rate}
(fraction) and \code{cutoff}.
}
\description{
Reports the number of compounds screened, the number of hits and the hit
rate.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
summarize_primary_hit(select_primary_hit(norm))

}
\seealso{
\code{\link[=select_primary_hit]{select_primary_hit()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_hts.R
\name{theme_hts}
\alias{theme_hts}
\title{Modern academic ggplot2 theme for diffHTS figures}
\usage{
theme_hts(
  base_size = 12,
  base_family = "",
  legend = "bottom",
  grid = c("y", "x", "both", "none")
)
}
\arguments{
\item{base_size}{Base font size in points. Default \code{12}.}

\item{base_family}{Base font family; defaults to the device sans-serif.}

\item{legend}{Legend position passed to \code{ggplot2::theme()}. Default
\code{"bottom"}.}

\item{grid}{Which major grid lines to keep: \code{"y"} (default), \code{"x"}, \code{"both"}
or \code{"none"}.}
}
\value{
A \link[ggplot2:theme]{ggplot2::theme} object that can be added to any ggplot.
}
\description{
A minimal, publication-ready theme: pure-white background, an L-shaped axis
(left and bottom only), light dashed horizontal major grid lines with no
minor grid, restrained dark-grey typography and generous margins. Pair it
with \code{\link[=hts_pal]{hts_pal()}} for discrete colours or a \code{viridis}/muted-blue gradient for
continuous ones.
}
\examples{
library(ggplot2)
ggplot(mtcars, aes(factor(cyl), mpg)) +
  geom_boxplot() +
  theme_hts()

}
\seealso{
\code{\link[=hts_pal]{hts_pal()}}, \code{\link[=plot_plate_qc]{plot_plate_qc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{calc_z_prime}
\alias{calc_z_prime}
\title{Compute per-plate Z' factors}
\usage{
calc_z_prime(
  hts_raw,
  signal_col = "signal",
  nc_label = "NC",
  pc_label = "PC",
  z_prime_min = 0.5
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column. Default \code{"signal"}.}

\item{nc_label, pc_label}{Well-type labels for the negative and positive
controls. Defaults \code{"NC"} and \code{"PC"}.}

\item{z_prime_min}{Acceptance threshold. Default \code{0.5}.}
}
\value{
A data frame with one row per plate: \code{plate_id}, \code{mean_nc},
\code{mean_pc}, \code{sd_nc}, \code{sd_pc}, \code{z_prime} and a logical \code{pass}.
}
\description{
Calculates the Z' (Z-prime) factor for each plate from its negative- and
positive-control populations and flags plates against an acceptance
threshold.
}
\examples{
raw <- read_hts_plate(hts_primary_raw)
calc_z_prime(raw)

}
\seealso{
\code{\link[=filter_valid_plates]{filter_valid_plates()}}, \code{\link[=plot_plate_qc]{plot_plate_qc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/delta.R
\name{compute_delta_auc}
\alias{compute_delta_auc}
\title{Differential (delta) AUC between two screening conditions}
\usage{
compute_delta_auc(
  data,
  drug_col = "drug_name",
  condition_col = "condition",
  auc_col = "auc",
  conditions = NULL
)
}
\arguments{
\item{data}{A long-format data frame with one row per drug and condition.}

\item{drug_col}{Name of the drug identifier column, as a string. Default
\code{"drug_name"}.}

\item{condition_col}{Name of the condition column, as a string. Default
\code{"condition"}. The column must contain exactly the two values given in
\code{conditions}.}

\item{auc_col}{Name of the AUC column, as a string. Default \code{"auc"}.}

\item{conditions}{Character vector of length two giving the baseline and
treatment condition, in the order \code{c(baseline, treatment)}. The delta is
computed as \code{treatment - baseline}. Defaults to the two levels found in
\code{condition_col}.}
}
\value{
A data frame with one row per drug containing the two per-condition
AUC columns (named \verb{auc_<condition>}), \code{delta_auc} and \code{zscore_delta_auc}
(the z-scored \code{delta_auc} across all drugs).
}
\description{
Contrasts drug potency between two experimental conditions by reshaping a
long per-drug/per-condition AUC table to one row per drug and computing the
differential AUC together with its within-experiment z-score. This is the
core scoring step for two-condition screens (for example irradiated versus
non-irradiated, or treated versus control).
}
\details{
A negative \code{delta_auc} indicates the treatment condition sensitises
cells to the drug relative to baseline (lower AUC = more killing). The
z-score standardises \code{delta_auc} across all drugs within the experiment so
that hits can be selected on a common scale (see \code{\link[=select_hits_sigma]{select_hits_sigma()}}).
}
\examples{
auc_long <- data.frame(
  drug_name = rep(c("DrugA", "DrugB", "DrugC"), each = 2),
  condition = rep(c("Gy0", "Gy2"), times = 3),
  auc = c(5.0, 3.1, 4.2, 4.0, 6.0, 2.0)
)
compute_delta_auc(auc_long, conditions = c("Gy0", "Gy2"))

}
\seealso{
\code{\link[=select_hits_cutoff]{select_hits_cutoff()}}, \code{\link[=select_hits_sigma]{select_hits_sigma()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hts_compound_meta}
\alias{hts_compound_meta}
\title{Example compound metadata}
\format{
A data frame with one row per compound and the following columns:
\describe{
\item{compound_id}{Compound identifier such as \code{"CPD001"} (character).}
\item{compound_name}{Human-readable compound name (character).}
\item{target}{Nominal molecular target (character).}
\item{moa}{Mechanism of action (character).}
\item{library_source}{Source library the compound came from (character).}
}
}
\usage{
hts_compound_meta
}
\description{
A small, simulated compound annotation table used to demonstrate hit
annotation. Keyed by \code{compound_id} to join onto screening results.
}
\details{
The data are simulated (see \code{data-raw/make_datasets.R}) and do not
correspond to any real compound.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{apply_plate_layout}
\alias{apply_plate_layout}
\title{Stamp a plate-map layout onto measured readings}
\usage{
apply_plate_layout(
  readings,
  layout,
  well_col = "well",
  signal_col = "signal",
  plate_col = "plate_id",
  plate_id = NULL
)
}
\arguments{
\item{readings}{A data frame (or \code{hts_raw}) of readings with at least a well
column and a signal column.}

\item{layout}{An \code{hts_layout} from \code{\link[=read_plate_layout]{read_plate_layout()}}, or a data frame with
\code{well}, \code{well_type}, \code{compound_id} and (optionally) \code{concentration}.}

\item{well_col, signal_col}{Names of the well and signal columns in
\code{readings}.}

\item{plate_col}{Name of the plate column. When present in both \code{readings}
and a multi-plate \code{layout}, wells are matched within each plate.}

\item{plate_id}{Optional plate identifier used when \code{readings} has no plate
column.}
}
\value{
An \code{hts_raw} object with columns \code{plate_id}, \code{well}, \code{row}, \code{col},
\code{well_type}, \code{compound_id}, \code{concentration} and \code{signal}, ready for the rest
of the pipeline.
}
\description{
Joins a user-defined layout from \code{\link[=read_plate_layout]{read_plate_layout()}} onto a table of raw
signal readings, matching by well address (and plate identifier when both
sides carry one), and returns a standard \code{hts_raw} object. This keeps the
\emph{what was measured} (signal) and the \emph{what each well is} (role, compound,
concentration) in separate files, exactly as a screening lab records them,
and lets the same signal export be re-interpreted against different layouts.
}
\examples{
f <- system.file("extdata", "plate_layout_example.csv", package = "diffHTS")
layout <- read_plate_layout(f, plate_id = "P01")
# Simulate a signal export for the same wells.
readings <- data.frame(well = layout$well,
                        signal = runif(nrow(layout), 1e4, 1e5))
raw <- apply_plate_layout(readings, layout, plate_id = "P01")
table(raw$well_type)

}
\seealso{
\code{\link[=read_plate_layout]{read_plate_layout()}}, \code{\link[=read_hts_plate]{read_hts_plate()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module6_auc_matrix.R
\name{build_auc_matrix}
\alias{build_auc_matrix}
\title{Build a compound-by-sample AUC matrix}
\usage{
build_auc_matrix(
  data,
  row_col = "compound_id",
  col_col = "cell_line",
  value_col = "auc",
  zscore = c("none", "row", "column")
)
}
\arguments{
\item{data}{A data frame with compound, sample and AUC columns (typically
\code{drc$meta} after \code{\link[=calc_drc_auc]{calc_drc_auc()}}).}

\item{row_col}{Compound column. Default \code{"compound_id"}.}

\item{col_col}{Sample column. Default \code{"cell_line"}.}

\item{value_col}{AUC column. Default \code{"auc"}.}

\item{zscore}{Standardisation: \code{"none"} (default), \code{"row"} or \code{"column"}.}
}
\value{
A numeric matrix (compounds x samples). The standardisation used is
stored in \code{attr(x, "zscore")}.
}
\description{
Reshapes long AUC results into a numeric matrix with compounds as rows and
samples (for example cell lines or batches) as columns, optionally
standardised to Z-scores for comparability.
}
\examples{
\donttest{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
build_auc_matrix(drc$meta)
}

}
\seealso{
\code{\link[=cluster_auc_matrix]{cluster_auc_matrix()}}, \code{\link[=plot_auc_heatmap]{plot_auc_heatmap()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{calc_sn_ratio}
\alias{calc_sn_ratio}
\title{Signal-to-noise ratio per plate}
\usage{
calc_sn_ratio(
  hts_raw,
  signal_col = "signal",
  nc_label = "NC",
  pc_label = "PC",
  sn_min = 3
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column. Default \code{"signal"}.}

\item{nc_label, pc_label}{Well-type labels for the negative and positive
controls. Defaults \code{"NC"} and \code{"PC"}.}

\item{sn_min}{Acceptance threshold. Default \code{3}.}
}
\value{
A data frame with one row per plate: \code{plate_id}, \code{mean_nc},
\code{mean_pc}, \code{sn} (window / low-control SD) and a logical \code{pass}.
}
\description{
The assay window divided by the noise of the low-signal control. \strong{What it is
for:} S/N asks whether the separation between the two controls is large
compared with the background scatter, i.e. whether a real change would rise
above the noise floor. It is more informative than S/B because it accounts
for variability; higher is better, with \verb{>= 3} a reasonable floor.
}
\examples{
calc_sn_ratio(read_hts_plate(hts_primary_raw))

}
\seealso{
\code{\link[=calc_sb_ratio]{calc_sb_ratio()}}, \code{\link[=calc_plate_qc]{calc_plate_qc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module5_drc.R
\name{extract_drc_params}
\alias{extract_drc_params}
\title{Extract dose-response parameters}
\usage{
extract_drc_params(drc)
}
\arguments{
\item{drc}{An \code{hts_drc} object from \code{\link[=fit_4pl_curve]{fit_4pl_curve()}}.}
}
\value{
A data frame with one row per curve: keys plus \code{ic50}, \code{emax} (\code{top}),
\code{bottom}, \code{hill}, \code{rsq} and \code{pass}. Curves that failed to fit yield \code{NA}
parameters.
}
\description{
Pulls the fitted potency and efficacy parameters from every curve of an
\code{hts_drc} object.
}
\examples{
drc <- fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line")
head(extract_drc_params(drc))

}
\seealso{
\code{\link[=fit_4pl_curve]{fit_4pl_curve()}}, \code{\link[=calc_drc_auc]{calc_drc_auc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dose_response.R
\name{four_pl}
\alias{four_pl}
\title{Four-parameter logistic (4PL) dose-response function}
\usage{
four_pl(x, top, ic50, hill, bottom)
}
\arguments{
\item{x}{Numeric vector of drug concentrations (on the same scale used to fit
the model, typically the raw concentration).}

\item{top}{Response of the upper asymptote (e.g. viability with no drug).}

\item{ic50}{Concentration producing the half-maximal response.}

\item{hill}{Hill slope controlling the steepness of the curve.}

\item{bottom}{Response of the lower asymptote (e.g. viability at saturating
drug).}
}
\value{
A numeric vector of predicted responses, the same length as \code{x}.
}
\description{
Evaluates the classic four-parameter logistic model used to describe
drug dose-response curves in high-throughput screens.
}
\details{
The model is
\deqn{f(x) = \mathrm{bottom} +
  \frac{\mathrm{top} - \mathrm{bottom}}{1 + (x / \mathrm{ic50})^{\mathrm{hill}}}}
}
\examples{
four_pl(x = c(0.01, 0.1, 1, 10), top = 1, ic50 = 1, hill = 1, bottom = 0)

}
\seealso{
\code{\link[=compute_auc]{compute_auc()}}, \code{\link[=fit_dose_response]{fit_dose_response()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_condition_scatter}
\alias{plot_condition_scatter}
\title{Scatter plot comparing two conditions or cell lines}
\usage{
plot_condition_scatter(data, x, y, drug_col = "drug_name", label_n = 5)
}
\arguments{
\item{data}{A data frame with one row per drug.}

\item{x, y}{Names of the two numeric score columns to place on the x and y
axes, as strings.}

\item{drug_col}{Name of the drug identifier column, as a string. Default
\code{"drug_name"}.}

\item{label_n}{Number of most-extreme drugs (by distance from the origin) to
label per quadrant. Default \code{5}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Plots a per-drug scatter of a score (for example delta AUC) under two
conditions or cell lines, colouring points by quadrant and labelling the most
extreme drugs in each quadrant. This is useful for spotting drugs that
behave differently between, say, a cancer and a normal cell line.
}
\details{
Point labelling uses \pkg{ggrepel} when available and falls back to
\code{\link[ggplot2:geom_text]{ggplot2::geom_text()}} otherwise. The axes are centred on the origin with a
fixed aspect ratio so the four quadrants are equal in size.
}
\examples{
df <- data.frame(
  drug_name = paste0("Drug", 1:8),
  MRC5 = c(-1, 0.5, -0.2, 1.2, -0.8, 0.3, -1.5, 0.9),
  A549 = c(-1.2, 0.1, 0.7, -0.9, -1.1, 1.0, -0.4, 0.6)
)
plot_condition_scatter(df, x = "MRC5", y = "A549")

}
\seealso{
\code{\link[=plot_delta_auc_heatmap]{plot_delta_auc_heatmap()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module6_auc_matrix.R
\name{cluster_auc_matrix}
\alias{cluster_auc_matrix}
\title{Hierarchically cluster an AUC matrix}
\usage{
cluster_auc_matrix(mat, k = 3, distance = "euclidean", method = "complete")
}
\arguments{
\item{mat}{A numeric matrix from \code{\link[=build_auc_matrix]{build_auc_matrix()}}.}

\item{k}{Number of compound clusters to cut. Default \code{3}.}

\item{distance}{Distance method for \code{\link[stats:dist]{stats::dist()}}. Default \code{"euclidean"}.}

\item{method}{Linkage method for \code{\link[stats:hclust]{stats::hclust()}}. Default \code{"complete"}.}
}
\value{
A list with \code{row_hclust}, \code{col_hclust} (or \code{NULL} if too few
columns), \code{clusters} (named integer vector of compound cluster ids) and
\code{k}.
}
\description{
Performs hierarchical clustering of the compounds (rows) and samples
(columns) of an AUC matrix and cuts the compound tree into groups.
}
\examples{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
cl <- cluster_auc_matrix(build_auc_matrix(drc$meta), k = 2)
table(cl$clusters)

}
\seealso{
\code{\link[=extract_cluster_hit]{extract_cluster_hit()}}, \code{\link[=plot_cluster_tree]{plot_cluster_tree()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module5_drc.R
\name{import_dose_response}
\alias{import_dose_response}
\title{Import gradient dose-response data}
\usage{
import_dose_response(
  data,
  compound_col = "compound_id",
  concentration_col = "concentration",
  response_col = "viability",
  group_cols = NULL
)
}
\arguments{
\item{data}{A data frame, or a path to a csv/txt/xlsx file.}

\item{compound_col, concentration_col, response_col}{Names of the compound,
concentration and response columns in \code{data}.}

\item{group_cols}{Optional grouping columns kept alongside each curve (for
example \code{"cell_line"}).}
}
\value{
An \code{hts_dose} object (data frame) with columns \code{compound_id},
\code{concentration}, \code{response}, any \code{group_cols}, and canonical class.
}
\description{
Standardises a secondary (dose-response) screening table into the columns
expected by the fitting engine, matching each compound to its concentration
gradient and response, ready for \code{\link[=fit_4pl_curve]{fit_4pl_curve()}}.
}
\examples{
dr <- import_dose_response(hts_dose_response, response_col = "viability",
  group_cols = "cell_line")
head(dr)

}
\seealso{
\code{\link[=fit_4pl_curve]{fit_4pl_curve()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{calc_robust_z_prime}
\alias{calc_robust_z_prime}
\title{Robust Z'-factor per plate}
\usage{
calc_robust_z_prime(
  hts_raw,
  signal_col = "signal",
  nc_label = "NC",
  pc_label = "PC",
  z_prime_min = 0.5
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column. Default \code{"signal"}.}

\item{nc_label, pc_label}{Well-type labels for the negative and positive
controls. Defaults \code{"NC"} and \code{"PC"}.}

\item{z_prime_min}{Acceptance threshold (\verb{>= 0.5} is the usual cut-off for an
excellent assay). Default \code{0.5}.}
}
\value{
A data frame with one row per plate: \code{plate_id}, \code{median_nc},
\code{median_pc}, \code{mad_nc}, \code{mad_pc}, \code{robust_z_prime} and a logical \code{pass}.
}
\description{
Median/MAD version of the Z'-factor. \strong{What it is for:} it answers the same
question as \code{\link[=calc_z_prime]{calc_z_prime()}} — how cleanly the positive and negative controls
separate — but because it uses the median and the median absolute deviation
instead of the mean and SD, a couple of splashed or crystallised control
wells cannot single-handedly sink an otherwise good plate. Prefer it when
control replicates occasionally contain outliers.
}
\examples{
calc_robust_z_prime(read_hts_plate(hts_primary_raw))

}
\seealso{
\code{\link[=calc_z_prime]{calc_z_prime()}}, \code{\link[=calc_plate_qc]{calc_plate_qc()}}, \code{\link[=plot_plate_qc]{plot_plate_qc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module3_replicate.R
\name{calc_replicate_correlation}
\alias{calc_replicate_correlation}
\title{Pairwise replicate correlation between plates}
\usage{
calc_replicate_correlation(
  data,
  value = "viability",
  compound_col = "compound_id",
  plate_col = "plate_id"
)
}
\arguments{
\item{data}{An \code{hts_norm} object or long data frame.}

\item{value}{Name of the value column. Default \code{"viability"}.}

\item{compound_col}{Name of the compound column. Default \code{"compound_id"}.}

\item{plate_col}{Name of the plate column. Default \code{"plate_id"}.}
}
\value{
A data frame with one row per plate pair: \code{plate_x}, \code{plate_y}, \code{n},
\code{pearson_r} and \code{r_squared}. The wide compound-by-plate matrix is attached
as \code{attr(x, "wide")} for use by \code{\link[=plot_replicate_scatter]{plot_replicate_scatter()}}.
}
\description{
Computes the Pearson correlation (and its \eqn{R^2}) between the per-compound
activity of every pair of replicate plates, the standard check that technical
replicates agree.
}
\examples{
norm <- norm_by_control(baseline_subtract(read_hts_plate(hts_primary_raw)))
calc_replicate_correlation(norm)

}
\seealso{
\code{\link[=calc_replicate_cv]{calc_replicate_cv()}}, \code{\link[=plot_replicate_scatter]{plot_replicate_scatter()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hts.R
\name{check_control_label}
\alias{check_control_label}
\title{Validate control-well labelling}
\usage{
check_control_label(data, type_col = "well_type", required = c("NC", "PC"))
}
\arguments{
\item{data}{A data frame of well-level readouts.}

\item{type_col}{Name of the well-type column, as a string. Default
\code{"well_type"}.}

\item{required}{Character vector of labels that must be present. Default
\code{c("NC", "PC")}; add \code{"Blank"} if background subtraction is planned.}
}
\value{
Invisibly, a list with \code{ok} (logical), \code{present} and \code{missing}
(character vectors) and \code{counts} (a table of well-type frequencies). A
warning is emitted when any required label is missing.
}
\description{
Checks that a data frame contains the expected negative-, positive- and
blank-control labels in its well-type column, so that normalisation and QC
can be carried out. It is a light-weight guard to run right after import.
}
\examples{
d <- data.frame(well_type = c("NC", "NC", "PC", "compound"))
check_control_label(d)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{read_hts_plate}
\alias{read_hts_plate}
\title{Import a plate readout file into a standard HTS object}
\usage{
read_hts_plate(
  file,
  plate_id = NULL,
  well_col = "well",
  signal_col = "signal",
  compound_col = "compound_id",
  conc_col = "concentration",
  type_col = "well_type",
  plate_col = "plate_id",
  nc_label = "NC",
  pc_label = "PC",
  blank_label = "Blank",
  sep = "\\t",
  sheet = 1
)
}
\arguments{
\item{file}{A path to a \code{.csv}, \code{.txt}/\code{.tsv} or \code{.xlsx}/\code{.xls} file, or an
existing data frame of well-level readings.}

\item{plate_id}{Optional plate identifier used when the data has no plate
column.}

\item{well_col, signal_col, compound_col, conc_col, type_col, plate_col}{Names of
the well, signal, compound, concentration, well-type and plate columns in
\code{file}. Missing optional columns are filled with \code{NA}.}

\item{nc_label, pc_label, blank_label}{Character vectors of the labels that
mark negative controls, positive controls and blanks (matched
case-insensitively in \code{type_col}, or in \code{compound_col} when \code{type_col} is
absent).}

\item{sep}{Field separator for \code{.txt}/\code{.tsv} files. Default tab.}

\item{sheet}{Worksheet passed to \pkg{readxl} for Excel files. Default \code{1}.}
}
\value{
An \code{hts_raw} object (a data frame subclass) with columns \code{plate_id},
\code{well}, \code{row}, \code{col}, \code{well_type}, \code{compound_id}, \code{concentration} and
\code{signal}.
}
\description{
Reads an instrument export (csv, txt/tsv or Excel) or an in-memory data frame
of well-level readouts and standardises it into an \code{hts_raw} object with a
fixed set of columns, ready for the rest of the pipeline. Well coordinates are
parsed from the well identifier and control wells are canonicalised to the
labels \code{"NC"}, \code{"PC"} and \code{"Blank"}.
}
\details{
Reading Excel files requires the suggested package \pkg{readxl}.
}
\examples{
raw <- read_hts_plate(hts_primary_raw)
table(raw$well_type)

}
\seealso{
\code{\link[=baseline_subtract]{baseline_subtract()}}, \code{\link[=calc_z_prime]{calc_z_prime()}}, \code{\link[=norm_by_control]{norm_by_control()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qc.R
\name{flag_qc_plates}
\alias{flag_qc_plates}
\title{Flag plates that fail quality-control thresholds}
\usage{
flag_qc_plates(qc, z_prime_min = 0.4, sb_min = 3, cv_max = 10)
}
\arguments{
\item{qc}{A data frame of plate quality metrics. Columns referenced by the
active thresholds must be present (\code{z_prime}, \code{sb_value} and, if supplied,
\code{cv_negative_ctrl}).}

\item{z_prime_min}{Minimum acceptable Z-prime. Plates below this are flagged
\code{"Bad"}. Default \code{0.4}.}

\item{sb_min}{Minimum acceptable signal-to-background ratio. Default \code{3}.}

\item{cv_max}{Maximum acceptable coefficient of variation (percent) of the
negative control. Only applied when a \code{cv_negative_ctrl} column is present.
Default \code{10}.}
}
\value{
The input data frame with added character flag columns
\code{z_primer_flag}, \code{sb_flag} and (when applicable) \code{cv_flag}, each taking the
value \code{"Bad"} or \code{"Normal"}.
}
\description{
Adds pass/fail flags to a table of plate quality metrics such as the one
produced by \code{\link[=calculate_qc_metrics]{calculate_qc_metrics()}}, using conventional acceptance
thresholds.
}
\examples{
qc <- data.frame(
  plateID = c("P01", "P02"),
  z_prime = c(0.62, 0.20),
  sb_value = c(5.1, 2.4),
  cv_negative_ctrl = c(6, 14)
)
flag_qc_plates(qc)

}
\seealso{
\code{\link[=calculate_qc_metrics]{calculate_qc_metrics()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module7_report.R
\name{generate_hts_report}
\alias{generate_hts_report}
\title{Generate an HTML screening report}
\usage{
generate_hts_report(
  params = list(),
  output_file = tempfile(fileext = ".html"),
  template = NULL,
  quiet = TRUE
)
}
\arguments{
\item{params}{A named list of objects made available to the report template
(passed as \code{params}), for example \code{list(hits = ranked, drc = drc)}.}

\item{output_file}{Output HTML path. Default a temporary file.}

\item{template}{Path to an \code{.Rmd} template. Defaults to the package template
in \code{inst/rmd/report_template.Rmd}.}

\item{quiet}{Passed to \code{\link[rmarkdown:render]{rmarkdown::render()}}. Default \code{TRUE}.}
}
\value{
The path to the rendered report, invisibly.
}
\description{
Renders a standard HTML analysis report from the bundled R Markdown template,
embedding the supplied QC, dose-response and heatmap results. Requires the
suggested \pkg{rmarkdown} package.
}
\examples{
\dontrun{
drc <- calc_drc_auc(fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line"))
params <- merge(extract_drc_params(drc), drc$meta[, c("curve_id", "auc")])
generate_hts_report(list(hits = rank_hit_compound(params)))
}

}
\seealso{
\code{\link[=export_hit_table]{export_hit_table()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hts.R
\name{convert_conc_log10}
\alias{convert_conc_log10}
\title{Convert concentrations to a log10 scale}
\usage{
convert_conc_log10(x, shift_min = FALSE)
}
\arguments{
\item{x}{Numeric vector of concentrations (factors/characters are coerced).}

\item{shift_min}{Logical; if \code{TRUE} the minimum finite \code{log10} value is
subtracted so that the series starts at zero (matching the pipeline's
\code{log10(conc) - min(log10(conc))} convention). Default \code{FALSE}.}
}
\value{
A numeric vector of \code{log10} concentrations.
}
\description{
Transforms a concentration vector to \code{log10}, the scale on which
dose-response curves are usually drawn and fitted. Non-positive values (for
example the zero-dose controls) become \code{NA}.
}
\examples{
convert_conc_log10(c(0.01, 0.1, 1, 10, 100))
convert_conc_log10(c(0.01, 0.1, 1, 10, 100), shift_min = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hts_dose_response}
\alias{hts_dose_response}
\title{Example gradient dose-response secondary screen}
\format{
A data frame with one row per measurement and the following columns:
\describe{
\item{compound_id}{Compound identifier such as \code{"CPD001"} (character).}
\item{cell_line}{Cell line, \code{"A549"}, \code{"H1299"} or \code{"MRC5"} (character).}
\item{concentration}{Compound concentration (micromolar) (numeric).}
\item{replicate}{Technical replicate index, \code{1} or \code{2} (integer).}
\item{viability}{Percent viability relative to vehicle control (numeric).}
}
}
\usage{
hts_dose_response
}
\description{
A small, simulated dose-response confirmation screen: ten compounds tested
across a concentration gradient in three cell lines with two replicates each.
Used to demonstrate the dose-response fitting, AUC and clustering modules.
}
\details{
The data are simulated (see \code{data-raw/make_datasets.R}); the cell
line \code{"MRC5"} is made comparatively resistant.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module5_drc.R
\name{fit_4pl_curve}
\alias{fit_4pl_curve}
\title{Fit four-parameter logistic curves per compound}
\usage{
fit_4pl_curve(
  data,
  response = "response",
  concentration = "concentration",
  compound_col = "compound_id",
  group_cols = NULL,
  rsq_min = 0.85
)
}
\arguments{
\item{data}{An \code{hts_dose} object from \code{\link[=import_dose_response]{import_dose_response()}} (or a long
data frame with \code{compound_id}, \code{concentration}, \code{response}).}

\item{response, concentration, compound_col}{Column names. Default
\code{"response"}, \code{"concentration"}, \code{"compound_id"}.}

\item{group_cols}{Optional grouping columns.}

\item{rsq_min}{Curves with \eqn{R^2} below this are flagged (not dropped
here; use \code{\link[=filter_low_quality_curve]{filter_low_quality_curve()}}). Default \code{0.85}.}
}
\value{
An \code{hts_drc} object: a list with \code{meta} (one row per curve: keys,
\code{n}, \code{rsq}, \code{converged}, \code{pass}), \code{fits} (named list of \code{dsr_4pl} models)
and \code{data} (the input). \code{group_cols} is stored as an attribute.
}
\description{
Fits a 4PL dose-response model to every compound (optionally split by
grouping columns such as cell line), reusing the package's self-contained
fitter, and records the goodness of fit \eqn{R^2}. No external fitting
package is required.
}
\examples{
dr <- import_dose_response(hts_dose_response, response_col = "viability",
  group_cols = "cell_line")
drc <- fit_4pl_curve(dr, group_cols = "cell_line")
head(drc$meta)

}
\seealso{
\code{\link[=extract_drc_params]{extract_drc_params()}}, \code{\link[=calc_drc_auc]{calc_drc_auc()}}, \code{\link[=plot_single_drc]{plot_single_drc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module5_drc.R
\name{filter_low_quality_curve}
\alias{filter_low_quality_curve}
\title{Drop low-quality dose-response curves}
\usage{
filter_low_quality_curve(drc, rsq_min = NULL)
}
\arguments{
\item{drc}{An \code{hts_drc} object.}

\item{rsq_min}{Minimum acceptable \eqn{R^2}. Defaults to the value stored at
fit time.}
}
\value{
The filtered \code{hts_drc} object; removed curve ids are stored in
\code{attr(drc, "removed")}.
}
\description{
Removes curves that failed to fit or whose \eqn{R^2} falls below the
threshold, leaving only trustworthy dose-response models.
}
\examples{
drc <- fit_4pl_curve(import_dose_response(hts_dose_response,
  response_col = "viability", group_cols = "cell_line"),
  group_cols = "cell_line")
clean <- filter_low_quality_curve(drc)
nrow(clean$meta)

}
\seealso{
\code{\link[=fit_4pl_curve]{fit_4pl_curve()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_qc_boxplot}
\alias{plot_qc_boxplot}
\title{Quality-control boxplot of control wells across plates}
\usage{
plot_qc_boxplot(
  data,
  plate_col = "plateID",
  value_col = "normalized_cell_count",
  group_col = "experiment_type",
  y_title = "Normalized cell count"
)
}
\arguments{
\item{data}{A long-format data frame of well-level readouts.}

\item{plate_col}{Name of the plate identifier column, as a string. Default
\code{"plateID"}.}

\item{value_col}{Name of the numeric readout column, as a string. Default
\code{"normalized_cell_count"}.}

\item{group_col}{Name of the control-type / grouping column, as a string.
Default \code{"experiment_type"}.}

\item{y_title}{Axis title for the readout. Default
\code{"Normalized cell count"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Draws grouped boxplots of a per-well readout across plates, split by control
type, to inspect assay consistency and separation between negative and
positive controls.
}
\examples{
plot_qc_boxplot(screen_doseresponse)

}
\seealso{
\code{\link[=calculate_qc_metrics]{calculate_qc_metrics()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module1_preqc.R
\name{calc_z_factor}
\alias{calc_z_factor}
\title{Z-factor per plate (sample-aware)}
\usage{
calc_z_factor(
  hts_raw,
  signal_col = "signal",
  nc_label = "NC",
  sample_label = "compound",
  z_factor_min = 0.5
)
}
\arguments{
\item{hts_raw}{An \code{hts_raw} object.}

\item{signal_col}{Name of the signal column. Default \code{"signal"}.}

\item{nc_label, sample_label}{Well-type labels for the negative control and
the test wells. Defaults \code{"NC"} and \code{"compound"}.}

\item{z_factor_min}{Acceptance threshold. Default \code{0.5}.}
}
\value{
A data frame with one row per plate: \code{plate_id}, \code{mean_sample},
\code{mean_nc}, \code{sd_sample}, \code{sd_nc}, \code{z_factor} and a logical \code{pass}.
}
\description{
The screening-window quality factor computed from the \strong{test-compound}
population against the negative control. \strong{What it is for:} unlike the
Z'-factor (controls only), the Z-factor tells you whether the assay window is
large enough relative to the spread of the actual library wells. Values above
\code{0.5} indicate an excellent screen and \code{0}--\code{0.5} a marginal one; it is
routinely low or negative in a single-point primary screen, because most
library compounds are inactive and therefore sit at the negative-control
level — that is expected, not a failure.
}
\examples{
calc_z_factor(read_hts_plate(hts_primary_raw))

}
\seealso{
\code{\link[=calc_z_prime]{calc_z_prime()}}, \code{\link[=calc_plate_qc]{calc_plate_qc()}}
}
